@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1997, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -26,13 +26,12 @@
  * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/box.c,v 3.7 1997/09/14 20:37:49 william Exp $
  */
-#ifndef lint
-static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/box.c,v 2.62 1995/07/10 05:48:15 william Exp $";
-#endif
 
 #include <stdio.h>
+#include <string.h>
 #include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -57,9 +56,11 @@ static char RCSid[] =
 #include "poly.e"
 #include "ps.e"
 #include "raster.e"
+#include "rect.e"
 #include "ruler.e"
 #include "select.e"
 #include "setup.e"
+#include "util.e"
 #include "xpixmap.e"
 
 int	boxDrawn = FALSE;
@@ -88,51 +89,44 @@ void DumpBoxPath (FP, ObjPtr, LtX, LtY, RbX, RbY, Width, Pen, Dash)
 {
    register int	i;
 
-   if (Width != 1) fprintf (FP, "   %1d setlinewidth\n", Width);
-   if (Dash != 0)
-   {
-      fprintf (FP, "   [");
-      for (i = 0; i < dashListLength[Dash]-1; i++)
-         fprintf (FP, "%1d ", (int)(dashList[Dash][i]));
-      fprintf (FP, "%1d] 0 setdash\n",
+   fprintf(FP, "   gsave\n");
+   if (!colorDump && useGray && Pen > BACKPAT) {
+      GrayCheck(Pen);
+      fprintf(FP, "      %s setgray\n", GrayStr(Pen));
+   }
+   fprintf(FP, "      newpath\n         %1d %1d moveto ", LtX, LtY);
+   fprintf(FP, "%1d %1d lineto ", RbX, LtY);
+   fprintf(FP, "%1d %1d lineto ", RbX, RbY);
+   fprintf(FP, "%1d %1d lineto\n", LtX, RbY);
+   fprintf(FP, "      closepath\n");
+
+   if (ObjPtr->ctm != NULL) {
+      fprintf(FP, "      tgiforigctm setmatrix\n");
+   }
+   if (Width != 1) fprintf(FP, "      %1d setlinewidth\n", Width);
+   if (Dash != 0) {
+      fprintf(FP, "      [");
+      for (i = 0; i < dashListLength[Dash]-1; i++) {
+         fprintf(FP, "%1d ", (int)(dashList[Dash][i]));
+      }
+      fprintf(FP, "%1d] 0 setdash\n",
             (int)(dashList[Dash][dashListLength[Dash]-1]));
    }
 
-   if (!colorDump && Pen > BACKPAT)
-   {
-      GrayCheck (Pen);
-      if (useGray)
-         fprintf (FP, "   %s setgray\n", GrayStr(Pen));
-      else
-      {
-         if (preDumpSetup) PSUseBWPattern();
-         fprintf (FP, "   pat%1d %s\n", Pen, patternStr);
+   switch (Pen) {
+   case SOLIDPAT: fprintf(FP, "      stroke\n"); break;
+   case BACKPAT: fprintf(FP, "      1 setgray stroke 0 setgray\n"); break;
+   default:
+      if (colorDump || !useGray) {
+         if (preDumpSetup) PSUseColorPattern();
+         fprintf(FP, "      flattenpath strokepath clip newpath\n");
+         DumpPatFill(FP, Pen, 8, ObjPtr->bbox, "      ");
+      } else {
+         fprintf(FP, "      stroke\n");
       }
+      break;
    }
-
-   fprintf (FP, "   newpath\n      %1d %1d moveto ", LtX, LtY);
-   fprintf (FP, "%1d %1d lineto ", RbX, LtY);
-   fprintf (FP, "%1d %1d lineto ", RbX, RbY);
-   fprintf (FP, "%1d %1d lineto\n", LtX, RbY);
-   fprintf (FP, "   closepath ");
-
-   switch (Pen)
-   {
-      case SOLIDPAT: fprintf (FP, "stroke\n"); break;
-      case BACKPAT: fprintf (FP, "1 setgray stroke 0 setgray\n"); break;
-      default:
-         if (colorDump)
-         {
-            if (preDumpSetup) PSUseColorPattern();
-            fprintf (FP, "flattenpath strokepath clip newpath\n");
-            DumpPatFill (FP, Pen, 8, ObjPtr->bbox, "   ");
-         }
-         else
-            fprintf (FP, "stroke\n");
-         break;
-   }
-   if (Dash != 0) fprintf (FP, "   [] 0 setdash\n");
-   if (Width != 1) fprintf (FP, "   1 setlinewidth\n");
+   fprintf(FP, "   grestore\n");
 }
  
 void DumpBoxObj (FP, ObjPtr)
@@ -142,11 +136,17 @@ void DumpBoxObj (FP, ObjPtr)
    register int	ltx, lty, rbx, rby;
    int		fill, width, pen, dash, color_index;
 
-   ltx = ObjPtr->obbox.ltx;
-   lty = ObjPtr->obbox.lty;
-   rbx = ObjPtr->obbox.rbx;
-   rby = ObjPtr->obbox.rby;
-
+   if (ObjPtr->ctm == NULL) {
+      ltx = ObjPtr->obbox.ltx;
+      lty = ObjPtr->obbox.lty;
+      rbx = ObjPtr->obbox.rbx;
+      rby = ObjPtr->obbox.rby;
+   } else {
+      ltx = ObjPtr->orig_obbox.ltx;
+      lty = ObjPtr->orig_obbox.lty;
+      rbx = ObjPtr->orig_obbox.rbx;
+      rby = ObjPtr->orig_obbox.rby;
+   }
    fill = ObjPtr->detail.b->fill;
    pen = ObjPtr->detail.b->pen;
    width = ObjPtr->detail.b->width;
@@ -155,11 +155,22 @@ void DumpBoxObj (FP, ObjPtr)
    if (fill == NONEPAT && pen == NONEPAT) return;
 
    fprintf (FP, "%% BOX\n");
+   if (ObjPtr->ctm != NULL) {
+      float m[6];
+
+      fprintf(FP, "gsave\n");
+      m[CTM_SX] = ((float)ObjPtr->ctm->m[CTM_SX])/((float)1000.0);
+      m[CTM_SY] = ((float)ObjPtr->ctm->m[CTM_SY])/((float)1000.0);
+      m[CTM_SIN] = ((float)ObjPtr->ctm->m[CTM_SIN])/((float)1000.0);
+      m[CTM_MSIN] = ((float)ObjPtr->ctm->m[CTM_MSIN])/((float)1000.0);
+      fprintf (FP, "   %1d %1d translate\n", ObjPtr->x, ObjPtr->y);
+      fprintf (FP, "   [%.3f %.3f %.3f %.3f %1d %1d] concat\n",
+            m[CTM_SX], m[CTM_SIN], m[CTM_MSIN], m[CTM_SY],
+            ObjPtr->ctm->m[CTM_TX], ObjPtr->ctm->m[CTM_TY]);
+      fprintf (FP, "   %1d neg %1d neg translate\n", ObjPtr->x, ObjPtr->y);
+   }
    color_index = ObjPtr->color;
-   if (colorDump)
-      DumpRGBColorLine(FP, color_index, 0, TRUE);
-   else
-      fprintf (FP, "0 setgray\n");
+   DumpRGBColorLine(FP, color_index, 0, TRUE);
 
    switch (fill)
    {
@@ -179,66 +190,54 @@ void DumpBoxObj (FP, ObjPtr)
          fprintf (FP, "%1d %1d lineto ", rbx, rby);
          fprintf (FP, "%1d %1d lineto\n", ltx, rby);
          fprintf (FP, "closepath 1 setgray fill\n");
-         if (colorDump)
-            DumpRGBColorLine(FP, color_index, 3, TRUE);
-         else
-            fprintf (FP, "0 setgray\n");
+         DumpRGBColorLine(FP, color_index, 3, TRUE);
          break;
       default:
          /* patterned */
          fprintf (FP, "gsave\n");
-         if (!colorDump)
-         {
-            GrayCheck (fill);
-            if (useGray)
-               fprintf (FP, "   %s setgray\n", GrayStr(fill));
-            else
-            {
-               if (preDumpSetup) PSUseBWPattern();
-               fprintf (FP, "   pat%1d %s\n", fill, patternStr);
-            }
-         }
-         else
-         {
+         if (colorDump || !useGray) {
             fprintf (FP, "   newpath\n   %1d %1d moveto ", ltx, lty);
             fprintf (FP, "%1d %1d lineto ", rbx, lty);
             fprintf (FP, "%1d %1d lineto ", rbx, rby);
             fprintf (FP, "%1d %1d lineto\n", ltx, rby);
             fprintf (FP, "   closepath 1 setgray fill\n");
             DumpRGBColorLine(FP, color_index, 3, TRUE);
+         } else {
+            GrayCheck (fill);
+            fprintf (FP, "   %s setgray\n", GrayStr(fill));
          }
          fprintf (FP, "   newpath\n      %1d %1d moveto ", ltx, lty);
          fprintf (FP, "%1d %1d lineto ", rbx, lty);
          fprintf (FP, "%1d %1d lineto ", rbx, rby);
          fprintf (FP, "%1d %1d lineto\n", ltx, rby);
-         if (colorDump)
-         {
+         if (colorDump || !useGray) {
             if (preDumpSetup) PSUseColorPattern();
             fprintf (FP, "   closepath eoclip newpath\n");
             DumpPatFill (FP, fill, 8, ObjPtr->bbox, "   ");
-         }
-         else
+         } else {
             fprintf (FP, "   closepath fill\n");
+         }
          fprintf (FP, "grestore\n");
          break;
    }
 
-   if (pen == NONEPAT) { fprintf (FP, "\n"); return; }
+   if (pen == NONEPAT) {
+      if (ObjPtr->ctm != NULL) fprintf(FP, "grestore\n");
+      fprintf (FP, "\n");
+      return;
+   }
 
    fprintf (FP, "gsave\n");
-   fprintf (FP, "   1.415 setmiterlimit\n");
+   fprintf (FP, "   10 setmiterlimit\n");
 
-   if (colorDump && pen > BACKPAT)
-   {
+   if ((colorDump || !useGray) && pen > BACKPAT) {
       DumpBoxPath (FP, ObjPtr, ltx, lty, rbx, rby, width, BACKPAT, 0);
-      if (colorDump)
-         DumpRGBColorLine(FP, color_index, 3, TRUE);
+      DumpRGBColorLine(FP, color_index, 3, TRUE);
    }
    DumpBoxPath (FP, ObjPtr, ltx, lty, rbx, rby, width, pen, dash);
 
-   fprintf (FP, "   1 setmiterlimit\n");
    fprintf (FP, "grestore\n");
-
+   if (ObjPtr->ctm != NULL) fprintf(FP, "grestore\n");
    fprintf (FP, "\n");
 }
 
@@ -269,18 +268,22 @@ void DrawBoxObj (win, XOff, YOff, ObjPtr)
 
    if (fill != 0)
    {
-      values.foreground = (fill == 2) ? myBgPixel : pixel;
+      values.foreground = GetDrawingBgPixel(fill, pixel);
       values.function = GXcopy;
       values.fill_style = FillOpaqueStippled;
       values.stipple = patPixmap[fill];
       XChangeGC (mainDisplay, drawGC,
             GCForeground | GCFunction | GCFillStyle | GCStipple, &values);
-      XFillRectangle (mainDisplay, win, drawGC, ltx, lty, rbx-ltx, rby-lty);
+      if (ObjPtr->ctm != NULL)
+         XFillPolygon (mainDisplay, win, drawGC, ObjPtr->rotated_obbox, 5,
+               Convex, CoordModeOrigin);
+      else
+         XFillRectangle (mainDisplay, win, drawGC, ltx, lty, rbx-ltx, rby-lty);
    }
 
    if (pen != 0)
    {
-      values.foreground = (pen == 2) ? myBgPixel : pixel;
+      values.foreground = GetDrawingBgPixel(pen, pixel);
       values.function = GXcopy;
       values.fill_style = FillOpaqueStippled;
       values.stipple = patPixmap[pen];
@@ -300,49 +303,60 @@ void DrawBoxObj (win, XOff, YOff, ObjPtr)
       XChangeGC (mainDisplay, drawGC,
             GCForeground | GCFunction | GCFillStyle | GCStipple | GCLineWidth |
             GCLineStyle | GCJoinStyle, &values);
-      XDrawRectangle (mainDisplay, win, drawGC, ltx, lty, rbx-ltx, rby-lty);
+      if (ObjPtr->ctm != NULL)
+         XDrawLines (mainDisplay, win, drawGC, ObjPtr->rotated_obbox, 5,
+               CoordModeOrigin);
+      else
+         XDrawRectangle (mainDisplay, win, drawGC, ltx, lty, rbx-ltx, rby-lty);
    }
 }
 
-static
-void CreateBoxObj (X1, Y1, X2, Y2)
-   int	X1, Y1, X2, Y2;
+int CreateBoxObj(X1, Y1, X2, Y2, CreateAbsolute)
+   int X1, Y1, X2, Y2, CreateAbsolute;
 {
-   struct BoxRec	* box_ptr;
-   struct ObjRec	* obj_ptr;
-   int			width, w, ltx, lty, rbx, rby;
+   struct BoxRec *box_ptr;
+   struct ObjRec *obj_ptr;
+   int width, w, ltx, lty, rbx, rby;
 
-   box_ptr = (struct BoxRec *) calloc (1, sizeof(struct BoxRec));
+   if (X1 == X2 && Y1 == Y2) return FALSE;
+   box_ptr = (struct BoxRec *)malloc(sizeof(struct BoxRec));
+   if (box_ptr == NULL) FailAllocMessage();
+   memset(box_ptr, 0, sizeof(struct BoxRec));
    box_ptr->fill = objFill;
    box_ptr->width = width = curWidthOfLine[lineWidth];
+   UtilStrCpy(box_ptr->width_spec, sizeof(box_ptr->width_spec),
+         curWidthOfLineSpec[lineWidth]);
    box_ptr->pen = penPat;
    box_ptr->dash = curDash;
 
-   obj_ptr = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
+   obj_ptr = (struct ObjRec *)malloc(sizeof(struct ObjRec));
+   if (obj_ptr == NULL) FailAllocMessage();
+   memset(obj_ptr, 0, sizeof(struct ObjRec));
 
-   if (X1 < X2)
-      if (Y1 < Y2)
-      {
+   if (X1 < X2) {
+      if (Y1 < Y2) {
          ltx = X1; lty = Y1; rbx = X2; rby = Y2;
-      }
-      else
-      {
+      } else {
          ltx = X1; lty = Y2; rbx = X2; rby = Y1;
       }
-   else
-      if (Y1 < Y2)
-      {
+   } else {
+      if (Y1 < Y2) {
          ltx = X2; lty = Y1; rbx = X1; rby = Y2;
-      }
-      else
-      {
+      } else {
          ltx = X2; lty = Y2; rbx = X1; rby = Y1;
       }
-
-   obj_ptr->bbox.ltx = obj_ptr->obbox.ltx = obj_ptr->x = ABS_X(ltx);
-   obj_ptr->bbox.lty = obj_ptr->obbox.lty = obj_ptr->y = ABS_Y(lty);
-   obj_ptr->bbox.rbx = obj_ptr->obbox.rbx = ABS_X(rbx);
-   obj_ptr->bbox.rby = obj_ptr->obbox.rby = ABS_Y(rby);
+   }
+   if (CreateAbsolute) {
+      obj_ptr->bbox.ltx = obj_ptr->obbox.ltx = obj_ptr->x = ltx;
+      obj_ptr->bbox.lty = obj_ptr->obbox.lty = obj_ptr->y = lty;
+      obj_ptr->bbox.rbx = obj_ptr->obbox.rbx = rbx;
+      obj_ptr->bbox.rby = obj_ptr->obbox.rby = rby;
+   } else {
+      obj_ptr->bbox.ltx = obj_ptr->obbox.ltx = obj_ptr->x = ABS_X(ltx);
+      obj_ptr->bbox.lty = obj_ptr->obbox.lty = obj_ptr->y = ABS_Y(lty);
+      obj_ptr->bbox.rbx = obj_ptr->obbox.rbx = ABS_X(rbx);
+      obj_ptr->bbox.rby = obj_ptr->obbox.rby = ABS_Y(rby);
+   }
    w = HALF_W(width);
    obj_ptr->bbox.ltx -= w;
    obj_ptr->bbox.lty -= w;
@@ -356,7 +370,10 @@ void CreateBoxObj (X1, Y1, X2, Y2)
    obj_ptr->locked = FALSE;
    obj_ptr->detail.b = box_ptr;
    obj_ptr->fattr = obj_ptr->lattr = NULL;
-   AddObj (NULL, topObj, obj_ptr);
+   obj_ptr->ctm = NULL;
+   obj_ptr->invisible = FALSE;
+   AddObj(NULL, topObj, obj_ptr);
+   return TRUE;
 }
 
 static XComposeStatus	c_stat;
@@ -395,7 +412,7 @@ void ContinueBox (OrigX, OrigY)
    PixelToMeasurementUnit(h_buf, 0);
    PixelToMeasurementUnit(x_buf, ABS_X(grid_x));
    PixelToMeasurementUnit(y_buf, ABS_Y(grid_y));
-   sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+   sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
    StartShowMeasureCursor (grid_x, grid_y, buf, TRUE);
    BeginIntervalRulers (grid_x, grid_y, grid_x, grid_y);
    XGrabPointer (mainDisplay, drawWindow, FALSE,
@@ -417,7 +434,7 @@ void ContinueBox (OrigX, OrigY)
          PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(saved_y-OrigY)));
          PixelToMeasurementUnit(x_buf, ABS_X(saved_x));
          PixelToMeasurementUnit(y_buf, ABS_Y(saved_y));
-         sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+         sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
          EndShowMeasureCursor (saved_x, saved_y, buf, TRUE);
          done = TRUE;
       }
@@ -454,7 +471,7 @@ void ContinueBox (OrigX, OrigY)
             PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(saved_y-OrigY)));
             PixelToMeasurementUnit(x_buf, ABS_X(saved_x));
             PixelToMeasurementUnit(y_buf, ABS_Y(saved_y));
-            sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+            sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
             ShowMeasureCursor (saved_x, saved_y, buf, TRUE);
             MyBox (drawWindow, drawGC, OrigX, OrigY, saved_x, saved_y);
             saved_x = grid_x;
@@ -464,7 +481,7 @@ void ContinueBox (OrigX, OrigY)
             PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(saved_y-OrigY)));
             PixelToMeasurementUnit(x_buf, ABS_X(saved_x));
             PixelToMeasurementUnit(y_buf, ABS_Y(saved_y));
-            sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+            sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
             ShowMeasureCursor (saved_x, saved_y, buf, TRUE);
          }
          DrawIntervalRulers (OrigX, OrigY, grid_x, grid_y);
@@ -473,11 +490,12 @@ void ContinueBox (OrigX, OrigY)
       else if (input.type == KeyPress)
       {
          KeySym	key_sym;
-         char	s[80];
+         char s[80];
+         int has_ch;
 
-         XLookupString (&(input.xkey), s, 80-1, &key_sym, &c_stat);
-         TranslateKeys (s, &key_sym);
-         if (s[0] == '\033' && (key_sym & 0xff) == '\033')
+         has_ch = XLookupString(&(input.xkey), s, sizeof(s), &key_sym, &c_stat);
+         TranslateKeys(s, &key_sym);
+         if (CharIsESC(&(input.xkey), s, key_sym, &has_ch))
          {
             XUngrabPointer (mainDisplay, CurrentTime);
             MyBox (drawWindow, drawGC, OrigX, OrigY, saved_x, saved_y);
@@ -486,7 +504,7 @@ void ContinueBox (OrigX, OrigY)
             PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(saved_y-OrigY)));
             PixelToMeasurementUnit(x_buf, ABS_X(saved_x));
             PixelToMeasurementUnit(y_buf, ABS_Y(saved_y));
-            sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+            sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
             EndShowMeasureCursor (saved_x, saved_y, buf, TRUE);
             abort = TRUE;
             done = TRUE;
@@ -495,7 +513,7 @@ void ContinueBox (OrigX, OrigY)
    }
    if (!abort && OrigX != grid_x && OrigY != grid_y)
    {
-      CreateBoxObj (OrigX, OrigY, grid_x, grid_y);
+      CreateBoxObj (OrigX, OrigY, grid_x, grid_y, FALSE);
       RecordNewObjCmd ();
       DrawBoxObj (drawWindow, drawOrigX, drawOrigY, topObj);
       boxDrawn = TRUE;
@@ -521,32 +539,57 @@ void DrawBox (input)
    }
 }
 
-void SaveBoxObj (FP, ObjPtr)
-   FILE			* FP;
-   struct ObjRec	* ObjPtr;
+void SaveBoxObj(FP, ObjPtr)
+   FILE *FP;
+   struct ObjRec *ObjPtr;
 {
-   if (fprintf (FP, "box('%s',", colorMenuItems[ObjPtr->color]) == EOF)
+   struct BoxRec *box_ptr=ObjPtr->detail.b;
+
+   if (fprintf(FP, "box('%s',", colorMenuItems[ObjPtr->color]) == EOF) {
       writeFileFailed = TRUE;
-   if (fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,",
+   }
+   if (fprintf(FP,
+         "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,'%s',",
          ObjPtr->obbox.ltx, ObjPtr->obbox.lty, ObjPtr->obbox.rbx,
-         ObjPtr->obbox.rby, ObjPtr->detail.b->fill, ObjPtr->detail.b->width,
-         ObjPtr->detail.b->pen, ObjPtr->id, ObjPtr->detail.b->dash,
-         ObjPtr->rotation, ObjPtr->locked) == EOF)
+         ObjPtr->obbox.rby, box_ptr->fill, box_ptr->width, box_ptr->pen,
+         ObjPtr->id, box_ptr->dash, ObjPtr->rotation, ObjPtr->locked,
+         ObjPtr->ctm!=NULL, ObjPtr->invisible, box_ptr->width_spec) == EOF) {
       writeFileFailed = TRUE;
-   SaveAttrs (FP, ObjPtr->lattr);
-   if (fprintf (FP, ")") == EOF) writeFileFailed = TRUE;
+   }
+   if (ObjPtr->ctm != NULL && fprintf(FP,
+         "[\n    %1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d],",
+         ObjPtr->x, ObjPtr->y,
+         ObjPtr->orig_obbox.ltx, ObjPtr->orig_obbox.lty,
+         ObjPtr->orig_obbox.rbx, ObjPtr->orig_obbox.rby,
+         ObjPtr->ctm->m[CTM_SX], ObjPtr->ctm->m[CTM_SIN],
+         ObjPtr->ctm->m[CTM_MSIN], ObjPtr->ctm->m[CTM_SY],
+         ObjPtr->ctm->m[CTM_TX], ObjPtr->ctm->m[CTM_TY]) == EOF) {
+      writeFileFailed = TRUE;
+   }
+   SaveAttrs(FP, ObjPtr->lattr);
+   if (fprintf(FP, ")") == EOF) writeFileFailed = TRUE;
 }
 
 #define GETVALUE(val,name) ScanValue("%d", &(val), name, "box")
+#define GETSTRNG(val,name) ScanValue("%s", (val), name, "box")
 
-void ReadBoxObj (Inbuf, ObjPtr)
+void ReadBoxObj (FP, Inbuf, ObjPtr)
+   FILE			* FP;
    char			* Inbuf;
    struct ObjRec	* * ObjPtr;
 {
    struct BoxRec	* box_ptr;
-   char			color_str[40], * s;
+   char			color_str[40], * s, width_spec[40];
    int			ltx, lty, rbx, rby, fill, width, pen, dash, w, id=0;
    int			new_alloc, rotation, locked=FALSE;
+   int			transformed=FALSE, invisible=FALSE;
+#ifdef _TGIF_WB
+
+   char                 aux[10000]; 
+
+   strcpy(aux,Inbuf);
+
+#endif /* _TGIF_WB */
 
    *ObjPtr = NULL;
 
@@ -557,6 +600,7 @@ void ReadBoxObj (Inbuf, ObjPtr)
 
    dash = 0;
    rotation = 0;
+   *width_spec = '\0';
    if (fileVersion <= 5)
    {
       if (GETVALUE (ltx,      "ltx") == INVALID ||
@@ -638,7 +682,7 @@ void ReadBoxObj (Inbuf, ObjPtr)
       }
       if (id >= objId) objId = id+1;
    }
-   else
+   else if (fileVersion <= 32)
    {
       if (GETVALUE (ltx,      "ltx") == INVALID ||
           GETVALUE (lty,      "lty") == INVALID ||
@@ -650,12 +694,73 @@ void ReadBoxObj (Inbuf, ObjPtr)
           GETVALUE (id,       "id") == INVALID ||
           GETVALUE (dash,     "dash") == INVALID ||
           GETVALUE (rotation, "rotation") == INVALID ||
-          GETVALUE (locked, "locked") == INVALID)
+          GETVALUE (locked,   "locked") == INVALID)
       {
          return;
       }
       if (id >= objId) objId = id+1;
    }
+   else
+   {
+#ifdef _TGIF_DBG
+      if (GETVALUE (ltx,         "ltx") == INVALID ||
+          GETVALUE (lty,         "lty") == INVALID ||
+          GETVALUE (rbx,         "rbx") == INVALID ||
+          GETVALUE (rby,         "rby") == INVALID ||
+          GETVALUE (fill,        "fill") == INVALID ||
+          GETVALUE (width,       "width") == INVALID ||
+          GETVALUE (pen,         "pen") == INVALID ||
+          GETVALUE (id,          "id") == INVALID ||
+          GETVALUE (dash,        "dash") == INVALID ||
+          GETVALUE (rotation,    "rotation") == INVALID ||
+          GETVALUE (locked,      "locked") == INVALID ||
+          GETVALUE (transformed, "transformed") == INVALID ||
+          GETVALUE (invisible,   "invisible") == INVALID)
+      {
+         return;
+      }
+      if (id >= objId) objId = id+1;
+      if (GETSTRNG(width_spec,  "width_spec") != INVALID) {
+         UtilRemoveQuotes(width_spec);
+      } else {
+         sprintf(width_spec, "%1d", width);
+         fprintf(stderr, "Bad width_spec in box... Fix this file!\n");
+      }
+#else /* ~_TGIF_DBG */
+      if (GETVALUE (ltx,         "ltx") == INVALID ||
+          GETVALUE (lty,         "lty") == INVALID ||
+          GETVALUE (rbx,         "rbx") == INVALID ||
+          GETVALUE (rby,         "rby") == INVALID ||
+          GETVALUE (fill,        "fill") == INVALID ||
+          GETVALUE (width,       "width") == INVALID ||
+          GETVALUE (pen,         "pen") == INVALID ||
+          GETVALUE (id,          "id") == INVALID ||
+          GETVALUE (dash,        "dash") == INVALID ||
+          GETVALUE (rotation,    "rotation") == INVALID ||
+          GETVALUE (locked,      "locked") == INVALID ||
+          GETVALUE (transformed, "transformed") == INVALID ||
+          GETVALUE (invisible,   "invisible") == INVALID ||
+          GETSTRNG (width_spec,  "width_spec") == INVALID)
+      {
+         return;
+      }
+      if (id >= objId) objId = id+1;
+      UtilRemoveQuotes(width_spec);
+#endif /* _TGIF_DBG */
+   }
+   if (fileVersion <= 16 && width <= 6) width = origWidthOfLine[width];
+   if (fileVersion <= 32) {
+      sprintf(width_spec, "%1d", width);
+   }
+   fill = UpgradePenFill (fill);
+   pen = UpgradePenFill (pen);
+
+   *ObjPtr = (struct ObjRec *)malloc(sizeof(struct ObjRec));
+   if (*ObjPtr == NULL) FailAllocMessage();
+   memset(*ObjPtr, 0, sizeof(struct ObjRec));
+   box_ptr = (struct BoxRec *)malloc(sizeof(struct BoxRec));
+   if (box_ptr == NULL) FailAllocMessage();
+   memset(box_ptr, 0, sizeof(struct BoxRec));
 
    if (ltx > rbx || lty > rby)
    {
@@ -666,20 +771,14 @@ void ReadBoxObj (Inbuf, ObjPtr)
       ltx = tmp_ltx; lty = tmp_lty; rbx = tmp_rbx; rby = tmp_rby;
    }
 
-   if (fileVersion <= 16 && width <= 6) width = origWidthOfLine[width];
-   fill = UpgradePenFill (fill);
-   pen = UpgradePenFill (pen);
-
-   *ObjPtr = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
-   box_ptr = (struct BoxRec *) calloc (1, sizeof(struct BoxRec));
-
    box_ptr->fill = fill;
    box_ptr->width = width;
+   UtilStrCpy(box_ptr->width_spec, sizeof(box_ptr->width_spec), width_spec);
    box_ptr->pen = pen;
    box_ptr->dash = dash;
    (*ObjPtr)->x = ltx;
    (*ObjPtr)->y = lty;
-   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc, TRUE);
+   (*ObjPtr)->color = QuickFindColorIndex(*ObjPtr, color_str, &new_alloc, TRUE);
    (*ObjPtr)->dirty = FALSE;
    (*ObjPtr)->id = id;
    (*ObjPtr)->rotation = rotation;
@@ -695,11 +794,62 @@ void ReadBoxObj (Inbuf, ObjPtr)
    (*ObjPtr)->bbox.rbx = rbx + w;
    (*ObjPtr)->bbox.rby = rby + w;
    (*ObjPtr)->detail.b = box_ptr;
+   (*ObjPtr)->ctm = NULL;
+   (*ObjPtr)->invisible = invisible;
+
+   if (fileVersion >= 33 && transformed)
+   {
+      int real_x=0, real_y=0;
+      struct BBRec orig_obbox;
+      char inbuf[MAXSTRING+1];
+      struct XfrmMtrxRec *ctm;
+
+#ifdef _TGIF_WB
+
+      s = FindChar ((int)'[', aux);
+      s++;
+      if (!bufferAsFileForWhiteBoard) {
+         fgets(inbuf, MAXSTRING, FP);
+      }
+
+#else /* ~_TGIF_WB */
+
+      fgets(inbuf, MAXSTRING, FP);
+
+#endif /* _TGIF_WB */
+
+      scanLineNum++;
+      InitScan(inbuf, "\t\n, ");
+
+      ctm = (struct XfrmMtrxRec *)malloc(sizeof(struct XfrmMtrxRec));
+      if (ctm == NULL) FailAllocMessage();
+      if (GETVALUE(real_x,           "real_x") == INVALID ||
+          GETVALUE(real_y,           "real_y") == INVALID ||
+          GETVALUE(orig_obbox.ltx,   "orig_obbox.ltx") == INVALID ||
+          GETVALUE(orig_obbox.lty,   "orig_obbox.lty") == INVALID ||
+          GETVALUE(orig_obbox.rbx,   "orig_obbox.rbx") == INVALID ||
+          GETVALUE(orig_obbox.rby,   "orig_obbox.rby") == INVALID ||
+          GETVALUE(ctm->m[CTM_SX],   "CTM_SX") == INVALID ||
+          GETVALUE(ctm->m[CTM_SIN],  "CTM_SIN") == INVALID ||
+          GETVALUE(ctm->m[CTM_MSIN], "CTM_MSIN") == INVALID ||
+          GETVALUE(ctm->m[CTM_SY],   "CTM_SY") == INVALID ||
+          GETVALUE(ctm->m[CTM_TX],   "CTM_TX") == INVALID ||
+          GETVALUE(ctm->m[CTM_TY],   "CTM_TY") == INVALID) {
+         return;
+      }
+      (*ObjPtr)->ctm = ctm;
+      if (ctm != NULL) {
+         memcpy(&(*ObjPtr)->orig_obbox, &orig_obbox, sizeof(struct BBRec));
+         (*ObjPtr)->x = real_x;
+         (*ObjPtr)->y = real_y;
+         GetTransformedOBBoxOffsetVs(*ObjPtr, (*ObjPtr)->rotated_obbox);
+      }
+   }
 }
 
 void FreeBoxObj (ObjPtr)
    struct ObjRec	* ObjPtr;
 {
-   cfree (ObjPtr->detail.b);
-   cfree (ObjPtr);
+   free(ObjPtr->detail.b);
+   free(ObjPtr);
 }
