@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,7 +29,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/cmd.c,v 2.37 1995/07/11 04:09:57 william Exp $";
+      "@(#)$Header: /n/opus/u/guest/william/src/tgif/v3/RCS/cmd.c,v 3.0 1996/05/06 16:04:09 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -42,9 +42,11 @@ static char RCSid[] =
 #ifndef _NO_EXTERN
 #include "cmd.e"
 #endif
+#include "color.e"
 #include "dialog.e"
 #include "drawing.e"
 #include "dup.e"
+#include "imgproc.e"
 #include "mark.e"
 #include "move.e"
 #include "msg.e"
@@ -53,15 +55,27 @@ static char RCSid[] =
 #include "select.e"
 #include "setup.e"
 #include "stk.e"
+#include "xpixmap.e"
+
+#ifdef _TGIF_WB
+#include "wb1.e"
+#endif /* _TGIF_WB */
 
 int		recordCmdIncludeTgifObj=FALSE;
+int		recordCmdUsesNewColormap=FALSE;
 int		undoingOrRedoing=FALSE;
 int		historyDepth=(-1), historyCount=0, defaultHistoryDepth=(-1);
-struct CmdRec	* firstCmd=NULL, * lastCmd=NULL, * curCmd=NULL;
+struct CmdRec	*firstCmd=NULL, *lastCmd=NULL, *curCmd=NULL;
 
-static
-void InsertCmd (PrevCmd, NextCmd, CmdPtr)
-   struct CmdRec	* PrevCmd, * NextCmd, * CmdPtr;
+struct SelRec	*topSelBeforeInCmd=NULL, *botSelBeforeInCmd=NULL;
+int		*stackingPosition=NULL;
+int		stackingCount=0;
+int		composingCommand=FALSE;
+Colormap	preparedColormap=None;
+
+
+void InsertCmd(PrevCmd, NextCmd, CmdPtr)
+   struct CmdRec *PrevCmd, *NextCmd, *CmdPtr;
    /* add CmdPtr between PrevCmd and NextCmd */
 {
    CmdPtr->prev = PrevCmd;
@@ -78,18 +92,17 @@ void InsertCmd (PrevCmd, NextCmd, CmdPtr)
       NextCmd->prev = CmdPtr;
 }
 
-static
 void DeleteARedoRecord(CmdPtr, percent_start, percent_end)
    struct CmdRec *CmdPtr;
-   float percent_start, percent_end;
+   double percent_start, percent_end;
 {
    register struct SelRec *sel_ptr, *next_sel;
    struct CmdRec *cmd_ptr, *prev_cmd;
    int num_records=0;
-   float inc=0;
+   double inc=0;
 
-   if (CmdPtr->pos_before != NULL) cfree(CmdPtr->pos_before);
-   if (CmdPtr->pos_after != NULL) cfree(CmdPtr->pos_after);
+   if (CmdPtr->pos_before != NULL) free(CmdPtr->pos_before);
+   if (CmdPtr->pos_after != NULL) free(CmdPtr->pos_after);
 
    if (percent_start >= 0.0) {
       sprintf(gszMsgBox, "Flushing undo buffers... %1d%%",
@@ -102,58 +115,58 @@ void DeleteARedoRecord(CmdPtr, percent_start, percent_end)
       for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel) {
          next_sel = sel_ptr->next;
          if (CmdPtr->undone) FreeObj(sel_ptr->obj);
-         cfree(sel_ptr);
+         free(sel_ptr);
       }
       break;
    case CMD_DELETE:
       for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel) {
          next_sel = sel_ptr->next;
          if (!CmdPtr->undone) FreeObj(sel_ptr->obj);
-         cfree(sel_ptr);
+         free(sel_ptr);
       }
       break;
    case CMD_MOVE:
       for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel) {
          next_sel = sel_ptr->next;
-         cfree(sel_ptr);
+         free(sel_ptr);
       }
       break;
    case CMD_STRETCH:
       for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel) {
          next_sel = sel_ptr->next;
          FreeObj(sel_ptr->obj);
-         cfree(sel_ptr);
+         free(sel_ptr);
       }
       break;
    case CMD_REPLACE:
       for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel) {
          next_sel = sel_ptr->next;
          FreeObj(sel_ptr->obj);
-         cfree(sel_ptr);
+         free(sel_ptr);
       }
       break;
    case CMD_ONE_TO_MANY:
       for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel) {
          next_sel = sel_ptr->next;
          if (!CmdPtr->undone) FreeObj(sel_ptr->obj);
-         cfree(sel_ptr);
+         free(sel_ptr);
       }
       for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel) {
          next_sel = sel_ptr->next;
          if (CmdPtr->undone) FreeObj(sel_ptr->obj);
-         cfree(sel_ptr);
+         free(sel_ptr);
       }
       break;
    case CMD_MANY_TO_ONE:
       for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel) {
          next_sel = sel_ptr->next;
          if (!CmdPtr->undone) FreeObj(sel_ptr->obj);
-         cfree(sel_ptr);
+         free(sel_ptr);
       }
       for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel) {
          next_sel = sel_ptr->next;
          if (CmdPtr->undone) FreeObj(sel_ptr->obj);
-         cfree(sel_ptr);
+         free(sel_ptr);
       }
       break;
    case CMD_COMPOSITE:
@@ -162,14 +175,14 @@ void DeleteARedoRecord(CmdPtr, percent_start, percent_end)
             num_records++;
          }
          if (num_records > 0) {
-            inc = (percent_end-percent_start)/((float)num_records);
+            inc = (percent_end-percent_start)/((double)num_records);
          }
       }
       for (cmd_ptr=CmdPtr->last; cmd_ptr!=NULL; cmd_ptr=prev_cmd) {
          prev_cmd = cmd_ptr->prev;
          if (percent_start >= 0) {
             DeleteARedoRecord(cmd_ptr, percent_start,
-                  min(((float)percent_start+inc),((float)100.0)));
+                  min(((double)percent_start+inc),((double)100.0)));
             percent_start += inc;
          } else {
             DeleteARedoRecord(cmd_ptr, percent_start, percent_end);
@@ -177,22 +190,20 @@ void DeleteARedoRecord(CmdPtr, percent_start, percent_end)
       }
       break;
    }
-   cfree (CmdPtr);
+   free(CmdPtr);
 }
 
-static
-void ClearRedoRecords (CmdPtr)
-   struct CmdRec	* CmdPtr;
+void ClearRedoRecords(CmdPtr)
+   struct CmdRec *CmdPtr;
 {
-   register struct CmdRec	*cmd_ptr, *prev_cmd;
+   register struct CmdRec *cmd_ptr, *prev_cmd;
 
-   for (cmd_ptr=lastCmd; cmd_ptr != curCmd; cmd_ptr=prev_cmd)
-   {
+   for (cmd_ptr=lastCmd; cmd_ptr != curCmd; cmd_ptr=prev_cmd) {
       prev_cmd = cmd_ptr->prev;
-      DeleteARedoRecord (cmd_ptr, (-1.0), (-1.0));
+      DeleteARedoRecord(cmd_ptr, (-1.0), (-1.0));
       historyCount--;
    }
-   if ((lastCmd = curCmd) == NULL) firstCmd = NULL;
+   if ((lastCmd=curCmd) == NULL) firstCmd = NULL;
 }
 
 void CleanUpCmds ()
@@ -204,7 +215,7 @@ void CleanUpCmds ()
       num_records++;
    }
    if (num_records > 0) {
-      float inc=(100.0/((float)num_records)), percent_start=0.0;
+      double inc=(100.0/((double)num_records)), percent_start=0.0;
 
       ShowInterrupt(1);
       SaveStatusStrings();
@@ -212,7 +223,7 @@ void CleanUpCmds ()
             percent_start += inc) {
          prev_cmd = cmd_ptr->prev;
          DeleteARedoRecord(cmd_ptr, percent_start,
-               min(((float)percent_start+inc),((float)100.0)));
+               min(((double)percent_start+inc),((double)100.0)));
       }
       RestoreStatusStrings();
       HideInterrupt();
@@ -221,53 +232,52 @@ void CleanUpCmds ()
    historyCount = 0;
 }
 
-static
-void CopySel (from_top_sel, count, to_top_sel, to_bot_sel)
-   struct SelRec	* from_top_sel;
-   int			count;
-   struct SelRec	* * to_top_sel, * * to_bot_sel;
+void CopySel(from_top_sel, count, to_top_sel, to_bot_sel)
+   struct SelRec *from_top_sel;
+   int count;
+   struct SelRec **to_top_sel, **to_bot_sel;
 {
-   register struct SelRec	* sel_ptr, * to_sel_ptr;
+   register struct SelRec *sel_ptr, *to_sel_ptr;
 
    *to_top_sel = *to_bot_sel = NULL;
 
-   for (sel_ptr=from_top_sel; sel_ptr!=NULL; sel_ptr=sel_ptr->next, count--)
-   {
-      to_sel_ptr = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+   for (sel_ptr=from_top_sel; sel_ptr!=NULL; sel_ptr=sel_ptr->next, count--) {
+      to_sel_ptr = (struct SelRec *)malloc(sizeof(struct SelRec));
+      if (to_sel_ptr == NULL) FailAllocMessage();
       to_sel_ptr->next = NULL;
       to_sel_ptr->prev = *to_bot_sel;
       to_sel_ptr->obj = sel_ptr->obj;
-      if (*to_bot_sel == NULL)
+      if (*to_bot_sel == NULL) {
          *to_top_sel = to_sel_ptr;
-      else
+      } else {
          (*to_bot_sel)->next = to_sel_ptr;
+      }
       *to_bot_sel = to_sel_ptr;
    }
-   if (count != 0)
-   {
-      Msg ("Possible fatal error.  Please read the terminal error output.");
-      fprintf (stderr, "Warning:  count!=0 in CopySel().\n");
-      fprintf (stderr, "\nUndo()/Redo() may crash %s.\n", TOOL_NAME);
-      fprintf (stderr, "Safest thing to do is to save and exit.\n");
-      fprintf (stderr, "Please try to reproduce this error and\n");
-      fprintf (stderr, "\tsend bug report to william@cs.ucla.edu.\n");
-      sprintf (gszMsgBox, "%s.\n%s %s.\n\n%s.\n\n%s %s.",
+   if (count != 0) {
+      Msg("Possible fatal error.  Please read the terminal error output.");
+      fprintf(stderr, "Warning:  count!=0 in CopySel().\n");
+      fprintf(stderr, "\nUndo()/Redo() may crash %s.\n", TOOL_NAME);
+      fprintf(stderr, "Safest thing to do is to save and exit.\n");
+      fprintf(stderr, "Please try to reproduce this error and\n");
+      fprintf(stderr, "\tsend bug report to william@cs.ucla.edu.\n");
+      sprintf(gszMsgBox, "%s.\n%s %s.\n\n%s.\n\n%s %s.",
             "Warning:  count!=0 in CopySel()",
             "Undo()/Redo() may crash", TOOL_NAME,
             "Safest thing to do is to save and exit",
             "Please try to reproduce this error and",
             "send bug report to william@cs.ucla.edu");
-      MsgBox (gszMsgBox, TOOL_NAME, STOP_MB);
-      XFlush (mainDisplay);
-      XSync (mainDisplay, False);
+      MsgBox(gszMsgBox, TOOL_NAME, STOP_MB);
+      XFlush(mainDisplay);
+      XSync(mainDisplay, False);
    }
 }
 
 static
-void LinkJustTheObjects (TopSel, BotSel)
-   struct SelRec	* TopSel, * BotSel;
+void LinkJustTheObjects(TopSel, BotSel)
+   struct SelRec *TopSel, *BotSel;
 {
-   register struct SelRec	* sel_ptr, * sel_ptr_next;
+   register struct SelRec *sel_ptr, *sel_ptr_next;
 
    sel_ptr=TopSel;
    sel_ptr_next=TopSel->next;
@@ -281,12 +291,12 @@ void LinkJustTheObjects (TopSel, BotSel)
 }
 
 static
-void UndoNewCmd (CmdPtr)
-   struct CmdRec	* CmdPtr;
+void UndoNewCmd(CmdPtr)
+   struct CmdRec *CmdPtr;
 {
-   register struct SelRec	* sel_ptr;
-   register struct ObjRec	* obj_ptr, * next_obj;
-   int				pos, count;
+   register struct SelRec *sel_ptr;
+   register struct ObjRec *obj_ptr, *next_obj;
+   int pos, count;
 
    topSel = CmdPtr->top_after;
    botSel = CmdPtr->bot_after;
@@ -316,10 +326,10 @@ void UndoNewCmd (CmdPtr)
 }
 
 static
-void RedoNewCmd (CmdPtr)
-   struct CmdRec	* CmdPtr;
+void RedoNewCmd(CmdPtr)
+   struct CmdRec *CmdPtr;
 {
-   struct ObjRec	* saved_top_obj, * saved_bot_obj;
+   struct ObjRec *saved_top_obj, *saved_bot_obj;
 
    CopySel (CmdPtr->top_after, CmdPtr->count_after, &topSel, &botSel);
 
@@ -352,14 +362,14 @@ void RedoNewCmd (CmdPtr)
 }
 
 static
-void UndoDeleteCmd (CmdPtr)
-   struct CmdRec	* CmdPtr;
+void UndoDeleteCmd(CmdPtr)
+   struct CmdRec *CmdPtr;
 {
-   register struct SelRec	* sel_ptr;
-   register struct ObjRec	* obj_ptr, * next_obj;
-   register int			pos;
-   struct ObjRec		* saved_top_obj, * saved_bot_obj;
-   int				count;
+   register struct SelRec *sel_ptr;
+   register struct ObjRec *obj_ptr, *next_obj;
+   register int pos;
+   struct ObjRec *saved_top_obj, *saved_bot_obj;
+   int count;
 
    CopySel (CmdPtr->top_before, CmdPtr->count_before, &topSel, &botSel);
 
@@ -404,12 +414,12 @@ void UndoDeleteCmd (CmdPtr)
 }
 
 static
-void RedoDeleteCmd (CmdPtr)
-   struct CmdRec	* CmdPtr;
+void RedoDeleteCmd(CmdPtr)
+   struct CmdRec *CmdPtr;
 {
-   register struct SelRec	* sel_ptr;
-   register struct ObjRec	* obj_ptr, * next_obj;
-   int				pos, count;
+   register struct SelRec *sel_ptr;
+   register struct ObjRec *obj_ptr, *next_obj;
+   int pos, count;
 
    topSel = CmdPtr->top_before;
    botSel = CmdPtr->bot_before;
@@ -439,12 +449,12 @@ void RedoDeleteCmd (CmdPtr)
 }
 
 static
-void UndoOrRedoMoveCmd (CmdPtr)
-   struct CmdRec	* CmdPtr;
+void UndoOrRedoMoveCmd(CmdPtr)
+   struct CmdRec *CmdPtr;
 {
-   register struct SelRec	* sel_ptr;
-   register struct ObjRec	* obj_ptr, * next_obj;
-   int				pos, count, dx, dy;
+   register struct SelRec *sel_ptr;
+   register struct ObjRec *obj_ptr, *next_obj;
+   int pos, count, dx, dy;
 
    dx = (CmdPtr->undone) ? CmdPtr->dx : -(CmdPtr->dx);
    dy = (CmdPtr->undone) ? CmdPtr->dy : -(CmdPtr->dy);
@@ -477,16 +487,193 @@ void UndoOrRedoMoveCmd (CmdPtr)
 }
 
 static
-void UndoOrRedoReplaceCmd (CmdPtr, HighLightSingleObj)
-   struct CmdRec	* CmdPtr;
-   int			HighLightSingleObj;
+void UpdateXPmObjectsInACmd(CmdPtr)
+   struct CmdRec *CmdPtr;
 {
-   register struct ObjRec	* obj_ptr, * next_obj;
-   register struct SelRec	* sel_ptr;
-   struct SelRec		* saved_top_sel, * saved_bot_sel;
-   struct ObjRec		* saved_top_obj, * saved_bot_obj;
-   int				ltx, lty, rbx, rby, pos, count;
-   int				* pos_table, max_count;
+   struct CmdRec *cmd_ptr;
+   struct SelRec *sel_ptr;
+
+   switch (CmdPtr->type) {
+   case CMD_COMPOSITE:
+      for (cmd_ptr=CmdPtr->first; cmd_ptr!=NULL; cmd_ptr=cmd_ptr->next) {
+         UpdateXPmObjectsInACmd(cmd_ptr);
+      }
+      break;
+   case CMD_NEW:
+      for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
+         UpdateXPmObjects(sel_ptr->obj);
+      }
+      break;
+   case CMD_DELETE: break;
+   case CMD_MOVE: break;
+   case CMD_STRETCH: break;
+   case CMD_REPLACE:
+      for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
+         UpdateXPmObjects(sel_ptr->obj);
+      }
+      break;
+   case CMD_ONE_TO_MANY: break;
+   case CMD_MANY_TO_ONE:
+      for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
+         UpdateXPmObjects(sel_ptr->obj);
+      }
+      for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
+         UpdateXPmObjects(sel_ptr->obj);
+      }
+      break;
+   case CMD_GOTO_PAGE: break;
+   }
+}
+
+static
+int UpdatePixelInACmd(CmdPtr, old_color_str)
+   struct CmdRec *CmdPtr;
+   char **old_color_str;
+{
+   struct CmdRec *cmd_ptr;
+   struct SelRec *sel_ptr;
+
+   switch (CmdPtr->type) {
+   case CMD_COMPOSITE:
+      for (cmd_ptr=CmdPtr->first; cmd_ptr!=NULL; cmd_ptr=cmd_ptr->next) {
+         UpdatePixelInACmd(cmd_ptr, old_color_str);
+      }
+      break;
+   case CMD_NEW:
+      for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
+         UpdatePixel(sel_ptr->obj, old_color_str);
+      }
+      break;
+   case CMD_DELETE: break;
+   case CMD_MOVE: break;
+   case CMD_STRETCH: break;
+   case CMD_REPLACE:
+      for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
+         UpdatePixel(sel_ptr->obj, old_color_str);
+      }
+      break;
+   case CMD_ONE_TO_MANY: break;
+   case CMD_MANY_TO_ONE:
+      for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
+         UpdatePixel(sel_ptr->obj, old_color_str);
+      }
+      for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=sel_ptr->next) {
+         UpdatePixel(sel_ptr->obj, old_color_str);
+      }
+      break;
+   case CMD_GOTO_PAGE: break;
+   }
+}
+
+static
+int RefreshColormap(nRedoing, CmdPtr)
+   int nRedoing;
+   struct CmdRec *CmdPtr;
+   /*
+    * This function mimics FlushColormap() in color.c.
+    */
+{
+   register int i;
+   int old_allocated_colors, changed=FALSE, saved_color_layers;
+   char **old_color_str;
+   struct ObjRec *obj_ptr;
+   struct PageRec *page_ptr;
+   Colormap colormap=XCopyColormapAndFree(mainDisplay, mainColormap);
+
+   mainColormap = colormap;
+   newColormapUsed = TRUE;
+   XSetWindowColormap(mainDisplay, mainWindow, mainColormap);
+
+   if (nRedoing) {
+      /* need to scan forward to pick up objects */
+      struct CmdRec *cmd_ptr;
+
+      for (page_ptr=firstPage; page_ptr != NULL; page_ptr=page_ptr->next) {
+         for (obj_ptr=page_ptr->bot; obj_ptr!=NULL; obj_ptr=obj_ptr->prev) {
+            UpdateXPmObjects(obj_ptr);
+         }
+      }
+      for (cmd_ptr=CmdPtr->next; cmd_ptr != NULL; cmd_ptr=cmd_ptr->next) {
+         UpdateXPmObjectsInACmd(cmd_ptr);
+      }
+   } else {
+      for (page_ptr=firstPage; page_ptr != NULL; page_ptr=page_ptr->next) {
+         for (obj_ptr=page_ptr->bot; obj_ptr!=NULL; obj_ptr=obj_ptr->prev) {
+            UpdateXPmObjects(obj_ptr);
+         }
+      }
+   }
+   old_color_str = (char**)malloc(maxColors*sizeof(char*));
+   if (old_color_str == NULL) FailAllocMessage();
+   old_allocated_colors = maxColors;
+   for (i = 0; i < maxColors; i++) {
+      old_color_str[i] =
+            (char*)malloc((strlen(colorMenuItems[i])+1)*sizeof(char));
+      if (old_color_str[i] == NULL) FailAllocMessage();
+      strcpy(old_color_str[i], colorMenuItems[i]);
+   }
+   initColorDontReload = TRUE;
+   CleanUpColors();
+   XFreeColormap(mainDisplay, mainColormap);
+   mainColormap = DefaultColormap(mainDisplay, mainScreen);
+   XSetWindowColormap(mainDisplay, mainWindow, mainColormap);
+   newColormapUsed = FALSE;
+   saved_color_layers = colorLayers;
+   InitColor();
+   initColorDontReload = FALSE;
+   colorLayers = saved_color_layers;
+
+   ShowColor(TRUE);
+
+   SaveStatusStrings();
+   gnUpdatePixelObjCount = 0;
+   if (nRedoing) {
+      /* need to scan forward to pick up objects */
+      struct CmdRec *cmd_ptr;
+
+      for (page_ptr=firstPage; page_ptr != NULL; page_ptr=page_ptr->next) {
+         for (obj_ptr=page_ptr->bot; obj_ptr!=NULL; obj_ptr=obj_ptr->prev) {
+            if (UpdatePixel(obj_ptr, old_color_str)) {
+               changed = TRUE;
+            }
+         }
+      }
+      for (cmd_ptr=CmdPtr->next; cmd_ptr != NULL; cmd_ptr=cmd_ptr->next) {
+         if (UpdatePixelInACmd(cmd_ptr, old_color_str)) {
+            changed = TRUE;
+         }
+      }
+   } else {
+      for (page_ptr=firstPage; page_ptr != NULL; page_ptr=page_ptr->next) {
+         for (obj_ptr=page_ptr->bot; obj_ptr!=NULL; obj_ptr=obj_ptr->prev) {
+            if (UpdatePixel(obj_ptr, old_color_str)) {
+               changed = TRUE;
+            }
+         }
+      }
+   }
+   RestoreStatusStrings();
+   for (i = 0; i < old_allocated_colors; i++) free(old_color_str[i]);
+   free(old_color_str);
+
+   DestroySubMenu(MENU_COLOR);
+   if (colorLayers) {
+      RedrawColorWindow();
+   }
+   return changed;
+}
+
+static
+void UndoOrRedoReplaceCmd(CmdPtr, HighLightSingleObj)
+   struct CmdRec *CmdPtr;
+   int HighLightSingleObj;
+{
+   register struct ObjRec *obj_ptr, *next_obj;
+   register struct SelRec *sel_ptr;
+   struct SelRec *saved_top_sel, *saved_bot_sel;
+   struct ObjRec *saved_top_obj, *saved_bot_obj;
+   int ltx, lty, rbx, rby, pos, count, *pos_table, max_count;
+   int need_clear_and_redraw=FALSE;
 
    LinkJustTheObjects (CmdPtr->top_before, CmdPtr->bot_before);
 
@@ -566,31 +753,42 @@ void UndoOrRedoReplaceCmd (CmdPtr, HighLightSingleObj)
          botSel = NULL;
       else
          topSel->prev = NULL;
-      cfree (sel_ptr);
+      free(sel_ptr);
+   }
+   if (CmdPtr->new_colormap) {
+      if (RefreshColormap(CmdPtr->undone, CmdPtr)) {
+         need_clear_and_redraw = TRUE;
+      }
    }
-
    UpdSelBBox ();
-   RedrawAreas (botObj, ltx-GRID_ABS_SIZE(1), lty-GRID_ABS_SIZE(1),
-         rbx+GRID_ABS_SIZE(1), rby+GRID_ABS_SIZE(1),
-         selLtX-GRID_ABS_SIZE(1), selLtY-GRID_ABS_SIZE(1),
-         selRbX+GRID_ABS_SIZE(1), selRbY+GRID_ABS_SIZE(1));
-   if (HighLightSingleObj && topSel != NULL)
-      HighLightForward ();
-   else
+   if (need_clear_and_redraw) {
+      ClearAndRedrawDrawWindow();
+   } else {
+      RedrawAreas (botObj, ltx-GRID_ABS_SIZE(1), lty-GRID_ABS_SIZE(1),
+            rbx+GRID_ABS_SIZE(1), rby+GRID_ABS_SIZE(1),
+            selLtX-GRID_ABS_SIZE(1), selLtY-GRID_ABS_SIZE(1),
+            selRbX+GRID_ABS_SIZE(1), selRbY+GRID_ABS_SIZE(1));
+   }
+   if (HighLightSingleObj && topSel != NULL) {
+      if (!need_clear_and_redraw) {
+         HighLightForward ();
+      }
+   } else {
       RemoveAllSel ();
+   }
    SetFileModified (TRUE);
    justDupped = FALSE;
 }
 
 static
-void UndoOrRedoOneToManyCmd (CmdPtr)
-   struct CmdRec	* CmdPtr;
+void UndoOrRedoOneToManyCmd(CmdPtr)
+   struct CmdRec *CmdPtr;
 {
-   register struct ObjRec	* obj_ptr, * next_obj;
-   register struct SelRec	* sel_ptr;
-   struct ObjRec		* saved_top_obj, * saved_bot_obj;
-   int				ltx, lty, rbx, rby, pos, count;
-   int				* pos_table, max_count;
+   register struct ObjRec *obj_ptr, *next_obj;
+   register struct SelRec *sel_ptr;
+   struct ObjRec *saved_top_obj, *saved_bot_obj;
+   int ltx, lty, rbx, rby, pos, count, *pos_table, max_count;
+   int need_clear_and_redraw=FALSE;
 
    if (CmdPtr->include_tgif_obj)
    {
@@ -671,23 +869,34 @@ void UndoOrRedoOneToManyCmd (CmdPtr)
          botSel = NULL;
       else
          topSel->prev = NULL;
-      cfree (sel_ptr);
+      free(sel_ptr);
+   }
+   if (CmdPtr->new_colormap) {
+      if (RefreshColormap(CmdPtr->undone, CmdPtr)) {
+         need_clear_and_redraw = TRUE;
+      }
    }
    UpdSelBBox ();
-   RedrawAreas (botObj, selLtX-GRID_ABS_SIZE(1), selLtY-GRID_ABS_SIZE(1),
-         selRbX+GRID_ABS_SIZE(1), selRbY+GRID_ABS_SIZE(1),
-         ltx-GRID_ABS_SIZE(1), lty-GRID_ABS_SIZE(1),
-         rbx+GRID_ABS_SIZE(1), rby+GRID_ABS_SIZE(1));
-   HighLightForward ();
+   if (need_clear_and_redraw) {
+      ClearAndRedrawDrawWindow();
+   } else {
+      RedrawAreas (botObj, selLtX-GRID_ABS_SIZE(1), selLtY-GRID_ABS_SIZE(1),
+            selRbX+GRID_ABS_SIZE(1), selRbY+GRID_ABS_SIZE(1),
+            ltx-GRID_ABS_SIZE(1), lty-GRID_ABS_SIZE(1),
+            rbx+GRID_ABS_SIZE(1), rby+GRID_ABS_SIZE(1));
+   }
+   if (!need_clear_and_redraw) {
+      HighLightForward ();
+   }
    SetFileModified (TRUE);
    justDupped = FALSE;
 }
 
 static
-void UndoOrRedoGotoPageCmd (CmdPtr)
-   struct CmdRec	* CmdPtr;
+void UndoOrRedoGotoPageCmd(CmdPtr)
+   struct CmdRec *CmdPtr;
 {
-   int	new_page_num;
+   int new_page_num;
 
    new_page_num = (CmdPtr->undone) ? CmdPtr->count_after : CmdPtr->count_before;
 
@@ -696,12 +905,11 @@ void UndoOrRedoGotoPageCmd (CmdPtr)
    ShowPage ();
 }
 
-static
-void UndoACmd (CmdPtr, HighLight)
-   struct CmdRec	* CmdPtr;
-   int			HighLight;
+void UndoACmd(CmdPtr, HighLight)
+   struct CmdRec *CmdPtr;
+   int HighLight;
 {
-   struct CmdRec	* cmd_ptr;
+   struct CmdRec *cmd_ptr;
 
    if (topSel!=NULL) { HighLightReverse (); RemoveAllSel (); }
    switch (CmdPtr->type)
@@ -741,12 +949,11 @@ void UndoCmd ()
    undoingOrRedoing = FALSE;
 }
 
-static
-void RedoACmd (CmdPtr, HighLight)
-   struct CmdRec	* CmdPtr;
-   int			HighLight;
+void RedoACmd(CmdPtr, HighLight)
+   struct CmdRec *CmdPtr;
+   int HighLight;
 {
-   struct CmdRec	* cmd_ptr;
+   struct CmdRec *cmd_ptr;
 
    if (topSel!=NULL) { HighLightReverse (); RemoveAllSel (); }
    switch (CmdPtr->type)
@@ -796,21 +1003,22 @@ void RedoCmd ()
 }
 
 struct CmdStkRec {
-   struct CmdRec	* first, * last, * cur;
-   int			history_count;
-   struct CmdStkRec	* next;
+   struct CmdRec *first, *last, *cur;
+   int history_count;
+   struct CmdStkRec *next;
 };
 
-static struct CmdStkRec	* topCompositeCmdStk=NULL;
-static int		composingCommand=FALSE;
+static struct CmdStkRec *topCompositeCmdStk=NULL;
 
-void StartCompositeCmd ()
+void StartCompositeCmd()
 {
-   struct CmdStkRec	* cmd_stk_ptr;
+   struct CmdStkRec *cmd_stk_ptr;
 
    if (historyDepth == 0) return;
 
-   cmd_stk_ptr = (struct CmdStkRec *) calloc (1, sizeof(struct CmdStkRec));
+   cmd_stk_ptr = (struct CmdStkRec *)malloc(sizeof(struct CmdStkRec));
+   if (cmd_stk_ptr == NULL) FailAllocMessage();
+   memset(cmd_stk_ptr, 0, sizeof(struct CmdStkRec));
 
    cmd_stk_ptr->next = topCompositeCmdStk;
    cmd_stk_ptr->first = firstCmd;
@@ -825,10 +1033,10 @@ void StartCompositeCmd ()
    composingCommand = TRUE;
 }
 
-void EndCompositeCmd ()
+void EndCompositeCmd()
 {
-   struct CmdRec	* composite_cmd=NULL;
-   int			empty=FALSE;
+   struct CmdRec *composite_cmd=NULL;
+   int empty=FALSE;
 
    if (historyDepth == 0) return;
 
@@ -836,7 +1044,9 @@ void EndCompositeCmd ()
       empty = TRUE;
    else
    {
-      composite_cmd = (struct CmdRec *) calloc (1, sizeof(struct CmdRec));
+      composite_cmd = (struct CmdRec *)malloc(sizeof(struct CmdRec));
+      if (composite_cmd == NULL) FailAllocMessage();
+      memset(composite_cmd, 0, sizeof(struct CmdRec));
       composite_cmd->type = CMD_COMPOSITE;
       composite_cmd->include_tgif_obj = FALSE;
       composite_cmd->first = firstCmd;
@@ -855,7 +1065,7 @@ void EndCompositeCmd ()
 
       cmd_stk_ptr = topCompositeCmdStk;
       topCompositeCmdStk = topCompositeCmdStk->next;
-      cfree(cmd_stk_ptr);
+      free(cmd_stk_ptr);
    } else {
       firstCmd = lastCmd = curCmd = NULL;
       historyCount = 0;
@@ -870,7 +1080,7 @@ void EndCompositeCmd ()
 
       if (++historyCount == historyDepth)
       {
-         struct CmdRec	* new_first_cmd = firstCmd->next;
+         struct CmdRec *new_first_cmd=firstCmd->next;
 
          new_first_cmd->prev = NULL;
          firstCmd->next = NULL;
@@ -907,20 +1117,16 @@ void EnableUndo()
    RestoreDefaultHistoryDepth();
 }
 
-static struct SelRec	* topSelBefore=NULL, * botSelBefore=NULL;
-static int		* stackingPosition=NULL;
-static int		stackingCount=0;
-
-static
-void PrepareStacking (TopSel, BotSel, NumObjs)
-   struct SelRec	* TopSel, * BotSel;
-   int			NumObjs;
+void PrepareStacking(TopSel, BotSel, NumObjs)
+   struct SelRec *TopSel, *BotSel;
+   int NumObjs;
 {
-   register int			pos;
-   register struct SelRec	* sel_ptr;
-   register struct ObjRec	* obj_ptr;
+   register int pos;
+   register struct SelRec *sel_ptr;
+   register struct ObjRec *obj_ptr;
 
-   stackingPosition = (int *) calloc (NumObjs, sizeof(int));
+   stackingPosition = (int*)malloc(NumObjs*sizeof(int));
+   if (stackingPosition == NULL) FailAllocMessage();
    stackingCount = 0;
    pos = 0;
    sel_ptr = TopSel;
@@ -951,15 +1157,17 @@ void PrepareStacking (TopSel, BotSel, NumObjs)
    }
 }
 
-void PrepareToRecord (CmdType, TopSel, BotSel, NumObjs)
-   int			CmdType, NumObjs;
-   struct SelRec	* TopSel, * BotSel;
+void PrepareToRecord(CmdType, TopSel, BotSel, NumObjs)
+   int CmdType, NumObjs;
+   struct SelRec *TopSel, *BotSel;
 {
-   register struct SelRec	* sel_ptr, * to_sel_ptr;
+   register struct SelRec *sel_ptr, *to_sel_ptr;
 
    if (historyDepth == 0) return;
 
-   topSelBefore = botSelBefore = NULL;
+   preparedColormap = (gnInImageProc ? mainColormap : None);
+
+   topSelBeforeInCmd = botSelBeforeInCmd = NULL;
    stackingPosition = NULL;
    stackingCount = 0;
    switch (CmdType)
@@ -968,8 +1176,8 @@ void PrepareToRecord (CmdType, TopSel, BotSel, NumObjs)
 
       case CMD_DELETE:
          PrepareStacking (TopSel, BotSel, NumObjs);
-         DupTheseObjects (TopSel, BotSel, &topSelBefore, &botSelBefore);
-         for (sel_ptr = TopSel, to_sel_ptr = topSelBefore; to_sel_ptr!=NULL;
+         DupTheseObjects (TopSel, BotSel, &topSelBeforeInCmd, &botSelBeforeInCmd);
+         for (sel_ptr = TopSel, to_sel_ptr = topSelBeforeInCmd; to_sel_ptr!=NULL;
                sel_ptr=sel_ptr->next, to_sel_ptr=to_sel_ptr->next)
          {
             CopyObjId (sel_ptr->obj, to_sel_ptr->obj);
@@ -981,8 +1189,8 @@ void PrepareToRecord (CmdType, TopSel, BotSel, NumObjs)
 
       case CMD_STRETCH:
          PrepareStacking (TopSel, BotSel, NumObjs);
-         DupTheseObjects (TopSel, BotSel, &topSelBefore, &botSelBefore);
-         for (sel_ptr = TopSel, to_sel_ptr = topSelBefore; to_sel_ptr!=NULL;
+         DupTheseObjects (TopSel, BotSel, &topSelBeforeInCmd, &botSelBeforeInCmd);
+         for (sel_ptr = TopSel, to_sel_ptr = topSelBeforeInCmd; to_sel_ptr!=NULL;
                sel_ptr=sel_ptr->next, to_sel_ptr=to_sel_ptr->next)
          {
             CopyObjId (sel_ptr->obj, to_sel_ptr->obj);
@@ -991,8 +1199,8 @@ void PrepareToRecord (CmdType, TopSel, BotSel, NumObjs)
          break;
       case CMD_REPLACE:
          PrepareStacking (TopSel, BotSel, NumObjs);
-         DupTheseObjects (TopSel, BotSel, &topSelBefore, &botSelBefore);
-         for (sel_ptr = TopSel, to_sel_ptr = topSelBefore; to_sel_ptr!=NULL;
+         DupTheseObjects (TopSel, BotSel, &topSelBeforeInCmd, &botSelBeforeInCmd);
+         for (sel_ptr = TopSel, to_sel_ptr = topSelBeforeInCmd; to_sel_ptr!=NULL;
                sel_ptr=sel_ptr->next, to_sel_ptr=to_sel_ptr->next)
          {
             CopyObjId (sel_ptr->obj, to_sel_ptr->obj);
@@ -1006,25 +1214,31 @@ void PrepareToRecord (CmdType, TopSel, BotSel, NumObjs)
    return;
 }
 
-static
-void FreeAfterSel (CmdPtr)
-   struct CmdRec	* CmdPtr;
+void FreeAfterSel(CmdPtr)
+   struct CmdRec *CmdPtr;
 {
-   register struct SelRec	* sel_ptr, * next_sel;
+   register struct SelRec *sel_ptr, *next_sel;
 
-   for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel)
-   {
+   for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel) {
       next_sel = sel_ptr->next;
-      cfree (sel_ptr);
+      free(sel_ptr);
    }
    CmdPtr->top_after = CmdPtr->bot_after = NULL;
 }
 
-void RecordCmd (CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
-   int			CmdType, NumObjs;
-   struct SelRec	* TopSel, * BotSel;
-   struct SubCmdRec	* SubCmdPtr;
+void RecordCmd(CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
+   int CmdType, NumObjs;
+   struct SelRec *TopSel, *BotSel;
+   struct SubCmdRec *SubCmdPtr;
 {
+#ifdef _TGIF_WB
+   if (ClientServer != 0)  /* connection is going on */
+   {
+      CheckConForRecord(CmdType, SubCmdPtr, TopSel, NumObjs, topSelBeforeInCmd);
+      if (ClientServer == 1) return;  /* connection ON and I'm client */
+   }
+#endif /* _TGIF_WB */
+
    if (historyDepth == 0) return;
 
    if (curCmd == NULL)
@@ -1034,7 +1248,7 @@ void RecordCmd (CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
 
    if (++historyCount == historyDepth && !composingCommand)
    {
-      struct CmdRec	* new_first_cmd = firstCmd->next;
+      struct CmdRec *new_first_cmd=firstCmd->next;
 
       new_first_cmd->prev = NULL;
       firstCmd->next = NULL;
@@ -1043,14 +1257,17 @@ void RecordCmd (CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
       firstCmd = new_first_cmd;
    }
 
-   curCmd = (struct CmdRec *) calloc (1, sizeof(struct CmdRec));
-   curCmd->top_before = topSelBefore;
-   curCmd->bot_before = botSelBefore;
+   curCmd = (struct CmdRec *)malloc(sizeof(struct CmdRec));
+   if (curCmd == NULL) FailAllocMessage();
+   memset(curCmd, 0, sizeof(struct CmdRec));
+   curCmd->top_before = topSelBeforeInCmd;
+   curCmd->bot_before = botSelBeforeInCmd;
    curCmd->pos_before = stackingPosition;
    curCmd->count_before = stackingCount;
    curCmd->type = CmdType;
    curCmd->undone = FALSE;
    curCmd->include_tgif_obj = recordCmdIncludeTgifObj;
+   curCmd->new_colormap = recordCmdUsesNewColormap;
 
    if (TopSel != NULL)
    {
@@ -1085,42 +1302,43 @@ void RecordCmd (CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
    curCmd = lastCmd;
 }
 
-void AbortPrepareCmd (CmdType)
-   int	CmdType;
+void AbortPrepareCmd(CmdType)
+   int CmdType;
 {
-   register struct SelRec	* sel_ptr, * next_sel;
+   register struct SelRec *sel_ptr, *next_sel;
 
    if (historyDepth == 0) return;
 
-   for (sel_ptr=topSelBefore; sel_ptr!=NULL; sel_ptr=next_sel)
+   for (sel_ptr=topSelBeforeInCmd; sel_ptr!=NULL; sel_ptr=next_sel)
    {
       next_sel = sel_ptr->next;
       switch (CmdType)
       {
          case CMD_REPLACE: FreeObj (sel_ptr->obj); break;
       }
-      cfree (sel_ptr);
+      free(sel_ptr);
    }
-   if (stackingPosition != NULL) cfree (stackingPosition);
+   if (stackingPosition != NULL) free(stackingPosition);
    stackingPosition = NULL;
    stackingCount = 0;
-   topSelBefore = botSelBefore = NULL;
+   topSelBeforeInCmd = botSelBeforeInCmd = NULL;
 }
 
-void RecordNewObjCmd ()
+void RecordNewObjCmd()
 {
    if (historyDepth == 0) return;
 
    if (topSel==NULL)
    {
-      struct SelRec	* sel_ptr;
+      struct SelRec *sel_ptr;
 
-      sel_ptr = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+      sel_ptr = (struct SelRec *)malloc(sizeof(struct SelRec));
+      if (sel_ptr == NULL) FailAllocMessage();
       sel_ptr->next = sel_ptr->prev = NULL;
       sel_ptr->obj = topObj;
       PrepareToRecord (CMD_NEW, NULL, NULL, 0);
       RecordCmd (CMD_NEW, NULL, sel_ptr, sel_ptr, 1);
-      cfree (sel_ptr);
+      free(sel_ptr);
    }
    else
    {
@@ -1129,35 +1347,38 @@ void RecordNewObjCmd ()
    }
 }
 
-void PrepareToReplaceAnObj (BeforeObjPtr)
-   struct ObjRec	* BeforeObjPtr;
+void PrepareToReplaceAnObj(BeforeObjPtr)
+   struct ObjRec *BeforeObjPtr;
 {
-   struct SelRec	* sel_ptr;
+   struct SelRec *sel_ptr;
 
    if (historyDepth == 0) return;
 
-   sel_ptr = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+   sel_ptr = (struct SelRec *)malloc(sizeof(struct SelRec));
+   if (sel_ptr == NULL) FailAllocMessage();
    sel_ptr->next = sel_ptr->prev = NULL;
    sel_ptr->obj = BeforeObjPtr;
    PrepareToRecord (CMD_REPLACE, sel_ptr, sel_ptr, 1);
-   cfree (sel_ptr);
+   free(sel_ptr);
 }
 
-void RecordReplaceAnObj (AfterObjPtr)
-   struct ObjRec	* AfterObjPtr;
+void RecordReplaceAnObj(AfterObjPtr)
+   struct ObjRec *AfterObjPtr;
 {
-   struct SelRec	* sel_ptr;
+   struct SelRec *sel_ptr;
 
    if (historyDepth == 0) return;
 
-   sel_ptr = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+   sel_ptr = (struct SelRec *)malloc(sizeof(struct SelRec));
+   if (sel_ptr == NULL) FailAllocMessage();
    sel_ptr->next = sel_ptr->prev = NULL;
    sel_ptr->obj = AfterObjPtr;
    RecordCmd (CMD_REPLACE, NULL, sel_ptr, sel_ptr, 1);
-   cfree (sel_ptr);
+   free(sel_ptr);
 }
 
-void ChangeReplaceOneCmdToDeleteCmd ()
+void ChangeReplaceOneCmdToDeleteCmd()
 {
-   RecordCmd (CMD_DELETE, NULL, NULL, NULL, 0);
+   RecordCmd(CMD_DELETE, NULL, NULL, NULL, 0);
 }
+
