@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -27,16 +27,12 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/const.h,v 2.131 1995/07/10 03:23:48 william Exp $
+ * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/const.h,v 3.9 1997/01/30 23:24:58 william Exp $
  */
 
 #ifndef _TGIF_CONST_H_
 #define _TGIF_CONST_H_
 
-#ifdef SES
-#define cfree free
-#endif /* ~SES */
-
 #if __STDC__ || defined(__cplusplus) || defined(c_plusplus)
 /* ANSI || C++ */
 #ifdef _NO_PROTO
@@ -49,7 +45,7 @@
 #define ARGS_DECL(args) ()
 #endif
 
-#include "tgif_debug.h"
+#include "tgif_dbg.h"
 
 #define TOOL_NAME "Tgif"
 
@@ -95,6 +91,15 @@
 #define OFFSET_Y(AbsY) ((zoomedIn) ? (((AbsY)-drawOrigY)<<zoomScale) : \
                                      (((AbsY)-drawOrigY)>>zoomScale))
 
+#define OFFSET_DOUBLE_X(AbsX) \
+      ((zoomedIn) ? \
+      (((AbsX)-((double)drawOrigX))*((double)(1<<zoomScale))) : \
+      (((AbsX)-((double)drawOrigX))/((double)(1<<zoomScale))))
+#define OFFSET_DOUBLE_Y(AbsY) \
+      ((zoomedIn) ? \
+      (((AbsY)-((double)drawOrigY))*((double)(1<<zoomScale))) : \
+      (((AbsY)-((double)drawOrigY))/((double)(1<<zoomScale))))
+
 #define ABS_X(OffsetX) ((zoomedIn) ? (((OffsetX)>>zoomScale)+drawOrigX) : \
                                      (((OffsetX)<<zoomScale)+drawOrigX))
 #define ABS_Y(OffsetY) ((zoomedIn) ? (((OffsetY)>>zoomScale)+drawOrigY) : \
@@ -103,6 +108,10 @@
 #define ZOOMED_SIZE(AbsSize) ((zoomedIn) ? ((AbsSize)<<zoomScale) : \
                                            ((AbsSize)>>zoomScale))
 
+#define ZOOMED_DOUBLE_SIZE(AbsSize) ((zoomedIn) ? \
+                                    ((AbsSize)*((double)(1<<zoomScale))) : \
+                                    ((AbsSize)/((double)(1<<zoomScale))))
+
 #define GRID_ZOOMED_SIZE(AbsSize) ((zoomedIn) ? (AbsSize) : \
                                                 ((AbsSize)>>zoomScale))
 
@@ -156,8 +165,9 @@
 #define DRAWRCBOX 7
 #define FREEHAND 8
 #define VERTEXMODE 9
+#define ROTATEMODE 10
 
-#define MAXCHOICES 10
+#define MAXCHOICES 11
 
 /* stipple patterns */
 
@@ -227,6 +237,18 @@
 
 #define MAXJUSTS 3
 
+#define PUSH_FONT 0
+#define PUSH_SIZE 1
+#define PUSH_STYLE 2
+#define PUSH_JUST 3
+#define PUSH_ROTATE 4
+#define PUSH_PEN 5
+#define PUSH_FILL 6
+#define PUSH_VSPACE 7
+#define PUSH_COLORINDEX 8
+#define PUSH_UNDERLINEON 9
+#define PUSH_UNDERLINE 10
+
 /* alignment */
 
 #define ALIGN_N 0
@@ -306,8 +328,9 @@
 #define TEXT_FILE 4
 #define EPSI_FILE 5
 #define GIF_FILE 6
+#define HTML_FILE 7
 
-#define MAXWHERETOPRINT 7
+#define MAXWHERETOPRINT 8
 
 /* measurement */
 
@@ -337,6 +360,13 @@
 #define ROTATE180 2
 #define ROTATE270 3
 
+#define CLOCKWISE90 (90<<6)
+#define COUNTER90 ((-90)<<6)
+#define CLOCKWISE180 (180<<6)
+#define COUNTER180 ((-180)<<6)
+#define CLOCKWISE270 (270<<6)
+#define COUNTER270 ((-270)<<6)
+
 /* flipping */
 
 #define NO_FLIP 0
@@ -365,17 +395,21 @@
 #define MENU_FONT 10
 #define MENU_STYLE 11
 #define MENU_SIZE 12
-#define MENU_LINEDASH 13
-#define MENU_LINESTYLE 14
-#define MENU_LINETYPE 15
-#define MENU_LINEWIDTH 16
-#define MENU_FILL 17
-#define MENU_PEN 18
-#define MENU_COLOR 19
-#define MENU_NAVIGATE 20
-#define MENU_SPECIAL 21
+#define MENU_SHAPE 13
+#define MENU_STRETCHTEXT 14
+#define MENU_LINEDASH 15
+#define MENU_LINESTYLE 16
+#define MENU_LINETYPE 17
+#define MENU_LINEWIDTH 18
+#define MENU_FILL 19
+#define MENU_PEN 20
+#define MENU_COLOR 21
+#define MENU_IMAGEPROC 22
+#define MENU_NAVIGATE 23
+#define MENU_SPECIAL 24
+#define MENU_HELP 25
 
-#define MAXMENUS 22
+#define MAXMENUS 26
 
 /* file menu */
 
@@ -388,27 +422,30 @@
 #define FILE_IMPORTXPM 6
 #define FILE_IMPORTEPS 7
 #define FILE_IMPORTGIF 8
-#define FILE_EMBEDEPS 9
-#define FILE_BROWSEXBM 10
-#define FILE_BROWSEXPM 11
-#define FILE_DOMAIN 12
-#define FILE_DUMP 13
-#define FILE_USR_DUMP 14
-#define FILE_DUMPSELECTED 15
-#define FILE_PRINTONE 16
-#define FILE_SETEXPORTTRIM 17
-#define FILE_INPUT_POLY 18
-#define FILE_INPUT_POLYGON 19
-#define FILE_SOLVE 20
-#define FILE_SIMULATE 21
-#define FILE_PROBE 22
-#define FILE_ANIMATE 23
-#define FILE_ESCAPE 24
-#define FILE_SAVESELAS 25
-#define FILE_SAVESYMINLIB 26
-#define FILE_QUIT 27
+#define FILE_IMPORTOTHERS 9
+#define FILE_EMBEDEPS 10
+#define FILE_BROWSEXBM 11
+#define FILE_BROWSEXPM 12
+#define FILE_BROWSEOTHERS 13
+#define FILE_DOMAIN 14
+#define FILE_DUMP 15
+#define FILE_USR_DUMP 16
+#define FILE_DUMPSELECTED 17
+#define FILE_PRINTONE 18
+#define FILE_SETEXPORTTRIM 19
+#define FILE_INPUT_POLY 20
+#define FILE_INPUT_POLYGON 21
+#define FILE_SET_TEMPLATE 22
+#define FILE_SOLVE 23
+#define FILE_SIMULATE 24
+#define FILE_PROBE 25
+#define FILE_ANIMATE 26
+#define FILE_ESCAPE 27
+#define FILE_SAVESELAS 28
+#define FILE_SAVESYMINLIB 29
+#define FILE_QUIT 30
 
-#define FILEMENUENTRIES 28
+#define FILEMENUENTRIES 31
 
 /* edit menu */
 
@@ -439,9 +476,22 @@
 #define EDIT_MAKE_REGULAR 24
 #define EDIT_BREAK_TEXT 25
 #define EDIT_SET_SEL_LINEWIDTH 26
-#define EDIT_ADD_COLOR 27
+#define EDIT_SET_SEL_FONT_SIZE 27
+#define EDIT_ADD_COLOR 28
+#define EDIT_BREAK_MAPS 29
+#define EDIT_LAYOUT_ON_ARC 30
+#define EDIT_PRECISE_ROTATE 31
+#define EDIT_JOIN_POLY 32
+#define EDIT_CUT_POLY 33
+#define EDIT_GET_BBOX 34
+#define EDIT_SET_SHAPE_SHADOW 35
+#define EDIT_NO_TRANSFORM 36
+#define EDIT_SET_EDIT_TEXT_SIZE 37
+#define EDIT_FIND_CASE 38
+#define EDIT_FIND_NOCASE 39
+#define EDIT_FIND_AGAIN 40
 
-#define MAXEDITMENUS 28
+#define MAXEDITMENUS 41
 
 /* layout menu */
 
@@ -465,11 +515,14 @@
 #define LAYOUT_TOGGLE_MAP_SHOWN 17
 #define LAYOUT_TOGGLE_USE_GRAY 18
 #define LAYOUT_TOGGLE_SHOW_MEASURE 19
-#define LAYOUT_TOGGLE_SHOW_MENUBAR 20
-#define LAYOUT_TOGGLE_SHOW_STATUS 21
-#define LAYOUT_TOGGLE_ONEMOTION_SELMOVE 22
+#define LAYOUT_TOGGLE_SET_MEASURE_UNIT 20
+#define LAYOUT_TOGGLE_SHOW_MENUBAR 21
+#define LAYOUT_TOGGLE_SHOW_STATUS 22
+#define LAYOUT_TOGGLE_ONEMOTION_SELMOVE 23
+/* make sure the LAYOUT_TOGGLE_COLOR_LAYERS is the last one */
+#define LAYOUT_TOGGLE_COLOR_LAYERS 24
 
-#define MAXLAYOUTMENUS 23
+#define MAXLAYOUTMENUS 25
 
 /* move mode stuff */
 
@@ -478,6 +531,13 @@
 
 #define MAXMOVEMODES 2
 
+/* stretchable text stuff */
+
+#define NO_STRETCHABLE_TEXT 0
+#define STRETCHABLE_TEXT 1
+
+#define MAXSTRETCHABLEMODES 2
+
 /* arrange menu */
 
 #define ARRANGE_FRONT 0
@@ -494,17 +554,19 @@
 #define FLIP_VERTICAL 11
 #define ROTATE_CLOCKWISE 12
 #define ROTATE_COUNTER 13
-#define ALIGN_OBJ_TOP 14
-#define ALIGN_OBJ_MIDDLE 15
-#define ALIGN_OBJ_BOTTOM 16
-#define ALIGN_OBJ_LEFT 17
-#define ALIGN_OBJ_CENTER 18
-#define ALIGN_OBJ_RIGHT 19
-#define ABUT_HORIZONTAL 20
-#define ABUT_VERTICAL 21
-#define ALIGN_CENTERANENDPOINT 22
+#define SET_TEXT_ROTATE 14
+#define SET_ROTATE_INC 15
+#define ALIGN_OBJ_TOP 16
+#define ALIGN_OBJ_MIDDLE 17
+#define ALIGN_OBJ_BOTTOM 18
+#define ALIGN_OBJ_LEFT 19
+#define ALIGN_OBJ_CENTER 20
+#define ALIGN_OBJ_RIGHT 21
+#define ABUT_HORIZONTAL 22
+#define ABUT_VERTICAL 23
+#define ALIGN_CENTERANENDPOINT 24
 
-#define MAXARRANGEMENUS 23
+#define MAXARRANGEMENUS 25
 
 /* nagivate menu */
 
@@ -545,8 +607,15 @@
 #define SPECIAL_EXPORT_ATTRS 22
 #define SPECIAL_MERGE_WITH_TABLE 23
 #define SPECIAL_EXPORT_TO_TABLE 24
+#ifdef _TGIF_WB
+#define SPECIAL_WHITE_BOARD 25
+#endif /* _TGIF_WB */
 
+#ifdef _TGIF_WB
+#define MAXSPECIALMENUS 26
+#else /* ~_TGIF_WB */
 #define MAXSPECIALMENUS 25
+#endif /* _TGIF_WB */
 
 /* page menu */
 
@@ -576,6 +645,21 @@
 
 #define MAXPAGELAYOUTMODES 2
 
+/* scroll stuff */
+
+#define SCROLL_LEFTEND 0
+#define SCROLL_LEFT 1
+#define SCROLL_RIGHT 2
+#define SCROLL_RIGHTEND 3
+#define SCROLL_UPEND 4
+#define SCROLL_UP 5
+#define SCROLL_CHECKALL 6
+#define SCROLL_UNCHECKALL 7
+#define SCROLL_DOWN 8
+#define SCROLL_DOWNEND 9
+
+#define MAXSCROLLBUTTONS 10
+
 /* pixel stuff */
 
 #ifdef sun
@@ -632,7 +716,7 @@
 
 #define ITEM_DSPED 10
 #define ITEM_LEN 30
-#define ROW_HEIGHT (defaultFontHeight+1)
+#define ROW_HEIGHT ((msgFontPtr==NULL)?(defaultFontHeight+1):(msgFontHeight+1))
 #define BUTTON_OK 0
 #define BUTTON_SETDIR 1
 #define BUTTON_CANCEL 2
@@ -695,6 +779,7 @@
 #define MB_ID_NO 4
 #define MB_ID_EXTRA 5
 
+#define MB_BTN_NONE		0
 #define MB_BTN_OK		1
 #define MB_BTN_YESNOCANCEL	2
 #define MB_BTN_OKCANCEL		3
@@ -704,6 +789,7 @@
 #define MB_ICON_STOP		0x0010
 #define MB_ICON_QUESTION	0x0020
 #define MB_ICON_INFORMATION	0x0040
+#define MB_ICON_DIALOG		0x0080
 
 #define MB_BTNMASK		0x0007
 #define MB_ICONMASK		0x00f0
@@ -716,8 +802,9 @@
 #define MB_PIXMAP_STOP 0
 #define MB_PIXMAP_QUESTION 1
 #define MB_PIXMAP_INFORMATION 2
+#define MB_PIXMAP_DIALOG 3
 
-#define MAX_MB_ICONS 3
+#define MAX_MB_ICONS 4
 
 /* expr stuff */
 
@@ -726,4 +813,31 @@
 #define DBL_VAL 2
 #define STR_VAL 3
 
+/* file version stuff */
+
+#define START_HAVING_ATTRS 8
+
+/* choice window width */
+
+#define MAXCHOICEWINDOWCOLS 13
+
+/* generic object stuff */
+
+#define TGO_STRING 0
+#define TGO_VISUAL 1
+#define TGO_MATH 2
+#define TGO_DATE 3
+#define TGO_USER 4
+
+/* choose a file stuff */
+
+#define NAMES_SIMPLE_SELECT_NAME 0
+#define NAMES_COMPLEX_SELECT_NAME 1
+#define NAMES_SELECT_FILE 2
+#define NAMES_EDIT_ATTR 3
+#define NAMES_EDIT_NAME 4
+
+#define NAMES_LOOP_MANY 0
+#define NAMES_LOOP_ONCE 1
+
 #endif /*_TGIF_CONST_H_*/
