@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,7 +29,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/dup.c,v 2.66 1995/05/15 01:21:32 william Exp $";
+      "@(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/dup.c,v 3.1 1996/05/10 06:33:46 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -38,6 +38,7 @@ static char RCSid[] =
 #include "types.h"
 
 #include "attr.e"
+#include "auxtext.e"
 #include "choice.e"
 #include "cmd.e"
 #include "drawing.e"
@@ -53,6 +54,8 @@ static char RCSid[] =
 #include "raster.e"
 #include "select.e"
 #include "setup.e"
+#include "text.e"
+#include "util.e"
 #include "xbitmap.e"
 #include "xpixmap.e"
 
@@ -130,6 +133,17 @@ void UnlockAnObj (ObjPtr)
       attr_ptr->obj->locked = FALSE;
 }
 
+void DupObjXfrmMtrx (FromObjPtr, ToObjPtr)
+   register struct ObjRec	* FromObjPtr, * ToObjPtr;
+{
+   ToObjPtr->ctm = NULL;
+   if (FromObjPtr->ctm == NULL) return;
+   ToObjPtr->ctm = (struct XfrmMtrxRec *)malloc(sizeof(struct XfrmMtrxRec));
+   if (ToObjPtr->ctm == NULL) FailAllocMessage();
+   memcpy(ToObjPtr->ctm, FromObjPtr->ctm, sizeof(struct XfrmMtrxRec));
+   memcpy(&ToObjPtr->orig_obbox,&FromObjPtr->orig_obbox,sizeof(struct BBRec));
+}
+
 void DupObjBasics (FromObjPtr, ToObjPtr)
    register struct ObjRec	* FromObjPtr, * ToObjPtr;
 {
@@ -150,220 +164,322 @@ void DupObjBasics (FromObjPtr, ToObjPtr)
    ToObjPtr->obbox.rbx = FromObjPtr->obbox.rbx;
    ToObjPtr->obbox.rby = FromObjPtr->obbox.rby;
    ToObjPtr->locked = FALSE;
+   ToObjPtr->invisible = FromObjPtr->invisible;
+
+   memcpy (&ToObjPtr->orig_obbox, &FromObjPtr->orig_obbox,
+         sizeof(struct BBRec));
+   memcpy (ToObjPtr->rotated_obbox, FromObjPtr->rotated_obbox,
+         5*sizeof(XPoint));
+   DupObjXfrmMtrx (FromObjPtr, ToObjPtr);
 }
 
-static
 void DupPolyObj (PolyPtr, ObjPtr)
    struct PolyRec	* PolyPtr;
    struct ObjRec	* ObjPtr;
 {
    register int			i, num_pts;
    register struct PolyRec	* poly_ptr;
-   register XPoint		* v;
+   register XPoint		*sv;
+   register IntPoint		*pv;
 
-   poly_ptr = (struct PolyRec *) calloc (1, sizeof(struct PolyRec));
+   poly_ptr = (struct PolyRec *)malloc(sizeof(struct PolyRec));
+   if (poly_ptr == NULL) FailAllocMessage();
+   memset(poly_ptr, 0, sizeof(struct PolyRec));
    num_pts = poly_ptr->n = PolyPtr->n;
-   v = (XPoint *) calloc (num_pts+1, sizeof(XPoint));
-   if (PolyPtr->smooth != NULL)
-      poly_ptr->smooth = (char *) calloc (num_pts+1, sizeof(char));
-   else
+   pv = (IntPoint*)malloc((num_pts+1)*sizeof(IntPoint));
+   if (pv == NULL) FailAllocMessage();
+   if (PolyPtr->smooth != NULL) {
+      poly_ptr->smooth = (char*)malloc((num_pts+1)*sizeof(char));
+      if (poly_ptr->smooth == NULL) FailAllocMessage();
+   } else {
       poly_ptr->smooth = NULL;
-   for (i = 0; i < num_pts; i++)
-   {
-      v[i].x = PolyPtr->vlist[i].x;
-      v[i].y = PolyPtr->vlist[i].y;
-      if (PolyPtr->smooth != NULL && poly_ptr->smooth != NULL)
-         poly_ptr->smooth[i] = PolyPtr->smooth[i];
    }
-   poly_ptr->vlist = v;
+   for (i = 0; i < num_pts; i++) {
+      pv[i].x = PolyPtr->vlist[i].x;
+      pv[i].y = PolyPtr->vlist[i].y;
+      if (PolyPtr->smooth != NULL && poly_ptr->smooth != NULL) {
+         poly_ptr->smooth[i] = PolyPtr->smooth[i];
+      }
+   }
+   poly_ptr->vlist = pv;
    poly_ptr->style = PolyPtr->style;
    poly_ptr->width = PolyPtr->width;
    poly_ptr->aw = PolyPtr->aw;
    poly_ptr->ah = PolyPtr->ah;
+   strcpy(poly_ptr->width_spec, PolyPtr->width_spec);
+   strcpy(poly_ptr->aw_spec, PolyPtr->aw_spec);
+   strcpy(poly_ptr->ah_spec, PolyPtr->ah_spec);
    poly_ptr->pen = PolyPtr->pen;
    poly_ptr->fill = PolyPtr->fill;
    poly_ptr->curved = PolyPtr->curved;
-   if (PolyPtr->sn != 0 && PolyPtr->svlist != NULL)
-   {
+   if (PolyPtr->sn != 0 && PolyPtr->svlist != NULL) {
       poly_ptr->sn = num_pts = PolyPtr->sn;
-      v = (XPoint *) calloc (num_pts+1, sizeof(XPoint));
-      for (i = 0; i < num_pts; i++)
-      {
-         v[i].x = PolyPtr->svlist[i].x;
-         v[i].y = PolyPtr->svlist[i].y;
+      sv = (XPoint*)malloc((num_pts+1)*sizeof(XPoint));
+      if (sv == NULL) FailAllocMessage();
+      for (i = 0; i < num_pts; i++) {
+         sv[i].x = PolyPtr->svlist[i].x;
+         sv[i].y = PolyPtr->svlist[i].y;
       }
-      poly_ptr->svlist = v;
-   }
-   else
-   {
+      poly_ptr->svlist = sv;
+   } else {
       poly_ptr->sn = 0;
       poly_ptr->svlist = NULL;
    }
    if (PolyPtr->curved == LT_INTSPLINE && PolyPtr->intn != 0 &&
-         PolyPtr->intvlist != NULL)
-   {
+         PolyPtr->intvlist != NULL) {
       poly_ptr->intn = num_pts = PolyPtr->intn;
-      v = (XPoint *) calloc (num_pts+1, sizeof(XPoint));
-      for (i = 0; i < num_pts; i++)
-      {
-         v[i].x = PolyPtr->intvlist[i].x;
-         v[i].y = PolyPtr->intvlist[i].y;
+      pv = (IntPoint*)malloc((num_pts+1)*sizeof(IntPoint));
+      if (pv == NULL) FailAllocMessage();
+      for (i = 0; i < num_pts; i++) {
+         pv[i].x = PolyPtr->intvlist[i].x;
+         pv[i].y = PolyPtr->intvlist[i].y;
       }
-      poly_ptr->intvlist = v;
-   }
-   else
-   {
+      poly_ptr->intvlist = pv;
+   } else {
       poly_ptr->intn = 0;
       poly_ptr->intvlist = NULL;
    }
    poly_ptr->dash = PolyPtr->dash;
+   if (ObjPtr->ctm != NULL && PolyPtr->rotated_n != 0 &&
+         PolyPtr->rotated_vlist != NULL) {
+      poly_ptr->rotated_n = num_pts = PolyPtr->rotated_n;
+      sv = (XPoint*)malloc((num_pts+1)*sizeof(XPoint));
+      if (sv == NULL) FailAllocMessage();
+      for (i = 0; i < num_pts; i++) {
+         sv[i].x = PolyPtr->rotated_vlist[i].x;
+         sv[i].y = PolyPtr->rotated_vlist[i].y;
+      }
+      poly_ptr->rotated_vlist = sv;
+   } else {
+      poly_ptr->rotated_n = 0;
+      poly_ptr->rotated_vlist = NULL;
+   }
 
    ObjPtr->detail.p = poly_ptr;
 }
 
-static
 void DupPolygonObj (PolygonPtr, ObjPtr)
    struct PolygonRec	* PolygonPtr;
    struct ObjRec	* ObjPtr;
 {
    register int			i, num_pts;
    register struct PolygonRec	* polygon_ptr;
-   XPoint			* v;
+   XPoint			*sv;
+   IntPoint			*pv;
 
-   polygon_ptr = (struct PolygonRec *) calloc (1, sizeof(struct PolygonRec));
+   polygon_ptr = (struct PolygonRec *)malloc(sizeof(struct PolygonRec));
+   if (polygon_ptr == NULL) FailAllocMessage();
+   memset(polygon_ptr, 0, sizeof(struct PolygonRec));
    num_pts = polygon_ptr->n = PolygonPtr->n;
-   v = (XPoint *) calloc (num_pts+1, sizeof(XPoint));
-   if (PolygonPtr->smooth != NULL)
-      polygon_ptr->smooth = (char *) calloc (num_pts+1, sizeof(char));
-   else
+   pv = (IntPoint*)malloc((num_pts+1)*sizeof(IntPoint));
+   if (pv == NULL) FailAllocMessage();
+   if (PolygonPtr->smooth != NULL) {
+      polygon_ptr->smooth = (char*)malloc((num_pts+1)*sizeof(char));
+      if (polygon_ptr->smooth == NULL) FailAllocMessage();
+   } else {
       polygon_ptr->smooth = NULL;
-   for (i = 0; i < num_pts; i++)
-   {
-      v[i].x = PolygonPtr->vlist[i].x;
-      v[i].y = PolygonPtr->vlist[i].y;
-      if (PolygonPtr->smooth != NULL && polygon_ptr->smooth != NULL)
-         polygon_ptr->smooth[i] = PolygonPtr->smooth[i];
    }
-   polygon_ptr->vlist = v;
+   for (i = 0; i < num_pts; i++) {
+      pv[i].x = PolygonPtr->vlist[i].x;
+      pv[i].y = PolygonPtr->vlist[i].y;
+      if (PolygonPtr->smooth != NULL && polygon_ptr->smooth != NULL) {
+         polygon_ptr->smooth[i] = PolygonPtr->smooth[i];
+      }
+   }
+   polygon_ptr->vlist = pv;
    polygon_ptr->fill = PolygonPtr->fill;
    polygon_ptr->width = PolygonPtr->width;
+   strcpy(polygon_ptr->width_spec, PolygonPtr->width_spec);
    polygon_ptr->pen = PolygonPtr->pen;
    polygon_ptr->curved = PolygonPtr->curved;
-   if (PolygonPtr->sn != 0 && PolygonPtr->svlist != NULL)
-   {
+   if (PolygonPtr->sn != 0 && PolygonPtr->svlist != NULL) {
       polygon_ptr->sn = num_pts = PolygonPtr->sn;
-      v = (XPoint *) calloc (num_pts+1, sizeof(XPoint));
-      for (i = 0; i < num_pts; i++)
-      {
-         v[i].x = PolygonPtr->svlist[i].x;
-         v[i].y = PolygonPtr->svlist[i].y;
+      sv = (XPoint*)malloc((num_pts+1)*sizeof(XPoint));
+      if (sv == NULL) FailAllocMessage();
+      for (i = 0; i < num_pts; i++) {
+         sv[i].x = PolygonPtr->svlist[i].x;
+         sv[i].y = PolygonPtr->svlist[i].y;
       }
-      polygon_ptr->svlist = v;
-   }
-   else
-   {
+      polygon_ptr->svlist = sv;
+   } else {
       polygon_ptr->sn = 0;
       polygon_ptr->svlist = NULL;
    }
    if (PolygonPtr->curved == LT_INTSPLINE && PolygonPtr->intn != 0 &&
-         PolygonPtr->intvlist != NULL)
-   {
+         PolygonPtr->intvlist != NULL) {
       polygon_ptr->intn = num_pts = PolygonPtr->intn;
-      v = (XPoint *) calloc (num_pts+1, sizeof(XPoint));
-      for (i = 0; i < num_pts; i++)
-      {
-         v[i].x = PolygonPtr->intvlist[i].x;
-         v[i].y = PolygonPtr->intvlist[i].y;
+      pv = (IntPoint*)malloc((num_pts+1)*sizeof(IntPoint));
+      if (pv == NULL) FailAllocMessage();
+      for (i = 0; i < num_pts; i++) {
+         pv[i].x = PolygonPtr->intvlist[i].x;
+         pv[i].y = PolygonPtr->intvlist[i].y;
       }
-      polygon_ptr->intvlist = v;
-   }
-   else
-   {
+      polygon_ptr->intvlist = pv;
+   } else {
       polygon_ptr->intn = 0;
       polygon_ptr->intvlist = NULL;
    }
    polygon_ptr->dash = PolygonPtr->dash;
+   if (ObjPtr->ctm != NULL && PolygonPtr->rotated_n != 0 &&
+         PolygonPtr->rotated_vlist != NULL) {
+      polygon_ptr->rotated_n = num_pts = PolygonPtr->rotated_n;
+      sv = (XPoint*)malloc((num_pts+1)*sizeof(XPoint));
+      if (sv == NULL) FailAllocMessage();
+      for (i = 0; i < num_pts; i++) {
+         sv[i].x = PolygonPtr->rotated_vlist[i].x;
+         sv[i].y = PolygonPtr->rotated_vlist[i].y;
+      }
+      polygon_ptr->rotated_vlist = sv;
+   } else {
+      polygon_ptr->rotated_n = 0;
+      polygon_ptr->rotated_vlist = NULL;
+   }
 
    ObjPtr->detail.g = polygon_ptr;
 }
 
-static
 void DupOvalObj (OvalPtr, ObjPtr)
    struct OvalRec	* OvalPtr;
    struct ObjRec	* ObjPtr;
 {
    register struct OvalRec	* oval_ptr;
 
-   oval_ptr = (struct OvalRec *) calloc (1, sizeof(struct OvalRec));
+   oval_ptr = (struct OvalRec *)malloc(sizeof(struct OvalRec));
+   if (oval_ptr == NULL) FailAllocMessage();
+   memset(oval_ptr, 0, sizeof(struct OvalRec));
    oval_ptr->fill = OvalPtr->fill;
    oval_ptr->width = OvalPtr->width;
+   strcpy(oval_ptr->width_spec, OvalPtr->width_spec);
    oval_ptr->pen = OvalPtr->pen;
    oval_ptr->dash = OvalPtr->dash;
+   if (ObjPtr->ctm != NULL && OvalPtr->rotated_n != 0 &&
+         OvalPtr->rotated_vlist != NULL)
+   {
+      int	i, num_pts;
+      XPoint	* v;
+
+      oval_ptr->rotated_n = num_pts = OvalPtr->rotated_n;
+      v = (XPoint*)malloc((num_pts+1)*sizeof(XPoint));
+      if (v == NULL) FailAllocMessage();
+      for (i = 0; i < num_pts; i++)
+      {
+         v[i].x = OvalPtr->rotated_vlist[i].x;
+         v[i].y = OvalPtr->rotated_vlist[i].y;
+      }
+      oval_ptr->rotated_vlist = v;
+   }
+   else
+   {
+      oval_ptr->rotated_n = 0;
+      oval_ptr->rotated_vlist = NULL;
+   }
 
    ObjPtr->detail.o = oval_ptr;
 }
 
-static
 void DupBoxObj (BoxPtr, ObjPtr)
    struct BoxRec	* BoxPtr;
    struct ObjRec	* ObjPtr;
 {
    register struct BoxRec	* box_ptr;
 
-   box_ptr = (struct BoxRec *) calloc (1, sizeof(struct BoxRec));
+   box_ptr = (struct BoxRec *)malloc(sizeof(struct BoxRec));
+   if (box_ptr == NULL) FailAllocMessage();
+   memset(box_ptr, 0, sizeof(struct BoxRec));
    box_ptr->fill = BoxPtr->fill;
    box_ptr->width = BoxPtr->width;
+   strcpy(box_ptr->width_spec, BoxPtr->width_spec);
    box_ptr->pen = BoxPtr->pen;
    box_ptr->dash = BoxPtr->dash;
 
    ObjPtr->detail.b = box_ptr;
 }
 
-static
 void DupRCBoxObj (RCBoxPtr, ObjPtr)
    struct RCBoxRec	* RCBoxPtr;
    struct ObjRec	* ObjPtr;
 {
    register struct RCBoxRec	* rcbox_ptr;
 
-   rcbox_ptr = (struct RCBoxRec *) calloc (1, sizeof(struct RCBoxRec));
+   rcbox_ptr = (struct RCBoxRec *)malloc(sizeof(struct RCBoxRec));
+   if (rcbox_ptr == NULL) FailAllocMessage();
+   memset(rcbox_ptr, 0, sizeof(struct RCBoxRec));
    rcbox_ptr->fill = RCBoxPtr->fill;
    rcbox_ptr->width = RCBoxPtr->width;
+   strcpy(rcbox_ptr->width_spec, RCBoxPtr->width_spec);
    rcbox_ptr->pen = RCBoxPtr->pen;
    rcbox_ptr->dash = RCBoxPtr->dash;
    rcbox_ptr->radius = RCBoxPtr->radius;
+   if (ObjPtr->ctm != NULL && RCBoxPtr->rotated_n != 0 &&
+         RCBoxPtr->rotated_vlist != NULL) {
+      int i, num_pts;
+      XPoint *v;
+
+      rcbox_ptr->rotated_n = num_pts = RCBoxPtr->rotated_n;
+      v = (XPoint*)malloc((num_pts+1)*sizeof(XPoint));
+      if (v == NULL) FailAllocMessage();
+      for (i = 0; i < num_pts; i++) {
+         v[i].x = RCBoxPtr->rotated_vlist[i].x;
+         v[i].y = RCBoxPtr->rotated_vlist[i].y;
+      }
+      rcbox_ptr->rotated_vlist = v;
+   } else {
+      rcbox_ptr->rotated_n = 0;
+      rcbox_ptr->rotated_vlist = NULL;
+   }
 
    ObjPtr->detail.rcb = rcbox_ptr;
 }
 
-static
 void DupArcObj (ArcPtr, ObjPtr)
    struct ArcRec	* ArcPtr;
    struct ObjRec	* ObjPtr;
 {
    register struct ArcRec	* arc_ptr;
 
-   arc_ptr = (struct ArcRec *) calloc (1, sizeof(struct ArcRec));
+   arc_ptr = (struct ArcRec *)malloc(sizeof(struct ArcRec));
+   if (arc_ptr == NULL) FailAllocMessage();
+   memset(arc_ptr, 0, sizeof(struct ArcRec));
    arc_ptr->fill = ArcPtr->fill;
    arc_ptr->width = ArcPtr->width;
    arc_ptr->aw = ArcPtr->aw;
    arc_ptr->ah = ArcPtr->ah;
+   strcpy(arc_ptr->width_spec, ArcPtr->width_spec);
+   strcpy(arc_ptr->aw_spec, ArcPtr->aw_spec);
+   strcpy(arc_ptr->ah_spec, ArcPtr->ah_spec);
    arc_ptr->pen = ArcPtr->pen;
    arc_ptr->dash = ArcPtr->dash;
    arc_ptr->style = ArcPtr->style;
 
-   arc_ptr->xc = ArcPtr->xc;         arc_ptr->yc = ArcPtr->yc;
-   arc_ptr->x1 = ArcPtr->x1;         arc_ptr->y1 = ArcPtr->y1;
-   arc_ptr->x2 = ArcPtr->x2;         arc_ptr->y2 = ArcPtr->y2;
+   arc_ptr->xc = ArcPtr->xc;             arc_ptr->yc = ArcPtr->yc;
+   arc_ptr->x1 = ArcPtr->x1;             arc_ptr->y1 = ArcPtr->y1;
+   arc_ptr->x2 = ArcPtr->x2;             arc_ptr->y2 = ArcPtr->y2;
    arc_ptr->dir = ArcPtr->dir;
-   arc_ptr->ltx = ArcPtr->ltx;       arc_ptr->lty = ArcPtr->lty;
-   arc_ptr->w = ArcPtr->w;           arc_ptr->h = ArcPtr->h;
-   arc_ptr->angle1 = ArcPtr->angle1; arc_ptr->angle2 = ArcPtr->angle2;
+   arc_ptr->ltx = ArcPtr->ltx;           arc_ptr->lty = ArcPtr->lty;
+   arc_ptr->w = ArcPtr->w;               arc_ptr->h = ArcPtr->h;
+   arc_ptr->angle1 = ArcPtr->angle1;     arc_ptr->angle2 = ArcPtr->angle2;
+   arc_ptr->a_angle1 = ArcPtr->a_angle1; arc_ptr->a_angle2 = ArcPtr->a_angle2;
+   if (ObjPtr->ctm != NULL && ArcPtr->rotated_n != 0 &&
+         ArcPtr->rotated_vlist != NULL) {
+      int i, num_pts;
+      XPoint *v;
+
+      arc_ptr->rotated_n = num_pts = ArcPtr->rotated_n;
+      v = (XPoint*)malloc((num_pts+2)*sizeof(XPoint));
+      if (v == NULL) FailAllocMessage();
+      for (i = 0; i < num_pts+2; i++) {
+         v[i].x = ArcPtr->rotated_vlist[i].x;
+         v[i].y = ArcPtr->rotated_vlist[i].y;
+      }
+      arc_ptr->rotated_vlist = v;
+   } else {
+      arc_ptr->rotated_n = 0;
+      arc_ptr->rotated_vlist = NULL;
+   }
 
    ObjPtr->detail.a = arc_ptr;
 }
 
-static
 void DupXBmObj (XBmPtr, ObjPtr)
    struct XBmRec	* XBmPtr;
    struct ObjRec	* ObjPtr;
@@ -374,7 +490,9 @@ void DupXBmObj (XBmPtr, ObjPtr)
    int				w, h, image_w, image_h;
    char				msg[MAXSTRING];
 
-   xbm_ptr = (struct XBmRec *) calloc (1, sizeof(struct XBmRec));
+   xbm_ptr = (struct XBmRec *)malloc(sizeof(struct XBmRec));
+   if (xbm_ptr == NULL) FailAllocMessage();
+   memset(xbm_ptr, 0, sizeof(struct XBmRec));
    ObjPtr->detail.xbm = xbm_ptr;
 
    xbm_ptr->image = NULL;
@@ -385,6 +503,7 @@ void DupXBmObj (XBmPtr, ObjPtr)
    xbm_ptr->rotate = XBmPtr->rotate;
    xbm_ptr->flip = XBmPtr->flip;
    xbm_ptr->cached_zoom = XBmPtr->cached_zoom;
+   xbm_ptr->cached_zoomed = XBmPtr->cached_zoomed;
    xbm_ptr->cached_rotate = XBmPtr->cached_rotate;
    xbm_ptr->cached_flip = XBmPtr->cached_flip;
    xbm_ptr->cached_w = XBmPtr->cached_w;
@@ -400,8 +519,11 @@ void DupXBmObj (XBmPtr, ObjPtr)
    xbm_ptr->save_epsf = XBmPtr->save_epsf;
    xbm_ptr->real_type = XBmPtr->real_type;
 
+   if (ObjPtr->ctm != NULL)
+      memcpy (&xbm_ptr->cached_ctm, &XBmPtr->cached_ctm,
+            sizeof(struct XfrmMtrxRec));
    if (XBmPtr->real_type==XBM_XBM || (XBmPtr->real_type==XBM_EPS &&
-         XBmPtr->bitmap != None))
+         XBmPtr->bitmap!=None))
    {
       bitmap = XCreatePixmap (mainDisplay, mainWindow, image_w, image_h, 1);
       if (bitmap == None)
@@ -415,8 +537,8 @@ void DupXBmObj (XBmPtr, ObjPtr)
       xbm_ptr->bitmap = bitmap;
       if (XBmPtr->cached_bitmap != None)
       {
-         w = ObjPtr->obbox.rbx - ObjPtr->obbox.ltx;
-         h = ObjPtr->obbox.rby - ObjPtr->obbox.lty;
+         w = ZOOMED_SIZE(ObjPtr->obbox.rbx - ObjPtr->obbox.ltx);
+         h = ZOOMED_SIZE(ObjPtr->obbox.rby - ObjPtr->obbox.lty);
 
          cached_bitmap = XCreatePixmap (mainDisplay, mainWindow, w, h, 1);
          if (cached_bitmap == None)
@@ -430,43 +552,46 @@ void DupXBmObj (XBmPtr, ObjPtr)
          xbm_ptr->cached_bitmap = cached_bitmap;
       }
    }
-   if (XBmPtr->real_type == XBM_EPS)
-   {
-      int	len=strlen(XBmPtr->filename);
-      int	num_epsf_lines=XBmPtr->num_epsf_lines;
+   if (XBmPtr->real_type == XBM_EPS) {
+      int len=strlen(XBmPtr->filename);
+      int num_epsf_lines=XBmPtr->num_epsf_lines;
 
       xbm_ptr->llx = XBmPtr->llx; xbm_ptr->lly = XBmPtr->lly;
       xbm_ptr->urx = XBmPtr->urx; xbm_ptr->ury = XBmPtr->ury;
-      xbm_ptr->filename = (char *) calloc (len+1, sizeof(char));
-      strcpy (xbm_ptr->filename, XBmPtr->filename);
-      strcpy (xbm_ptr->write_date, XBmPtr->write_date);
+      xbm_ptr->filename = (char*)malloc((len+1)*sizeof(char));
+      if (xbm_ptr->filename == NULL) FailAllocMessage();
+      strcpy(xbm_ptr->filename, XBmPtr->filename);
+      strcpy(xbm_ptr->write_date, XBmPtr->write_date);
       xbm_ptr->num_epsf_lines = num_epsf_lines;
-      xbm_ptr->epsflines = (char * *) calloc (num_epsf_lines, sizeof(char *));
-      for (i=0; i < num_epsf_lines; i++)
-      {
+      xbm_ptr->epsflines = (char**)malloc(num_epsf_lines*sizeof(char*));
+      if (xbm_ptr->epsflines == NULL) FailAllocMessage();
+      for (i=0; i < num_epsf_lines; i++) {
          len = strlen (XBmPtr->epsflines[i]);
-         xbm_ptr->epsflines[i] = (char *) calloc (len+1, sizeof(char));
-         strcpy (xbm_ptr->epsflines[i], XBmPtr->epsflines[i]);
+         xbm_ptr->epsflines[i] = (char*)malloc((len+1)*sizeof(char));
+         if (xbm_ptr->epsflines[i] == NULL) FailAllocMessage();
+         strcpy(xbm_ptr->epsflines[i], XBmPtr->epsflines[i]);
       }
    }
 }
 
-static
 void DupXPmObj (XPmPtr, ObjPtr)
    struct XPmRec	* XPmPtr;
    struct ObjRec	* ObjPtr;
 {
    register struct XPmRec	* xpm_ptr;
    register int			j, i;
-   Pixmap			pixmap, cached_pixmap;
+   Pixmap			pixmap, bitmap, cached_pixmap, cached_bitmap;
    int				w, h, ncolors, * pixels, len, chars_per_pixel;
    int				image_w, image_h;
    char				* color_char, * * color_str, msg[MAXSTRING];
 
-   xpm_ptr = (struct XPmRec *) calloc (1, sizeof(struct XPmRec));
+   xpm_ptr = (struct XPmRec *)malloc(sizeof(struct XPmRec));
+   if (xpm_ptr == NULL) FailAllocMessage();
+   memset(xpm_ptr, 0, sizeof(struct XPmRec));
    ObjPtr->detail.xpm = xpm_ptr;
 
    xpm_ptr->image = NULL;
+   xpm_ptr->bitmap_image = NULL;
    xpm_ptr->data = NULL;
    xpm_ptr->fill = XPmPtr->fill;
    xpm_ptr->image_w = image_w = XPmPtr->image_w;
@@ -474,72 +599,119 @@ void DupXPmObj (XPmPtr, ObjPtr)
    xpm_ptr->rotate = XPmPtr->rotate;
    xpm_ptr->flip = XPmPtr->flip;
    xpm_ptr->cached_zoom = XPmPtr->cached_zoom;
+   xpm_ptr->cached_zoomed = XPmPtr->cached_zoomed;
    xpm_ptr->cached_rotate = XPmPtr->cached_rotate;
    xpm_ptr->cached_flip = XPmPtr->cached_flip;
    xpm_ptr->cached_w = XPmPtr->cached_w;
    xpm_ptr->cached_h = XPmPtr->cached_h;
+   xpm_ptr->cached_color = (-1);
 
    xpm_ptr->pixmap = None;
+   xpm_ptr->bitmap = None;
    xpm_ptr->cached_pixmap = None;
+   xpm_ptr->cached_bitmap = None;
+   xpm_ptr->clip_mask = None;
 
-   pixmap = XCreatePixmap (mainDisplay,mainWindow,image_w,image_h,mainDepth);
-   if (pixmap == None)
-   {
-      sprintf (msg, "Can not copy pixmap of size %1dx%1d.", image_w, image_h);
-      Msg (msg);
+   if (ObjPtr->ctm != NULL)
+      memcpy (&xpm_ptr->cached_ctm, &XPmPtr->cached_ctm,
+            sizeof(struct XfrmMtrxRec));
+   pixmap = XCreatePixmap(mainDisplay,mainWindow,image_w,image_h,mainDepth);
+   if (pixmap == None) {
+      sprintf(msg, "Can not copy pixmap of size %1dx%1d.", image_w, image_h);
+      Msg(msg);
       return;
    }
    XCopyArea (mainDisplay,XPmPtr->pixmap,pixmap,xpmGC,0,0,image_w,image_h,0,0);
    xpm_ptr->pixmap = pixmap;
-   if (XPmPtr->cached_pixmap != None)
-   {
-      w = ObjPtr->obbox.rbx - ObjPtr->obbox.ltx;
-      h = ObjPtr->obbox.rby - ObjPtr->obbox.lty;
 
-      cached_pixmap = XCreatePixmap (mainDisplay, mainWindow, w, h, mainDepth);
-      if (cached_pixmap == None)
-      {
-         sprintf (msg, "Can not copy pixmap of size %1dx%1d.", w, h);
-         Msg (msg);
+   bitmap = XCreatePixmap(mainDisplay,mainWindow,image_w,image_h,1);
+   if (bitmap == None) {
+      sprintf(msg, "Can not copy bitmap of size %1dx%1d.", image_w, image_h);
+      Msg(msg);
+      return;
+   }
+   XCopyArea (mainDisplay,XPmPtr->bitmap,bitmap,xbmGC,0,0,image_w,image_h,0,0);
+   xpm_ptr->bitmap = bitmap;
+
+   w = ZOOMED_SIZE(ObjPtr->obbox.rbx - ObjPtr->obbox.ltx);
+   h = ZOOMED_SIZE(ObjPtr->obbox.rby - ObjPtr->obbox.lty);
+
+   if (XPmPtr->cached_pixmap != None) {
+      cached_pixmap = XCreatePixmap(mainDisplay, mainWindow, w, h, mainDepth);
+      if (cached_pixmap == None) {
+         sprintf(msg, "Can not copy pixmap of size %1dx%1d.", w, h);
+         Msg(msg);
          return;
       }
-      XCopyArea (mainDisplay, XPmPtr->cached_pixmap, cached_pixmap, xpmGC, 0, 0,
+      XCopyArea(mainDisplay, XPmPtr->cached_pixmap, cached_pixmap, xpmGC, 0, 0,
             w, h, 0, 0);
       xpm_ptr->cached_pixmap = cached_pixmap;
    }
+   if (XPmPtr->cached_bitmap != None) {
+      cached_bitmap = XCreatePixmap(mainDisplay, mainWindow, w, h, 1);
+      if (cached_bitmap == None) {
+         sprintf(msg, "Can not copy bitmap of size %1dx%1d.", w, h);
+         Msg(msg);
+         return;
+      }
+      XCopyArea(mainDisplay, XPmPtr->cached_bitmap, cached_bitmap, xbmGC, 0, 0,
+            w, h, 0, 0);
+      xpm_ptr->cached_bitmap = cached_bitmap;
+   }
 
    chars_per_pixel = xpm_ptr->chars_per_pixel = XPmPtr->chars_per_pixel;
    xpm_ptr->first_pixel_is_bg = XPmPtr->first_pixel_is_bg;
 
    ncolors = xpm_ptr->ncolors = XPmPtr->ncolors;
-   color_char = xpm_ptr->color_char = (char *) calloc (ncolors*chars_per_pixel,
-         sizeof(char));
-   color_str = xpm_ptr->color_str = (char * *) calloc (ncolors, sizeof(char *));
-   pixels = xpm_ptr->pixels = (int *) calloc (ncolors, sizeof(int));
-   for (i = 0; i < ncolors; i++)
-   {
-      for (j = 0; j < chars_per_pixel; j++)
+   color_char = xpm_ptr->color_char =
+         (char*)malloc(ncolors*chars_per_pixel*sizeof(char));
+   color_str = xpm_ptr->color_str = (char**)malloc(ncolors*sizeof(char*));
+   pixels = xpm_ptr->pixels = (int*)malloc(ncolors*sizeof(int));
+   if (color_char==NULL || color_str==NULL || pixels==NULL) FailAllocMessage();
+   for (i = 0; i < ncolors; i++) {
+      for (j = 0; j < chars_per_pixel; j++) {
          color_char[i*chars_per_pixel+j] =
                XPmPtr->color_char[i*chars_per_pixel+j];
+      }
       pixels[i] = XPmPtr->pixels[i];
 
       len = strlen (XPmPtr->color_str[i]);
-      color_str[i] = (char *) calloc (len+1, sizeof(char));
-      strcpy (color_str[i], XPmPtr->color_str[i]);
+      color_str[i] = (char*)malloc((len+1)*sizeof(char));
+      if (color_str[i] == NULL) FailAllocMessage();
+      strcpy(color_str[i], XPmPtr->color_str[i]);
    }
    xpm_ptr->red = xpm_ptr->green = xpm_ptr->blue = NULL;
+   if (ObjPtr->ctm != NULL && XPmPtr->clip_mask != None)
+   {
+      w = ZOOMED_SIZE(ObjPtr->obbox.rbx - ObjPtr->obbox.ltx);
+      h = ZOOMED_SIZE(ObjPtr->obbox.rby - ObjPtr->obbox.lty);
+
+      pixmap = XCreatePixmap (mainDisplay, mainWindow, w, h, 1);
+      if (pixmap == None)
+      {
+         sprintf (msg, "Can not copy pixmap of size %1dx%1d.", w, h);
+         Msg (msg);
+         return;
+      }
+      XCopyArea (mainDisplay, XPmPtr->clip_mask, pixmap, xbmGC, 0, 0,
+            w, h, 0, 0);
+      xpm_ptr->clip_mask = pixmap;
+   }
 }
 
-void DupTextObj (TextPtr, ObjPtr)
+void DupTextObj (TextPtr, FromObjPtr, ToObjPtr)
    struct TextRec	* TextPtr;
-   struct ObjRec	* ObjPtr;
+   struct ObjRec	* FromObjPtr, * ToObjPtr;
 {
    register int			i, num_lines;
    register struct TextRec	* text_ptr;
    struct StrRec		* first_str;
    struct StrRec		* from_str_ptr, * to_str_ptr;
+   char				msg[MAXSTRING];
 
-   text_ptr = (struct TextRec *) calloc (1, sizeof(struct TextRec));
+   text_ptr = (struct TextRec *)malloc(sizeof(struct TextRec));
+   if (text_ptr == NULL) FailAllocMessage();
+   memset(text_ptr, 0, sizeof(struct TextRec));
    text_ptr->just = TextPtr->just;
    text_ptr->v_space = TextPtr->v_space;
    num_lines = text_ptr->lines = TextPtr->lines;
@@ -547,8 +719,8 @@ void DupTextObj (TextPtr, ObjPtr)
    first_str = NULL;
    for (i = 0; i < num_lines; i++, from_str_ptr = from_str_ptr->prev)
    {
-      to_str_ptr = (struct StrRec *) calloc (1, sizeof(struct StrRec));
-      strcpy (to_str_ptr->s, from_str_ptr->s);
+      to_str_ptr = NewStr ();
+      DynStrCpy (&to_str_ptr->dyn_str, &from_str_ptr->dyn_str);
       to_str_ptr->next = first_str;
       if (first_str == NULL)
          text_ptr->last = to_str_ptr;
@@ -567,24 +739,59 @@ void DupTextObj (TextPtr, ObjPtr)
    text_ptr->fill = TextPtr->fill;
    text_ptr->asc = TextPtr->asc;
    text_ptr->des = TextPtr->des;
-   text_ptr->cached_zoom = 0;
+   text_ptr->underline_on = TextPtr->underline_on;
+   text_ptr->underline = TextPtr->underline;
+   text_ptr->min_lbearing = TextPtr->min_lbearing;
+   text_ptr->max_rextra = TextPtr->max_rextra;
+   text_ptr->double_byte = TextPtr->double_byte;
+   text_ptr->direction = TextPtr->direction;
+   if (TextPtr->custom_screen_font_name == NULL)
+      text_ptr->custom_screen_font_name = NULL;
+   else
+      text_ptr->custom_screen_font_name =
+            UtilStrDup(TextPtr->custom_screen_font_name);
+   text_ptr->cached_zoom = TextPtr->cached_zoom;
    text_ptr->cached_bitmap = None;
    text_ptr->image = NULL;
    text_ptr->cached_zoomed = TextPtr->cached_zoomed;
+   text_ptr->cached_rotate = TextPtr->cached_rotate;
 
    text_ptr->read_only = TextPtr->read_only;
    text_ptr->orig_w = TextPtr->orig_w;
    text_ptr->orig_h = TextPtr->orig_h;
    text_ptr->font_name = NULL;
 
-   ObjPtr->detail.t = text_ptr;
+   ToObjPtr->detail.t = text_ptr;
+
+   if (ToObjPtr->ctm != NULL)
+      memcpy (&text_ptr->cached_ctm, &TextPtr->cached_ctm,
+            sizeof(struct XfrmMtrxRec));
+   if (TextPtr->cached_bitmap != None)
+   {
+      int	w=ZOOMED_SIZE(FromObjPtr->bbox.rbx - FromObjPtr->bbox.ltx - 2);
+      int	h=ZOOMED_SIZE(FromObjPtr->bbox.rby - FromObjPtr->bbox.lty - 2);
+
+      if (w == 0) w = 1;
+      if (h == 0) h = 1;
+      text_ptr->cached_bitmap = XCreatePixmap (mainDisplay,mainWindow,w,h,1);
+      if (text_ptr->cached_bitmap == None)
+      {
+         sprintf (msg, "Can not create bitmap of size %1dx%1d.", w, h);
+         Msg (msg);
+         return;
+      }
+      XCopyArea (mainDisplay, TextPtr->cached_bitmap, text_ptr->cached_bitmap,
+            rotateGC, 0, 0, w, h, 0, 0);
+   }
+   if (ToObjPtr->ctm != NULL) {
+      memcpy(&text_ptr->orig_bbox,&TextPtr->orig_bbox,sizeof(struct BBRec));
+   }
 }
 
 #ifdef _NO_EXTERN
 struct ObjRec	* DupObj ARGS_DECL((struct ObjRec *));
 #endif /* _NO_EXTERN */
 
-static
 void DupGroupObj (GroupPtr, ObjPtr)
    struct GroupRec	* GroupPtr;
    struct ObjRec	* ObjPtr;
@@ -593,8 +800,9 @@ void DupGroupObj (GroupPtr, ObjPtr)
    struct ObjRec		* top_obj;
    struct ObjRec		* from_obj_ptr, * to_obj_ptr;
 
-   group_ptr = (struct GroupRec *) calloc (1, sizeof(struct GroupRec));
-   if (group_ptr == NULL) FailAllocMessage ();
+   group_ptr = (struct GroupRec *)malloc(sizeof(struct GroupRec));
+   if (group_ptr == NULL) FailAllocMessage();
+   memset(group_ptr, 0, sizeof(struct GroupRec));
    top_obj = NULL;
    from_obj_ptr = GroupPtr->last;
    for ( ; from_obj_ptr != NULL; from_obj_ptr = from_obj_ptr->prev)
@@ -618,8 +826,9 @@ struct ObjRec * DupObj (ObjPtr)
 {
    struct ObjRec	* obj_ptr;
 
-   obj_ptr = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
-   if (obj_ptr == NULL) FailAllocMessage ();
+   obj_ptr = (struct ObjRec *)malloc(sizeof(struct ObjRec));
+   if (obj_ptr == NULL) FailAllocMessage();
+   memset(obj_ptr, 0, sizeof(struct ObjRec));
    DupObjBasics (ObjPtr, obj_ptr);
 
    switch (ObjPtr->type)
@@ -636,7 +845,7 @@ struct ObjRec * DupObj (ObjPtr)
          DupOvalObj (ObjPtr->detail.o, obj_ptr);
          DupAttrs (ObjPtr, obj_ptr);
          break;
-      case OBJ_TEXT: DupTextObj (ObjPtr->detail.t, obj_ptr); break;
+      case OBJ_TEXT: DupTextObj (ObjPtr->detail.t, ObjPtr, obj_ptr); break;
       case OBJ_POLYGON:
          DupPolygonObj (ObjPtr->detail.g, obj_ptr);
          DupAttrs (ObjPtr, obj_ptr);
@@ -746,28 +955,26 @@ void DupSelObj ()
    SetFileModified (TRUE);
 }
 
-void DupTheseObjects (TopSel, BotSel, NewTopSel, NewBotSel)
-   struct SelRec	* TopSel, * BotSel, * * NewTopSel, * * NewBotSel;
+void DupTheseObjects(TopSel, BotSel, NewTopSel, NewBotSel)
+   struct SelRec *TopSel, *BotSel, **NewTopSel, **NewBotSel;
 {
-   struct SelRec	* sel_ptr, * new_sel_ptr;
-   struct ObjRec	* obj_ptr, * top_obj;
+   struct SelRec *sel_ptr, *new_sel_ptr;
+   struct ObjRec *obj_ptr, *top_obj;
 
    *NewTopSel = *NewBotSel = NULL;
    if (TopSel == NULL) return;
 
    top_obj = NULL;
-   for (sel_ptr = BotSel; sel_ptr != NULL; sel_ptr = sel_ptr->prev)
-   {
-      obj_ptr = DupObj (sel_ptr->obj);
+   for (sel_ptr=BotSel; sel_ptr != NULL; sel_ptr=sel_ptr->prev) {
+      obj_ptr = DupObj(sel_ptr->obj);
       obj_ptr->next = top_obj;
-      new_sel_ptr = (struct SelRec *) calloc (1, sizeof (struct SelRec));
-      if (new_sel_ptr == NULL) FailAllocMessage ();
+      new_sel_ptr = (struct SelRec *)malloc(sizeof(struct SelRec));
+      if (new_sel_ptr == NULL) FailAllocMessage();
       new_sel_ptr->next = *NewTopSel;
       new_sel_ptr->obj = obj_ptr;
-      if (top_obj == NULL)
+      if (top_obj == NULL) {
          *NewBotSel = new_sel_ptr;
-      else
-      {
+      } else {
          top_obj->prev = obj_ptr;
          (*NewTopSel)->prev = new_sel_ptr;
       }
