@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1997, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -26,11 +26,9 @@
  * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/expr.c,v 3.1 1997/09/14 20:37:51 william Exp $
  */
-#ifndef lint
-static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/expr.c,v 2.19 1995/06/28 03:16:13 william Exp $";
-#endif
 
 #include <stdio.h>
 #include <string.h>
@@ -204,9 +202,10 @@ int OpPush(op_code, cur_depth)
    int op_code;
    int cur_depth;
 {
-   struct OpRec *op_ptr=(struct OpRec *)calloc(1,sizeof(struct OpRec));
+   struct OpRec *op_ptr=(struct OpRec *)malloc(sizeof(struct OpRec));
    
    if (op_ptr == NULL) return FailAllocMessage();
+   memset(op_ptr, 0, sizeof(struct OpRec));
    op_ptr->op_code = op_code;
    op_ptr->prec = OpPrec(op_code)+cur_depth;
    op_ptr->next = topOpStk;
@@ -218,9 +217,10 @@ static
 int ValPush(v_ptr)
    struct VRec *v_ptr;
 {
-   struct ValRec *val_ptr=(struct ValRec *)calloc(1,sizeof(struct ValRec));
+   struct ValRec *val_ptr=(struct ValRec *)malloc(sizeof(struct ValRec));
    
    if (val_ptr == NULL) return FailAllocMessage();
+   memset(val_ptr, 0, sizeof(struct ValRec));
    memcpy(&val_ptr->v, v_ptr, sizeof(val_ptr->v));
    val_ptr->next = topValStk;
    topValStk = val_ptr;
@@ -234,7 +234,7 @@ int PopOpStk()
    struct OpRec *op_ptr=topOpStk;
    
    topOpStk = topOpStk->next;
-   cfree(op_ptr);
+   free(op_ptr);
    return op_code;
 }
 
@@ -253,13 +253,13 @@ void PopValStk(v_ptr)
          v_ptr->val.d = val_ptr->v.val.d;
       }
    }
-   cfree(val_ptr);
+   free(val_ptr);
 }
 
 void CleanUpExpr()
 {
-   if (pszCurExpr != NULL) { cfree(pszCurExpr); pszCurExpr = NULL; }
-   if (pszExpr != NULL) { cfree(pszExpr); pszExpr = NULL; }
+   if (pszCurExpr != NULL) { free(pszCurExpr); pszCurExpr = NULL; }
+   if (pszExpr != NULL) { free(pszExpr); pszExpr = NULL; }
    while (topOpStk != NULL) PopOpStk();
    while (topValStk != NULL) PopValStk(NULL);
 }
@@ -279,7 +279,7 @@ int ExprAtomType(expr)
    c_ptr = dup_str;
    if (*c_ptr == '-') c_ptr++;
    if (strtok(c_ptr, "0123456789") == NULL) rc = INT_VAL;
-   cfree(dup_str);
+   free(dup_str);
    if (rc != INVALID) return rc;
 
    dup_str = UtilStrDup(expr);
@@ -288,7 +288,7 @@ int ExprAtomType(expr)
    c_ptr = dup_str;
    if (*c_ptr == '-') c_ptr++;
    rc = (strtok(c_ptr, ".0123456789") == NULL ? DBL_VAL : STR_VAL);
-   cfree(dup_str);
+   free(dup_str);
    return rc;
 }
 
@@ -775,7 +775,7 @@ int Operate()
    }
    for (i=0; i < arity; i++) {
       if (v[i].vtype == STR_VAL && v[i].val.s != NULL) {
-         cfree(v[i].val.s);
+         free(v[i].val.s);
          v[i].val.s = NULL;
       }
    }
