@@ -5,7 +5,7 @@
  *
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -33,7 +33,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/ftp.c,v 2.19 1995/05/15 01:22:00 william Exp $";
+      "@(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/ftp.c,v 3.0 1996/05/06 22:06:00 william Exp $";
 #endif
 
 #include <sys/types.h>
@@ -46,6 +46,9 @@ static char RCSid[] =
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#ifdef isc /* SunSoft/Interactive UNIX */
+#include <net/errno.h>
+#endif
 
 #include "const.h"
 
@@ -62,7 +65,7 @@ static int gnReadyToReceiveData=FALSE;
 void FtpFreeBuf(buf)
    char *buf;
 {
-   cfree(buf);
+   free(buf);
 }
 
 void FtpDebug(val)
@@ -76,7 +79,7 @@ int FtpDoConnect(psz_host, us_port, pn_socket)
    int us_port, *pn_socket;
 {
    int rc, len=strlen(psz_host)+80;
-   char *msg=(char*)calloc(len+1, sizeof(char));
+   char *msg=(char*)malloc((len+1)*sizeof(char));
 
    if (msg == NULL) {
       fprintf(stderr, "Memory allocation failed.\n");
@@ -93,7 +96,7 @@ int FtpDoConnect(psz_host, us_port, pn_socket)
       sprintf(msg, "Fail to connect to FTP server on \"%s\".", psz_host);
    }
    ShowRemoteStatus(msg);
-   cfree(msg);
+   free(msg);
 
    return rc;
 }
@@ -108,7 +111,7 @@ char *AppendSimpleString(buf, value)
       int new_len=cur_len+2;
 
       if (buf == NULL) {
-         buf = (char*)calloc(new_len+1, sizeof(char));
+         buf = (char*)malloc((new_len+1)*sizeof(char));
       } else {
          buf = (char*)realloc(buf, new_len+1);
       }
@@ -118,7 +121,7 @@ char *AppendSimpleString(buf, value)
       int new_len=cur_len+strlen(value)+2;
 
       if (buf == NULL) {
-         buf = (char*)calloc(new_len+1, sizeof(char));
+         buf = (char*)malloc((new_len+1)*sizeof(char));
       } else {
          buf = (char*)realloc(buf, new_len+1);
       }
@@ -189,7 +192,7 @@ int FtpDoRead(n_socket, ppsz_buf, pn_buf_sz)
 {
    int buf_sz=0x400, len=0, end_of_file=FALSE;
    int rc=(-1);
-   char *buf=(char*)calloc(buf_sz, sizeof(char));
+   char *buf=(char*)malloc(buf_sz*sizeof(char));
 
    if (pn_buf_sz != NULL) *pn_buf_sz = 0;
    *ppsz_buf = NULL;
@@ -212,11 +215,11 @@ int FtpDoRead(n_socket, ppsz_buf, pn_buf_sz)
       if (bytes_read <= 0) {
          if (bytes_read < 0 && (errno == ENOTCONN || errno == ECONNRESET ||
                errno == EPIPE)) {
-            cfree(buf);
+            free(buf);
             fprintf(stderr, "Network read error.\n");
             return TG_REMOTE_STATUS_READ;
          } else if (bytes_read < 0) {
-            cfree(buf);
+            free(buf);
             fprintf(stderr, "Network error.\n");
             return TG_REMOTE_STATUS_NET;
          }
@@ -225,7 +228,7 @@ int FtpDoRead(n_socket, ppsz_buf, pn_buf_sz)
          len += bytes_read;
       }
       if (!end_of_file && UserAbortComm()) {
-         if (buf != NULL) cfree(buf);
+         if (buf != NULL) free(buf);
          ShowRemoteStatus("FTP: aborted by the user.");
          return TG_REMOTE_STATUS_INTR;
       } else {
@@ -261,7 +264,7 @@ int FtpDoRead(n_socket, ppsz_buf, pn_buf_sz)
    } while (!end_of_file);
    if (debugFtp >= 2) fprintf(stderr, "    rc = %1d in FtpDoRead().\n", rc);
    if (rc == 421) {
-      cfree(buf);
+      free(buf);
       fprintf(stderr, "Connection terminated by the FTP server.\n");
       return TG_REMOTE_STATUS_TERM;
    } else {
@@ -285,7 +288,7 @@ int FtpSendSimpleCmd(n_socket, value)
       return TG_REMOTE_STATUS_MEM;
    }
    status = FtpDoWrite(n_socket, cmd);
-   cfree(cmd);
+   free(cmd);
    return status;
 }
 
@@ -311,15 +314,16 @@ int FtpSendRetrieveCmd(n_socket, psz_path)
    char *psz_path;
 {
    int status=TG_REMOTE_STATUS_OK;
-   char *cmd=(char*)calloc(strlen("RETR")+1+strlen(psz_path)+2+1, sizeof(char));
+   char *cmd;
 
+   cmd = (char*)malloc((strlen("RETR")+1+strlen(psz_path)+2+1)*sizeof(char));
    if (cmd == NULL) {
       fprintf(stderr, "Memory allocation failed.\n");
       return TG_REMOTE_STATUS_MEM;
    }
    sprintf(cmd, "RETR %s\r\n", psz_path);
    status = FtpDoWrite(n_socket, cmd);
-   cfree(cmd);
+   free(cmd);
    return status;
 }
 
@@ -336,15 +340,16 @@ int FtpSendCWDCmd(n_socket, psz_path)
    char *psz_path;
 {
    int status=TG_REMOTE_STATUS_OK;
-   char *cmd=(char*)calloc(strlen("CWD")+1+strlen(psz_path)+2+1, sizeof(char));
+   char *cmd;
 
+   cmd = (char*)malloc((strlen("CWD")+1+strlen(psz_path)+2+1)*sizeof(char));
    if (cmd == NULL) {
       fprintf(stderr, "Memory allocation failed.\n");
       return TG_REMOTE_STATUS_MEM;
    }
    sprintf(cmd, "CWD %s\r\n", psz_path);
    status = FtpDoWrite(n_socket, cmd);
-   cfree(cmd);
+   free(cmd);
    return status;
 }
 
