@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1997, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -26,11 +26,9 @@
  * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/grid.c,v 3.10 1997/10/03 19:09:04 william Exp $
  */
-#ifndef lint
-static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/grid.c,v 2.113 1995/07/10 05:15:33 william Exp $";
-#endif
 
 #include <stdio.h>
 #include <X11/Xlib.h>
@@ -58,7 +56,9 @@ static char RCSid[] =
 #include "scroll.e"
 #include "select.e"
 #include "setup.e"
+#include "stretch.e"
 #include "text.e"
+#include "util.e"
 
 #define ENGLISH_GRID_STEP 8
 #define METRIC_GRID_STEP 5
@@ -76,6 +76,8 @@ int	usePaperSizeStoredInFile = FALSE;
 int	oneMotionSelectMove = FALSE;
 int	queryZoomInPoint=FALSE;
 
+static int zoomInDontRedraw=FALSE;
+
 char * layoutMenuStr[] =
       {  "+Grid          #i",
          "-Grid          #d",
@@ -97,9 +99,12 @@ char * layoutMenuStr[] =
          "HideBit/Pixmap   ",
          "UseGrayScale     ",
          "ShowMeasurement  ",
+         "SetMeasureUnit   ",
          "ShowMenubar      ",
          "ShowStatus       ",
-         "ClickSelClickMove"
+         "ClickSelClickMove",
+         "UseColorLayers   ",
+         NULL
       };
 #define MAX_LAYOUT_STR 20 /* longest string length in layoutMenuStr[] */
 static char * layoutMenuDescription[] =
@@ -123,9 +128,11 @@ static char * layoutMenuDescription[] =
          "Toggle between hiding/showing detailed bitmap/pixmap objects",
          "Use gray scales for tiling patterns to speed up printing",
          "Show cursor measurements",
+         "Set the unit of measurements",
          "Show menubar window",
          "Show status window",
          "Toggle between click-select-move and click-select-click-move modes",
+         "Use color layers",
          NULL
       };
 #define MAX_LAYOUT_DESC 80 /* longest string length in layoutMenuStr[] */
@@ -361,22 +368,24 @@ static char snapOffStr[]         = "SnapOff       ^#g";
 static char snapOnDesc[]         = "Turn on snapping to grid points";
 static char snapOffDesc[]        = "Turn off snapping to grid points";
 
-static char printerStr[]      = "Printer       ^#x";
-static char latexFigStr[]     = "LaTeXFig      ^#x";
-static char psFileStr[]       = "RawPSFile     ^#x";
-static char xbmStr[]          = "XBitmap       ^#x";
-static char xpmStr[]          = "XPixmap       ^#x";
-static char textFileStr[]     = "TextFile      ^#x";
-static char epsiFileStr[]     = "EPSI          ^#x";
-static char gifStr[]          = "GIF/ISMAP     ^#x";
-static char printerDesc[]     = "Set printing mode to print to a printer";
-static char latexFigDesc[]    = "Set printing mode to generate EPS files";
-static char psFileDesc[]      = "Set printing mode to generate PS files";
-static char xbmDesc[]         = "Set printing mode to generate bitmap files";
-static char xpmDesc[]         = "Set printing mode to generate pixmap files";
-static char textFileDesc[]    = "Set printing mode to generate ASCII files";
-static char epsiFileDesc[]    = "Set printing mode to generate EPSI files";
-static char gifDesc[]         = "Set printing mode to generate GIF/ISMAP files";
+static char printerStr[]   = "Printer       ^#x";
+static char latexFigStr[]  = "LaTeXFig      ^#x";
+static char psFileStr[]    = "RawPSFile     ^#x";
+static char xbmStr[]       = "XBitmap       ^#x";
+static char xpmStr[]       = "XPixmap       ^#x";
+static char textFileStr[]  = "TextFile      ^#x";
+static char epsiFileStr[]  = "EPSI          ^#x";
+static char gifStr[]       = "GIF/ISMAP     ^#x";
+static char htmlStr[]      = "HTML/USEMAP   ^#x";
+static char printerDesc[]  = "Set printing mode to print to a printer";
+static char latexFigDesc[] = "Set printing mode to generate EPS files";
+static char psFileDesc[]   = "Set printing mode to generate PS files";
+static char xbmDesc[]      = "Set printing mode to generate bitmap files";
+static char xpmDesc[]      = "Set printing mode to generate pixmap files";
+static char textFileDesc[] = "Set printing mode to generate ASCII files";
+static char epsiFileDesc[] = "Set printing mode to generate EPSI files";
+static char gifDesc[]      = "Set printing mode to generate GIF/ISMAP files";
+static char htmlDesc[]     = "Set printing mode to generate HTML files";
 
 static char bwPostScriptStr[]    = "BlkWhtPS      ^#k";
 static char colorPostScriptStr[] = "ColorPS       ^#k";
@@ -423,6 +432,11 @@ static char showStatusDesc[]      = "Show status window";
 static char oneMotionSelMoveStr[] = "OneMotionSelMove";
 static char clickSelClickMoveStr[]= "ClickSelClickMove";
 
+static char noColorLayersStr[]    = "NoColorLayers";
+static char colorLayersStr[]      = "UseColorLayers";
+static char noColorLayersDesc[]   = "Do not use color layers";
+static char colorLayersDesc[]     = "Use color layers";
+
 void ToggleGridSystem ()
 {
    int	saved_orig_x, saved_orig_y;
@@ -558,6 +572,24 @@ void ToggleUseGray ()
    UpdateSubMenu (MENU_LAYOUT);
 }
 
+void SetMeasureUnit()
+{
+   char buf[MAXSTRING+1], spec[MAXSTRING+1];
+
+   GetUnitSpec(gszMsgBox);
+   sprintf(buf, "( current spec: '%s' )", gszMsgBox);
+   sprintf(gszMsgBox, "Please enter '%s' (%s):",
+         "<num> {pixel|cm|in}/<unit>", "e.g., 36 in/yd or 0.1 in/mil");
+   Dialog(gszMsgBox, buf, spec);
+   UtilTrimBlanks(spec);
+   if (*spec == '\0') return;
+   if (SetUnit(spec)) {
+      RedrawRulers();
+      ShowUnitMsg();
+      SetFileModified(TRUE);
+   }
+}
+
 void ToggleShowMenubar ()
 {
    noMenubar = !noMenubar;
@@ -583,27 +615,30 @@ void ToggleShowStatus ()
 void ToggleWhereToPrint ()
 {
    whereToPrint = (whereToPrint+1) % MAXWHERETOPRINT;
-   switch (whereToPrint)
-   {
-      case PRINTER: Msg ("Print device set to printer."); break;
-      case LATEX_FIG:
-         Msg ("Will print Encapsulated PostScript (LaTeX-Figure) file.");
-         break;
-      case PS_FILE: Msg ("Will print raw PostScript file."); break;
-      case XBM_FILE:
-         if (colorDump)
-            Msg ("Will print X11 Pixmap file.");
-         else
-            Msg ("Will print X11 Bitmap file.");
-         break;
-      case TEXT_FILE: Msg ("Will print ASCII text file."); break;
-      case EPSI_FILE:
-         Msg ("Will print EPSI file (EPS with preview bitmap).");
-         break;
-      case GIF_FILE: Msg ("Will print GIF/ISMAP file."); break;
+   switch (whereToPrint) {
+   case PRINTER: Msg("Print device set to printer."); break;
+   case LATEX_FIG:
+      Msg("Will print Encapsulated PostScript (LaTeX-Figure) file.");
+      break;
+   case PS_FILE: Msg("Will print raw PostScript file."); break;
+   case XBM_FILE:
+      if (colorDump) {
+         Msg("Will print X11 Pixmap file.");
+      } else {
+         Msg("Will print X11 Bitmap file.");
+      }
+      break;
+   case TEXT_FILE: Msg("Will print ASCII text file."); break;
+   case EPSI_FILE:
+      Msg("Will print EPSI file (EPS with preview bitmap).");
+      break;
+   case GIF_FILE: Msg("Will print GIF/ISMAP file."); break;
+   case HTML_FILE:
+      Msg("Will print HTML (with Client-side imagemap) file.");
+      break;
    }
-   ShowWhereToPrint ();
-   UpdateSubMenu (MENU_LAYOUT);
+   ShowWhereToPrint();
+   UpdateSubMenu(MENU_LAYOUT);
    if (colorDump) {
       SetMouseStatus(colorPrintMouseStatus[whereToPrint].l,
             colorPrintMouseStatus[whereToPrint].m,
@@ -625,6 +660,35 @@ void ToggleOneMotionSelectMove ()
    UpdateSubMenu (MENU_LAYOUT);
 }
 
+void ToggleColorLayers ()
+{
+   colorLayers = !colorLayers;
+   if (colorLayers) {
+      XMapWindow(mainDisplay, colorWindow);
+      XMapWindow(mainDisplay, colorDummyWindow);
+      Msg("Color layers enabled.");
+   } else {
+      XUnmapWindow(mainDisplay, colorWindow);
+      XUnmapWindow(mainDisplay, colorDummyWindow);
+      Msg("Color layers disabled.");
+   }
+   Reconfigure(TRUE);
+   UpdateSubMenu(MENU_LAYOUT);
+   ClearAndRedrawDrawWindow();
+}
+
+void ToggleStretchableText ()
+{
+   stretchableText = !stretchableText;
+   if (stretchableText) {
+      Msg("Text objects are stretchable.");
+   } else {
+      Msg("Text objects are not stretchable.");
+   }
+   ShowStretchableTextMode();
+   UpdateSubMenu(MENU_STRETCHTEXT);
+}
+
 static
 void PostZoom (xc, yc)
    int	xc, yc;
@@ -715,17 +779,19 @@ void PostZoom (xc, yc)
       drawOrigY = 0;
 
    UpdDrawWinBBox ();
-   SetWatchCursor (drawWindow);
-   SetWatchCursor (mainWindow);
-   RedrawRulers ();
-   RedrawScrollBars ();
-   AdjSplineVs ();
-   AdjCaches ();
-   ShowZoom ();
-   /* use to be ClearAndRedrawDrawWindowDontDrawCurText() */
-   ClearAndRedrawDrawWindowNoCurT ();
-   SetDefaultCursor (mainWindow);
-   ShowCursor ();
+   if (!zoomInDontRedraw) {
+      SetWatchCursor (drawWindow);
+      SetWatchCursor (mainWindow);
+      RedrawRulers ();
+      RedrawScrollBars ();
+      AdjSplineVs ();
+      AdjCaches ();
+      ShowZoom ();
+      /* use to be ClearAndRedrawDrawWindowDontDrawCurText() */
+      ClearAndRedrawDrawWindowNoCurT ();
+      SetDefaultCursor (mainWindow);
+      ShowCursor ();
+   }
    justDupped = FALSE;
 }
 
@@ -756,26 +822,41 @@ void DefaultZoom ()
    if (curChoice==DRAWTEXT && textCursorShown) PostZoomCurText (xc, yc);
 }
 
+static int zoomInAbsX=0, zoomInAbsY=0;
+static int useZoomInAtCursor=FALSE;
+
 void ZoomIn ()
 {
    int	xc, yc;
 
    if (zoomedIn && zoomScale == MAX_ZOOMED_IN)
    {
-      Msg ("Already at highest magnification, can no longer zoom in.");
+      MsgBox("Already at highest magnification, can no longer zoom in.",
+            TOOL_NAME, INFO_MB);
       return;
    }
 
    if (curChoice==DRAWTEXT && textCursorShown) {
+      if (useZoomInAtCursor) {
+         TwoLineMsg("Zooming in around text being edited...", gszMsgBox);
+         sprintf(gszMsgBox, "    %s %s",
+               "(the cursor position is ignored even",
+               "though <Cntrl><Shift>Btn1 is used)");
+      }
       PrepareZoomCurText(&xc, &yc);
+   } else if (useZoomInAtCursor) {
+      xc = zoomInAbsX;
+      yc = zoomInAbsY;
    } else if (queryZoomInPoint==TRUE || (queryZoomInPoint==INVALID &&
          topSel==NULL)) {
-      MakeQuiescent();
+      if (queryZoomInPoint==INVALID && topSel==NULL) {
+         MakeQuiescent();
+      }
       SaveStatusStrings();
       TwoLineMsg ("Left button selects a zoom center,",
             "    other buttons cancel zoom.");
       SetMouseStatus("Select zoom center", "Cancel", "Cancel");
-      if (MagLoop(&xc, &yc) != Button1) {
+      if (DrawWindowLoop(&xc, &yc, magCursor, FALSE) != Button1) {
          RestoreStatusStrings();
          Msg("ZoomIn canceled by the user.");
          return;
@@ -817,6 +898,16 @@ void ZoomIn ()
    if (curChoice==DRAWTEXT && textCursorShown) PostZoomCurText (xc, yc);
 }
 
+void ZoomInAtCursor(abs_x, abs_y)
+   int abs_x, abs_y;
+{
+   zoomInAbsX = abs_x;
+   zoomInAbsY = abs_y;
+   useZoomInAtCursor = TRUE;
+   ZoomIn();
+   useZoomInAtCursor = FALSE;
+}
+
 void ZoomWayOut ()
 {
    int	saved_zoom_scale = zoomScale, saved_zoomed_in = zoomedIn;
@@ -860,42 +951,53 @@ void ZoomWayOut ()
    PostZoom (0, 0);
 }
 
-void ZoomOut ()
+void ZoomOut()
 {
-   int	xc, yc;
+   int xc, yc;
 
-   if (paperWidth <= drawWinW && paperHeight <= drawWinH)
-   {
-      Msg ("Already at paper boundaries, can no longer zoom out.");
+   if (paperWidth <= drawWinW && paperHeight <= drawWinH) {
+      strcpy(gszMsgBox, "Already at paper boundaries, can no longer zoom out.");
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return;
    }
+   if (curChoice==DRAWTEXT && textCursorShown) {
+      PrepareZoomCurText(&xc, &yc);
+   } else if (queryZoomInPoint==BAD) {
+      Window root_win, child_win;
+      int root_x, root_y;
+      unsigned int status;
 
-   if (curChoice==DRAWTEXT && textCursorShown)
-      PrepareZoomCurText (&xc, &yc);
-   else
-   {
+      XQueryPointer(mainDisplay, drawWindow, &root_win, &child_win,
+            &root_x, &root_y, &xc, &yc, &status);
+      xc = ABS_X(xc);
+      yc = ABS_Y(yc);
+   } else {
       xc = (topSel==NULL) ? ((drawOrigX<<1)+drawWinW)>>1 : (selLtX+selRbX)>>1;
       yc = (topSel==NULL) ? ((drawOrigY<<1)+drawWinH)>>1 : (selLtY+selRbY)>>1;
    }
-
-   if (curChoice == VERTEXMODE)
-   {
-      TieLooseEnds ();
-      SetCurChoice (NOTHING);
+   if (curChoice == VERTEXMODE) {
+      TieLooseEnds();
+      SetCurChoice(NOTHING);
    }
-
-   if (!zoomedIn)
+   if (!zoomedIn) {
       zoomScale++;
-   else if (zoomScale == 1)
-   {
+   } else if (zoomScale == 1) {
       zoomedIn = FALSE;
       zoomScale--;
-   }
-   else
+   } else {
       zoomScale--;
+   }
+   PostZoom(xc, yc);
+   if (curChoice==DRAWTEXT && textCursorShown) PostZoomCurText(xc, yc);
+}
 
-   PostZoom (xc, yc);
-   if (curChoice==DRAWTEXT && textCursorShown) PostZoomCurText (xc, yc);
+void CenterAtCursor(abs_x, abs_y)
+   int abs_x, abs_y;
+{
+   zoomInDontRedraw = TRUE;
+   ZoomInAtCursor(abs_x, abs_y);
+   zoomInDontRedraw = FALSE;
+   ZoomOut();
 }
 
 void PreciseZoom (zoomed_in, zoom_scale, force_zoom)
@@ -928,7 +1030,7 @@ void PreciseZoom (zoomed_in, zoom_scale, force_zoom)
 
 void SetPSPageWidthHeight ()
 {
-   if (mainWindow != None)
+   if (psYOffStr != NULL && psYOff != NULL)
    {
       sprintf (psYOffStr[0], "%.2f",
             (float)(((float)onePageHeight)/((float)PIX_PER_INCH)));
@@ -959,9 +1061,9 @@ int UpdPageStyle (NewPageStyle)
 
    pageStyle = NewPageStyle;
    onePageWidth =
-         round(psPageWidthInInch[NewPageStyle]*PIX_PER_INCH*100/printMag);
+         round(psPageWidthInInch[NewPageStyle]*PIX_PER_INCH*100.0/printMag);
    onePageHeight =
-         round(psPageHeightInInch[NewPageStyle]*PIX_PER_INCH*100/printMag);
+         round(psPageHeightInInch[NewPageStyle]*PIX_PER_INCH*100.0/printMag);
    paperWidth = onePageWidth * paperCol;
    paperHeight = onePageHeight * paperRow;
    if (drawOrigX + drawWinW > paperWidth || drawOrigX + drawWinW > old_w)
@@ -1134,9 +1236,11 @@ void LayoutSubMenu (index)
       case LAYOUT_TOGGLE_MAP_SHOWN: ToggleMapShown (); break;
       case LAYOUT_TOGGLE_USE_GRAY: ToggleUseGray (); break;
       case LAYOUT_TOGGLE_SHOW_MEASURE: ToggleShowMeasurement (); break;
+      case LAYOUT_TOGGLE_SET_MEASURE_UNIT: SetMeasureUnit (); break;
       case LAYOUT_TOGGLE_SHOW_MENUBAR: ToggleShowMenubar (); break;
       case LAYOUT_TOGGLE_SHOW_STATUS: ToggleShowStatus (); break;
       case LAYOUT_TOGGLE_ONEMOTION_SELMOVE: ToggleOneMotionSelectMove (); break;
+      case LAYOUT_TOGGLE_COLOR_LAYERS: ToggleColorLayers (); break;
    }
 }
 
@@ -1146,25 +1250,22 @@ void SetUpLayoutMenuStr (menu_strings, menu_description)
    register int	i;
    char		* str_ptr, * desc_ptr;
 
-   str_ptr = (char *) calloc (MAX_LAYOUT_STR*MAXLAYOUTMENUS, sizeof(char));
-   desc_ptr = (char *) calloc (MAX_LAYOUT_DESC*MAXLAYOUTMENUS, sizeof(char));
+   str_ptr = (char*)malloc(MAX_LAYOUT_STR*MAXLAYOUTMENUS*sizeof(char));
+   desc_ptr = (char*)malloc(MAX_LAYOUT_DESC*MAXLAYOUTMENUS*sizeof(char));
+   if (str_ptr == NULL || desc_ptr == NULL) FailAllocMessage();
    for (i = 0; i < MAXLAYOUTMENUS; i++, str_ptr += MAX_LAYOUT_STR,
-         desc_ptr += MAX_LAYOUT_DESC)
-   {
+         desc_ptr += MAX_LAYOUT_DESC) {
       menu_strings[i] = str_ptr;
       strcpy (menu_strings[i], layoutMenuStr[i]);
       menu_description[i] = desc_ptr;
       strcpy (menu_description[i], layoutMenuDescription[i]);
    }
-   if (gridShown)
-   {
-      strcpy (menu_strings[LAYOUT_GRID], hideGridStr);
-      strcpy (menu_description[LAYOUT_GRID], hideGridDesc);
-   }
-   else
-   {
-      strcpy (menu_strings[LAYOUT_GRID], showGridStr);
-      strcpy (menu_description[LAYOUT_GRID], showGridDesc);
+   if (gridShown) {
+      strcpy(menu_strings[LAYOUT_GRID], hideGridStr);
+      strcpy(menu_description[LAYOUT_GRID], hideGridDesc);
+   } else {
+      strcpy(menu_strings[LAYOUT_GRID], showGridStr);
+      strcpy(menu_description[LAYOUT_GRID], showGridDesc);
    }
    if (gridOn)
    {
@@ -1211,6 +1312,10 @@ void SetUpLayoutMenuStr (menu_strings, menu_description)
          strcpy (menu_description[LAYOUT_TOGGLE_WHERE_TO_PRINT], gifDesc);
          break;
       case GIF_FILE:
+         strcpy (menu_strings[LAYOUT_TOGGLE_WHERE_TO_PRINT], htmlStr);
+         strcpy (menu_description[LAYOUT_TOGGLE_WHERE_TO_PRINT], htmlDesc);
+         break;
+      case HTML_FILE:
          strcpy (menu_strings[LAYOUT_TOGGLE_WHERE_TO_PRINT], printerStr);
          strcpy (menu_description[LAYOUT_TOGGLE_WHERE_TO_PRINT], printerDesc);
          break;
@@ -1299,40 +1404,48 @@ void SetUpLayoutMenuStr (menu_strings, menu_description)
       strcpy (menu_strings[LAYOUT_TOGGLE_SHOW_STATUS], hideStatusStr);
       strcpy (menu_description[LAYOUT_TOGGLE_SHOW_STATUS], hideStatusDesc);
    }
-   if (oneMotionSelectMove)
+   if (oneMotionSelectMove) {
       strcpy (menu_strings[LAYOUT_TOGGLE_ONEMOTION_SELMOVE],
             clickSelClickMoveStr);
-   else
+   } else {
       strcpy (menu_strings[LAYOUT_TOGGLE_ONEMOTION_SELMOVE],
             oneMotionSelMoveStr);
+   }
+   if (colorLayers) {
+      strcpy(menu_strings[LAYOUT_TOGGLE_COLOR_LAYERS], noColorLayersStr);
+      strcpy (menu_description[LAYOUT_TOGGLE_SHOW_STATUS], noColorLayersDesc);
+   } else {
+      strcpy(menu_strings[LAYOUT_TOGGLE_COLOR_LAYERS], colorLayersStr);
+      strcpy (menu_description[LAYOUT_TOGGLE_SHOW_STATUS], colorLayersDesc);
+   }
 }
 
 int LayoutMenu (X, Y, TrackMenubar)
    int	X, Y, TrackMenubar;
 {
-   int		index, * fore_colors, * valid, * init_rv;
-   char		* menu_strings[MAXLAYOUTMENUS];
-   char		* menu_description[MAXLAYOUTMENUS];
+   int index, *fore_colors, *valid, *init_rv;
+   char *menu_strings[MAXLAYOUTMENUS];
+   char *menu_description[MAXLAYOUTMENUS];
 
-   SetUpLayoutMenuStr (menu_strings, menu_description);
+   SetUpLayoutMenuStr(menu_strings, menu_description);
 
-   DefaultColorArrays (MAXLAYOUTMENUS, &fore_colors, &valid, &init_rv, NULL);
+   DefaultColorArrays(colorDisplay ? MAXLAYOUTMENUS : MAXLAYOUTMENUS-1,
+         &fore_colors, &valid, &init_rv, NULL);
 
-   switch (pageStyle)
-   {
-      case PORTRAIT: init_rv[LAYOUT_PORT] = TRUE; break;
-      case LANDSCAPE: init_rv[LAYOUT_LAND] = TRUE; break;
+   switch (pageStyle) {
+   case PORTRAIT: init_rv[LAYOUT_PORT] = TRUE; break;
+   case LANDSCAPE: init_rv[LAYOUT_LAND] = TRUE; break;
    }
    activeMenu = MENU_LAYOUT;
-   index = TextMenuLoop (X, Y, menu_strings, MAXLAYOUTMENUS,
-         fore_colors, valid, init_rv, menu_description, SINGLECOLOR,
-         TrackMenubar);
+   index = TextMenuLoop(X, Y, menu_strings,
+         colorDisplay ? MAXLAYOUTMENUS : MAXLAYOUTMENUS-1, fore_colors, valid,
+         init_rv, menu_description, SINGLECOLOR, TrackMenubar);
 
-   cfree (menu_strings[0]);
-   cfree (menu_description[0]);
+   free(menu_strings[0]);
+   free(menu_description[0]);
 
-   if (index >= 0) LayoutSubMenu (index);
-   return (index);
+   if (index >= 0) LayoutSubMenu(index);
+   return index;
 }
 
 void GridXY (X, Y, GridX, GridY)
@@ -1410,7 +1523,7 @@ int MoveModeMenu (X, Y, TrackMenubar)
    int	index, * fore_colors, * valid, * init_rv;
 
    DefaultColorArrays (MAXMOVEMODES, &fore_colors, &valid, &init_rv, NULL);
-   cfree (valid);
+   free(valid);
    init_rv[moveMode] = TRUE;
    activeMenu = MENU_MOVEMODE;
    index = PxMpMenuLoop (X, Y, choiceImageW, choiceImageH, MAXMOVEMODES, 1,
@@ -1420,3 +1533,40 @@ int MoveModeMenu (X, Y, TrackMenubar)
    if (index >= 0) MoveModeSubMenu (index);
    return (index);
 }
+
+void StretchableTextModeSubMenu(index)
+   int index;
+{
+   stretchableText = index;
+   switch (stretchableText) {
+   case NO_STRETCHABLE_TEXT: Msg("Text objects are not stretchable."); break;
+   case STRETCHABLE_TEXT: Msg("Text objects are stretchable."); break;
+   }
+   ShowStretchableTextMode();
+   UpdateSubMenu(MENU_STRETCHTEXT);
+}
+
+static char * stretchableTextModeMenuDescription[] =
+{
+   "Make text objects stretchable",
+   "Make text objects not stretchable",
+   NULL
+};
+
+int StretchableTextModeMenu(X, Y, TrackMenubar)
+   int X, Y, TrackMenubar;
+{
+   int index, *fore_colors, *valid, *init_rv;
+
+   DefaultColorArrays(MAXSTRETCHABLEMODES, &fore_colors, &valid, &init_rv,
+         NULL);
+   free(valid);
+   init_rv[stretchableText] = TRUE;
+   activeMenu = MENU_STRETCHTEXT;
+   index = PxMpMenuLoop(X, Y, choiceImageW, choiceImageH, MAXSTRETCHABLEMODES,
+         1, MAXSTRETCHABLEMODES, fore_colors, stretchableModePixmap, init_rv,
+         stretchableTextModeMenuDescription, SINGLECOLOR, TrackMenubar);
+
+   if (index >= 0) StretchableTextModeSubMenu(index);
+   return (index);
+}
