@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,7 +29,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/group.c,v 2.30 1995/05/15 01:22:05 william Exp $";
+      "@(#)$Header: /n/opus/u/guest/william/src/tgif/v3/RCS/group.c,v 3.0 1996/05/06 16:05:24 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -62,13 +62,17 @@ void CreateGroupObj (TopObjPtr, BotObjPtr)
    struct GroupRec	* group_ptr;
    struct ObjRec	* obj_ptr;
 
-   group_ptr = (struct GroupRec *) calloc (1, sizeof(struct GroupRec));
+   group_ptr = (struct GroupRec *)malloc(sizeof(struct GroupRec));
+   if (group_ptr == NULL) FailAllocMessage();
+   memset(group_ptr, 0, sizeof(struct GroupRec));
    group_ptr->first = TopObjPtr;
    group_ptr->last = BotObjPtr;
    group_ptr->rotate = ROTATE0;
    group_ptr->flip = NO_FLIP;
    group_ptr->deck_index = (-1);
-   obj_ptr = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
+   obj_ptr = (struct ObjRec *)malloc(sizeof(struct ObjRec));
+   if (obj_ptr == NULL) FailAllocMessage();
+   memset(obj_ptr, 0, sizeof(struct ObjRec));
    obj_ptr->x = selObjLtX; /* note:  selLtX, selLtY are absolute */
    obj_ptr->y = selObjLtY;
    obj_ptr->id = objId++;
@@ -84,6 +88,8 @@ void CreateGroupObj (TopObjPtr, BotObjPtr)
    obj_ptr->obbox.rby = selObjRbY;
    obj_ptr->detail.r = group_ptr;
    obj_ptr->fattr = obj_ptr->lattr = NULL;
+   obj_ptr->ctm = NULL;
+   obj_ptr->invisible = FALSE;
    if (numObjLocked > 0) obj_ptr->locked = TRUE;
    AddObj (NULL, topObj, obj_ptr);
 }
@@ -96,8 +102,10 @@ void SaveGroupObj (FP, ObjPtr, Level)
    if (fprintf (FP, "group([\n") == EOF) writeFileFailed = TRUE;
    Save (FP, ObjPtr->detail.r->last, Level+1, INVALID);
    if (fprintf (FP, "],\n") == EOF) writeFileFailed = TRUE;
-   if (fprintf (FP, "%1d,%1d,", ObjPtr->id, ObjPtr->locked) == EOF)
+   if (fprintf (FP, "%1d,%1d,%1d,", ObjPtr->id, ObjPtr->locked,
+         ObjPtr->invisible) == EOF) {
       writeFileFailed = TRUE;
+   }
    SaveAttrs (FP, ObjPtr->lattr);
    if (fprintf (FP, ")") == EOF) writeFileFailed = TRUE;
 }
@@ -110,8 +118,10 @@ void SaveCompObj (FP, ObjPtr, Level)
    if (fprintf (FP, "sym([\n") == EOF) writeFileFailed = TRUE;
    Save (FP, ObjPtr->detail.r->last, Level+1, INVALID);
    if (fprintf (FP, "],\n") == EOF) writeFileFailed = TRUE;
-   if (fprintf (FP, "%1d,%1d,", ObjPtr->id, ObjPtr->locked) == EOF)
+   if (fprintf (FP, "%1d,%1d,%1d,", ObjPtr->id, ObjPtr->locked,
+         ObjPtr->invisible) == EOF) {
       writeFileFailed = TRUE;
+   }
    SaveAttrs (FP, ObjPtr->lattr);
    if (fprintf (FP, ")") == EOF) writeFileFailed = TRUE;
 }
@@ -124,10 +134,11 @@ void SaveIconObj (FP, ObjPtr, Level)
    if (fprintf (FP, "icon([\n") == EOF) writeFileFailed = TRUE;
    Save (FP, ObjPtr->detail.r->last, Level+1, INVALID);
    if (fprintf (FP, "],\n") == EOF) writeFileFailed = TRUE;
-   if (fprintf (FP, "\"%s\",%1d,%1d,%1d,%1d,", ObjPtr->detail.r->s, ObjPtr->id,
-         ObjPtr->detail.r->rotate, ObjPtr->detail.r->flip, ObjPtr->locked) ==
-         EOF)
+   if (fprintf (FP, "\"%s\",%1d,%1d,%1d,%1d,%1d,",
+         ObjPtr->detail.r->s, ObjPtr->id, ObjPtr->detail.r->rotate,
+         ObjPtr->detail.r->flip, ObjPtr->locked, ObjPtr->invisible) == EOF) {
       writeFileFailed = TRUE;
+   }
    SaveAttrs (FP, ObjPtr->lattr);
    if (fprintf (FP, ")") == EOF) writeFileFailed = TRUE;
 }
@@ -141,119 +152,124 @@ void ReadGroupObj (FP, ObjType, ObjPtr)
    struct ObjRec	* top_obj = NULL, * bot_obj = NULL, * obj_ptr;
    int			ltx, lty, rbx, rby, id=0, locked=FALSE;
    int			obj_ltx, obj_lty, obj_rbx, obj_rby, rotate=0, flip=0;
+   int			invisible=FALSE;
    char			line[MAXSTRING+1], * s, * s1, tmp_str[MAXSTRING+1];
 
    *ObjPtr = NULL;
-   while (ReadObj (FP, &obj_ptr))
-   {
+   while (ReadObj(FP, &obj_ptr)) {
       if (obj_ptr == NULL) return;
 
       obj_ptr->next = top_obj;
-      if (top_obj == NULL)
+      if (top_obj == NULL) {
          bot_obj = obj_ptr;
-      else
+      } else {
          top_obj->prev = obj_ptr;
+      }
       top_obj = obj_ptr;
    }
    if (top_obj == NULL) return;
 
-   if (fileVersion <= 20 && (ObjType==OBJ_GROUP || ObjType==OBJ_SYM))
+   if (fileVersion <= 20 && (ObjType==OBJ_GROUP || ObjType==OBJ_SYM)) {
       id = objId++;
-   else
-   {
-      if (fgets (line, MAXSTRING, FP) == NULL) return;
+   } else {
+      if (fgets(line, MAXSTRING, FP) == NULL) return;
       scanLineNum++;
 
-      switch (ObjType)
-      {
-         case OBJ_GROUP:
-            InitScan (line, "\t\n, []");
-            if (fileVersion <= 25)
-            {
-               if (ScanValue ("%d", &id, "id", "group") == INVALID) return;
+      switch (ObjType) {
+      case OBJ_GROUP:
+         InitScan(line, "\t\n, []");
+         if (fileVersion <= 25) {
+            if (ScanValue("%d", &id, "id", "group") == INVALID) return;
+         } else if (fileVersion <= 32) {
+            if (ScanValue("%d", &id, "id", "group") == INVALID ||
+                ScanValue("%d", &locked, "locked", "group") == INVALID) {
+               return;
+            }
+         } else {
+            if (ScanValue("%d", &id, "id", "group") == INVALID ||
+                ScanValue("%d", &locked, "locked", "group") == INVALID ||
+                ScanValue("%d", &invisible, "invisible", "group") == INVALID) {
+               return;
+            }
+         }
+         if (id >= objId) objId = id+1;
+         break;
+      case OBJ_SYM:
+         InitScan(line, "\t\n, []");
+         if (fileVersion <= 25) {
+            if (ScanValue("%d", &id, "id", "sym") == INVALID) return;
+         } else if (fileVersion <= 32) {
+            if (ScanValue("%d", &id, "id", "sym") == INVALID ||
+                ScanValue("%d", &locked, "locked", "sym") == INVALID) {
+               return;
+            }
+         } else {
+            if (ScanValue("%d", &id, "id", "sym") == INVALID ||
+                ScanValue("%d", &locked, "locked", "sym") == INVALID ||
+                ScanValue("%d", &invisible, "invisible", "group") == INVALID) {
+               return;
+            }
+         }
+         if (id >= objId) objId = id+1;
+         break;
+      case OBJ_ICON:
+         strcpy(tmp_str, FindChar((int)'"', line));
+         s = FindChar((int)'"', tmp_str);
+         if (fileVersion == INVALID) return;
 
-               if (id >= objId) objId = id+1;
+         if (fileVersion <= 12) {
+            s1 = FindChar((int)',', s);
+            InitScan(s1, "\t\n, ");
+            if (ScanValue("%d", &id, "id", "icon") == INVALID) return;
+         } else if (fileVersion <= 25) {
+            s1 = FindChar((int)',', s);
+            InitScan(s1, "\t\n, ");
+            if (ScanValue("%d", &id, "id", "icon") == INVALID ||
+                ScanValue("%d", &rotate, "rotation", "icon") == INVALID ||
+                ScanValue("%d", &flip, "flip", "icon") == INVALID) {
+               return;
             }
-            else
-            {
-               if (ScanValue ("%d", &id, "id", "group") == INVALID ||
-                   ScanValue ("%d", &locked, "locked", "group") == INVALID)
-                  return;
-
-               if (id >= objId) objId = id+1;
+         } else if (fileVersion <= 32) {
+            s1 = FindChar((int)',', s);
+            InitScan(s1, "\t\n, ");
+            if (ScanValue("%d", &id, "id", "icon") == INVALID ||
+                ScanValue("%d", &rotate, "rotation", "icon") == INVALID ||
+                ScanValue("%d", &flip, "flip", "icon") == INVALID ||
+                ScanValue("%d", &locked, "locked", "icon") == INVALID) {
+               return;
             }
-            break;
-         case OBJ_SYM:
-            InitScan (line, "\t\n, []");
-            if (fileVersion <= 25)
-            {
-               if (ScanValue ("%d", &id, "id", "sym") == INVALID) return;
-
-               if (id >= objId) objId = id+1;
+         } else {
+            s1 = FindChar((int)',', s);
+            InitScan(s1, "\t\n, ");
+            if (ScanValue("%d", &id, "id", "icon") == INVALID ||
+                ScanValue("%d", &rotate, "rotation", "icon") == INVALID ||
+                ScanValue("%d", &flip, "flip", "icon") == INVALID ||
+                ScanValue("%d", &locked, "locked", "icon") == INVALID ||
+                ScanValue("%d", &invisible, "invisible", "icon") == INVALID) {
+               return;
             }
-            else
-            {
-               if (ScanValue ("%d", &id, "id", "sym") == INVALID ||
-                   ScanValue ("%d", &locked, "locked", "sym") == INVALID)
-                  return;
-
-               if (id >= objId) objId = id+1;
-            }
-            break;
-         case OBJ_ICON:
-            strcpy(tmp_str, FindChar ((int)'"', line));
-            s = FindChar ((int)'"', tmp_str);
-            if (fileVersion == INVALID) return;
-
-            if (fileVersion <= 12)
-            {
-               s1 = FindChar ((int)',', s);
-               InitScan (s1, "\t\n, ");
-               if (ScanValue ("%d", &id, "id", "icon") == INVALID) return;
-
-               if (id >= objId) objId = id+1;
-            }
-            else if (fileVersion <= 25)
-            {
-               s1 = FindChar ((int)',', s);
-               InitScan (s1, "\t\n, ");
-               if (ScanValue ("%d", &id, "id", "icon") == INVALID ||
-                   ScanValue ("%d", &rotate, "rotation", "icon") == INVALID ||
-                   ScanValue ("%d", &flip, "flip", "icon") == INVALID)
-               {
-                  return;
-               }
-               if (id >= objId) objId = id+1;
-            }
-            else
-            {
-               s1 = FindChar ((int)',', s);
-               InitScan (s1, "\t\n, ");
-               if (ScanValue ("%d", &id, "id", "icon") == INVALID ||
-                   ScanValue ("%d", &rotate, "rotation", "icon") == INVALID ||
-                   ScanValue ("%d", &flip, "flip", "icon") == INVALID ||
-                   ScanValue ("%d", &locked, "locked", "icon") == INVALID)
-               {
-                  return;
-               }
-               if (id >= objId) objId = id+1;
-            }
-            *(--s) = '\0';
-            break;
+         }
+         if (id >= objId) objId = id+1;
+         *(--s) = '\0';
+         break;
       }
    }
 
-   *ObjPtr = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
+   *ObjPtr = (struct ObjRec *)malloc(sizeof(struct ObjRec));
+   if (*ObjPtr == NULL) FailAllocMessage();
+   memset(*ObjPtr, 0, sizeof(struct ObjRec));
 
    top_obj->prev = NULL;
 
-   group_ptr = (struct GroupRec *) calloc (1, sizeof(struct GroupRec));
+   group_ptr = (struct GroupRec *)malloc(sizeof(struct GroupRec));
+   if (group_ptr == NULL) FailAllocMessage();
+   memset(group_ptr, 0, sizeof(struct GroupRec));
    group_ptr->first = top_obj;
    group_ptr->last = bot_obj;
    group_ptr->rotate = rotate;
    group_ptr->flip = flip;
    group_ptr->deck_index = (-1);
-   if (ObjType == OBJ_ICON) strcpy (group_ptr->s, tmp_str);
+   if (ObjType == OBJ_ICON) strcpy(group_ptr->s, tmp_str);
 
    ltx = top_obj->bbox.ltx;
    lty = top_obj->bbox.lty;
@@ -263,8 +279,7 @@ void ReadGroupObj (FP, ObjType, ObjPtr)
    obj_lty = top_obj->obbox.lty;
    obj_rbx = top_obj->obbox.rbx;
    obj_rby = top_obj->obbox.rby;
-   for (obj_ptr = top_obj->next; obj_ptr != NULL; obj_ptr = obj_ptr->next)
-   {
+   for (obj_ptr = top_obj->next; obj_ptr != NULL; obj_ptr = obj_ptr->next) {
       if (obj_ptr->bbox.ltx < ltx) ltx = obj_ptr->bbox.ltx;
       if (obj_ptr->bbox.lty < lty) lty = obj_ptr->bbox.lty;
       if (obj_ptr->bbox.rbx > rbx) rbx = obj_ptr->bbox.rbx;
@@ -290,6 +305,8 @@ void ReadGroupObj (FP, ObjType, ObjPtr)
    (*ObjPtr)->obbox.rbx = obj_rbx;
    (*ObjPtr)->obbox.rby = obj_rby;
    (*ObjPtr)->detail.r = group_ptr;
+   (*ObjPtr)->ctm = NULL;
+   (*ObjPtr)->invisible = invisible;
 }
 
 void FreeGroupObj (ObjPtr)
@@ -297,13 +314,12 @@ void FreeGroupObj (ObjPtr)
 {
    register struct ObjRec	* ptr, * next_obj;
 
-   for (ptr = ObjPtr->detail.r->first; ptr != NULL; ptr = next_obj)
-   {
+   for (ptr = ObjPtr->detail.r->first; ptr != NULL; ptr = next_obj) {
       next_obj = ptr->next;
-      FreeObj (ptr);
+      FreeObj(ptr);
    }
-   cfree (ObjPtr->detail.r);
-   cfree (ObjPtr);
+   free(ObjPtr->detail.r);
+   free(ObjPtr);
 }
 
 static
@@ -319,15 +335,16 @@ void UngroupObj (ObjPtr)
 
    tmpTopSel = tmpBotSel = NULL;
 
-   for ( ; obj_ptr != NULL; obj_ptr = obj_ptr->prev)
-   {
-      sel_ptr = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+   for ( ; obj_ptr != NULL; obj_ptr=obj_ptr->prev) {
+      sel_ptr = (struct SelRec *)malloc(sizeof(struct SelRec));
+      if (sel_ptr == NULL) FailAllocMessage();
       sel_ptr->obj = obj_ptr;
       sel_ptr->next = tmpTopSel;
-      if (tmpTopSel == NULL)
+      if (tmpTopSel == NULL) {
          tmpBotSel = sel_ptr;
-      else
+      } else {
          tmpTopSel->prev = sel_ptr;
+      }
       tmpTopSel = sel_ptr;
    }
    tmpTopSel->prev = NULL;
@@ -401,8 +418,8 @@ void UngroupSelObj ()
          else
             sel_ptr->next->prev = tmpBotSel;
 
-         cfree (sel_ptr);
-         cfree (obj_ptr);
+         free(sel_ptr);
+         free(obj_ptr);
       }
    }
    EndCompositeCmd ();
