@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,7 +29,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/mainmenu.c,v 2.15 1995/05/15 01:22:11 william Exp $";
+      "@(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/mainmenu.c,v 3.1 1996/07/08 14:53:17 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -47,6 +47,8 @@ static char RCSid[] =
 #include "file.e"
 #include "font.e"
 #include "grid.e"
+#include "help.e"
+#include "imgproc.e"
 #include "mainloop.e"
 #ifndef _NO_EXTERN
 #include "mainmenu.e"
@@ -58,7 +60,9 @@ static char RCSid[] =
 #include "pattern.e"
 #include "raster.e"
 #include "setup.e"
+#include "shape.e"
 #include "special.e"
+#include "stretch.e"
 #include "text.e"
 
 extern int	atoi ARGS_DECL((char *));
@@ -135,7 +139,7 @@ void InitMainMenu ()
 
    XSelectInput (mainDisplay, mainMenuWindow, StructureNotifyMask |
          ExposureMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask |
-         EnterWindowMask);
+         EnterWindowMask | LeaveWindowMask);
 
    sizehints.flags = PSize | PMinSize | PMaxSize;
    sizehints.width = sizehints.min_width = sizehints.max_width =
@@ -283,6 +287,9 @@ int MainMenuEventHandler (input)
       SetMouseStatus ("Move pinned menu", "Execute Menu Command",
             "Close pinned menu");
    }
+   else if (input->type == LeaveNotify)
+   {
+   }
    else if (input->type == ConfigureNotify)
    {
       mainMenuX = input->xconfigure.x;
@@ -300,30 +307,33 @@ int MainMenuEventHandler (input)
             index = button_ev->y/defaultFontHeight;
             x = button_ev->x_root;
             y = button_ev->y_root;
-            switch (index)
-            {
-               case MENU_MODE: ModeMenu (x, y, FALSE); break;
-               case MENU_FILE: rc = FileMenu (x, y, FALSE); break;
-               case MENU_EDIT: EditMenu (x, y, FALSE); break;
-               case MENU_LAYOUT: LayoutMenu (x, y, FALSE); break;
-               case MENU_MOVEMODE: MoveModeMenu (x, y, FALSE); break;
-               case MENU_ARRANGE: ArrangeMenu (x, y, FALSE); break;
-               case MENU_HORIALIGN: HoriAlignMenu (x, y, FALSE); break;
-               case MENU_VERTALIGN: VertAlignMenu (x, y, FALSE); break;
-               case MENU_FONT: FontMenu (x, y, FALSE); break;
-               case MENU_STYLE: StyleMenu (x, y, FALSE); break;
-               case MENU_SIZE: SizeMenu (x, y, FALSE); break;
-               case MENU_LINEDASH: LineDashMenu (x, y, FALSE); break;
-               case MENU_LINESTYLE: LineStyleMenu (x, y, FALSE); break;
-               case MENU_LINETYPE: LineTypeMenu (x, y, FALSE); break;
-               case MENU_LINEWIDTH: LineWidthMenu (x, y, FALSE); break;
-               case MENU_FILL: FillMenu (x, y, FALSE); break;
-               case MENU_PEN: PenMenu (x, y, FALSE); break;
-               case MENU_COLOR: ColorMenu (x, y, FALSE); break;
-               case MENU_NAVIGATE: NavigateMenu (x, y, FALSE); break;
-               case MENU_SPECIAL: SpecialMenu (x, y, FALSE); break;
-               case MENU_PAGE: PageMenu (x, y, FALSE); break;
-               case MENU_PAGELAYOUT: PageLayoutMenu (x, y, FALSE); break;
+            switch (index) {
+            case MENU_MODE: ModeMenu (x, y, FALSE); break;
+            case MENU_FILE: rc = FileMenu (x, y, FALSE); break;
+            case MENU_EDIT: EditMenu (x, y, FALSE); break;
+            case MENU_LAYOUT: LayoutMenu (x, y, FALSE); break;
+            case MENU_MOVEMODE: MoveModeMenu (x, y, FALSE); break;
+            case MENU_ARRANGE: ArrangeMenu (x, y, FALSE); break;
+            case MENU_HORIALIGN: HoriAlignMenu (x, y, FALSE); break;
+            case MENU_VERTALIGN: VertAlignMenu (x, y, FALSE); break;
+            case MENU_FONT: FontMenu (x, y, FALSE); break;
+            case MENU_STYLE: StyleMenu (x, y, FALSE); break;
+            case MENU_SIZE: SizeMenu (x, y, FALSE); break;
+            case MENU_SHAPE: ShapeMenu (x, y, FALSE); break;
+            case MENU_STRETCHTEXT: StretchableTextModeMenu (x, y, FALSE); break;
+            case MENU_LINEDASH: LineDashMenu (x, y, FALSE); break;
+            case MENU_LINESTYLE: LineStyleMenu (x, y, FALSE); break;
+            case MENU_LINETYPE: LineTypeMenu (x, y, FALSE); break;
+            case MENU_LINEWIDTH: LineWidthMenu (x, y, FALSE); break;
+            case MENU_FILL: FillMenu (x, y, FALSE); break;
+            case MENU_PEN: PenMenu (x, y, FALSE); break;
+            case MENU_COLOR: ColorMenu (x, y, FALSE); break;
+            case MENU_IMAGEPROC: ImageProcMenu (x, y, FALSE); break;
+            case MENU_NAVIGATE: NavigateMenu (x, y, FALSE); break;
+            case MENU_SPECIAL: SpecialMenu (x, y, FALSE); break;
+            case MENU_PAGE: PageMenu (x, y, FALSE); break;
+            case MENU_PAGELAYOUT: PageLayoutMenu (x, y, FALSE); break;
+            case MENU_HELP: HelpMenu (x, y, FALSE); break;
             }
             break;
       }
@@ -357,35 +367,31 @@ void SaveStackingOrder ()
       hash_value = mainMenuWindow & 0xff;
       checkExtra[hash_value>>3] |= (1<<(hash_value&0x7));
    }
-   for (i = 0; i < numExtraWins; i++)
+   for (i = 0; i < numExtraWins; i++) {
       if (extraWinInfo[i].raise && extraWinInfo[i].mapped &&
-            extraWinInfo[i].window != None)
-      {
+            extraWinInfo[i].window != None) {
          hash_value = extraWinInfo[i].window & 0xff;
          checkExtra[hash_value>>3] |= (1<<(hash_value&0x7));
       }
+   }
 
-   XQueryTree (mainDisplay, rootWindow, &root_win, &parent_win, &child_wins,
+   XQueryTree(mainDisplay, rootWindow, &root_win, &parent_win, &child_wins,
          &num_child);
 
    numStacking = 0;
-   if (stackingWins != NULL) cfree (stackingWins);
-   stackingWins = (Window *) calloc (numExtraWins+1, sizeof(Window));
+   if (stackingWins != NULL) free(stackingWins);
+   stackingWins = (Window*)malloc((numExtraWins+1)*sizeof(Window));
+   if (stackingWins == NULL) FailAllocMessage();
 
-   for (i = 0; i < num_child; i++)
-   {
+   for (i = 0; i < num_child; i++) {
       hash_value = child_wins[i] & 0xff;
-      if (checkExtra[hash_value>>3] & (1<<(hash_value&0x7)))
-      {
-         if (child_wins[i] == mainMenuWindow)
+      if (checkExtra[hash_value>>3] & (1<<(hash_value&0x7))) {
+         if (child_wins[i] == mainMenuWindow) {
             stackingWins[numStacking++] = mainMenuWindow;
-         else
-         {
-            for (j = 0; j < numExtraWins; j++)
-            {
+         } else {
+            for (j = 0; j < numExtraWins; j++) {
                if (extraWinInfo[j].raise && extraWinInfo[j].mapped &&
-                     extraWinInfo[j].window == child_wins[i])
-               {
+                     extraWinInfo[j].window == child_wins[i]) {
                   stackingWins[numStacking++] = child_wins[i];
                   break;
                }
@@ -497,9 +503,9 @@ void PaintTextSubMenu (window,win_w,Strings,Entries,ForeColors,Valid,InitRV)
                strlen(Strings[i]));
       }
    }
-   cfree (ForeColors);
-   cfree (Valid);
-   cfree (InitRV);
+   free(ForeColors);
+   free(Valid);
+   free(InitRV);
 }
 
 static
@@ -546,8 +552,8 @@ void PaintPxMpSubMenu (window, W, H, Rows, Cols, Entries, ForeColors, PxMp,
       }
    XSetForeground (mainDisplay, rasterGC, myFgPixel);
 
-   cfree (ForeColors);
-   cfree (InitRV);
+   free(ForeColors);
+   free(InitRV);
 }
 
 static
@@ -564,7 +570,7 @@ void RedrawSubMenu (menu_index)
    {
       case MENU_MODE:
          DefaultColorArrays (MAXCHOICES, &fore_colors, &valid, &init_rv, NULL);
-         cfree (valid);
+         free(valid);
          init_rv[curChoice] = TRUE;
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, MAXCHOICES, 1,
                MAXCHOICES, fore_colors, choicePixmap, init_rv, SINGLECOLOR);
@@ -605,17 +611,18 @@ void RedrawSubMenu (menu_index)
          break;
       case MENU_LAYOUT:
          SetUpLayoutMenuStr (layout_menu_strings, layout_menu_description);
-         DefaultColorArrays (MAXLAYOUTMENUS, &fore_colors, &valid, &init_rv,
-               NULL);
+         DefaultColorArrays (colorDisplay ? MAXLAYOUTMENUS : MAXLAYOUTMENUS-1,
+               &fore_colors, &valid, &init_rv, NULL);
          switch (pageStyle)
          {
             case PORTRAIT: init_rv[LAYOUT_PORT] = TRUE; break;
             case LANDSCAPE: init_rv[LAYOUT_LAND] = TRUE; break;
          }
          PaintTextSubMenu (win, subMenuInfo[menu_index].w, layout_menu_strings,
-               MAXLAYOUTMENUS, fore_colors, valid, init_rv);
-         cfree (layout_menu_strings[0]);
-         cfree (layout_menu_description[0]);
+               colorDisplay ? MAXLAYOUTMENUS : MAXLAYOUTMENUS-1,
+               fore_colors, valid, init_rv);
+         free(layout_menu_strings[0]);
+         free(layout_menu_description[0]);
          break;
       case MENU_ARRANGE:
          DefaultColorArrays (MAXARRANGEMENUS, &fore_colors, &valid, &init_rv,
@@ -626,35 +633,50 @@ void RedrawSubMenu (menu_index)
       case MENU_MOVEMODE:
          DefaultColorArrays (MAXMOVEMODES, &fore_colors, &valid, &init_rv,
                NULL);
-         cfree (valid);
+         free(valid);
          init_rv[moveMode] = TRUE;
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, MAXMOVEMODES, 1,
                MAXMOVEMODES, fore_colors, moveModePixmap, init_rv, SINGLECOLOR);
          break;
       case MENU_HORIALIGN:
          DefaultColorArrays (MAXALIGNS, &fore_colors, &valid, &init_rv, NULL);
-         cfree (valid);
+         free(valid);
          init_rv[horiAlign] = TRUE;
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, MAXALIGNS, 1,
                MAXALIGNS, fore_colors, alignHoriPixmap, init_rv, SINGLECOLOR);
          break;
       case MENU_VERTALIGN:
          DefaultColorArrays (MAXALIGNS, &fore_colors, &valid, &init_rv, NULL);
-         cfree (valid);
+         free(valid);
          init_rv[vertAlign] = TRUE;
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, MAXALIGNS, 1,
                MAXALIGNS, fore_colors, alignVertPixmap, init_rv, SINGLECOLOR);
          break;
       case MENU_FILL:
          DefaultColorArrays (MAXPATTERNS, &fore_colors, &valid, &init_rv, NULL);
-         cfree (valid);
+         free(valid);
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, 8, 4,
                MAXPATTERNS, fore_colors, patPixmap, init_rv, SINGLECOLOR);
          break;
+      case MENU_SHAPE:
+         DefaultColorArrays (numShapes, &fore_colors, &valid, &init_rv, NULL);
+         free(valid);
+         PaintPxMpSubMenu (win, choiceImageW, choiceImageH, 8, 3,
+               numShapes, fore_colors, shapePixmap, init_rv, SINGLECOLOR);
+         break;
+      case MENU_STRETCHTEXT:
+         DefaultColorArrays (MAXSTRETCHABLEMODES, &fore_colors, &valid,
+               &init_rv, NULL);
+         free(valid);
+         init_rv[stretchableText] = TRUE;
+         PaintPxMpSubMenu (win, choiceImageW, choiceImageH,
+               MAXSTRETCHABLEMODES, 1, MAXSTRETCHABLEMODES, fore_colors,
+               stretchableModePixmap, init_rv, SINGLECOLOR);
+         break;
       case MENU_LINEWIDTH:
          DefaultColorArrays (maxLineWidths, &fore_colors, &valid, &init_rv,
                NULL);
-         cfree (valid);
+         free(valid);
          init_rv[lineWidth] = TRUE;
          PaintPxMpSubMenu (win, menuImageW, menuImageH, maxLineWidths, 1,
                maxLineWidths, fore_colors, lineWidthPixmap, init_rv,
@@ -663,7 +685,7 @@ void RedrawSubMenu (menu_index)
       case MENU_LINESTYLE:
          DefaultColorArrays (MAXLINESTYLES, &fore_colors, &valid, &init_rv,
                NULL);
-         cfree (valid);
+         free(valid);
          init_rv[lineStyle] = TRUE;
          PaintPxMpSubMenu (win, menuImageW, menuImageH, MAXLINESTYLES, 1,
                MAXLINESTYLES, fore_colors, lineStylePixmap, init_rv,
@@ -672,7 +694,7 @@ void RedrawSubMenu (menu_index)
       case MENU_LINETYPE:
          DefaultColorArrays (MAXLINETYPES, &fore_colors, &valid, &init_rv,
                NULL);
-         cfree (valid);
+         free(valid);
          init_rv[curSpline] = TRUE;
          PaintPxMpSubMenu (win, menuImageW, menuImageH, MAXLINETYPES, 1,
                MAXLINETYPES, fore_colors, lineTypePixmap, init_rv,
@@ -680,14 +702,14 @@ void RedrawSubMenu (menu_index)
          break;
       case MENU_LINEDASH:
          DefaultColorArrays (MAXDASHES, &fore_colors, &valid, &init_rv, NULL);
-         cfree (valid);
+         free(valid);
          init_rv[curDash] = TRUE;
          PaintPxMpSubMenu (win, menuImageW, menuImageH, MAXDASHES, 1,
                MAXDASHES, fore_colors, dashPixmap, init_rv, SINGLECOLOR);
          break;
       case MENU_PEN:
          DefaultColorArrays (MAXPATTERNS, &fore_colors, &valid, &init_rv, NULL);
-         cfree (valid);
+         free(valid);
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, 8, 4,
                MAXPATTERNS, fore_colors, patPixmap, init_rv, SINGLECOLOR);
          break;
@@ -713,7 +735,7 @@ void RedrawSubMenu (menu_index)
       case MENU_PAGELAYOUT:
          DefaultColorArrays (MAXPAGELAYOUTMODES, &fore_colors, &valid, &init_rv,
                NULL);
-         cfree (valid);
+         free(valid);
          init_rv[pageLayoutMode] = TRUE;
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, MAXPAGELAYOUTMODES,
                1, MAXPAGELAYOUTMODES, fore_colors, pageLayoutPixmap, init_rv,
@@ -723,7 +745,13 @@ void RedrawSubMenu (menu_index)
          SetUpColorMenuPixmap (&fore_colors, &init_rv, &pixmap, &rows, &cols);
          PaintPxMpSubMenu (win, choiceImageW, choiceImageH, rows, cols,
                maxColors, fore_colors, pixmap, init_rv, MULTICOLOR);
-         cfree (pixmap);
+         free(pixmap);
+         break;
+      case MENU_IMAGEPROC:
+         DefaultColorArrays (numImageProc, &fore_colors, &valid, &init_rv,
+               NULL);
+         PaintTextSubMenu (win, subMenuInfo[menu_index].w, imageProcMenuStr,
+               numImageProc, fore_colors, valid, init_rv);
          break;
       case MENU_NAVIGATE:
          DefaultColorArrays (MAXNAVIGATEMENUS, &fore_colors, &valid, &init_rv,
@@ -738,6 +766,11 @@ void RedrawSubMenu (menu_index)
          PaintTextSubMenu (win, subMenuInfo[menu_index].w, specialMenuStr,
                MAXSPECIALMENUS, fore_colors, valid, init_rv);
          break;
+      case MENU_HELP:
+         DefaultColorArrays (numHelp, &fore_colors, &valid, &init_rv, NULL);
+         PaintTextSubMenu (win, subMenuInfo[menu_index].w, helpMenuStr,
+               numHelp, fore_colors, valid, init_rv);
+         break;
    }
 }
     
@@ -764,39 +797,42 @@ static
 void GetRowCol (menu_index, x, y, row, col, unit_w, unit_h, num_rows)
    int	menu_index, x, y, * row, * col, * unit_w, * unit_h, * num_rows;
 {
-   switch (menu_index)
-   {
-      case MENU_FILE:
-      case MENU_EDIT:
-      case MENU_STYLE:
-      case MENU_SIZE:
-      case MENU_FONT:
-      case MENU_LAYOUT:
-      case MENU_ARRANGE:
-      case MENU_NAVIGATE:
-      case MENU_SPECIAL:
-      case MENU_PAGE:
-         *unit_w = subMenuInfo[menu_index].w;
-         *unit_h = defaultFontHeight;
-         break;
-      case MENU_LINEWIDTH:
-      case MENU_LINESTYLE:
-      case MENU_LINETYPE:
-      case MENU_LINEDASH:
-         *unit_w = subMenuInfo[menu_index].w;
-         *unit_h = menuImageH;
-         break;
-      case MENU_MODE:
-      case MENU_MOVEMODE:
-      case MENU_HORIALIGN:
-      case MENU_VERTALIGN:
-      case MENU_COLOR:
-      case MENU_FILL:
-      case MENU_PEN:
-      case MENU_PAGELAYOUT:
-         *unit_w = choiceImageW;
-         *unit_h = choiceImageH;
-         break;
+   switch (menu_index) {
+   case MENU_FILE:
+   case MENU_EDIT:
+   case MENU_STYLE:
+   case MENU_SIZE:
+   case MENU_FONT:
+   case MENU_LAYOUT:
+   case MENU_ARRANGE:
+   case MENU_IMAGEPROC:
+   case MENU_NAVIGATE:
+   case MENU_SPECIAL:
+   case MENU_PAGE:
+   case MENU_HELP:
+      *unit_w = subMenuInfo[menu_index].w;
+      *unit_h = defaultFontHeight;
+      break;
+   case MENU_LINEWIDTH:
+   case MENU_LINESTYLE:
+   case MENU_LINETYPE:
+   case MENU_LINEDASH:
+      *unit_w = subMenuInfo[menu_index].w;
+      *unit_h = menuImageH;
+      break;
+   case MENU_MODE:
+   case MENU_MOVEMODE:
+   case MENU_HORIALIGN:
+   case MENU_VERTALIGN:
+   case MENU_SHAPE:
+   case MENU_STRETCHTEXT:
+   case MENU_COLOR:
+   case MENU_FILL:
+   case MENU_PEN:
+   case MENU_PAGELAYOUT:
+      *unit_w = choiceImageW;
+      *unit_h = choiceImageH;
+      break;
    }
    *row = (y>=0 && y<=subMenuInfo[menu_index].h) ? y/(*unit_h) : INVALID;
    *col = (x>=0 && x<=subMenuInfo[menu_index].w) ? x/(*unit_w) : INVALID;
@@ -831,127 +867,121 @@ void ComputeSubMenuWinXY (win, x, y)
 }
 
 static
-int SubMenuEventHandler (input)
-   XEvent	* input;
+int SubMenuEventHandler(input)
+   XEvent *input;
 {
-   register int	menu_index;
-   int		done;
-   int		down_row, down_col, up_row, up_col, unit_w, unit_h, num_rows;
-   XEvent	ev;
-   XButtonEvent	* button_ev;
-   Window	win;
+   register int menu_index;
+   int done, down_row, down_col, up_row, up_col, unit_w, unit_h, num_rows;
+   XButtonEvent *button_ev;
+   Window win;
 
-   if (input->type==MapNotify || input->type==Expose)
-   {
-      SubMenuExposeHandler (input);
-      return (INVALID);
+   if (input->type==MapNotify || input->type==Expose) {
+      SubMenuExposeHandler(input);
+      return INVALID;
    }
-
    win = input->xany.window;
-   for (menu_index = 0; menu_index < MAXMENUS; menu_index++)
-      if (subMenuInfo[menu_index].win == win && win != None)
+   for (menu_index = 0; menu_index < MAXMENUS; menu_index++) {
+      if (subMenuInfo[menu_index].win == win && win != None) {
          break;
-   if (menu_index == MAXMENUS) return (INVALID);
+      }
+   }
+   if (menu_index == MAXMENUS) return INVALID;
 
-   if (input->type == ConfigureNotify)
-   {
-      ComputeSubMenuWinXY (win, &(subMenuInfo[menu_index].x),
+   if (input->type == ConfigureNotify) {
+      ComputeSubMenuWinXY(win, &(subMenuInfo[menu_index].x),
             &(subMenuInfo[menu_index].y));
-   }
-   else if (input->type == EnterNotify)
-   {
-      SetMouseStatus ("Move pinned menu", "Execute Menu Command",
+   } else if (input->type == EnterNotify) {
+      SetMouseStatus("Move pinned menu", "Execute Menu Command",
             "Close pinned menu");
-   }
-   else if (input->type == ButtonPress)
-   {
+   } else if (input->type == LeaveNotify) {
+   } else if (input->type == ButtonPress) {
       button_ev = &(input->xbutton);
-      switch (button_ev->button)
-      {
-         case Button1:
-            RepositionSubMenuWindow (win, menu_index, button_ev->x_root,
-                  button_ev->y_root);
-            break;
-         case Button2:
-            GetRowCol (menu_index, button_ev->x, button_ev->y, &down_row,
-                  &down_col, &unit_w, &unit_h, &num_rows);
-            XRaiseWindow (mainDisplay, win);
-            XFillRectangle (mainDisplay, win, revDefaultGC, down_col*unit_w,
-                  down_row*unit_h, unit_w, unit_h);
+      switch (button_ev->button) {
+      case Button1:
+         RepositionSubMenuWindow(win, menu_index, button_ev->x_root,
+               button_ev->y_root);
+         break;
+      case Button2:
+         GetRowCol(menu_index, button_ev->x, button_ev->y, &down_row,
+               &down_col, &unit_w, &unit_h, &num_rows);
+         XRaiseWindow(mainDisplay, win);
+         XFillRectangle(mainDisplay, win, revDefaultGC, down_col*unit_w,
+               down_row*unit_h, unit_w, unit_h);
 
-            done = FALSE;
-            while (!done)
-            {
-               XNextEvent (mainDisplay, &ev);
+         done = FALSE;
+         while (!done) {
+            XEvent ev;
 
-               if (ev.type == Expose || ev.type == VisibilityNotify)
-                  ExposeEventHandler (&ev, TRUE);
-               else if (ev.type == ButtonRelease)
-               {
-                  done = TRUE;
-                  XFillRectangle (mainDisplay, win, revDefaultGC,
-                        down_col*unit_w, down_row*unit_h, unit_w, unit_h);
+            XNextEvent(mainDisplay, &ev);
 
-                  GetRowCol (menu_index, ev.xbutton.x, ev.xbutton.y,
-                        &up_row, &up_col, &unit_w, &unit_h, &num_rows);
+            if (ev.type == Expose || ev.type == VisibilityNotify) {
+               ExposeEventHandler(&ev, TRUE);
+            } else if (ev.type == ButtonRelease) {
+               done = TRUE;
+               XFillRectangle(mainDisplay, win, revDefaultGC,
+                     down_col*unit_w, down_row*unit_h, unit_w, unit_h);
 
-                  if (up_row!=INVALID && up_col!=INVALID &&
-                        up_row==down_row && up_col==down_col)
-                  {
-                     switch (menu_index)
-                     {
-                        case MENU_MODE: SetCurChoice (up_row); break;
-                        case MENU_FILE: return (FileSubMenu (up_row));
-                        case MENU_EDIT: EditSubMenu (up_row); break;
-                        case MENU_STYLE: StyleSubMenu (up_row); break;
-                        case MENU_SIZE: ChangeFontSize (up_row); break;
-                        case MENU_FONT: ChangeFont (up_row); break;
-                        case MENU_LAYOUT: LayoutSubMenu (up_row); break;
-                        case MENU_ARRANGE: ArrangeSubMenu (up_row); break;
-                        case MENU_MOVEMODE: MoveModeSubMenu (up_row); break;
-                        case MENU_HORIALIGN: HoriAlignSubMenu (up_row); break;
-                        case MENU_VERTALIGN: VertAlignSubMenu (up_row); break;
-                        case MENU_FILL:
-                           ChangeAllSelFill (up_col*num_rows+up_row, TRUE);
-                           break;
-                        case MENU_LINEWIDTH:
-                           ChangeAllSelLineWidth (up_row, TRUE); break;
-                        case MENU_LINESTYLE:
-                           ChangeAllSelLineStyle (up_row, TRUE); break;
-                        case MENU_LINETYPE:
-                           ChangeAllSelLineType (up_row, TRUE); break;
-                        case MENU_LINEDASH:
-                           ChangeAllSelDashes (up_row, TRUE); break;
-                        case MENU_PEN:
-                           ChangeAllSelPen (up_col*num_rows+up_row, TRUE);
-                           break;
-                        case MENU_COLOR:
-                           ChangeAllSelColor (up_col*num_rows+up_row, TRUE);
-                           break;
-                        case MENU_NAVIGATE: NavigateSubMenu (up_row); break;
-                        case MENU_SPECIAL: SpecialSubMenu (up_row); break;
-                        case MENU_PAGE: PageSubMenu (up_row); break;
-                        case MENU_PAGELAYOUT: PageLayoutSubMenu (up_row); break;
-                     }
+               GetRowCol(menu_index, ev.xbutton.x, ev.xbutton.y,
+                     &up_row, &up_col, &unit_w, &unit_h, &num_rows);
+
+               if (up_row!=INVALID && up_col!=INVALID &&
+                     up_row==down_row && up_col==down_col) {
+                  switch (menu_index) {
+                  case MENU_MODE: SetCurChoice(up_row); break;
+                  case MENU_FILE: return FileSubMenu(up_row);
+                  case MENU_EDIT: EditSubMenu(up_row); break;
+                  case MENU_STYLE: StyleSubMenu(up_row); break;
+                  case MENU_SIZE: ChangeFontSize(up_row); break;
+                  case MENU_FONT: ChangeFont(up_row); break;
+                  case MENU_LAYOUT: LayoutSubMenu(up_row); break;
+                  case MENU_ARRANGE: ArrangeSubMenu(up_row); break;
+                  case MENU_MOVEMODE: MoveModeSubMenu(up_row); break;
+                  case MENU_HORIALIGN: HoriAlignSubMenu(up_row); break;
+                  case MENU_VERTALIGN: VertAlignSubMenu(up_row); break;
+                  case MENU_SHAPE: ShapeSubMenu(up_col*num_rows+up_row); break;
+                  case MENU_STRETCHTEXT:
+                     StretchableTextModeSubMenu(up_row);
+                     break;
+                  case MENU_FILL:
+                     ChangeAllSelFill(up_col*num_rows+up_row, TRUE);
+                     break;
+                  case MENU_LINEWIDTH:
+                     ChangeAllSelLineWidth(up_row, TRUE); break;
+                  case MENU_LINESTYLE:
+                     ChangeAllSelLineStyle(up_row, TRUE); break;
+                  case MENU_LINETYPE:
+                     ChangeAllSelLineType(up_row, TRUE); break;
+                  case MENU_LINEDASH:
+                     ChangeAllSelDashes(up_row, TRUE); break;
+                  case MENU_PEN:
+                     ChangeAllSelPen(up_col*num_rows+up_row, TRUE);
+                     break;
+                  case MENU_COLOR:
+                     ChangeAllSelColor(up_col*num_rows+up_row, TRUE);
+                     break;
+                  case MENU_IMAGEPROC: ImageProcSubMenu(up_row); break;
+                  case MENU_NAVIGATE: NavigateSubMenu(up_row); break;
+                  case MENU_SPECIAL: SpecialSubMenu(up_row); break;
+                  case MENU_PAGE: PageSubMenu(up_row); break;
+                  case MENU_PAGELAYOUT: PageLayoutSubMenu(up_row); break;
+                  case MENU_HELP: HelpSubMenu(up_row); break;
                   }
                }
             }
-            break;
+         }
+         break;
       }
-   }
-   else if (input->type == ButtonRelease)
-   {
+   } else if (input->type == ButtonRelease) {
       button_ev = &(input->xbutton);
       if (button_ev->button == Button3 &&
             button_ev->x>=0 && button_ev->x<=subMenuInfo[menu_index].w &&
-            button_ev->y>=0 && button_ev->y<=subMenuInfo[menu_index].h)
-      {
-         XDestroyWindow (mainDisplay, subMenuInfo[menu_index].win);
+            button_ev->y>=0 && button_ev->y<=subMenuInfo[menu_index].h) {
+         XDestroyWindow(mainDisplay, subMenuInfo[menu_index].win);
          subMenuInfo[menu_index].win = None;
          extraWinInfo[subMenuInfo[menu_index].extra_index].window = None;
       }
    }
-   return (INVALID);
+   return INVALID;
 }
 
 static
@@ -1000,11 +1030,11 @@ void RealizeSubMenuWindow (win_x, win_y, win_w, win_h)
    XMapWindow (mainDisplay, win);
    XSelectInput (mainDisplay, win, StructureNotifyMask |
          ExposureMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask |
-         EnterWindowMask);
+         EnterWindowMask | LeaveWindowMask);
 #else
    XSelectInput (mainDisplay, win, StructureNotifyMask |
          ExposureMask | ButtonPressMask | ButtonReleaseMask | KeyPressMask |
-         EnterWindowMask);
+         EnterWindowMask | LeaveWindowMask);
    XMapWindow (mainDisplay, win);
 #endif
 
@@ -1025,35 +1055,56 @@ void RealizeSubMenuWindow (win_x, win_y, win_w, win_h)
    subMenuInfo[activeMenu].win = win;
 }
 
-void UpdateSubMenu (menu_index)
-   int	menu_index;
+int HandleSubMenuEvent(win, menu_index, input)
+   Window win;
+   int menu_index;
+   XEvent *input;
 {
-   if (subMenuInfo[menu_index].win != None) RedrawSubMenu (menu_index);
+   if (subMenuInfo[menu_index].win != None &&
+         subMenuInfo[menu_index].win == win) {
+      return (*(extraWinInfo[subMenuInfo[menu_index].extra_index].ev_handler))(
+            input);
+   }
+   return INVALID;
 }
 
-void UpdateAllSubMenus ()
+int WindowIsSubMenu(win, menu_index)
+   Window win;
+   int menu_index;
 {
-   UpdateSubMenu (MENU_HORIALIGN);
-   UpdateSubMenu (MENU_VERTALIGN);
-   UpdateSubMenu (MENU_STYLE);
-   UpdateSubMenu (MENU_FONT);
-   UpdateSubMenu (MENU_SIZE);
-   UpdateSubMenu (MENU_LINEWIDTH);
-   UpdateSubMenu (MENU_LINESTYLE);
-   UpdateSubMenu (MENU_LINETYPE);
-   UpdateSubMenu (MENU_LINEDASH);
-   UpdateSubMenu (MENU_LAYOUT);
-   UpdateSubMenu (MENU_MOVEMODE);
-   UpdateSubMenu (MENU_PAGE);
-   UpdateSubMenu (MENU_PAGELAYOUT);
+   return (subMenuInfo[menu_index].win != None &&
+         subMenuInfo[menu_index].win == win);
 }
 
-void DestroySubMenu (menu_index)
-   int	menu_index;
+void UpdateSubMenu(menu_index)
+   int menu_index;
 {
-   if (subMenuInfo[menu_index].win != None)
-   {
-      XDestroyWindow (mainDisplay, subMenuInfo[menu_index].win);
+   if (subMenuInfo[menu_index].win != None) RedrawSubMenu(menu_index);
+}
+
+void UpdateAllSubMenus()
+{
+   UpdateSubMenu(MENU_HORIALIGN);
+   UpdateSubMenu(MENU_VERTALIGN);
+   UpdateSubMenu(MENU_STYLE);
+   UpdateSubMenu(MENU_FONT);
+   UpdateSubMenu(MENU_SIZE);
+   UpdateSubMenu(MENU_LINEWIDTH);
+   UpdateSubMenu(MENU_LINESTYLE);
+   UpdateSubMenu(MENU_LINETYPE);
+   UpdateSubMenu(MENU_LINEDASH);
+   UpdateSubMenu(MENU_LAYOUT);
+   UpdateSubMenu(MENU_MOVEMODE);
+   UpdateSubMenu(MENU_STRETCHTEXT);
+   UpdateSubMenu(MENU_PAGE);
+   UpdateSubMenu(MENU_PAGELAYOUT);
+}
+
+void DestroySubMenu(menu_index)
+   int menu_index;
+{
+   if (subMenuInfo[menu_index].win != None) {
+      XDestroyWindow(mainDisplay, subMenuInfo[menu_index].win);
       subMenuInfo[menu_index].win = None;
       extraWinInfo[subMenuInfo[menu_index].extra_index].window = None;
    }
