@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,7 +29,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/msg.c,v 2.65 1995/05/30 04:38:50 william Exp $";
+      "@(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/msg.c,v 3.6 1996/12/26 06:19:28 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -38,27 +38,38 @@ static char RCSid[] =
 #include <X11/Xlib.h>
 #include "const.h"
 #include "types.h"
+#include "patchlvl.h"
 
 #include "button.e"
-#include "copypaste.e"
+#include "cutpaste.e"
 #include "cursor.e"
 #include "dialog.e"
 #include "file.e"
 #include "font.e"
+#include "grid.e"
 #include "mainloop.e"
 #include "menu.e"
 #ifndef _NO_EXTERN
 #include "msg.e"
 #endif
+#include "navigate.e"
+#include "pattern.e"
+#include "ps.e"
 #include "raster.e"
 #include "setup.e"
 #include "util.e"
+#include "version.e"
+#include "xbitmap.e"
+
+extern char *getenv ARGS_DECL((char *));
 
 #define MSG_ROWS 2
 
-char	scanFileName[MAXPATHLENGTH];
+char	scanFileName[MAXPATHLENGTH+1];
 int	scanLineNum = 0;
 
+char	progName[MAXPATHLENGTH+1];
+
 struct MsgRec {
    char			* s;
    struct MsgRec	* next, * prev;
@@ -72,11 +83,11 @@ static int		firstCharIndex = 0;
 
 static int		reverseMouseStatusButtons=FALSE;
 
-int FailAllocMessage ()
+int FailAllocMessage()
 {
-   fprintf (stderr, "Out of virtual memory and can not calloc().\n");
-   fflush (stderr);
-   return (FALSE);
+   fprintf(stderr, "Out of virtual memory and can not malloc().\n");
+   fflush(stderr);
+   return FALSE;
 }
 
 static
@@ -89,8 +100,10 @@ void AddMsg (Msg)
    firstCharIndex = 0;
    if (*Msg == '\0') { topMsgNumber = msgCount; return; }
 
-   s = (char *) calloc (strlen (Msg) + 1, sizeof(char));
-   msg_ptr = (struct MsgRec *) calloc (1, sizeof(struct MsgRec));
+   s = (char*)malloc((strlen(Msg)+1)*sizeof(char));
+   if (s == NULL) FailAllocMessage();
+   msg_ptr = (struct MsgRec *)malloc(sizeof(struct MsgRec));
+   if (msg_ptr == NULL) FailAllocMessage();
 
    strcpy (s, Msg);
    msg_ptr->s = s;
@@ -109,15 +122,14 @@ void AddMsg (Msg)
    botMsg = msg_ptr;
 }
 
-void CleanUpMsg ()
+void CleanUpMsg()
 {
-   register struct MsgRec	* msg_ptr, * prev_msg;
+   register struct MsgRec *msg_ptr, *prev_msg;
 
-   for (msg_ptr = botMsg; msg_ptr != NULL; msg_ptr = prev_msg)
-   {
+   for (msg_ptr=botMsg; msg_ptr != NULL; msg_ptr=prev_msg) {
       prev_msg = msg_ptr->prev;
-      cfree (msg_ptr->s);
-      cfree (msg_ptr);
+      free(msg_ptr->s);
+      free(msg_ptr);
    }
    topMsg = botMsg = mostRecentTopMsgPtr = NULL;
    msgCount = topMsgNumber = firstCharIndex = 0;
@@ -275,32 +287,53 @@ void RedrawStatusStrings ()
 
    if (PRTGIF || noStatusWindow) return;
 
-   if (oneLineStatus)
-   {
-      int	y;
+   if (oneLineStatus) {
+      int y;
 
       XClearWindow (mainDisplay, statusWindow);
-      if (defaultFontHeight+(brdrW<<1)+2 > btn1_height)
-         y = 2 + brdrW + defaultFontAsc;
-      else
-         y = ((statusWindowH-defaultFontHeight)>>1)+defaultFontAsc;
-      XDrawString (mainDisplay, statusWindow, defaultGC, 2, y,
-            oneLineStatusStr, strlen (oneLineStatusStr));
-   }
-   else
-   {
-      int	left=0, w=(int)(statusWindowW/3), right=0;
+      if (msgFontPtr == NULL) {
+         if (defaultFontHeight+(brdrW<<1)+2 > btn1_height) {
+            y = 2 + brdrW + defaultFontAsc;
+         } else {
+            y = ((statusWindowH-defaultFontHeight)>>1)+defaultFontAsc;
+         }
+         XDrawString(mainDisplay, statusWindow, defaultGC, 2, y,
+               oneLineStatusStr, strlen (oneLineStatusStr));
+      } else {
+         if (msgFontHeight+(brdrW<<1)+2 > btn1_height) {
+            y = 2 + brdrW + msgFontAsc;
+         } else {
+            y = ((statusWindowH-msgFontHeight)>>1)+msgFontAsc;
+         }
+         XSetFont(mainDisplay, defaultGC, msgFontPtr->fid);
+         XDrawString(mainDisplay, statusWindow, defaultGC, 2, y,
+               oneLineStatusStr, strlen(oneLineStatusStr));
+         XSetFont(mainDisplay, defaultGC, defaultFontPtr->fid);
+      }
+   } else {
+      int left=0, w=(int)(statusWindowW/3), right=0;
 
-      for (i=0; i < MAX_STATUS_BTNS; i++)
-      {
+      if (msgFontPtr != NULL) {
+         XSetFont(mainDisplay, defaultGC, msgFontPtr->fid);
+      }
+      for (i=0; i < MAX_STATUS_BTNS; i++) {
          right += w;
          if (right >= statusWindowW) right = statusWindowW-1;
          XClearWindow (mainDisplay, statusSubWindow[i]);
-         XDrawString (mainDisplay, statusSubWindow[i], defaultGC,
-               2, 1+defaultFontAsc, btnStatusStr[i], strlen (btnStatusStr[i]));
+         if (msgFontPtr == NULL) {
+            XDrawString (mainDisplay, statusSubWindow[i], defaultGC, 2,
+                  1+defaultFontAsc, btnStatusStr[i], strlen(btnStatusStr[i]));
+         } else {
+            XDrawString (mainDisplay, statusSubWindow[i], defaultGC, 2,
+                  1+msgFontAsc, btnStatusStr[i], strlen(btnStatusStr[i]));
+         }
          left += w;
       }
+      if (msgFontPtr != NULL) {
+         XSetFont(mainDisplay, defaultGC, defaultFontPtr->fid);
+      }
    }
+   if (btn1_bits) { }
 }
 
 void RedrawStatusWindow ()
@@ -310,14 +343,22 @@ void RedrawStatusWindow ()
 
    if (PRTGIF || noStatusWindow) return;
 
-   if (!oneLineStatus)
-   {
-      int	left=0, w=(int)(statusWindowW/3), y, right=0;
+   if (!oneLineStatus) {
+      int left=0, w=(int)(statusWindowW/3), y, right=0;
 
-      if (defaultFontHeight+(brdrW<<1)+2 > btn1_height)
-         y = (statusWindowH-btn1_height)>>1;
-      else
-         y = 1;
+      if (msgFontPtr == NULL) {
+         if (defaultFontHeight+(brdrW<<1)+2 > btn1_height) {
+            y = (statusWindowH-btn1_height)>>1;
+         } else {
+            y = 1;
+         }
+      } else {
+         if (msgFontHeight+(brdrW<<1)+2 > btn1_height) {
+            y = (statusWindowH-btn1_height)>>1;
+         } else {
+            y = 1;
+         }
+      }
       for (i=0; i < MAX_STATUS_BTNS; i++)
       {
          int	x=left+(brdrW<<2);
@@ -388,7 +429,7 @@ void SetMouseStatus (Left, Middle, Right)
       }
       if (Middle != NULL) {
          if (Middle != btnStatusStr[1] &&
-               strcmp(Left, btnStatusStr[1]) != 0) {
+               strcmp(Middle, btnStatusStr[1]) != 0) {
             UtilStrCpy (btnStatusStr[1], MAXSTRING+1, Middle);
          }
       } else {
@@ -396,7 +437,7 @@ void SetMouseStatus (Left, Middle, Right)
       }
       if (Right != NULL) {
          if (Right != btnStatusStr[right_index] &&
-               strcmp(Left, btnStatusStr[right_index]) != 0) {
+               strcmp(Right, btnStatusStr[right_index]) != 0) {
             UtilStrCpy (btnStatusStr[right_index], MAXSTRING+1, Right);
          }
       } else {
@@ -437,38 +478,56 @@ void SetStringStatus (StatusStr)
       RedrawStatusStrings ();
 }
 
-static char	savedBtnStatusStr[MAX_STATUS_BTNS][MAXSTRING+1];
-static char	savedOneLineStatusStr[MAXSTRING+1];
-static int	savedOneLineStatus=FALSE;
+typedef struct StatusInfoRec {
+   char btn_str[MAX_STATUS_BTNS][MAXSTRING+1];
+   char one_line_str[MAXSTRING+1];
+   int one_line_status;
+   struct StatusInfoRec *next;
+} *StatusInfoPtr;
+
+static struct StatusInfoRec *topStatusInfo=NULL;
 
 void SaveStatusStrings ()
 {
-   register int	i;
+   struct StatusInfoRec *psi;
+   int i;
 
    if (PRTGIF || noStatusWindow) return;
-   for (i=0; i < MAX_STATUS_BTNS; i++)
-      UtilStrCpy (savedBtnStatusStr[i], MAXSTRING+1, btnStatusStr[i]);
-   UtilStrCpy (savedOneLineStatusStr, MAXSTRING+1, oneLineStatusStr);
-   savedOneLineStatus = oneLineStatus;
+
+   psi = (struct StatusInfoRec *)malloc(sizeof(struct StatusInfoRec));
+   if (psi == NULL) FailAllocMessage();
+   psi->next = topStatusInfo;
+   for (i=0; i < MAX_STATUS_BTNS; i++) {
+      UtilStrCpy(psi->btn_str[i], MAXSTRING+1, btnStatusStr[i]);
+   }
+   UtilStrCpy(psi->one_line_str, MAXSTRING+1, oneLineStatusStr);
+   psi->one_line_status = oneLineStatus;
+   topStatusInfo = psi;
 }
 
 void RestoreStatusStrings ()
 {
-   register int	i;
+   struct StatusInfoRec *psi;
+   int i;
 
-   if (PRTGIF || noStatusWindow) return;
-   for (i=0; i < MAX_STATUS_BTNS; i++)
-      UtilStrCpy (btnStatusStr[i], MAXSTRING+1, savedBtnStatusStr[i]);
-   UtilStrCpy (oneLineStatusStr, MAXSTRING+1, savedOneLineStatusStr);
-   if (savedOneLineStatus != oneLineStatus)
-   {
-      if (savedOneLineStatus)
-         SetStringStatus (oneLineStatusStr);
-      else
-         SetMouseStatus (btnStatusStr[0], btnStatusStr[1], btnStatusStr[2]);
+   if (PRTGIF || noStatusWindow || topStatusInfo == NULL) return;
+
+   for (i=0; i < MAX_STATUS_BTNS; i++) {
+      UtilStrCpy(btnStatusStr[i], MAXSTRING+1, topStatusInfo->btn_str[i]);
    }
-   else
-      RedrawStatusWindow ();
+   UtilStrCpy(oneLineStatusStr, MAXSTRING+1, topStatusInfo->one_line_str);
+   if (topStatusInfo->one_line_status != oneLineStatus) {
+      if (topStatusInfo->one_line_status) {
+         SetStringStatus(oneLineStatusStr);
+      } else {
+         SetMouseStatus(btnStatusStr[0], btnStatusStr[1], btnStatusStr[2]);
+      }
+   } else {
+      RedrawStatusWindow();
+   }
+   psi = topStatusInfo->next;
+   free(topStatusInfo);
+   topStatusInfo = psi;
 }
 
 void SaveStatusStringsIntoBuf(ppsz_buf, pn_one_line)
@@ -629,6 +688,12 @@ void InitStatus ()
 
 void CleanUpStatus()
 {
+   while (topStatusInfo != NULL) {
+      struct StatusInfoRec *next_si=topStatusInfo->next;
+
+      free(topStatusInfo);
+      topStatusInfo = next_si;
+   }
 }
 
 static char	* scanVal = NULL;
@@ -848,8 +913,10 @@ void Usage (tool_name)
    fprintf (stderr, "\t[-bd <color>] \\\n");
    fprintf (stderr, "\t[-rv] [-nv] \\\n");
    fprintf (stderr, "\t[-bw] \\\n");
+   fprintf (stderr, "\t[-reqcolor] \\\n");
    fprintf (stderr, "\t[-cwo] \\\n");
    fprintf (stderr, "\t[-hyper] \\\n");
+   fprintf (stderr, "\t[-a4] \\\n");
    fprintf (stderr, "\t[-geometry <geom>] [=<geom>] \\\n");
    fprintf (stderr, "\t[<filename>[.obj]]\n");
    fprintf (stderr, "or:\n");
@@ -857,15 +924,274 @@ void Usage (tool_name)
    fprintf (stderr, "\t[-eps] [-p] \\\n");
    fprintf (stderr, "\t[-ps] [-f] \\\n");
    fprintf (stderr, "\t[-text] \\\n");
+   fprintf (stderr, "\t[-raw[+h]] \\\n");
    fprintf (stderr, "\t[-gray] \\\n");
+   fprintf (stderr, "\t[-color | -reqcolor] \\\n");
    fprintf (stderr, "\t[-adobe | -adobe=<number>/<number] \\\n");
    fprintf (stderr, "\t[-page <number>] \\\n");
    fprintf (stderr, "\t[-print_cmd \"<command>\"] \\\n");
    fprintf (stderr, "\t[-one_file_per_page] \\\n");
    fprintf (stderr, "\t[-pepsc] \\\n");
+   fprintf (stderr, "\t[-a4] \\\n");
    fprintf (stderr, "\t[-bop_hook \"<string>\"] \\\n");
    fprintf (stderr, "\t[-eop_hook \"<string>\"] \\\n");
    fprintf (stderr, "\t[-odir] \\\n");
    fprintf (stderr, "\t[<file1>[.obj] <file2>[.obj] ...]\n");
    exit (-1);
 }
+
+int ProcessPrTgifOptions(argc, argv, from_prtgif)
+   int argc, from_prtgif;
+   char *argv[];
+{
+   char	*c_ptr;
+   int saved_argc=argc;
+   char **saved_argv=argv;
+
+   cmdLineOneFilePerPage = FALSE;
+   for (argc--, argv++; argc > 0; argc--, argv++) {
+      if (**argv == '-') {
+         if (strcmp(*argv, "-raw") == 0) {
+            cmdLineDumpURL = TRUE;
+            cmdLineDumpURLWithHeader = FALSE;
+         } else if (strcmp(*argv, "-raw+h") == 0) {
+            cmdLineDumpURL = TRUE;
+            cmdLineDumpURLWithHeader = TRUE;
+         }
+      }
+   }
+   argc = saved_argc;
+   argv = saved_argv;
+   if (!cmdLineDumpURL) {
+      if (TGIF_PATCHLEVEL == 0) {
+         fprintf(stderr, "%s Version %s\n", TOOL_NAME, versionString);
+      } else {
+         fprintf(stderr, "%s Version %s (patchlevel %1d)\n", TOOL_NAME,
+               versionString, TGIF_PATCHLEVEL);
+      }
+      fprintf(stderr, "%s\n\n", copyrightString);
+      fflush(stderr);
+   }
+   if (!from_prtgif) {
+      if ((c_ptr=getenv("TGIFPATH")) == NULL)
+         strcpy(drawPath, TGIF_PATH);
+      else
+         if (((int)strlen(c_ptr)) >= MAXSTRING) {
+            /* must be an error */
+            strcpy(drawPath, TGIF_PATH);
+         } else {
+            strcpy(drawPath, c_ptr);
+         }
+#ifdef PRINT_CMD
+      strcpy(printCommand, PRINT_CMD);
+#else
+#ifdef VMS
+      strcpy(printCommand, "print");
+#else
+#ifdef SYSV
+      strcpy(printCommand, "lp -dpostscript");
+#else
+      strcpy(printCommand, "lpr");
+#endif /* SYSV */
+#endif /* VMS */
+#endif /* PRINT_CMD */
+   }
+
+   whereToPrint = PRINTER;
+   usePsAdobeString = FALSE;
+   *adobeString = '\0';
+   *epsfString = '\0';
+
+   c_ptr = printCommand;
+   c_ptr += strlen(printCommand);
+   for (argc--, argv++; argc > 0; argc--, argv++) {
+      if (**argv == '-') {
+         int len;
+
+         if (strcmp(*argv, "-p") == 0 || strcmp(*argv, "-eps") == 0) {
+            whereToPrint = LATEX_FIG;
+         } else if (strcmp(*argv,"-f") == 0 || strcmp(*argv,"-ps") == 0) {
+            whereToPrint = PS_FILE;
+         } else if (strcmp(*argv, "-text") == 0) {
+            whereToPrint = TEXT_FILE;
+         } else if (strcmp(*argv, "-raw") == 0) {
+            cmdLineDumpURL = TRUE;
+            cmdLineDumpURLWithHeader = FALSE;
+         } else if (strcmp(*argv, "-raw+h") == 0) {
+            cmdLineDumpURL = TRUE;
+            cmdLineDumpURLWithHeader = TRUE;
+         } else if (strcmp(*argv, "-page") == 0) {
+            argc--; argv++;
+            if (argc <= 0) {
+               fprintf(stderr, "Missing page number.\n");
+               Usage(progName);
+            }
+            cmdLineHasPageNum = TRUE;
+            strcpy(cmdLinePageNumStr, *argv);
+            if (sscanf(cmdLinePageNumStr, "%d", &cmdLinePageNum) != 1 ||
+                  cmdLinePageNum <= 0) {
+               cmdLineHasPageNum = FALSE;
+               fprintf(stderr, "Invalid page number.\n");
+               Usage(progName);
+            }
+         } else if (strcmp(*argv, "-one_file_per_page") == 0) {
+            cmdLineOneFilePerPage = TRUE;
+         } else if (strcmp(*argv, "-pepsc") == 0) {
+            stripEPSComments = FALSE;
+         } else if (strcmp(*argv, "-a4") == 0) {
+            cmdLineA4 = TRUE;
+         } else if (strcmp(*argv, "-bop_hook") == 0) {
+            argc--; argv++;
+            if (argc <= 0) {
+               fprintf(stderr, "Missing argument for -bop_hook.\n");
+               Usage(progName);
+            }
+            SetBopHook(*argv);
+         } else if (strcmp(*argv, "-eop_hook") == 0) {
+            argc--; argv++;
+            if (argc <= 0) {
+               fprintf(stderr, "Missing argument for -bop_hooe.\n");
+               Usage(progName);
+            }
+            SetEopHook(*argv);
+         } else if (strcmp(*argv, "-gray") == 0) {
+            useGray = TRUE;
+         } else if (strcmp(*argv, "-bw") == 0) {
+         } else if (strcmp(*argv, "-color") == 0) {
+            cmdLineRequestedColor = FALSE;
+            cmdLineColor = TRUE;
+         } else if (strcmp(*argv, "-reqcolor") == 0) {
+            cmdLineRequestedColor = TRUE;
+            cmdLineColor = TRUE;
+         } else if (strcmp(*argv, "-adobe") == 0) {
+            usePsAdobeString = TRUE;
+         } else if (strncmp(*argv, "-adobe=", 7) == 0) {
+            char tmp_str[80];
+
+            strcpy(tmp_str, &(*argv)[7]);
+            if (!ParsePsAdobeString(tmp_str, &usePsAdobeString, adobeString,
+                  epsfString))
+            {
+               fprintf(stderr, "%s %s*UsePsAdobeString:  '%s', %s.\n",
+                     "Invalid", TOOL_NAME, tmp_str, "'false' is used");
+               usePsAdobeString = FALSE;
+            }
+         } else if (strcmp(*argv, "-tgif_path") == 0) {
+            argc--; argv++;
+            if (argc <= 0) {
+               fprintf(stderr, "Missing tgif_path.\n");
+               Usage(progName);
+            }
+            len = strlen(*argv);
+            if (**argv == '"' && (*argv)[len-1] == '"') {
+               strcpy(drawPath, &(*argv)[1]);
+               drawPath[strlen(drawPath)-1] = '\0';
+            } else {
+               strcpy(drawPath, *argv);
+            }
+            if (!from_prtgif) {
+               fprintf(stderr, "Warning:  -tgif_path should not be used!\n");
+            }
+         } else if (strcmp(*argv, "-print_cmd") == 0) {
+            argc--; argv++;
+            if (argc <= 0) {
+               fprintf(stderr, "Missing print_cmd.\n");
+               Usage(progName);
+            }
+            len = strlen(*argv);
+            if (**argv == '"' && (*argv)[len-1] == '"') {
+               strcpy(printCommand, &(*argv)[1]);
+               printCommand[strlen(printCommand)-1] = '\0';
+            } else {
+               strcpy(printCommand, *argv);
+            }
+         } else if (strncmp(*argv, "-o", 2) == 0) {
+            strcpy (outputDir, &(*argv)[2]);
+         } else if (strcmp(*argv, "-prtgif") != 0 &&
+               strcmp(*argv, "-print") != 0 &&
+               strcmp(*argv, "-PRINT") != 0) {
+            strcat(c_ptr++, " ");
+            strcat(c_ptr, *argv);
+            c_ptr += strlen(*argv);
+         }
+      } else {
+         break;
+      }
+   }
+   if (cmdLineOneFilePerPage) {
+      if (whereToPrint == PRINTER) {
+         fprintf(stderr, "%s %s.\n",
+               "Printing to printer is incompatible with",
+               "-one_file_page_page");
+         Usage(progName);
+      }
+   }
+   return argc;
+}
+
+int ProcessTgifOptions(argc, argv, file_name)
+   int argc;
+   char *argv[], *file_name;
+   /* All these strangeness with strings are related to */
+   /*	Prolog's foreign function interface. */
+{
+   register int i;
+
+   *file_name = '\0';
+   for (i=1; i < argc; i++) {
+      if (strcmp(argv[i], "-display") == 0) {
+         if (++i == argc) {
+            fprintf(stderr, "Missing display name.\n");
+            Usage(progName);
+         }
+         displayName = argv[i];
+      } else if (strcmp(argv[i], "-bw") == 0) {
+         cmdLineBW = TRUE;
+      } else if (strcmp(argv[i], "-reqcolor") == 0) {
+         cmdLineRequestedColor = TRUE;
+      } else if (strcmp(argv[i], "-rv") == 0) {
+         cmdLineRV = TRUE;
+      } else if (strcmp(argv[i], "-nv") == 0) {
+         cmdLineRV = FALSE;
+      } else if (strcmp(argv[i], "-cwo") == 0) {
+         cmdLineCWO = TRUE;
+      } else if (strcmp(argv[i], "-hyper") == 0) {
+         inHyperSpace = TRUE;
+      } else if (strcmp(argv[i], "-a4") == 0) {
+         cmdLineA4 = TRUE;
+      } else if (strcmp(argv[i], "-fg") == 0) {
+         if (++i == argc) {
+            fprintf(stderr, "Missing foreground color.\n");
+            Usage(progName);
+         }
+         cmdLineForeground = (char*)malloc(80*sizeof(char));
+         if (cmdLineForeground == NULL) FailAllocMessage();
+         strcpy(cmdLineForeground, argv[i]);
+      } else if (strcmp(argv[i], "-bg") == 0) {
+         if (++i == argc)
+         { fprintf(stderr, "Missing background color.\n"); Usage(progName); }
+         cmdLineBackground = (char*)malloc(80*sizeof(char));
+         if (cmdLineBackground == NULL) FailAllocMessage();
+         strcpy(cmdLineBackground, argv[i]);
+      } else if (strcmp(argv[i], "-bd") == 0) {
+         if (++i == argc)
+         { fprintf(stderr, "Missing border color.\n"); Usage(progName); }
+         cmdLineBorder = (char*)malloc(80*sizeof(char));
+         if (cmdLineBorder == NULL) FailAllocMessage();
+         strcpy(cmdLineBorder, argv[i]);
+      } else if (strcmp(argv[i], "-geometry") == 0) {
+         if (++i == argc)
+         { fprintf(stderr, "Missing geometry spec.\n"); Usage(progName); }
+         strcat(geometrySpec, argv[i]);
+         geometrySpecified = TRUE;
+      } else if (*argv[i] == '=') {
+         strcpy(geometrySpec, argv[i]);
+         geometrySpecified = TRUE;
+      } else if (*argv[i] != '-') {
+         strcpy(file_name, argv[i]);
+      } else {
+         Usage(progName);
+      }
+   }
+   return TRUE;
+}
