@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1997, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -26,13 +26,12 @@
  * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/oval.c,v 3.8 1997/09/14 20:37:56 william Exp $
  */
-#ifndef lint
-static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/oval.c,v 2.64 1995/07/10 05:48:15 william Exp $";
-#endif
 
 #include <stdio.h>
+#include <string.h>
 #include <math.h>
 #include <sys/types.h>
 #include <X11/Xlib.h>
@@ -58,9 +57,12 @@ static char RCSid[] =
 #include "poly.e"
 #include "ps.e"
 #include "raster.e"
+#include "rect.e"
 #include "ruler.e"
 #include "select.e"
 #include "setup.e"
+#include "spline.e"
+#include "util.e"
 #include "xpixmap.e"
 
 int	ovalDrawn = FALSE;
@@ -117,48 +119,40 @@ void DumpOvalPath (FP, ObjPtr, Xc, Yc, A, B, Width, Pen, Dash)
    register int	i;
    char		s[MAXSTRING];
 
-   if (Width != 1) fprintf (FP, "   %1d setlinewidth\n", Width);
-   if (Dash != 0)
-   {
-      fprintf (FP, "   [");
-      for (i = 0; i < dashListLength[Dash]-1; i++)
-         fprintf (FP, "%1d ", (int)(dashList[Dash][i]));
-      fprintf (FP, "%1d] 0 setdash\n",
+   fprintf(FP, "   gsave\n");
+   if (!colorDump && useGray && Pen > BACKPAT) {
+      GrayCheck(Pen);
+      fprintf(FP, "      %s setgray\n", GrayStr(Pen));
+   }
+   GetPSEllipseStr(Xc, Yc, A, B, s);
+   fprintf(FP, "      %s\n", s);
+
+   if (ObjPtr->ctm != NULL) {
+      fprintf(FP, "      tgiforigctm setmatrix\n");
+   }
+   if (Width != 1) fprintf(FP, "      %1d setlinewidth\n", Width);
+   if (Dash != 0) {
+      fprintf(FP, "      [");
+      for (i = 0; i < dashListLength[Dash]-1; i++) {
+         fprintf(FP, "%1d ", (int)(dashList[Dash][i]));
+      }
+      fprintf(FP, "%1d] 0 setdash\n",
             (int)(dashList[Dash][dashListLength[Dash]-1]));
    }
-
-   if (!colorDump && Pen > BACKPAT)
-   {
-      GrayCheck (Pen);
-      if (useGray)
-         fprintf (FP, "   %s setgray\n", GrayStr(Pen));
-      else
-      {
-         if (preDumpSetup) PSUseBWPattern();
-         fprintf (FP, "   pat%1d %s\n", Pen, patternStr);
+   switch (Pen) {
+   case SOLIDPAT: fprintf(FP, "      stroke\n"); break;
+   case BACKPAT: fprintf(FP, "      1 setgray stroke 0 setgray\n"); break;
+   default:
+      if (colorDump || !useGray) {
+         if (preDumpSetup) PSUseColorPattern();
+         fprintf(FP, "      flattenpath strokepath clip newpath\n");
+         DumpPatFill(FP, Pen, 8, ObjPtr->bbox, "      ");
+      } else {
+         fprintf(FP, "      stroke\n");
       }
+      break;
    }
-
-   GetPSEllipseStr (Xc, Yc, A, B, s);
-   fprintf (FP, "   %s ", s);
-
-   switch (Pen)
-   {
-      case SOLIDPAT: fprintf (FP, "stroke\n"); break;
-      case BACKPAT: fprintf (FP, "1 setgray stroke 0 setgray\n"); break;
-      default:
-         if (colorDump)
-         {
-            if (preDumpSetup) PSUseColorPattern();
-            fprintf (FP, "flattenpath strokepath clip newpath\n");
-            DumpPatFill (FP, Pen, 8, ObjPtr->bbox, "   ");
-         }
-         else
-            fprintf (FP, "stroke\n");
-         break;
-   }
-   if (Dash != 0) fprintf (FP, "   [] 0 setdash\n");
-   if (Width != 1) fprintf (FP, "   1 setlinewidth\n");
+   fprintf(FP, "   grestore\n");
 }
  
 void DumpOvalObj (FP, ObjPtr)
@@ -168,25 +162,42 @@ void DumpOvalObj (FP, ObjPtr)
    int	ltx, lty, rbx, rby, xc, yc, a, b;
    int	fill, width, pen, dash, color_index;
    char	s[MAXSTRING];
+   struct OvalRec *oval_ptr=ObjPtr->detail.o;
 
-   ltx = ObjPtr->obbox.ltx; lty = ObjPtr->obbox.lty;
-   rbx = ObjPtr->obbox.rbx; rby = ObjPtr->obbox.rby;
+   if (ObjPtr->ctm == NULL) {
+      ltx = ObjPtr->obbox.ltx; lty = ObjPtr->obbox.lty;
+      rbx = ObjPtr->obbox.rbx; rby = ObjPtr->obbox.rby;
+   } else {
+      ltx = ObjPtr->orig_obbox.ltx; lty = ObjPtr->orig_obbox.lty;
+      rbx = ObjPtr->orig_obbox.rbx; rby = ObjPtr->orig_obbox.rby;
+   }
    a = (rbx - ltx) / 2; xc = ltx + a;
    b = (rby - lty) / 2; yc = lty + b;
 
-   fill = ObjPtr->detail.o->fill;
-   width = ObjPtr->detail.o->width;
-   pen = ObjPtr->detail.o->pen;
-   dash = ObjPtr->detail.o->dash;
+   fill = oval_ptr->fill;
+   width = oval_ptr->width;
+   pen = oval_ptr->pen;
+   dash = oval_ptr->dash;
 
    if (fill == NONEPAT && pen == NONEPAT) return;
 
    fprintf (FP, "%% OVAL\n");
+   if (ObjPtr->ctm != NULL) {
+      float m[6];
+
+      fprintf(FP, "gsave\n");
+      m[CTM_SX] = ((float)ObjPtr->ctm->m[CTM_SX])/((float)1000.0);
+      m[CTM_SY] = ((float)ObjPtr->ctm->m[CTM_SY])/((float)1000.0);
+      m[CTM_SIN] = ((float)ObjPtr->ctm->m[CTM_SIN])/((float)1000.0);
+      m[CTM_MSIN] = ((float)ObjPtr->ctm->m[CTM_MSIN])/((float)1000.0);
+      fprintf (FP, "   %1d %1d translate\n", ObjPtr->x, ObjPtr->y);
+      fprintf (FP, "   [%.3f %.3f %.3f %.3f %1d %1d] concat\n",
+            m[CTM_SX], m[CTM_SIN], m[CTM_MSIN], m[CTM_SY],
+            ObjPtr->ctm->m[CTM_TX], ObjPtr->ctm->m[CTM_TY]);
+      fprintf (FP, "   %1d neg %1d neg translate\n", ObjPtr->x, ObjPtr->y);
+   }
    color_index = ObjPtr->color;
-   if (colorDump)
-      DumpRGBColorLine(FP, color_index, 0, TRUE);
-   else
-      fprintf (FP, "0 setgray\n");
+   DumpRGBColorLine(FP, color_index, 0, TRUE);
 
    GetPSEllipseStr (xc, yc, a, b, s);
    switch (fill)
@@ -200,15 +211,12 @@ void DumpOvalObj (FP, ObjPtr)
          /* solid white oval */
          fprintf (FP, "%s\n", s);
          fprintf (FP, "closepath 1 setgray fill\n");
-         if (colorDump)
-            DumpRGBColorLine(FP, color_index, 3, TRUE);
-         else
-            fprintf (FP, "0 setgray\n");
+         DumpRGBColorLine(FP, color_index, 3, TRUE);
          break;
       default:
          /* patterned */
          fprintf (FP, "gsave\n");
-         if (colorDump)
+         if (colorDump || !useGray)
          {
             if (preDumpSetup) PSUseColorPattern();
             fprintf (FP, "   %s\n", s);
@@ -221,35 +229,123 @@ void DumpOvalObj (FP, ObjPtr)
          else
          {
             GrayCheck (fill);
-            if (useGray)
-               fprintf (FP, "   %s setgray\n", GrayStr(fill));
-            else
-            {
-               if (preDumpSetup) PSUseBWPattern();
-               fprintf (FP, "   pat%1d %s\n",fill, patternStr);
-            }
+            fprintf (FP, "   %s setgray\n", GrayStr(fill));
             fprintf (FP, "   %s fill\n", s);
          }
          fprintf (FP, "grestore\n");
          break;
    }
-
-   if (pen == NONEPAT) { fprintf (FP, "\n"); return; }
+   if (pen == NONEPAT) {
+      if (ObjPtr->ctm != NULL) fprintf(FP, "grestore\n");
+      fprintf (FP, "\n");
+      return;
+   }
 
    fprintf (FP, "gsave\n");
 
-   if (colorDump && pen > BACKPAT)
+   if ((colorDump || !useGray) && pen > BACKPAT)
    {
       DumpOvalPath (FP, ObjPtr, xc, yc, a, b, width, BACKPAT, 0);
-      if (colorDump)
-         DumpRGBColorLine(FP, color_index, 3, TRUE);
+      DumpRGBColorLine(FP, color_index, 3, TRUE);
    }
    DumpOvalPath (FP, ObjPtr, xc, yc, a, b, width, pen, dash);
 
    fprintf (FP, "grestore\n");
-
+   if (ObjPtr->ctm != NULL) fprintf(FP, "grestore\n");
    fprintf (FP, "\n");
 }
+
+int NeedsToCacheOvalObj (ObjPtr)
+   struct ObjRec	* ObjPtr;
+{
+   return (ObjPtr->ctm != NULL);
+}
+
+static double	ovalXMag1=(double)0.0, ovalYMag1=(double)0.0;
+static double	ovalXMag2=(double)0.0, ovalYMag2=(double)0.0;
+
+static
+void MakeCachedOval(ObjPtr)
+   struct ObjRec *ObjPtr;
+{
+   register int		i;
+   int			sn, cntrln;
+   int			saved_zoomedin, saved_zoomscale, saved_x, saved_y;
+   struct OvalRec	* oval_ptr=ObjPtr->detail.o;
+   XPoint		*sv=NULL;
+   DoublePoint		*tmp_sv=NULL, *pv=NULL, *cntrlv=NULL;
+   struct BBRec		obbox;
+   double		w, h, cx, cy;
+
+   if (ovalXMag1 == (double)0.0) {
+      ovalXMag1 = (double)cos((double)(28.6*M_PI/180.0));
+      ovalYMag1 = (double)sin((double)(28.6*M_PI/180.0));
+      ovalXMag2 = (double)cos((double)(61.4*M_PI/180.0));
+      ovalYMag2 = (double)sin((double)(61.4*M_PI/180.0));
+   }
+   if (ObjPtr->ctm == NULL) return;
+
+   if (oval_ptr->rotated_vlist != NULL) free(oval_ptr->rotated_vlist);
+   oval_ptr->rotated_n = 0;
+   oval_ptr->rotated_vlist = NULL;
+   tmp_sv = (DoublePoint*)malloc((13+1)*sizeof(DoublePoint));
+   pv = (DoublePoint*)malloc((13+1)*sizeof(DoublePoint));
+   if (tmp_sv == NULL || pv == NULL) FailAllocMessage();
+
+   obbox.ltx = ObjPtr->orig_obbox.ltx - ObjPtr->x;
+   obbox.lty = ObjPtr->orig_obbox.lty - ObjPtr->y;
+   obbox.rbx = ObjPtr->orig_obbox.rbx - ObjPtr->x;
+   obbox.rby = ObjPtr->orig_obbox.rby - ObjPtr->y;
+   cx = (((double)(obbox.ltx+obbox.rbx))/2.0);
+   cy = (((double)(obbox.lty+obbox.rby))/2.0);
+   w = (((double)(obbox.rbx-obbox.ltx))/2.0);
+   h = (((double)(obbox.rby-obbox.lty))/2.0);
+
+   tmp_sv[0].x  = (double)obbox.rbx;          tmp_sv[0].y  = (double)cy;
+   tmp_sv[1].x  = ((double)cx)+(w*ovalXMag1); tmp_sv[1].y  = ((double)cy)-(h*ovalYMag1);
+   tmp_sv[2].x  = ((double)cx)+(w*ovalXMag2); tmp_sv[2].y  = ((double)cy)-(h*ovalYMag2);
+   tmp_sv[3].x  = (double)cx;                 tmp_sv[3].y  = (double)obbox.lty;
+   tmp_sv[4].x  = ((double)cx)-(w*ovalXMag2); tmp_sv[4].y  = ((double)cy)-(h*ovalYMag2);
+   tmp_sv[5].x  = ((double)cx)-(w*ovalXMag1); tmp_sv[5].y  = ((double)cy)-(h*ovalYMag1);
+   tmp_sv[6].x  = (double)obbox.ltx;          tmp_sv[6].y  = (double)cy;
+   tmp_sv[7].x  = ((double)cx)-(w*ovalXMag1); tmp_sv[7].y  = ((double)cy)+(h*ovalYMag1);
+   tmp_sv[8].x  = ((double)cx)-(w*ovalXMag2); tmp_sv[8].y  = ((double)cy)+(h*ovalYMag2);
+   tmp_sv[9].x  = (double)cx;                 tmp_sv[9].y  = (double)obbox.rby;
+   tmp_sv[10].x = ((double)cx)+(w*ovalXMag2); tmp_sv[10].y = ((double)cy)+(h*ovalYMag2);
+   tmp_sv[11].x = ((double)cx)+(w*ovalXMag1); tmp_sv[11].y = ((double)cy)+(h*ovalYMag1);
+   tmp_sv[12].x = tmp_sv[0].x;                tmp_sv[12].y = tmp_sv[0].y;
+   for (i=0; i < 13; i++) {
+      double x, y;
+
+      TransformDoublePointThroughCTM(tmp_sv[i].x, tmp_sv[i].y, ObjPtr->ctm,
+            &x, &y);
+      tmp_sv[i].x = pv[i].x = OFFSET_DOUBLE_X(x + (double)ObjPtr->x);
+      tmp_sv[i].y = pv[i].y = OFFSET_DOUBLE_Y(y + (double)ObjPtr->y);
+   }
+   saved_zoomedin = zoomedIn;
+   saved_zoomscale = zoomScale;
+   saved_x = drawOrigX;
+   saved_y = drawOrigY;
+   zoomedIn = FALSE;
+   zoomScale = 0;
+   drawOrigX = drawOrigY = 0;
+   sv = MakeDoubleIntSplinePolygonVertex(&sn, &cntrln, &cntrlv,
+         drawOrigX, drawOrigY, 13, pv);
+   zoomedIn = saved_zoomedin;
+   zoomScale = saved_zoomscale;
+   drawOrigX = saved_x;
+   drawOrigY = saved_y;
+   free(pv);
+   if (sv != NULL) {
+      free(tmp_sv);
+      oval_ptr->rotated_n = sn;
+      oval_ptr->rotated_vlist = sv;
+   } else {
+      oval_ptr->rotated_n = 13;
+   }
+
+   if (cntrlv != NULL) free(cntrlv);
+}
  
 void DrawOvalObj (window, XOff, YOff, ObjPtr)
    Window		window;
@@ -259,6 +355,7 @@ void DrawOvalObj (window, XOff, YOff, ObjPtr)
    int			fill, width, pen, dash, pixel, real_x_off, real_y_off;
    struct BBRec		bbox;
    XGCValues		values;
+   struct OvalRec	*oval_ptr=ObjPtr->detail.o;
 
    real_x_off = (zoomedIn ? XOff : (XOff>>zoomScale)<<zoomScale);
    real_y_off = (zoomedIn ? YOff : (YOff>>zoomScale)<<zoomScale);
@@ -267,26 +364,34 @@ void DrawOvalObj (window, XOff, YOff, ObjPtr)
    bbox.rbx = ZOOMED_SIZE(ObjPtr->obbox.rbx - real_x_off);
    bbox.rby = ZOOMED_SIZE(ObjPtr->obbox.rby - real_y_off);
 
-   fill = ObjPtr->detail.o->fill;
-   width = ObjPtr->detail.o->width;
-   pen = ObjPtr->detail.o->pen;
-   dash = ObjPtr->detail.o->dash;
+   fill = oval_ptr->fill;
+   width = oval_ptr->width;
+   pen = oval_ptr->pen;
+   dash = oval_ptr->dash;
    pixel = colorPixels[ObjPtr->color];
 
+   if (NeedsToCacheOvalObj(ObjPtr) && oval_ptr->rotated_vlist==NULL) {
+      MakeCachedOval(ObjPtr);
+   }
    if (fill != 0)
    {
-      values.foreground = (fill == 2) ? myBgPixel : pixel;
+      values.foreground = GetDrawingBgPixel(fill, pixel);
       values.function = GXcopy;
       values.fill_style = FillOpaqueStippled;
       values.stipple = patPixmap[fill];
       XChangeGC (mainDisplay, drawGC,
             GCForeground | GCFunction | GCFillStyle | GCStipple, &values);
-      XFillArc (mainDisplay, window, drawGC, bbox.ltx, bbox.lty,
-            bbox.rbx-bbox.ltx, bbox.rby-bbox.lty, 0, 360*64);
+      if (ObjPtr->ctm != NULL)
+         XFillPolygon (mainDisplay, window, drawGC,
+               oval_ptr->rotated_vlist, oval_ptr->rotated_n,
+               Convex, CoordModeOrigin);
+      else
+         XFillArc (mainDisplay, window, drawGC, bbox.ltx, bbox.lty,
+               bbox.rbx-bbox.ltx, bbox.rby-bbox.lty, 0, 360*64);
    }
    if (pen != 0)
    {
-      values.foreground = (pen == 2) ? myBgPixel : pixel;
+      values.foreground = GetDrawingBgPixel(pen, pixel);
       values.function = GXcopy;
       values.fill_style = FillOpaqueStippled;
       values.stipple = patPixmap[pen];
@@ -309,31 +414,51 @@ void DrawOvalObj (window, XOff, YOff, ObjPtr)
       XChangeGC (mainDisplay, drawGC,
             GCForeground | GCFunction | GCFillStyle | GCStipple | GCLineWidth |
             GCLineStyle, &values);
-      XDrawArc (mainDisplay, window, drawGC, bbox.ltx, bbox.lty,
-            bbox.rbx-bbox.ltx, bbox.rby-bbox.lty, 0, 360*64);
+      if (ObjPtr->ctm != NULL)
+         XDrawLines (mainDisplay, window, drawGC,
+               oval_ptr->rotated_vlist, oval_ptr->rotated_n,
+               CoordModeOrigin);
+      else
+         XDrawArc (mainDisplay, window, drawGC, bbox.ltx, bbox.lty,
+               bbox.rbx-bbox.ltx, bbox.rby-bbox.lty, 0, 360*64);
    }
 }
 
-static
-void CreateOvalObj (BBox)
-   struct BBRec	BBox;
+void CreateOvalObj(BBox, CreateAbsolute)
+   struct BBRec	*BBox;
+   int CreateAbsolute;
 {
    struct OvalRec	* oval_ptr;
    struct ObjRec	* obj_ptr;
    int			width, w;
 
-   oval_ptr = (struct OvalRec *) calloc (1, sizeof(struct OvalRec));
+   oval_ptr = (struct OvalRec *)malloc(sizeof(struct OvalRec));
+   if (oval_ptr == NULL) FailAllocMessage();
+   memset(oval_ptr, 0, sizeof(struct OvalRec));
    oval_ptr->fill = objFill;
    oval_ptr->width = width = curWidthOfLine[lineWidth];
+   UtilStrCpy(oval_ptr->width_spec, sizeof(oval_ptr->width_spec),
+         curWidthOfLineSpec[lineWidth]);
    oval_ptr->pen = penPat;
    oval_ptr->dash = curDash;
+   oval_ptr->rotated_n = 0;
+   oval_ptr->rotated_vlist = NULL;
 
-   obj_ptr = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
+   obj_ptr = (struct ObjRec *)malloc(sizeof(struct ObjRec));
+   if (obj_ptr == NULL) FailAllocMessage();
+   memset(obj_ptr, 0, sizeof(struct ObjRec));
 
-   obj_ptr->bbox.ltx = obj_ptr->obbox.ltx = obj_ptr->x = ABS_X(BBox.ltx);
-   obj_ptr->bbox.lty = obj_ptr->obbox.lty = obj_ptr->y = ABS_Y(BBox.lty);
-   obj_ptr->bbox.rbx = obj_ptr->obbox.rbx = ABS_X(BBox.rbx);
-   obj_ptr->bbox.rby = obj_ptr->obbox.rby = ABS_Y(BBox.rby);
+   if (CreateAbsolute) {
+      obj_ptr->bbox.ltx = obj_ptr->obbox.ltx = obj_ptr->x = BBox->ltx;
+      obj_ptr->bbox.lty = obj_ptr->obbox.lty = obj_ptr->y = BBox->lty;
+      obj_ptr->bbox.rbx = obj_ptr->obbox.rbx = BBox->rbx;
+      obj_ptr->bbox.rby = obj_ptr->obbox.rby = BBox->rby;
+   } else {
+      obj_ptr->bbox.ltx = obj_ptr->obbox.ltx = obj_ptr->x = ABS_X(BBox->ltx);
+      obj_ptr->bbox.lty = obj_ptr->obbox.lty = obj_ptr->y = ABS_Y(BBox->lty);
+      obj_ptr->bbox.rbx = obj_ptr->obbox.rbx = ABS_X(BBox->rbx);
+      obj_ptr->bbox.rby = obj_ptr->obbox.rby = ABS_Y(BBox->rby);
+   }
    w = HALF_W(width);
    obj_ptr->bbox.ltx -= w;
    obj_ptr->bbox.lty -= w;
@@ -347,6 +472,8 @@ void CreateOvalObj (BBox)
    obj_ptr->locked = FALSE;
    obj_ptr->detail.o = oval_ptr;
    obj_ptr->fattr = obj_ptr->lattr = NULL;
+   obj_ptr->ctm = NULL;
+   obj_ptr->invisible = FALSE;
    AddObj (NULL, topObj, obj_ptr);
 }
 
@@ -388,10 +515,10 @@ void ContinueOval (OrigX, OrigY)
    PixelToMeasurementUnit(h_buf, 0);
    PixelToMeasurementUnit(x_buf, ABS_X(grid_x));
    PixelToMeasurementUnit(y_buf, ABS_Y(grid_y));
-   sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
-   StartShowMeasureCursor (grid_x, grid_y, buf, TRUE);
-   BeginIntervalRulers (grid_x, grid_y, grid_x, grid_y);
-   XGrabPointer (mainDisplay, drawWindow, FALSE,
+   sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
+   StartShowMeasureCursor(grid_x, grid_y, buf, TRUE);
+   BeginIntervalRulers(grid_x, grid_y, grid_x, grid_y);
+   XGrabPointer(mainDisplay, drawWindow, FALSE,
          PointerMotionMask | ButtonReleaseMask,
          GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
    
@@ -410,7 +537,7 @@ void ContinueOval (OrigX, OrigY)
          PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(bbox.rby-OrigY)));
          PixelToMeasurementUnit(x_buf, ABS_X(bbox.rbx));
          PixelToMeasurementUnit(y_buf, ABS_Y(bbox.rby));
-         sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+         sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
          EndShowMeasureCursor (bbox.rbx, bbox.rby, buf, TRUE);
          done = TRUE;
       }
@@ -447,7 +574,7 @@ void ContinueOval (OrigX, OrigY)
             PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(bbox.rby-OrigY)));
             PixelToMeasurementUnit(x_buf, ABS_X(bbox.rbx));
             PixelToMeasurementUnit(y_buf, ABS_Y(bbox.rby));
-            sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+            sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
             ShowMeasureCursor (bbox.rbx, bbox.rby, buf, TRUE);
             MyOval (drawWindow, drawGC, bbox);
             bbox.rbx = grid_x;
@@ -457,7 +584,7 @@ void ContinueOval (OrigX, OrigY)
             PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(bbox.rby-OrigY)));
             PixelToMeasurementUnit(x_buf, ABS_X(bbox.rbx));
             PixelToMeasurementUnit(y_buf, ABS_Y(bbox.rby));
-            sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+            sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
             ShowMeasureCursor (bbox.rbx, bbox.rby, buf, TRUE);
          }
          DrawIntervalRulers (OrigX, OrigY, grid_x, grid_y);
@@ -466,11 +593,12 @@ void ContinueOval (OrigX, OrigY)
       else if (input.type == KeyPress)
       {
          KeySym	key_sym;
-         char	s[80];
+         char s[80];
+         int has_ch;
 
-         XLookupString (&(input.xkey), s, 80-1, &key_sym, &c_stat);
-         TranslateKeys (s, &key_sym);
-         if (s[0] == '\033' && (key_sym & 0xff) == '\033')
+         has_ch = XLookupString(&(input.xkey), s, sizeof(s), &key_sym, &c_stat);
+         TranslateKeys(s, &key_sym);
+         if (CharIsESC(&(input.xkey), s, key_sym, &has_ch))
          {
             XUngrabPointer (mainDisplay, CurrentTime);
             MyOval (drawWindow, drawGC, bbox);
@@ -479,7 +607,7 @@ void ContinueOval (OrigX, OrigY)
             PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(bbox.rby-OrigY)));
             PixelToMeasurementUnit(x_buf, ABS_X(bbox.rbx));
             PixelToMeasurementUnit(y_buf, ABS_Y(bbox.rby));
-            sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+            sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
             EndShowMeasureCursor (bbox.rbx, bbox.rby, buf, TRUE);
             abort = TRUE;
             done = TRUE;
@@ -496,7 +624,7 @@ void ContinueOval (OrigX, OrigY)
       {
          end_y = bbox.lty; bbox.lty = bbox.rby; bbox.rby = end_y;
       }
-      CreateOvalObj (bbox);
+      CreateOvalObj(&bbox, FALSE);
       RecordNewObjCmd ();
       DrawOvalObj (drawWindow, drawOrigX, drawOrigY, topObj);
       ovalDrawn = TRUE;
@@ -526,28 +654,54 @@ void SaveOvalObj (FP, ObjPtr)
    FILE			* FP;
    struct ObjRec	* ObjPtr;
 {
-   if (fprintf (FP, "oval('%s',", colorMenuItems[ObjPtr->color]) == EOF)
+   struct OvalRec *oval_ptr=ObjPtr->detail.o;
+
+   if (fprintf (FP, "oval('%s',", colorMenuItems[ObjPtr->color]) == EOF) {
       writeFileFailed = TRUE;
-   if (fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,",
+   }
+   if (fprintf(FP,
+         "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,'%s',",
          ObjPtr->obbox.ltx, ObjPtr->obbox.lty, ObjPtr->obbox.rbx,
-         ObjPtr->obbox.rby, ObjPtr->detail.o->fill, ObjPtr->detail.o->width,
-         ObjPtr->detail.o->pen, ObjPtr->id, ObjPtr->detail.o->dash,
-         ObjPtr->rotation, ObjPtr->locked) == EOF)
+         ObjPtr->obbox.rby, oval_ptr->fill, oval_ptr->width,
+         oval_ptr->pen, ObjPtr->id, oval_ptr->dash,
+         ObjPtr->rotation, ObjPtr->locked, ObjPtr->ctm!=NULL,
+         ObjPtr->invisible, oval_ptr->width_spec) == EOF) {
       writeFileFailed = TRUE;
+   }
+   if (ObjPtr->ctm != NULL && fprintf(FP,
+         "[\n    %1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d],",
+         ObjPtr->x, ObjPtr->y,
+         ObjPtr->orig_obbox.ltx, ObjPtr->orig_obbox.lty,
+         ObjPtr->orig_obbox.rbx, ObjPtr->orig_obbox.rby,
+         ObjPtr->ctm->m[CTM_SX], ObjPtr->ctm->m[CTM_SIN],
+         ObjPtr->ctm->m[CTM_MSIN], ObjPtr->ctm->m[CTM_SY],
+         ObjPtr->ctm->m[CTM_TX], ObjPtr->ctm->m[CTM_TY]) == EOF) {
+      writeFileFailed = TRUE;
+   }
    SaveAttrs (FP, ObjPtr->lattr);
    if (fprintf (FP, ")") == EOF) writeFileFailed = TRUE;
 }
 
 #define GETVALUE(val,name) ScanValue("%d", &(val), name, "oval")
+#define GETSTRNG(val,name) ScanValue("%s", (val), name, "oval")
 
-void ReadOvalObj (Inbuf, ObjPtr)
+void ReadOvalObj (FP, Inbuf, ObjPtr)
+   FILE			* FP;
    char			* Inbuf;
    struct ObjRec	* * ObjPtr;
 {
    struct OvalRec	* oval_ptr;
-   char			color_str[40], * s;
+   char			color_str[40], * s, width_spec[40];
    int			ltx, lty, rbx, rby, fill, width, pen, dash, w, id=0;
    int			new_alloc, rotation, locked=FALSE;
+   int			transformed=FALSE, invisible=FALSE;
+#ifdef _TGIF_WB
+ 
+   char                 aux[10000];
+ 
+   strcpy(aux,Inbuf);
+
+#endif /* _TGIF_WB */
 
    *ObjPtr = NULL;
 
@@ -558,6 +712,7 @@ void ReadOvalObj (Inbuf, ObjPtr)
 
    dash = 0;
    rotation = 0;
+   *width_spec = '\0';
    if (fileVersion <= 5)
    {
       if (GETVALUE (ltx,      "ltx") == INVALID ||
@@ -639,7 +794,7 @@ void ReadOvalObj (Inbuf, ObjPtr)
       }
       if (id >= objId) objId = id+1;
    }
-   else
+   else if (fileVersion <= 32)
    {
       if (GETVALUE (ltx,      "ltx") == INVALID ||
           GETVALUE (lty,      "lty") == INVALID ||
@@ -657,6 +812,54 @@ void ReadOvalObj (Inbuf, ObjPtr)
       }
       if (id >= objId) objId = id+1;
    }
+   else
+   {
+#ifdef _TGIF_DBG
+      if (GETVALUE (ltx,         "ltx") == INVALID ||
+          GETVALUE (lty,         "lty") == INVALID ||
+          GETVALUE (rbx,         "rbx") == INVALID ||
+          GETVALUE (rby,         "rby") == INVALID ||
+          GETVALUE (fill,        "fill") == INVALID ||
+          GETVALUE (width,       "width") == INVALID ||
+          GETVALUE (pen,         "pen") == INVALID ||
+          GETVALUE (id,          "id") == INVALID ||
+          GETVALUE (dash,        "dash") == INVALID ||
+          GETVALUE (rotation,    "rotation") == INVALID ||
+          GETVALUE (locked,      "locked") == INVALID ||
+          GETVALUE (transformed, "transformed") == INVALID ||
+          GETVALUE (invisible,   "invisible") == INVALID)
+      {
+         return;
+      }
+      if (id >= objId) objId = id+1;
+      if (GETSTRNG(width_spec,  "width_spec") != INVALID) {
+         UtilRemoveQuotes(width_spec);
+      } else {
+         sprintf(width_spec, "%1d", width);
+         fprintf(stderr, "Bad width_spec in oval... Fix this file!\n");
+      }
+#else /* ~_TGIF_DBG */
+      if (GETVALUE (ltx,         "ltx") == INVALID ||
+          GETVALUE (lty,         "lty") == INVALID ||
+          GETVALUE (rbx,         "rbx") == INVALID ||
+          GETVALUE (rby,         "rby") == INVALID ||
+          GETVALUE (fill,        "fill") == INVALID ||
+          GETVALUE (width,       "width") == INVALID ||
+          GETVALUE (pen,         "pen") == INVALID ||
+          GETVALUE (id,          "id") == INVALID ||
+          GETVALUE (dash,        "dash") == INVALID ||
+          GETVALUE (rotation,    "rotation") == INVALID ||
+          GETVALUE (locked,      "locked") == INVALID ||
+          GETVALUE (transformed, "transformed") == INVALID ||
+          GETVALUE (invisible,   "invisible") == INVALID ||
+          GETSTRNG (width_spec,  "width_spec") == INVALID)
+      {
+         return;
+      }
+      if (id >= objId) objId = id+1;
+      UtilRemoveQuotes(width_spec);
+#endif /* _TGIF_DBG */
+   }
 
    if (ltx > rbx || lty > rby)
    {
@@ -668,19 +871,31 @@ void ReadOvalObj (Inbuf, ObjPtr)
    }
 
    if (fileVersion <= 16 && width <= 6) width = origWidthOfLine[width];
+   if (fileVersion <= 32) {
+      sprintf(width_spec, "%1d", width);
+   }
    fill = UpgradePenFill (fill);
    pen = UpgradePenFill (pen);
 
-   * ObjPtr = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
-   oval_ptr = (struct OvalRec *) calloc (1, sizeof(struct OvalRec));
+   *ObjPtr = (struct ObjRec *)malloc(sizeof(struct ObjRec));
+   if (*ObjPtr == NULL) FailAllocMessage();
+   memset(*ObjPtr, 0, sizeof(struct ObjRec));
+   oval_ptr = (struct OvalRec *)malloc(sizeof(struct OvalRec));
+   if (oval_ptr == NULL) FailAllocMessage();
+   memset(oval_ptr, 0, sizeof(struct OvalRec));
 
    oval_ptr->fill = fill;
    oval_ptr->width = width;
+   UtilStrCpy(oval_ptr->width_spec, sizeof(oval_ptr->width_spec), width_spec);
    oval_ptr->pen = pen;
    oval_ptr->dash = dash;
+
+   oval_ptr->rotated_n = 0;
+   oval_ptr->rotated_vlist = NULL;
+
    (*ObjPtr)->x = ltx;
    (*ObjPtr)->y = lty;
-   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc, TRUE);
+   (*ObjPtr)->color = QuickFindColorIndex(*ObjPtr, color_str, &new_alloc, TRUE);
    (*ObjPtr)->dirty = FALSE;
    (*ObjPtr)->id = id;
    (*ObjPtr)->rotation = rotation;
@@ -696,11 +911,65 @@ void ReadOvalObj (Inbuf, ObjPtr)
    (*ObjPtr)->bbox.rbx = rbx + w;
    (*ObjPtr)->bbox.rby = rby + w;
    (*ObjPtr)->detail.o = oval_ptr;
+   (*ObjPtr)->ctm = NULL;
+   (*ObjPtr)->invisible = invisible;
+
+   if (fileVersion >= 33 && transformed)
+   {
+      int real_x=0, real_y=0;
+      struct BBRec orig_obbox;
+      char inbuf[MAXSTRING+1];
+      struct XfrmMtrxRec *ctm;
+
+#ifdef _TGIF_WB
+ 
+      s = FindChar ((int)'[', aux);
+      s++;
+      if (!bufferAsFileForWhiteBoard) {
+         fgets(inbuf, MAXSTRING, FP);
+      }
+ 
+#else /* ~_TGIF_WB */
+ 
+      fgets(inbuf, MAXSTRING, FP);
+ 
+#endif /* _TGIF_WB */
+
+      scanLineNum++;
+      InitScan(inbuf, "\t\n, ");
+
+      ctm = (struct XfrmMtrxRec *)malloc(sizeof(struct XfrmMtrxRec));
+      if (ctm == NULL) FailAllocMessage();
+      if (GETVALUE(real_x,           "real_x") == INVALID ||
+          GETVALUE(real_y,           "real_y") == INVALID ||
+          GETVALUE(orig_obbox.ltx,   "orig_obbox.ltx") == INVALID ||
+          GETVALUE(orig_obbox.lty,   "orig_obbox.lty") == INVALID ||
+          GETVALUE(orig_obbox.rbx,   "orig_obbox.rbx") == INVALID ||
+          GETVALUE(orig_obbox.rby,   "orig_obbox.rby") == INVALID ||
+          GETVALUE(ctm->m[CTM_SX],   "CTM_SX") == INVALID ||
+          GETVALUE(ctm->m[CTM_SIN],  "CTM_SIN") == INVALID ||
+          GETVALUE(ctm->m[CTM_MSIN], "CTM_MSIN") == INVALID ||
+          GETVALUE(ctm->m[CTM_SY],   "CTM_SY") == INVALID ||
+          GETVALUE(ctm->m[CTM_TX],   "CTM_TX") == INVALID ||
+          GETVALUE(ctm->m[CTM_TY],   "CTM_TY") == INVALID) {
+         return;
+      }
+      (*ObjPtr)->ctm = ctm;
+      if (ctm != NULL) {
+         memcpy(&(*ObjPtr)->orig_obbox, &orig_obbox, sizeof(struct BBRec));
+         (*ObjPtr)->x = real_x;
+         (*ObjPtr)->y = real_y;
+         GetTransformedOBBoxOffsetVs(*ObjPtr, (*ObjPtr)->rotated_obbox);
+      }
+   }
 }
 
 void FreeOvalObj (ObjPtr)
    struct ObjRec	* ObjPtr;
 {
-   cfree (ObjPtr->detail.o);
-   cfree (ObjPtr);
+   if (ObjPtr->detail.o->rotated_vlist != NULL) {
+      free(ObjPtr->detail.o->rotated_vlist);
+   }
+   free(ObjPtr->detail.o);
+   free(ObjPtr);
 }
