@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,7 +29,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/ps.c,v 2.14 1995/05/15 01:22:48 william Exp $";
+      "@(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/ps.c,v 3.0 1996/05/06 16:06:59 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -44,9 +44,10 @@ static char RCSid[] =
 #endif
 #include "util.e"
 
-#define DEF_PS_DICT_COUNT 1
+#define DEF_PS_DICT_COUNT 2
 
 int preDumpSetup=FALSE;
+int needsTgifOrigCTM=TRUE;
 
 static int psDictCount=DEF_PS_DICT_COUNT;
 
@@ -124,7 +125,7 @@ void PSUseColorImage()
 {
    if (preDumpSetup && !psUseColorImage) {
       psUseColorImage = TRUE;
-      psDictCount += 9;
+      psDictCount += 11;
    }
 }
 
@@ -176,7 +177,7 @@ void PSUseReencode(font_name)
             return;
          }
       }
-      p_reencode = (struct ReEncodeRec *)calloc(1, sizeof(struct ReEncodeRec));
+      p_reencode = (struct ReEncodeRec *)malloc(sizeof(struct ReEncodeRec));
       if (p_reencode == NULL) FailAllocMessage();
       p_reencode->font_name = UtilStrDup(font_name);
       p_reencode->next = topReEncodeInfo;
@@ -192,8 +193,8 @@ void CleanUpReEncodeInfo()
 
    for ( ; topReEncodeInfo != NULL; topReEncodeInfo=p_next_reencode) {
       p_next_reencode = topReEncodeInfo->next;
-      cfree(topReEncodeInfo->font_name);
-      cfree(topReEncodeInfo);
+      free(topReEncodeInfo->font_name);
+      free(topReEncodeInfo);
    }
 }
 
@@ -202,6 +203,8 @@ void ResetPSInfo()
    psDictCount = DEF_PS_DICT_COUNT;
    preDumpSetup = TRUE;
 
+   needsTgifOrigCTM = TRUE;
+
    psUsePSAdobe=TRUE;
 #ifdef INVERT_CTM_BUG
    UseMinRadius();
@@ -485,6 +488,37 @@ static char *psColorImageMacro[] = {
   "   } def",
   " } ifelse",
   "",
+  "/tgiftranscolorspot",
+  " { tgifxpmdict begin",
+  "      /index exch def",
+  "      pixels index 3 mul 3 getinterval aload pop",
+  "      dup 0 lt { pop pop pop false } { setrgbcolor true } ifelse",
+  "   end",
+  " } def",
+  "",
+  "/tgiftranscolorimage",
+  " { tgifcolordict begin",
+  "      pop pop pop pop pop",
+  "      /ih exch def",
+  "      /iw exch def",
+  "      /x 0 def",
+  "      /y 0 def",
+  "      1 1 ih",
+  "       { pop 1 1 iw",
+  "          { pop currentfile",
+  "            tgifbwpicstr readhexstring pop 0 get tgiftranscolorspot",
+  "            { x y moveto 1 0 rlineto 0 1 rlineto -1 0 rlineto",
+  "              closepath fill",
+  "            }",
+  "            if",
+  "            /x x 1 add def",
+  "          } for",
+  "         /y y 1 add def",
+  "         /x 0 def",
+  "       } for",
+  "   end",
+  " } def",
+  "",
   NULL
 };
 
@@ -514,6 +548,7 @@ static char *psColorPatternMacro[] = {
   "      /retstr widthlim 1 add string def",
   "      /i 0 def",
   "",
+  "      tgiforigctm setmatrix",
   "      ltx lty translate",
   "      w h true [1 0 0 1 0 0] {tgifpatproc} imagemask",
   "      ltx neg lty neg translate",
