@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,10 +29,11 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/rcbox.c,v 2.63 1995/07/10 05:48:15 william Exp $";
+      "@(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/rcbox.c,v 3.5 1996/10/30 22:49:44 william Exp $";
 #endif
 
 #include <stdio.h>
+#include <string.h>
 #include <sys/types.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -55,12 +56,15 @@ static char RCSid[] =
 #include "poly.e"
 #include "ps.e"
 #include "raster.e"
+#include "rect.e"
 #ifndef _NO_EXTERN
 #include "rcbox.e"
 #endif
 #include "ruler.e"
 #include "select.e"
 #include "setup.e"
+#include "spline.e"
+#include "util.e"
 #include "xpixmap.e"
 
 int	rcBoxDrawn = FALSE;
@@ -201,48 +205,42 @@ void DumpRCBoxPath (FP, ObjPtr, ltx, lty, rbx, rby, r, width, pen, dash)
 {
    register int	i;
 
-   if (width != 1) fprintf (FP, "   %1d setlinewidth\n", width);
+   fprintf(FP, "   gsave\n");
+   if (!colorDump && useGray && pen > BACKPAT)
+   {
+      GrayCheck(pen);
+      fprintf(FP, "      %s setgray\n", GrayStr(pen));
+   }
+   DumpRCBoxPSPath(FP, ltx, lty, rbx, rby, r, "      ", "         ");
+   fprintf(FP, "      closepath\n");
+
+   if (ObjPtr->ctm != NULL) {
+      fprintf(FP, "      tgiforigctm setmatrix\n");
+   }
    if (dash != 0)
    {
-      fprintf (FP, "   [");
-      for (i = 0; i < dashListLength[dash]-1; i++)
-         fprintf (FP, "%1d ", (int)(dashList[dash][i]));
-      fprintf (FP, "%1d] 0 setdash\n",
+      fprintf(FP, "      [");
+      for (i = 0; i < dashListLength[dash]-1; i++) {
+         fprintf(FP, "%1d ", (int)(dashList[dash][i]));
+      }
+      fprintf(FP, "%1d] 0 setdash\n",
             (int)(dashList[dash][dashListLength[dash]-1]));
    }
-
-   if (!colorDump && pen > BACKPAT)
-   {
-      GrayCheck (pen);
-      if (useGray)
-         fprintf (FP, "   %s setgray\n", GrayStr(pen));
-      else
-      {
-         if (preDumpSetup) PSUseBWPattern();
-         fprintf (FP, "   pat%1d %s\n", pen, patternStr);
+   if (width != 1) fprintf(FP, "      %1d setlinewidth\n", width);
+   switch (pen) {
+   case SOLIDPAT: fprintf(FP, "      stroke\n"); break;
+   case BACKPAT: fprintf(FP, "      1 setgray stroke 0 setgray\n"); break;
+   default:
+      if (colorDump || !useGray) {
+         if (preDumpSetup) PSUseColorPattern();
+         fprintf(FP, "      flattenpath strokepath clip newpath\n");
+         DumpPatFill(FP, pen, 8, ObjPtr->bbox, "      ");
+      } else {
+         fprintf(FP, "      stroke\n");
       }
+      break;
    }
-
-   DumpRCBoxPSPath (FP, ltx, lty, rbx, rby, r, "   ", "      ");
-   fprintf (FP, "   closepath ");
-
-   switch (pen)
-   {
-      case SOLIDPAT: fprintf (FP, "stroke\n"); break;
-      case BACKPAT: fprintf (FP, "1 setgray stroke 0 setgray\n"); break;
-      default:
-         if (colorDump)
-         {
-            if (preDumpSetup) PSUseColorPattern();
-            fprintf (FP, "flattenpath strokepath clip newpath\n");
-            DumpPatFill (FP, pen, 8, ObjPtr->bbox, "   ");
-         }
-         else
-            fprintf (FP, "stroke\n");
-         break;
-   }
-   if (dash != 0) fprintf (FP, "   [] 0 setdash\n");
-   if (width != 1) fprintf (FP, "   1 setlinewidth\n");
+   fprintf(FP, "   grestore\n");
 }
  
 void DumpRCBoxObj (FP, ObjPtr)
@@ -252,10 +250,13 @@ void DumpRCBoxObj (FP, ObjPtr)
    register int	ltx, lty, rbx, rby;
    int		fill, width, pen, dash, color_index, r;
 
-   ltx = ObjPtr->obbox.ltx;
-   lty = ObjPtr->obbox.lty;
-   rbx = ObjPtr->obbox.rbx;
-   rby = ObjPtr->obbox.rby;
+   if (ObjPtr->ctm == NULL) {
+      ltx = ObjPtr->obbox.ltx; lty = ObjPtr->obbox.lty;
+      rbx = ObjPtr->obbox.rbx; rby = ObjPtr->obbox.rby;
+   } else {
+      ltx = ObjPtr->orig_obbox.ltx; lty = ObjPtr->orig_obbox.lty;
+      rbx = ObjPtr->orig_obbox.rbx; rby = ObjPtr->orig_obbox.rby;
+   }
 
    fill = ObjPtr->detail.rcb->fill;
    pen = ObjPtr->detail.rcb->pen;
@@ -266,11 +267,22 @@ void DumpRCBoxObj (FP, ObjPtr)
    if (fill == NONEPAT && pen == NONEPAT) return;
 
    fprintf (FP, "%% RCBOX\n");
+   if (ObjPtr->ctm != NULL) {
+      float m[6];
+
+      fprintf(FP, "gsave\n");
+      m[CTM_SX] = ((float)ObjPtr->ctm->m[CTM_SX])/((float)1000.0);
+      m[CTM_SY] = ((float)ObjPtr->ctm->m[CTM_SY])/((float)1000.0);
+      m[CTM_SIN] = ((float)ObjPtr->ctm->m[CTM_SIN])/((float)1000.0);
+      m[CTM_MSIN] = ((float)ObjPtr->ctm->m[CTM_MSIN])/((float)1000.0);
+      fprintf (FP, "   %1d %1d translate\n", ObjPtr->x, ObjPtr->y);
+      fprintf (FP, "   [%.3f %.3f %.3f %.3f %1d %1d] concat\n",
+            m[CTM_SX], m[CTM_SIN], m[CTM_MSIN], m[CTM_SY],
+            ObjPtr->ctm->m[CTM_TX], ObjPtr->ctm->m[CTM_TY]);
+      fprintf (FP, "   %1d neg %1d neg translate\n", ObjPtr->x, ObjPtr->y);
+   }
    color_index = ObjPtr->color;
-   if (colorDump)
-      DumpRGBColorLine(FP, color_index, 0, TRUE);
-   else
-      fprintf (FP, "0 setgray\n");
+   DumpRGBColorLine(FP, color_index, 0, TRUE);
 
    switch (fill)
    {
@@ -282,32 +294,23 @@ void DumpRCBoxObj (FP, ObjPtr)
       case BACKPAT:
          DumpRCBoxPSPath (FP, ltx, lty, rbx, rby, r, "", "   ");
          fprintf (FP, "closepath 1 setgray fill\n");
-         if (colorDump)
-            DumpRGBColorLine(FP, color_index, 3, TRUE);
-         else
-            fprintf (FP, "0 setgray\n");
+         DumpRGBColorLine(FP, color_index, 3, TRUE);
          break;
       default:
          fprintf (FP, "gsave\n");
-         if (!colorDump)
-         {
-            GrayCheck (fill);
-            if (useGray)
-               fprintf (FP, "   %s setgray\n", GrayStr(fill));
-            else
-            {
-               if (preDumpSetup) PSUseBWPattern();
-               fprintf (FP, "   pat%1d %s\n", fill, patternStr);
-            }
-         }
-         else
+         if (colorDump || !useGray)
          {
             DumpRCBoxPSPath (FP, ltx, lty, rbx, rby, r, "   ", "      ");
             fprintf (FP, "   closepath 1 setgray fill\n");
             DumpRGBColorLine(FP, color_index, 3, TRUE);
          }
+         else
+         {
+            GrayCheck (fill);
+            fprintf (FP, "   %s setgray\n", GrayStr(fill));
+         }
          DumpRCBoxPSPath (FP, ltx, lty, rbx, rby, r, "   ", "      ");
-         if (colorDump)
+         if (colorDump || !useGray)
          {
             if (preDumpSetup) PSUseColorPattern();
             fprintf (FP, "   closepath eoclip newpath\n");
@@ -318,24 +321,108 @@ void DumpRCBoxObj (FP, ObjPtr)
          fprintf (FP, "grestore\n");
          break;
    }
-
-   if (pen == NONEPAT) { fprintf (FP, "\n"); return; }
-
+   if (pen == NONEPAT) {
+      if (ObjPtr->ctm != NULL) fprintf(FP, "grestore\n");
+      fprintf (FP, "\n");
+      return;
+   }
    fprintf (FP, "gsave\n");
 
-   if (colorDump && pen > BACKPAT)
+   if ((colorDump || !useGray) && pen > BACKPAT)
    {
       DumpRCBoxPath (FP, ObjPtr, ltx, lty, rbx, rby, r, width, BACKPAT, 0);
-      if (colorDump)
-         DumpRGBColorLine(FP, color_index, 3, TRUE);
+      DumpRGBColorLine(FP, color_index, 3, TRUE);
    }
    DumpRCBoxPath (FP, ObjPtr, ltx, lty, rbx, rby, r, width, pen, dash);
 
    fprintf (FP, "grestore\n");
-
+   if (ObjPtr->ctm != NULL) fprintf(FP, "grestore\n");
    fprintf (FP, "\n");
 }
 
+int NeedsToCacheRCBoxObj (ObjPtr)
+   struct ObjRec	* ObjPtr;
+{
+   return (ObjPtr->ctm != NULL);
+}
+
+static
+void MakeCachedRCBox(ObjPtr)
+   struct ObjRec *ObjPtr;
+{
+   struct RCBoxRec *rcbox_ptr=ObjPtr->detail.rcb;
+   XPoint *sv=NULL;
+   IntPoint *pv;
+   char *smooth=NULL;
+   struct BBRec obbox;
+   int i, w, h, sn;
+   int r=rcbox_ptr->radius, num_vs;
+
+   if (ObjPtr->ctm == NULL) return;
+
+   obbox.ltx = ObjPtr->orig_obbox.ltx - ObjPtr->x;
+   obbox.lty = ObjPtr->orig_obbox.lty - ObjPtr->y;
+   obbox.rbx = ObjPtr->orig_obbox.rbx - ObjPtr->x;
+   obbox.rby = ObjPtr->orig_obbox.rby - ObjPtr->y;
+   w = obbox.rbx - obbox.ltx;
+   h = obbox.rby - obbox.lty;
+   num_vs = (w < (r<<1) || h < (r<<1)) ? 5 : 13;
+
+   if (rcbox_ptr->rotated_vlist != NULL) free(rcbox_ptr->rotated_vlist);
+   rcbox_ptr->rotated_n = 0;
+   rcbox_ptr->rotated_vlist = (XPoint*)malloc((num_vs+1)*sizeof(XPoint));
+   pv = (IntPoint*)malloc((num_vs+1)*sizeof(IntPoint));
+   if (rcbox_ptr->rotated_vlist == NULL || pv == NULL) FailAllocMessage();
+   if (num_vs == 13) {
+      smooth = (char*)malloc((num_vs+1)*sizeof(char));
+      if (smooth == NULL) FailAllocMessage();
+   }
+   if (num_vs == 5) {
+      pv[0].x = obbox.ltx; pv[0].y = obbox.lty;
+      pv[1].x = obbox.rbx; pv[1].y = obbox.lty;
+      pv[2].x = obbox.rbx; pv[2].y = obbox.rby;
+      pv[3].x = obbox.ltx; pv[3].y = obbox.rby;
+      pv[4].x = pv[0].x; pv[4].y = pv[0].y;
+   } else {
+      pv[0].x  = obbox.ltx;   pv[0].y  = obbox.lty;
+      pv[1].x  = obbox.ltx+r; pv[1].y  = obbox.lty;
+      pv[2].x  = obbox.rbx-r; pv[2].y  = obbox.lty;
+      pv[3].x  = obbox.rbx;   pv[3].y  = obbox.lty;
+      pv[4].x  = obbox.rbx;   pv[4].y  = obbox.lty+r;
+      pv[5].x  = obbox.rbx;   pv[5].y  = obbox.rby-r;
+      pv[6].x  = obbox.rbx;   pv[6].y  = obbox.rby;
+      pv[7].x  = obbox.rbx-r; pv[7].y  = obbox.rby;
+      pv[8].x  = obbox.ltx+r; pv[8].y  = obbox.rby;
+      pv[9].x  = obbox.ltx;   pv[9].y  = obbox.rby;
+      pv[10].x = obbox.ltx;   pv[10].y = obbox.rby-r;
+      pv[11].x = obbox.ltx;   pv[11].y = obbox.lty+r;
+      pv[12].x = pv[0].x; pv[12].y = pv[0].y;
+      for (i=0; i < num_vs; i++) smooth[i] = FALSE;
+      smooth[0] = smooth[3] = smooth[6] = smooth[9] = smooth[12] = TRUE;
+   }
+   for (i=0; i < num_vs; i++) {
+      int x, y;
+
+      TransformPointThroughCTM(pv[i].x, pv[i].y, ObjPtr->ctm, &x, &y);
+      pv[i].x = x + ObjPtr->x;
+      pv[i].y = y + ObjPtr->y;
+      rcbox_ptr->rotated_vlist[i].x = (short)OFFSET_X(pv[i].x);
+      rcbox_ptr->rotated_vlist[i].y = (short)OFFSET_Y(pv[i].y);
+   }
+   if (num_vs == 13) {
+      sv = MakeMultiSplinePolygonVertex(&sn, smooth, drawOrigX, drawOrigY,
+            num_vs, pv);
+      if (sv == NULL) FailAllocMessage();
+      free(rcbox_ptr->rotated_vlist);
+      rcbox_ptr->rotated_n = sn;
+      rcbox_ptr->rotated_vlist = sv;
+   } else {
+      rcbox_ptr->rotated_n = num_vs;
+   }
+   free(pv);
+   if (smooth != NULL) free(smooth);
+}
+
 void DrawRCBoxObj (win, XOff, YOff, ObjPtr)
    Window		win;
    int			XOff, YOff;
@@ -353,6 +440,9 @@ void DrawRCBoxObj (win, XOff, YOff, ObjPtr)
    radius = ZOOMED_SIZE(rcbox_ptr->radius);
    pixel = colorPixels[ObjPtr->color];
 
+   if (NeedsToCacheRCBoxObj(ObjPtr) && rcbox_ptr->rotated_vlist==NULL) {
+      MakeCachedRCBox(ObjPtr);
+   }
    if (fill == 0 && pen == 0) return;
 
    real_x_off = (zoomedIn ? XOff : (XOff>>zoomScale)<<zoomScale);
@@ -362,22 +452,24 @@ void DrawRCBoxObj (win, XOff, YOff, ObjPtr)
    rbx = ZOOMED_SIZE(ObjPtr->obbox.rbx - real_x_off);
    rby = ZOOMED_SIZE(ObjPtr->obbox.rby - real_y_off);
 
-   SetRCBoxVertex (ltx, lty, rbx, rby, radius);
+   SetRCBoxVertex(ltx, lty, rbx, rby, radius);
 
-   if (fill != 0)
-   {
-      values.foreground = (fill == 2) ? myBgPixel : pixel;
+   if (fill != 0) {
+      values.foreground = GetDrawingBgPixel(fill, pixel);
       values.function = GXcopy;
       values.fill_style = FillOpaqueStippled;
       values.stipple = patPixmap[fill];
-      XChangeGC (mainDisplay, drawGC,
+      XChangeGC(mainDisplay, drawGC,
             GCForeground | GCFunction | GCFillStyle | GCStipple, &values);
-      MyFillRCBox (win, drawGC, ltx, lty, rbx, rby, radius);
+      if (ObjPtr->ctm != NULL) {
+         XFillPolygon(mainDisplay, win, drawGC, rcbox_ptr->rotated_vlist,
+               rcbox_ptr->rotated_n, Convex, CoordModeOrigin);
+      } else {
+         MyFillRCBox(win, drawGC, ltx, lty, rbx, rby, radius);
+      }
    }
-
-   if (pen != 0)
-   {
-      values.foreground = (pen == 2) ? myBgPixel : pixel;
+   if (pen != 0) {
+      values.foreground = GetDrawingBgPixel(pen, pixel);
       values.function = GXcopy;
       values.fill_style = FillOpaqueStippled;
       values.stipple = patPixmap[pen];
@@ -389,18 +481,22 @@ void DrawRCBoxObj (win, XOff, YOff, ObjPtr)
       if (values.line_width <= 1) values.line_width = 0;
 #endif
 #endif
-      if (dash != 0)
-      {
-         XSetDashes (mainDisplay, drawGC, 0, dashList[dash],
+      if (dash != 0) {
+         XSetDashes(mainDisplay, drawGC, 0, dashList[dash],
                dashListLength[dash]);
          values.line_style = LineOnOffDash;
-      }
-      else
+      } else {
          values.line_style = LineSolid;
-      XChangeGC (mainDisplay, drawGC,
+      }
+      XChangeGC(mainDisplay, drawGC,
             GCForeground | GCFunction | GCFillStyle | GCStipple | GCLineWidth |
             GCLineStyle, &values);
-      MyRCBox (win, drawGC, ltx, lty, rbx, rby, radius);
+      if (ObjPtr->ctm != NULL) {
+         XDrawLines(mainDisplay, win, drawGC, rcbox_ptr->rotated_vlist,
+               rcbox_ptr->rotated_n, CoordModeOrigin);
+      } else {
+         MyRCBox(win, drawGC, ltx, lty, rbx, rby, radius);
+      }
    }
 }
 
@@ -412,14 +508,23 @@ void CreateRCBoxObj (X1, Y1, X2, Y2)
    struct ObjRec	* obj_ptr;
    int			width, w, ltx, lty, rbx, rby;
 
-   rcbox_ptr = (struct RCBoxRec *) calloc (1, sizeof(struct RCBoxRec));
+   rcbox_ptr = (struct RCBoxRec *)malloc(sizeof(struct RCBoxRec));
+   if (rcbox_ptr == NULL) FailAllocMessage();
+   memset(rcbox_ptr, 0, sizeof(struct RCBoxRec));
    rcbox_ptr->fill = objFill;
    rcbox_ptr->width = width = curWidthOfLine[lineWidth];
+   UtilStrCpy(rcbox_ptr->width_spec, sizeof(rcbox_ptr->width_spec),
+         curWidthOfLineSpec[lineWidth]);
    rcbox_ptr->pen = penPat;
    rcbox_ptr->dash = curDash;
    rcbox_ptr->radius = rcbRadius;
 
-   obj_ptr = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
+   rcbox_ptr->rotated_n = 0;
+   rcbox_ptr->rotated_vlist = NULL;
+
+   obj_ptr = (struct ObjRec *)malloc(sizeof(struct ObjRec));
+   if (obj_ptr == NULL) FailAllocMessage();
+   memset(obj_ptr, 0, sizeof(struct ObjRec));
 
    if (X1 < X2)
       if (Y1 < Y2)
@@ -457,6 +562,8 @@ void CreateRCBoxObj (X1, Y1, X2, Y2)
    obj_ptr->locked = FALSE;
    obj_ptr->detail.rcb = rcbox_ptr;
    obj_ptr->fattr = obj_ptr->lattr = NULL;
+   obj_ptr->ctm = NULL;
+   obj_ptr->invisible = FALSE;
    AddObj (NULL, topObj, obj_ptr);
 }
 
@@ -496,10 +603,10 @@ void ContinueRCBox (OrigX, OrigY)
    PixelToMeasurementUnit(h_buf, 0);
    PixelToMeasurementUnit(x_buf, ABS_X(grid_x));
    PixelToMeasurementUnit(y_buf, ABS_Y(grid_y));
-   sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
-   StartShowMeasureCursor (grid_x, grid_y, buf, TRUE);
-   BeginIntervalRulers (grid_x, grid_y, grid_x, grid_y);
-   XGrabPointer (mainDisplay, drawWindow, FALSE,
+   sprintf(buf, "x=%s\ny=%s\nw=%s\nh=%s", w_buf, h_buf, x_buf, y_buf);
+   StartShowMeasureCursor(grid_x, grid_y, buf, TRUE);
+   BeginIntervalRulers(grid_x, grid_y, grid_x, grid_y);
+   XGrabPointer(mainDisplay, drawWindow, FALSE,
          PointerMotionMask | ButtonReleaseMask,
          GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
    
@@ -519,7 +626,7 @@ void ContinueRCBox (OrigX, OrigY)
          PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(saved_y-OrigY)));
          PixelToMeasurementUnit(x_buf, ABS_X(saved_x));
          PixelToMeasurementUnit(y_buf, ABS_Y(saved_y));
-         sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+         sprintf(buf, "x=%s\ny=%s\nw=%s\nh=%s", w_buf, h_buf, x_buf, y_buf);
          EndShowMeasureCursor (saved_x, saved_y, buf, TRUE);
          done = TRUE;
       }
@@ -556,7 +663,7 @@ void ContinueRCBox (OrigX, OrigY)
             PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(saved_y-OrigY)));
             PixelToMeasurementUnit(x_buf, ABS_X(saved_x));
             PixelToMeasurementUnit(y_buf, ABS_Y(saved_y));
-            sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+            sprintf(buf, "x=%s\ny=%s\nw=%s\nh=%s", w_buf, h_buf, x_buf, y_buf);
             ShowMeasureCursor (saved_x, saved_y, buf, TRUE);
             MyRCBox (drawWindow,drawGC,OrigX,OrigY,saved_x,saved_y,radius);
             saved_x = grid_x;
@@ -567,7 +674,7 @@ void ContinueRCBox (OrigX, OrigY)
             PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(saved_y-OrigY)));
             PixelToMeasurementUnit(x_buf, ABS_X(saved_x));
             PixelToMeasurementUnit(y_buf, ABS_Y(saved_y));
-            sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+            sprintf(buf, "x=%s\ny=%s\nw=%s\nh=%s", w_buf, h_buf, x_buf, y_buf);
             ShowMeasureCursor (saved_x, saved_y, buf, TRUE);
          }
          DrawIntervalRulers (OrigX, OrigY, grid_x, grid_y);
@@ -576,11 +683,12 @@ void ContinueRCBox (OrigX, OrigY)
       else if (input.type == KeyPress)
       {
          KeySym	key_sym;
-         char	s[80];
+         char s[80];
+         int has_ch;
 
-         XLookupString (&(input.xkey), s, 80-1, &key_sym, &c_stat);
+         has_ch = XLookupString(&(input.xkey), s, sizeof(s), &key_sym, &c_stat);
          TranslateKeys (s, &key_sym);
-         if (s[0] == '\033' && (key_sym & 0xff) == '\033')
+         if (CharIsESC(&(input.xkey), s, key_sym, &has_ch))
          {
             XUngrabPointer (mainDisplay, CurrentTime);
             MyRCBox (drawWindow,drawGC,OrigX,OrigY,saved_x,saved_y,radius);
@@ -589,7 +697,7 @@ void ContinueRCBox (OrigX, OrigY)
             PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(saved_y-OrigY)));
             PixelToMeasurementUnit(x_buf, ABS_X(saved_x));
             PixelToMeasurementUnit(y_buf, ABS_Y(saved_y));
-            sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+            sprintf(buf, "x=%s\ny=%s\nw=%s\nh=%s", w_buf, h_buf, x_buf, y_buf);
             EndShowMeasureCursor (saved_x, saved_y, buf, TRUE);
             abort = TRUE;
             done = TRUE;
@@ -624,34 +732,59 @@ void DrawRCBox (input)
    }
 }
 
-void SaveRCBoxObj (FP, ObjPtr)
-   FILE			* FP;
-   struct ObjRec	* ObjPtr;
+void SaveRCBoxObj(FP, ObjPtr)
+   FILE *FP;
+   struct ObjRec *ObjPtr;
 {
-   if (fprintf (FP, "rcbox('%s',", colorMenuItems[ObjPtr->color]) == EOF)
+   struct RCBoxRec *rcbox_ptr=ObjPtr->detail.rcb;
+
+   if (fprintf(FP, "rcbox('%s',", colorMenuItems[ObjPtr->color]) == EOF) {
       writeFileFailed = TRUE;
-   if (fprintf (FP, "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,",
-         ObjPtr->obbox.ltx,
-         ObjPtr->obbox.lty, ObjPtr->obbox.rbx, ObjPtr->obbox.rby,
-         ObjPtr->detail.rcb->fill, ObjPtr->detail.rcb->width,
-         ObjPtr->detail.rcb->pen, ObjPtr->detail.rcb->dash,
-         ObjPtr->detail.rcb->radius, ObjPtr->id, ObjPtr->rotation,
-         ObjPtr->locked) == EOF)
+   }
+   if (fprintf(FP,
+         "%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,'%s',",
+         ObjPtr->obbox.ltx, ObjPtr->obbox.lty, ObjPtr->obbox.rbx,
+         ObjPtr->obbox.rby, rcbox_ptr->fill, rcbox_ptr->width, rcbox_ptr->pen,
+         rcbox_ptr->dash, rcbox_ptr->radius, ObjPtr->id, ObjPtr->rotation,
+         ObjPtr->locked, ObjPtr->ctm!=NULL, ObjPtr->invisible,
+         rcbox_ptr->width_spec) == EOF) {
       writeFileFailed = TRUE;
-   SaveAttrs (FP, ObjPtr->lattr);
-   if (fprintf (FP, ")") == EOF) writeFileFailed = TRUE;
+   }
+   if (ObjPtr->ctm != NULL && fprintf(FP,
+         "[\n    %1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d,%1d],",
+         ObjPtr->x, ObjPtr->y,
+         ObjPtr->orig_obbox.ltx, ObjPtr->orig_obbox.lty,
+         ObjPtr->orig_obbox.rbx, ObjPtr->orig_obbox.rby,
+         ObjPtr->ctm->m[CTM_SX], ObjPtr->ctm->m[CTM_SIN],
+         ObjPtr->ctm->m[CTM_MSIN], ObjPtr->ctm->m[CTM_SY],
+         ObjPtr->ctm->m[CTM_TX], ObjPtr->ctm->m[CTM_TY]) == EOF) {
+      writeFileFailed = TRUE;
+   }
+   SaveAttrs(FP, ObjPtr->lattr);
+   if (fprintf(FP, ")") == EOF) writeFileFailed = TRUE;
 }
 
 #define GETVALUE(val,name) ScanValue("%d", &(val), name, "rcbox")
+#define GETSTRNG(val,name) ScanValue("%s", (val), name, "rcbox")
 
-void ReadRCBoxObj (Inbuf, ObjPtr)
+void ReadRCBoxObj (FP, Inbuf, ObjPtr)
+   FILE			* FP;
    char			* Inbuf;
    struct ObjRec	* * ObjPtr;
 {
    struct RCBoxRec	* rcbox_ptr;
-   char			color_str[40], * s, msg[MAXSTRING];
+   char			color_str[40], * s, msg[MAXSTRING], width_spec[40];
    int			ltx, lty, rbx, rby, fill, width, pen, dash, radius;
    int			rotation, new_alloc, id=0, w, locked=FALSE;
+   int			transformed=FALSE, invisible=FALSE;
+#ifdef _TGIF_WB
+ 
+   char                 aux[10000];
+ 
+   strcpy(aux,Inbuf);
+ 
+#endif /* _TGIF_WB */
+
 
    *ObjPtr = NULL;
 
@@ -661,6 +794,7 @@ void ReadRCBoxObj (Inbuf, ObjPtr)
    InitScan (s, ", \t\n");
 
    rotation = 0;
+   *width_spec = '\0';
    if (fileVersion <= 8)
    {
       sprintf (msg, "Invalid rcbox version (%1d).", fileVersion);
@@ -705,7 +839,7 @@ void ReadRCBoxObj (Inbuf, ObjPtr)
       }
       if (id >= objId) objId = id+1;
    }
-   else
+   else if (fileVersion <= 32)
    {
       if (GETVALUE (ltx,      "ltx") == INVALID ||
           GETVALUE (lty,      "lty") == INVALID ||
@@ -724,6 +858,29 @@ void ReadRCBoxObj (Inbuf, ObjPtr)
       }
       if (id >= objId) objId = id+1;
    }
+   else
+   {
+      if (GETVALUE (ltx,         "ltx") == INVALID ||
+          GETVALUE (lty,         "lty") == INVALID ||
+          GETVALUE (rbx,         "rbx") == INVALID ||
+          GETVALUE (rby,         "rby") == INVALID ||
+          GETVALUE (fill,        "fill") == INVALID ||
+          GETVALUE (width,       "width") == INVALID ||
+          GETVALUE (pen,         "pen") == INVALID ||
+          GETVALUE (dash,        "dash") == INVALID ||
+          GETVALUE (radius,      "radius") == INVALID ||
+          GETVALUE (id,          "id") == INVALID ||
+          GETVALUE (rotation,    "rotation") == INVALID ||
+          GETVALUE (locked,      "locked") == INVALID ||
+          GETVALUE (transformed, "transformed") == INVALID ||
+          GETVALUE (invisible,   "invisible") == INVALID ||
+          GETSTRNG (width_spec,  "width_spec") == INVALID)
+      {
+         return;
+      }
+      if (id >= objId) objId = id+1;
+      UtilRemoveQuotes(width_spec);
+   }
 
    if (ltx > rbx || lty > rby)
    {
@@ -735,20 +892,32 @@ void ReadRCBoxObj (Inbuf, ObjPtr)
    }
 
    if (fileVersion <= 16 && width <= 6) width = origWidthOfLine[width];
+   if (fileVersion <= 32) {
+      sprintf(width_spec, "%1d", width);
+   }
    fill = UpgradePenFill (fill);
    pen = UpgradePenFill (pen);
 
-   *ObjPtr = (struct ObjRec *) calloc (1, sizeof(struct ObjRec));
-   rcbox_ptr = (struct RCBoxRec *) calloc (1, sizeof(struct RCBoxRec));
+   *ObjPtr = (struct ObjRec *)malloc(sizeof(struct ObjRec));
+   if (*ObjPtr == NULL) FailAllocMessage();
+   memset(*ObjPtr, 0, sizeof(struct ObjRec));
+   rcbox_ptr = (struct RCBoxRec *)malloc(sizeof(struct RCBoxRec));
+   if (rcbox_ptr == NULL) FailAllocMessage();
+   memset(rcbox_ptr, 0, sizeof(struct RCBoxRec));
 
    rcbox_ptr->fill = fill;
    rcbox_ptr->width = width;
+   UtilStrCpy(rcbox_ptr->width_spec, sizeof(rcbox_ptr->width_spec), width_spec);
    rcbox_ptr->pen = pen;
    rcbox_ptr->dash = dash;
    rcbox_ptr->radius = radius;
+
+   rcbox_ptr->rotated_n = 0;
+   rcbox_ptr->rotated_vlist = NULL;
+
    (*ObjPtr)->x = ltx;
    (*ObjPtr)->y = lty;
-   (*ObjPtr)->color = QuickFindColorIndex (color_str, &new_alloc, TRUE);
+   (*ObjPtr)->color = QuickFindColorIndex(*ObjPtr, color_str, &new_alloc, TRUE);
    (*ObjPtr)->dirty = FALSE;
    (*ObjPtr)->id = id;
    (*ObjPtr)->rotation = rotation;
@@ -764,11 +933,61 @@ void ReadRCBoxObj (Inbuf, ObjPtr)
    (*ObjPtr)->bbox.rbx = rbx + w;
    (*ObjPtr)->bbox.rby = rby + w;
    (*ObjPtr)->detail.rcb = rcbox_ptr;
+   (*ObjPtr)->ctm = NULL;
+   (*ObjPtr)->invisible = invisible;
+   if (fileVersion >= 33 && transformed)
+   {
+      int real_x=0, real_y=0;
+      struct BBRec orig_obbox;
+      char inbuf[MAXSTRING+1];
+      struct XfrmMtrxRec *ctm;
+
+#ifdef _TGIF_WB
+ 
+      s = FindChar ((int)'[', aux);
+      s++;
+ 
+#else /* ~_TGIF_WB */
+ 
+      fgets(inbuf, MAXSTRING, FP);
+ 
+#endif /* _TGIF_WB */
+
+      scanLineNum++;
+      InitScan(inbuf, "\t\n, ");
+
+      ctm = (struct XfrmMtrxRec *)malloc(sizeof(struct XfrmMtrxRec));
+      if (ctm == NULL) FailAllocMessage();
+      if (GETVALUE(real_x,           "real_x") == INVALID ||
+          GETVALUE(real_y,           "real_y") == INVALID ||
+          GETVALUE(orig_obbox.ltx,   "orig_obbox.ltx") == INVALID ||
+          GETVALUE(orig_obbox.lty,   "orig_obbox.lty") == INVALID ||
+          GETVALUE(orig_obbox.rbx,   "orig_obbox.rbx") == INVALID ||
+          GETVALUE(orig_obbox.rby,   "orig_obbox.rby") == INVALID ||
+          GETVALUE(ctm->m[CTM_SX],   "CTM_SX") == INVALID ||
+          GETVALUE(ctm->m[CTM_SIN],  "CTM_SIN") == INVALID ||
+          GETVALUE(ctm->m[CTM_MSIN], "CTM_MSIN") == INVALID ||
+          GETVALUE(ctm->m[CTM_SY],   "CTM_SY") == INVALID ||
+          GETVALUE(ctm->m[CTM_TX],   "CTM_TX") == INVALID ||
+          GETVALUE(ctm->m[CTM_TY],   "CTM_TY") == INVALID) {
+         return;
+      }
+      (*ObjPtr)->ctm = ctm;
+      if (ctm != NULL) {
+         memcpy(&(*ObjPtr)->orig_obbox, &orig_obbox, sizeof(struct BBRec));
+         (*ObjPtr)->x = real_x;
+         (*ObjPtr)->y = real_y;
+         GetTransformedOBBoxOffsetVs(*ObjPtr, (*ObjPtr)->rotated_obbox);
+      }
+   }
 }
 
 void FreeRCBoxObj (ObjPtr)
    struct ObjRec	* ObjPtr;
 {
-   cfree (ObjPtr->detail.rcb);
-   cfree (ObjPtr);
+   if (ObjPtr->detail.rcb->rotated_vlist != NULL) {
+      free(ObjPtr->detail.rcb->rotated_vlist);
+   }
+   free(ObjPtr->detail.rcb);
+   free(ObjPtr);
 }
