@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,7 +29,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/rect.c,v 2.39 1995/05/15 01:22:54 william Exp $";
+      "@(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/rect.c,v 3.1 1996/10/30 22:51:48 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -39,6 +39,10 @@ static char RCSid[] =
 #include "types.h"
 
 #include "arc.e"
+#include "color.e"
+#include "dialog.e"
+#include "drawing.e"
+#include "msg.e"
 #include "poly.e"
 #ifndef _NO_EXTERN
 #include "rect.e"
@@ -46,6 +50,232 @@ static char RCSid[] =
 #include "setup.e"
 #include "spline.e"
 
+void SetRotateVs (vs, ltx, lty, rbx, rby)
+   XPoint	* vs; /* array of 5 points */
+   int		ltx, lty, rbx, rby;
+{
+   vs[0].x = vs[4].x = ltx; vs[0].y = vs[4].y = lty;
+   vs[1].x = rbx; vs[1].y = lty;
+   vs[2].x = rbx; vs[2].y = rby;
+   vs[3].x = ltx; vs[3].y = rby;
+}
+
+void SetBBRec (bbox, ltx, lty, rbx, rby)
+   struct BBRec	* bbox;
+   int		ltx, lty, rbx, rby;
+{
+   bbox->ltx = ltx; bbox->lty = lty; bbox->rbx = rbx; bbox->rby = rby;
+}
+
+void ConcatCTM (ctm, orig_ctm, new_ctm)
+   struct XfrmMtrxRec	* ctm, * orig_ctm, * new_ctm;
+   /* Note: CTM_SX, CTM_SIN, CTM_MSIN, and CTM_SY are scaled 1000 times */
+   /*       while CTM_TX and CTM_TY are not scaled */
+{
+   new_ctm->m[0] = round(((float)(ctm->m[0]*orig_ctm->m[0] +
+         ctm->m[1]*orig_ctm->m[2]))/1000.0);
+   new_ctm->m[1] = round(((float)(ctm->m[0]*orig_ctm->m[1] +
+         ctm->m[1]*orig_ctm->m[3]))/1000.0);
+   new_ctm->m[2] = round(((float)(ctm->m[2]*orig_ctm->m[0] +
+         ctm->m[3]*orig_ctm->m[2]))/1000.0);
+   new_ctm->m[3] = round(((float)(ctm->m[2]*orig_ctm->m[1] +
+         ctm->m[3]*orig_ctm->m[3]))/1000.0);
+   new_ctm->m[4] = round(((float)(ctm->m[4]*orig_ctm->m[0] +
+         ctm->m[5]*orig_ctm->m[2]))/1000.0) + orig_ctm->m[4];
+   new_ctm->m[5] = round(((float)(ctm->m[4]*orig_ctm->m[1] +
+         ctm->m[5]*orig_ctm->m[3]))/1000.0) + orig_ctm->m[5];
+}
+
+void TransformPointThroughCTM (X, Y, ctm, NewX, NewY)
+   int			X, Y, * NewX, * NewY;
+   struct XfrmMtrxRec	* ctm;
+{
+   *NewX = round(((float)(X*ctm->m[0] + Y*ctm->m[2]))/1000.0) + ctm->m[CTM_TX];
+   *NewY = round(((float)(X*ctm->m[1] + Y*ctm->m[3]))/1000.0) + ctm->m[CTM_TY];
+}
+
+void ReverseTransformPointThroughCTM (X, Y, ctm, NewX, NewY)
+   int			X, Y, * NewX, * NewY;
+   struct XfrmMtrxRec	* ctm;
+   /*
+    *           [ a b 0 ]
+    * Let ctm = [ c d 0 ]
+    *           [ e f 1 ]
+    *
+    *          d(X-e) - c(Y-f)
+    * *NewX = -----------------
+    *              ad - bc
+    *
+    *          b(X-e) - a(Y-f)
+    * *NewY = -----------------
+    *              bc - ad
+    */
+{
+   int		ad_bc=ctm->m[0]*ctm->m[3]-ctm->m[1]*ctm->m[2];
+   int		term_1=X-ctm->m[CTM_TX], term_2=Y-ctm->m[CTM_TY];
+   float	x, y;
+
+   x = ((float)1000.0) *
+         ((float)(ctm->m[3]*term_1 - ctm->m[2]*term_2)) / ((float)ad_bc);
+   y = ((float)1000.0) *
+         ((float)(ctm->m[0]*term_2 - ctm->m[1]*term_1)) / ((float)(ad_bc));
+   *NewX = round(x);
+   *NewY = round(y);
+}
+
+void TransformObjectV(ObjPtr, VIn, VOut)
+   struct ObjRec *ObjPtr;
+   IntPoint *VIn, *VOut;
+{
+   int x, y;
+
+   TransformPointThroughCTM(VIn->x-ObjPtr->x, VIn->y-ObjPtr->y, ObjPtr->ctm,
+         &x, &y);
+   VOut->x = x + ObjPtr->x;
+   VOut->y = y + ObjPtr->y;
+}
+
+void ReversedTransformObjectV(ObjPtr, VIn, VOut)
+   struct ObjRec *ObjPtr;
+   IntPoint *VIn, *VOut;
+{
+   int x, y;
+
+   ReverseTransformPointThroughCTM(VIn->x-ObjPtr->x, VIn->y-ObjPtr->y,
+         ObjPtr->ctm, &x, &y);
+   VOut->x = x + ObjPtr->x;
+   VOut->y = y + ObjPtr->y;
+}
+
+void TransformOffsetBBoxThroughCTM (bbox, ctm, vs)
+   struct BBRec		* bbox;
+   struct XfrmMtrxRec	* ctm;
+   IntPoint		* vs; /* array of 5 points */
+{
+   int	x, y;
+
+   TransformPointThroughCTM (bbox->ltx, bbox->lty, ctm, &x, &y);
+   vs[0].x = vs[4].x = x; vs[0].y = vs[4].y = y;
+   TransformPointThroughCTM (bbox->rbx, bbox->lty, ctm, &x, &y);
+   vs[1].x = x; vs[1].y = y;
+   TransformPointThroughCTM (bbox->rbx, bbox->rby, ctm, &x, &y);
+   vs[2].x = x; vs[2].y = y;
+   TransformPointThroughCTM (bbox->ltx, bbox->rby, ctm, &x, &y);
+   vs[3].x = x; vs[3].y = y;
+}
+
+void GetTransformedOBBoxOffsetVs (ObjPtr, Vs)
+   struct ObjRec	* ObjPtr;
+   XPoint		* Vs; /* array of 5 points */
+{
+   int		x, y;
+   struct BBRec	obbox;
+   IntPoint	vs[5];
+
+   if (ObjPtr->ctm == NULL) return;
+
+   obbox.ltx = ObjPtr->orig_obbox.ltx - ObjPtr->x;
+   obbox.lty = ObjPtr->orig_obbox.lty - ObjPtr->y;
+   obbox.rbx = ObjPtr->orig_obbox.rbx - ObjPtr->x;
+   obbox.rby = ObjPtr->orig_obbox.rby - ObjPtr->y;
+   TransformOffsetBBoxThroughCTM (&obbox, ObjPtr->ctm, vs);
+   x = OFFSET_X(vs[0].x + ObjPtr->x);
+   y = OFFSET_Y(vs[0].y + ObjPtr->y);
+   Vs[0].x = Vs[4].x = x; Vs[0].y = Vs[4].y = y;
+   x = OFFSET_X(vs[1].x + ObjPtr->x);
+   y = OFFSET_Y(vs[1].y + ObjPtr->y);
+   Vs[1].x = x; Vs[1].y = y;
+   x = OFFSET_X(vs[2].x + ObjPtr->x);
+   y = OFFSET_Y(vs[2].y + ObjPtr->y);
+   Vs[2].x = x; Vs[2].y = y;
+   x = OFFSET_X(vs[3].x + ObjPtr->x);
+   y = OFFSET_Y(vs[3].y + ObjPtr->y);
+   Vs[3].x = x; Vs[3].y = y;
+}
+
+void GetTransformedOBBoxAbsVs (ObjPtr, Vs)
+   struct ObjRec	* ObjPtr;
+   IntPoint		* Vs; /* array of 5 points */
+{
+   int		x, y;
+   struct BBRec	obbox;
+   IntPoint	vs[5];
+
+   if (ObjPtr->ctm == NULL) return;
+
+   obbox.ltx = ObjPtr->orig_obbox.ltx - ObjPtr->x;
+   obbox.lty = ObjPtr->orig_obbox.lty - ObjPtr->y;
+   obbox.rbx = ObjPtr->orig_obbox.rbx - ObjPtr->x;
+   obbox.rby = ObjPtr->orig_obbox.rby - ObjPtr->y;
+   TransformOffsetBBoxThroughCTM (&obbox, ObjPtr->ctm, vs);
+   x = vs[0].x + ObjPtr->x; y = vs[0].y + ObjPtr->y;
+   Vs[0].x = Vs[4].x = x; Vs[0].y = Vs[4].y = y;
+   x = vs[1].x + ObjPtr->x; y = vs[1].y + ObjPtr->y;
+   Vs[1].x = x; Vs[1].y = y;
+   x = vs[2].x + ObjPtr->x; y = vs[2].y + ObjPtr->y;
+   Vs[2].x = x; Vs[2].y = y;
+   x = vs[3].x + ObjPtr->x; y = vs[3].y + ObjPtr->y;
+   Vs[3].x = x; Vs[3].y = y;
+}
+
+void SetCTM(ObjPtr, ctm)
+   struct ObjRec *ObjPtr;
+   struct XfrmMtrxRec *ctm;
+   /* ObjPtr->ctm must be NULL */
+{
+   IntPoint abs_obj_obbox_vs[5];
+   struct XfrmMtrxRec new_ctm;
+   int ltx, lty, rbx, rby;
+
+   if (ObjPtr->ctm != NULL) {
+      sprintf(gszMsgBox, "Error: SetCTM() is called with ObjPtr->ctm != NULL");
+      MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+      free(ObjPtr->ctm);
+   }
+   memcpy(&ObjPtr->orig_obbox, &ObjPtr->obbox, sizeof(struct BBRec));
+   if (ObjPtr->type == OBJ_TEXT) {
+      memcpy(&ObjPtr->detail.t->orig_bbox, &ObjPtr->bbox,
+            sizeof(struct BBRec));
+   }
+   ObjPtr->ctm = (struct XfrmMtrxRec *)malloc(sizeof(struct XfrmMtrxRec));
+   if (ObjPtr->ctm == NULL) FailAllocMessage();
+   ObjPtr->ctm->m[CTM_SX] = ObjPtr->ctm->m[CTM_SY] = 1000;
+   ObjPtr->ctm->m[CTM_SIN] = ObjPtr->ctm->m[CTM_MSIN] = 0;
+   ObjPtr->ctm->m[CTM_TX] = ObjPtr->ctm->m[CTM_TY] = 0;
+   ConcatCTM(ObjPtr->ctm, ctm, &new_ctm);
+   memcpy(ObjPtr->ctm, &new_ctm, sizeof(struct XfrmMtrxRec));
+
+   GetTransformedOBBoxAbsVs(ObjPtr, abs_obj_obbox_vs);
+
+   ltx = min(min(abs_obj_obbox_vs[0].x,abs_obj_obbox_vs[1].x),
+         min(abs_obj_obbox_vs[2].x,abs_obj_obbox_vs[3].x));
+   rbx = max(max(abs_obj_obbox_vs[0].x,abs_obj_obbox_vs[1].x),
+         max(abs_obj_obbox_vs[2].x,abs_obj_obbox_vs[3].x));
+   lty = min(min(abs_obj_obbox_vs[0].y,abs_obj_obbox_vs[1].y),
+         min(abs_obj_obbox_vs[2].y,abs_obj_obbox_vs[3].y));
+   rby = max(max(abs_obj_obbox_vs[0].y,abs_obj_obbox_vs[1].y),
+         max(abs_obj_obbox_vs[2].y,abs_obj_obbox_vs[3].y));
+
+   ObjPtr->obbox.ltx = ltx; ObjPtr->obbox.lty = lty;
+   ObjPtr->obbox.rbx = rbx; ObjPtr->obbox.rby = rby;
+   GetTransformedOBBoxOffsetVs(ObjPtr, ObjPtr->rotated_obbox);
+}
+
+int BBoxIntersect (Rect1, Rect2)
+   struct BBRec	Rect1, Rect2;
+{
+   if (Rect1.ltx < Rect2.ltx)
+      if (Rect1.lty < Rect2.lty)
+         return (Rect1.rbx >= Rect2.ltx && Rect1.rby >= Rect2.lty);
+      else
+         return (Rect1.rbx >= Rect2.ltx && Rect1.lty <= Rect2.rby);
+   else
+      if (Rect1.lty < Rect2.lty)
+         return (Rect1.ltx <= Rect2.rbx && Rect1.rby >= Rect2.lty);
+      else
+         return (Rect1.ltx <= Rect2.rbx && Rect1.lty <= Rect2.rby);
+}
+
 int IntersectRect(BBox1, BBox2, BBox3)
    struct BBRec	BBox1, BBox2, *BBox3;
    /* returns the intersection of BBox1 and BBox2 in BBox3 */
@@ -69,21 +299,6 @@ int Inside (BBox1, BBox2)
          BBox1.rbx <= BBox2.rbx && BBox1.rby <= BBox2.rby);
 }
 
-int BBoxIntersect (Rect1, Rect2)
-   struct BBRec	Rect1, Rect2;
-{
-   if (Rect1.ltx < Rect2.ltx)
-      if (Rect1.lty < Rect2.lty)
-         return (Rect1.rbx >= Rect2.ltx && Rect1.rby >= Rect2.lty);
-      else
-         return (Rect1.rbx >= Rect2.ltx && Rect1.lty <= Rect2.rby);
-   else
-      if (Rect1.lty < Rect2.lty)
-         return (Rect1.ltx <= Rect2.rbx && Rect1.rby >= Rect2.lty);
-      else
-         return (Rect1.ltx <= Rect2.rbx && Rect1.lty <= Rect2.rby);
-}
-
 int PointInBBox (X, Y, Rect)
    int	X, Y;
    struct BBRec	Rect;
@@ -202,6 +417,21 @@ int PointOnPoly (X, Y, NumPts, V, W)
    return (FALSE);
 }
 
+int FindGoodText (XOff, YOff, TextObj)
+   int			XOff, YOff;
+   struct ObjRec	* TextObj;
+   /* XOff and YOff are screen offsets */
+{
+   if (colorLayers && TextObj->tmp_parent == NULL &&
+         !ObjInVisibleLayer(TextObj)) {
+      return FALSE;
+   }
+   if (TextObj->ctm == NULL)
+      return TRUE;
+   else
+      return (PointInPolygon (XOff, YOff, 5, TextObj->rotated_obbox));
+}
+
 int FindGoodXBm (XOff, YOff, XBmObj)
    int			XOff, YOff;
    struct ObjRec	* XBmObj;
@@ -209,11 +439,18 @@ int FindGoodXBm (XOff, YOff, XBmObj)
 {
    struct BBRec	bbox;
 
+   if (colorLayers && XBmObj->tmp_parent == NULL &&
+         !ObjInVisibleLayer(XBmObj)) {
+      return FALSE;
+   }
    bbox.ltx = OFFSET_X(XBmObj->obbox.ltx)-3;
    bbox.lty = OFFSET_Y(XBmObj->obbox.lty)-3;
    bbox.rbx = OFFSET_X(XBmObj->obbox.rbx)+3;
    bbox.rby = OFFSET_Y(XBmObj->obbox.rby)+3;
-   return (PointInBBox (XOff, YOff, bbox));
+   if (XBmObj->ctm == NULL)
+      return (PointInBBox (XOff, YOff, bbox));
+   else
+      return (PointInPolygon (XOff, YOff, 5, XBmObj->rotated_obbox));
 }
 
 int FindGoodXPm (XOff, YOff, XPmObj)
@@ -223,11 +460,18 @@ int FindGoodXPm (XOff, YOff, XPmObj)
 {
    struct BBRec	bbox;
 
+   if (colorLayers && XPmObj->tmp_parent == NULL &&
+         !ObjInVisibleLayer(XPmObj)) {
+      return FALSE;
+   }
    bbox.ltx = OFFSET_X(XPmObj->obbox.ltx)-3;
    bbox.lty = OFFSET_Y(XPmObj->obbox.lty)-3;
    bbox.rbx = OFFSET_X(XPmObj->obbox.rbx)+3;
    bbox.rby = OFFSET_Y(XPmObj->obbox.rby)+3;
-   return (PointInBBox (XOff, YOff, bbox));
+   if (XPmObj->ctm == NULL)
+      return (PointInBBox (XOff, YOff, bbox));
+   else
+      return (PointInPolygon (XOff, YOff, 5, XPmObj->rotated_obbox));
 }
 
 int FindGoodBox (XOff, YOff, BoxObj)
@@ -238,20 +482,35 @@ int FindGoodBox (XOff, YOff, BoxObj)
    struct BBRec	bbox;
    int		w;
 
+   if (colorLayers && BoxObj->tmp_parent == NULL &&
+         !ObjInVisibleLayer(BoxObj)) {
+      return FALSE;
+   }
    bbox.ltx = OFFSET_X(BoxObj->obbox.ltx)-3;
    bbox.lty = OFFSET_Y(BoxObj->obbox.lty)-3;
    bbox.rbx = OFFSET_X(BoxObj->obbox.rbx)+3;
    bbox.rby = OFFSET_Y(BoxObj->obbox.rby)+3;
    if (!PointInBBox (XOff, YOff, bbox)) return (FALSE);
 
-   if (BoxObj->detail.b->fill != NONEPAT) return (TRUE);
+   if (BoxObj->ctm == NULL)
+   {
+      if (BoxObj->detail.b->fill != NONEPAT) return (TRUE);
 
-   w = HALF_W(BoxObj->detail.b->width);
-   bbox.ltx = OFFSET_X(BoxObj->obbox.ltx+w)+3;
-   bbox.lty = OFFSET_Y(BoxObj->obbox.lty+w)+3;
-   bbox.rbx = OFFSET_X(BoxObj->obbox.rbx-w)-3;
-   bbox.rby = OFFSET_Y(BoxObj->obbox.rby-w)-3;
-   return (!PointInBBox (XOff, YOff, bbox));
+      w = HALF_W(BoxObj->detail.b->width);
+      bbox.ltx = OFFSET_X(BoxObj->obbox.ltx+w)+3;
+      bbox.lty = OFFSET_Y(BoxObj->obbox.lty+w)+3;
+      bbox.rbx = OFFSET_X(BoxObj->obbox.rbx-w)-3;
+      bbox.rby = OFFSET_Y(BoxObj->obbox.rby-w)-3;
+      return (!PointInBBox (XOff, YOff, bbox));
+   }
+   else
+   {
+      if (BoxObj->detail.b->fill != NONEPAT)
+         if (PointInPolygon (XOff, YOff, 5, BoxObj->rotated_obbox))
+            return (TRUE);
+      return (PointOnPoly (XOff, YOff, 5, BoxObj->rotated_obbox,
+            ZOOMED_HALF_W(BoxObj->detail.b->width)));
+   }
 }
 
 int FindGoodRCBox (XOff, YOff, RCBoxObj)
@@ -259,33 +518,52 @@ int FindGoodRCBox (XOff, YOff, RCBoxObj)
    struct ObjRec	* RCBoxObj;
    /* XOff and YOff are screen offsets */
 {
-   register struct BBRec	* obbox;
-   register struct BBRec	bbox;
-   int				w, r;
+   struct RCBoxRec *rcbox_ptr=RCBoxObj->detail.rcb;
+   int fill=rcbox_ptr->fill;
 
-   obbox = &(RCBoxObj->obbox);
+   if (colorLayers && RCBoxObj->tmp_parent == NULL &&
+         !ObjInVisibleLayer(RCBoxObj)) {
+      return FALSE;
+   }
+   if (RCBoxObj->ctm == NULL) {
+      register struct BBRec *obbox;
+      register struct BBRec bbox;
+      int w, r;
 
-   bbox.ltx = OFFSET_X(obbox->ltx)-3;
-   bbox.lty = OFFSET_Y(obbox->lty)-3;
-   bbox.rbx = OFFSET_X(obbox->rbx)+3;
-   bbox.rby = OFFSET_Y(obbox->rby)+3;
-   if (!PointInBBox (XOff, YOff, bbox)) return (FALSE);
+      obbox = &(RCBoxObj->obbox);
 
-   if (RCBoxObj->detail.b->fill != NONEPAT) return (TRUE);
+      bbox.ltx = OFFSET_X(obbox->ltx)-3;
+      bbox.lty = OFFSET_Y(obbox->lty)-3;
+      bbox.rbx = OFFSET_X(obbox->rbx)+3;
+      bbox.rby = OFFSET_Y(obbox->rby)+3;
+      if (!PointInBBox(XOff, YOff, bbox)) return (FALSE);
 
-   r = RCBoxObj->detail.rcb->radius;
-   w = HALF_W(RCBoxObj->detail.rcb->width);
+      if (fill != NONEPAT) return (TRUE);
 
-   bbox.ltx = OFFSET_X(obbox->ltx+w)+3; bbox.lty = OFFSET_Y(obbox->lty+r)+3;
-   bbox.rbx = OFFSET_X(obbox->rbx-w)-3; bbox.rby = OFFSET_Y(obbox->rby-r)-3;
+      r = rcbox_ptr->radius;
+      w = HALF_W(rcbox_ptr->width);
 
-   if (PointInBBox (XOff, YOff, bbox))
-      return (FALSE);
-   else
-   {
-      bbox.ltx = OFFSET_X(obbox->ltx+r)+3; bbox.lty = OFFSET_Y(obbox->lty+w)+3;
-      bbox.rbx = OFFSET_X(obbox->rbx-r)-3; bbox.rby = OFFSET_Y(obbox->rby-w)-3;
-      return (!PointInBBox (XOff, YOff, bbox));
+      bbox.ltx = OFFSET_X(obbox->ltx+w)+3; bbox.lty = OFFSET_Y(obbox->lty+r)+3;
+      bbox.rbx = OFFSET_X(obbox->rbx-w)-3; bbox.rby = OFFSET_Y(obbox->rby-r)-3;
+
+      if (PointInBBox(XOff, YOff, bbox)) {
+         return (FALSE);
+      } else {
+         bbox.ltx = OFFSET_X(obbox->ltx+r)+3;
+         bbox.lty = OFFSET_Y(obbox->lty+w)+3;
+         bbox.rbx = OFFSET_X(obbox->rbx-r)-3;
+         bbox.rby = OFFSET_Y(obbox->rby-w)-3;
+         return (!PointInBBox(XOff, YOff, bbox));
+      }
+   } else {
+      if (fill != NONEPAT) {
+         if (PointInPolygon(XOff, YOff, rcbox_ptr->rotated_n,
+               rcbox_ptr->rotated_vlist)) {
+            return TRUE;
+         }
+      }
+      return (PointOnPoly(XOff, YOff, rcbox_ptr->rotated_n,
+            rcbox_ptr->rotated_vlist, ZOOMED_HALF_W(rcbox_ptr->width)));
    }
 }
 
@@ -296,49 +574,66 @@ int FindGoodOval (XOff, YOff, OvalObj)
 {
    struct OvalRec	* oval_ptr=OvalObj->detail.o;
    int			w, ltx, lty, rbx, rby;
-   double		cx, cy, rx, ry, tmp_x, tmp_y;
-   double		x1 = 0.0, x2 = 0.0, y1 = 0.0, y2 = 0.0;
    int			fill=oval_ptr->fill;
 
+   if (colorLayers && OvalObj->tmp_parent == NULL &&
+         !ObjInVisibleLayer(OvalObj)) {
+      return FALSE;
+   }
    ltx = OFFSET_X(OvalObj->obbox.ltx); lty = OFFSET_Y(OvalObj->obbox.lty);
    rbx = OFFSET_X(OvalObj->obbox.rbx); rby = OFFSET_Y(OvalObj->obbox.rby);
 
    if (ltx==rbx && lty==rby) return (FALSE);
 
-   cx = (ltx + rbx) / 2; cy = (lty + rby) / 2;
-   rx = (rbx - ltx) / 2; ry = (rby - lty) / 2;
-
-   if (rx >= ry)
-   {  /* flat oval */
-      tmp_y = (double)sqrt(fabs((double)(ry*ry*(1 -
-            (((double)XOff)-cx)*(((double)XOff)-cx)/rx/rx))));
-      y1 = cy - tmp_y;
-      y2 = cy + tmp_y;
-   }
-   else
-   {  /* tall oval */
-      tmp_x = (double)sqrt(fabs((double)(rx*rx*(1 -
-            (((double)YOff)-cy)*(((double)YOff)-cy)/ry/ry))));
-      x1 = cx - tmp_x;
-      x2 = cx + tmp_x;
-   }
-
-   if (fill != NONEPAT)
+   if (OvalObj->ctm == NULL)
    {
+      double	cx, cy, rx, ry, tmp_x, tmp_y;
+      double	x1=0.0, x2=0.0, y1=0.0, y2=0.0;
+
+      cx = ((double)(ltx+rbx)/2.0); cy = ((double)(lty+rby)/2.0);
+      rx = ((double)(rbx-ltx)/2.0); ry = ((double)(rby-lty)/2.0);
+
       if (rx >= ry)
       {  /* flat oval */
-         if (YOff >= y1-4 && y2+4 >= YOff) return (TRUE);
+         tmp_y = (double)sqrt(fabs((double)(ry*ry*(1 -
+               (((double)XOff)-cx)*(((double)XOff)-cx)/rx/rx))));
+         y1 = cy - tmp_y;
+         y2 = cy + tmp_y;
       }
       else
       {  /* tall oval */
-         if (XOff >= x1-4 && x2+4 >= XOff) return (TRUE);
+         tmp_x = (double)sqrt(fabs((double)(rx*rx*(1 -
+               (((double)YOff)-cy)*(((double)YOff)-cy)/ry/ry))));
+         x1 = cx - tmp_x;
+         x2 = cx + tmp_x;
       }
+
+      if (fill != NONEPAT)
+      {
+         if (rx >= ry)
+         {  /* flat oval */
+            if (YOff >= y1-4 && y2+4 >= YOff) return (TRUE);
+         }
+         else
+         {  /* tall oval */
+            if (XOff >= x1-4 && x2+4 >= XOff) return (TRUE);
+         }
+      }
+      w = ZOOMED_HALF_W(oval_ptr->width)+4;
+      if (rx >= ry)
+         return ((fabs((double)(YOff-y1))<=w) || (fabs((double)(YOff-y2))<=w));
+      else
+         return ((fabs((double)(XOff-x1))<=w) || (fabs((double)(XOff-x2))<=w));
    }
-   w = ZOOMED_HALF_W(oval_ptr->width)+4;
-   if (rx >= ry)
-      return ((fabs((double)(YOff-y1))<=w) || (fabs((double)(YOff-y2))<=w));
    else
-      return ((fabs((double)(XOff-x1))<=w) || (fabs((double)(XOff-x2))<=w));
+   {
+      if (fill != NONEPAT)
+         if (PointInPolygon (XOff, YOff, oval_ptr->rotated_n,
+               oval_ptr->rotated_vlist))
+            return TRUE;
+      return (PointOnPoly (XOff, YOff, oval_ptr->rotated_n,
+            oval_ptr->rotated_vlist, ZOOMED_HALF_W(oval_ptr->width)));
+   }
 }
 
 int FindGoodPoly (XOff, YOff, PolyObj)
@@ -346,37 +641,65 @@ int FindGoodPoly (XOff, YOff, PolyObj)
    struct ObjRec	* PolyObj;
    /* XOff and YOff are screen offsets */
 {
-   int			n, dx, dy;
+   int			sn=0, dx, dy;
    double		len, sin, cos, aw, ah;
-   XPoint		* v, tmp_v[4];
+   XPoint		tmp_v[4], *sv=NULL;
    struct PolyRec	* poly_ptr = PolyObj->detail.p;
 
-   if (poly_ptr->fill != NONEPAT)
-   {
-      n = poly_ptr->sn;
-      v = poly_ptr->svlist;
-      v[n].x = v[0].x; v[n].y = v[0].y;
-      if (PointInPolygon (XOff, YOff, n+1, v)) return (TRUE);
+   if (colorLayers && PolyObj->tmp_parent == NULL &&
+         !ObjInVisibleLayer(PolyObj)) {
+      return FALSE;
    }
-   if (poly_ptr->style == LS_PLAIN)
+   if (poly_ptr->fill != NONEPAT) {
+      if (PolyObj->ctm == NULL) {
+         sn = poly_ptr->sn;
+         sv = poly_ptr->svlist;
+         sv[sn].x = sv[0].x; sv[sn].y = sv[0].y;
+         if (PointInPolygon(XOff, YOff, sn+1, sv)) return (TRUE);
+      } else {
+         sn = poly_ptr->rotated_n;
+         sv = poly_ptr->rotated_vlist;
+         sv[sn].x = sv[0].x; sv[sn].y = sv[0].y;
+         if (PointInPolygon(XOff, YOff, sn+1, sv)) return (TRUE);
+      }
+   }
+   if (poly_ptr->style == LS_PLAIN) {
+      if (PolyObj->ctm == NULL) {
+         if (PointOnPoly(XOff, YOff, poly_ptr->sn, poly_ptr->svlist,
+               ZOOMED_HALF_W(poly_ptr->width)))
+            return (TRUE);
+      } else {
+         if (PointOnPoly(XOff, YOff, poly_ptr->rotated_n,
+               poly_ptr->rotated_vlist, ZOOMED_HALF_W(poly_ptr->width)))
+            return (TRUE);
+      }
+   } else {
+      if (PolyObj->ctm == NULL) {
+         if (PointOnPoly(XOff, YOff, poly_ptr->asn, poly_ptr->asvlist,
+               ZOOMED_HALF_W(poly_ptr->width)))
+            return (TRUE);
+      } else {
+         if (PointOnPoly(XOff, YOff, poly_ptr->rotated_n,
+               poly_ptr->rotated_vlist, ZOOMED_HALF_W(poly_ptr->width)))
+            return (TRUE);
+      }
+   }
+   if (PolyObj->ctm == NULL)
    {
-      if (PointOnPoly (XOff, YOff, poly_ptr->sn, poly_ptr->svlist,
-            ZOOMED_HALF_W(poly_ptr->width)))
-         return (TRUE);
+      sv = poly_ptr->svlist;
+      sn = poly_ptr->sn;
    }
    else
    {
-      if (PointOnPoly (XOff, YOff, poly_ptr->asn, poly_ptr->asvlist,
-            ZOOMED_HALF_W(poly_ptr->width)))
-         return (TRUE);
+      sv = poly_ptr->rotated_vlist;
+      sn = poly_ptr->rotated_n;
    }
-   v = poly_ptr->vlist;
-   n = poly_ptr->n;
    aw = poly_ptr->aw;
    ah = poly_ptr->ah;
 
-   dx = v[1].x - v[0].x;
-   dy = v[1].y - v[0].y;
+   dx = (int)(sv[1].x - sv[0].x);
+   dy = (int)(sv[1].y - sv[0].y);
+
    if ((poly_ptr->style & LS_LEFT) && (dx != 0 || dy != 0))
    {
       len = (double)sqrt((double)(((double)dx)*((double)dx) +
@@ -384,17 +707,17 @@ int FindGoodPoly (XOff, YOff, PolyObj)
       sin = ((double)dy) / len;
       cos = ((double)dx) / len;
 
-      tmp_v[0].x = tmp_v[3].x = OFFSET_X(v[0].x);
-      tmp_v[0].y = tmp_v[3].y = OFFSET_Y(v[0].y);
-      tmp_v[1].x = OFFSET_X(round(v[0].x+aw*cos-ah*sin));
-      tmp_v[1].y = OFFSET_Y(round(v[0].y+aw*sin+ah*cos));
-      tmp_v[2].x = OFFSET_X(round(v[0].x+aw*cos+ah*sin));
-      tmp_v[2].y = OFFSET_Y(round(v[0].y+aw*sin-ah*cos));
+      tmp_v[0].x = tmp_v[3].x = OFFSET_X(sv[0].x);
+      tmp_v[0].y = tmp_v[3].y = OFFSET_Y(sv[0].y);
+      tmp_v[1].x = OFFSET_X(round(sv[0].x+aw*cos-ah*sin));
+      tmp_v[1].y = OFFSET_Y(round(sv[0].y+aw*sin+ah*cos));
+      tmp_v[2].x = OFFSET_X(round(sv[0].x+aw*cos+ah*sin));
+      tmp_v[2].y = OFFSET_Y(round(sv[0].y+aw*sin-ah*cos));
 
       if (PointInPolygon (XOff, YOff, 4, tmp_v)) return (TRUE);
    }
-   dx = v[n-1].x - v[n-2].x;
-   dy = v[n-1].y - v[n-2].y;
+   dx = (int)(sv[sn-1].x - sv[sn-2].x);
+   dy = (int)(sv[sn-1].y - sv[sn-2].y);
    if ((poly_ptr->style & LS_RIGHT) && (dx != 0 || dy != 0))
    {
       len = (double)sqrt((double)(((double)dx)*((double)dx) +
@@ -402,12 +725,12 @@ int FindGoodPoly (XOff, YOff, PolyObj)
       sin = ((double)dy) / len;
       cos = ((double)dx) / len;
 
-      tmp_v[0].x = tmp_v[3].x = OFFSET_X(v[n-1].x);
-      tmp_v[0].y = tmp_v[3].y = OFFSET_Y(v[n-1].y);
-      tmp_v[1].x = OFFSET_X(round(v[n-1].x-aw*cos+ah*sin));
-      tmp_v[1].y = OFFSET_Y(round(v[n-1].y-aw*sin-ah*cos));
-      tmp_v[2].x = OFFSET_X(round(v[n-1].x-aw*cos-ah*sin));
-      tmp_v[2].y = OFFSET_Y(round(v[n-1].y-aw*sin+ah*cos));
+      tmp_v[0].x = tmp_v[3].x = OFFSET_X(sv[sn-1].x);
+      tmp_v[0].y = tmp_v[3].y = OFFSET_Y(sv[sn-1].y);
+      tmp_v[1].x = OFFSET_X(round(sv[sn-1].x-aw*cos+ah*sin));
+      tmp_v[1].y = OFFSET_Y(round(sv[sn-1].y-aw*sin-ah*cos));
+      tmp_v[2].x = OFFSET_X(round(sv[sn-1].x-aw*cos-ah*sin));
+      tmp_v[2].y = OFFSET_Y(round(sv[sn-1].y-aw*sin+ah*cos));
 
       if (PointInPolygon (XOff, YOff, 4, tmp_v)) return (TRUE);
    }
@@ -421,12 +744,29 @@ int FindGoodPolygon (XOff, YOff, PolygonObj)
 {
    struct PolygonRec	* polygon_ptr = PolygonObj->detail.g;
 
-   if (polygon_ptr->fill != NONEPAT)
-      if (PointInPolygon (XOff, YOff, polygon_ptr->sn, polygon_ptr->svlist))
-         return (TRUE);
+   if (colorLayers && PolygonObj->tmp_parent == NULL &&
+         !ObjInVisibleLayer(PolygonObj)) {
+      return FALSE;
+   }
+   if (PolygonObj->ctm == NULL)
+   {
+      if (polygon_ptr->fill != NONEPAT)
+         if (PointInPolygon (XOff, YOff, polygon_ptr->sn, polygon_ptr->svlist))
+            return (TRUE);
 
-   return (PointOnPoly (XOff, YOff, polygon_ptr->sn, polygon_ptr->svlist,
-         ZOOMED_HALF_W(polygon_ptr->width)));
+      return (PointOnPoly (XOff, YOff, polygon_ptr->sn, polygon_ptr->svlist,
+            ZOOMED_HALF_W(polygon_ptr->width)));
+   }
+   else
+   {
+      if (polygon_ptr->fill != NONEPAT)
+         if (PointInPolygon (XOff, YOff, polygon_ptr->rotated_n,
+               polygon_ptr->rotated_vlist))
+            return (TRUE);
+
+      return (PointOnPoly (XOff, YOff, polygon_ptr->rotated_n,
+            polygon_ptr->rotated_vlist, ZOOMED_HALF_W(polygon_ptr->width)));
+   }
 }
 
 static
@@ -713,6 +1053,21 @@ int FindGoodArc (XOff, YOff, ArcObj)
    int			pass_theta1, just_pass_theta1, angle, dir, full_circle;
    XPoint		tmp_v[4];
 
+   if (colorLayers && ArcObj->tmp_parent == NULL &&
+         !ObjInVisibleLayer(ArcObj)) {
+      return FALSE;
+   }
+   if (ArcObj->ctm != NULL) {
+      if (fill != NONEPAT) {
+         if (PointInPolygon(XOff, YOff, arc_ptr->rotated_n+2,
+               arc_ptr->rotated_vlist)) {
+            return TRUE;
+         }
+      }
+      return (PointOnPoly(XOff, YOff, arc_ptr->rotated_n,
+            arc_ptr->rotated_vlist, ZOOMED_HALF_W(arc_ptr->width)));
+   }
+
    ltx = OFFSET_X(arc_ptr->ltx);
    lty = OFFSET_Y(arc_ptr->lty);
    rbx = OFFSET_X(arc_ptr->ltx+arc_ptr->w);
@@ -1492,7 +1847,11 @@ int FindGoodObj (XOff, YOff, FirstObj, SubObj, ImmediateChildObj)
    for (obj_ptr = FirstObj; obj_ptr != NULL; obj_ptr = obj_ptr->next)
    {
       obj_ptr->tmp_child = NULL;
+      obj_ptr->tmp_parent = NULL;
       if (ImmediateChildObj != NULL) *ImmediateChildObj = obj_ptr;
+      if (colorLayers && !ObjInVisibleLayer(obj_ptr)) {
+         continue;
+      }
       for (attr_ptr=obj_ptr->fattr; attr_ptr!=NULL; attr_ptr=attr_ptr->next)
          if (attr_ptr->shown &&
                XOff >= OFFSET_X(attr_ptr->obj->bbox.ltx)-3 &&
@@ -1514,7 +1873,8 @@ int FindGoodObj (XOff, YOff, FirstObj, SubObj, ImmediateChildObj)
          switch (obj_ptr->type)
          {
             case OBJ_TEXT:
-               return (TRUE);
+               if (FindGoodText (XOff, YOff, obj_ptr)) return (TRUE);
+               break;
             case OBJ_XBM:
                if (FindGoodXBm (XOff, YOff, obj_ptr)) return (TRUE);
                break;
@@ -1543,6 +1903,14 @@ int FindGoodObj (XOff, YOff, FirstObj, SubObj, ImmediateChildObj)
             case OBJ_GROUP:
             case OBJ_SYM:
             case OBJ_ICON:
+               if (colorLayers) {
+                  struct ObjRec *tmp_obj;
+
+                  for (tmp_obj=obj_ptr->detail.r->first; tmp_obj != NULL;
+                        tmp_obj=tmp_obj->next) {
+                     tmp_obj->tmp_parent = obj_ptr;
+                  }
+               }
                if (FindGoodObj (XOff, YOff, obj_ptr->detail.r->first, SubObj,
                      &next_level_child))
                {
