@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,7 +29,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/select.c,v 2.134 1995/05/29 05:36:24 william Exp $";
+      "@(#)$Header: /n/opus/u/guest/william/src/tgif/v3/RCS/select.c,v 3.2 1996/05/24 19:54:50 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -94,6 +94,27 @@ int		numObjLocked=0;
 struct SelRec	* topSel = NULL, * botSel = NULL;
 struct VSelRec	* topVSel = NULL, * botVSel = NULL;
 
+int CountSelectedVertices()
+{
+   struct VSelRec *vsel_ptr;
+   int count = 0;
+
+   for (vsel_ptr=topVSel; vsel_ptr != NULL; vsel_ptr=vsel_ptr->next) {
+      int i, n=vsel_ptr->n;
+
+      count += n;
+      if (vsel_ptr->obj->type == OBJ_POLYGON) {
+         for (i = 0; i < n; i++) {
+            if (vsel_ptr->v_index[i] == 0) {
+               count--;
+               break;
+            }
+         }
+      }
+   }
+   return count;
+}
+
 void CalcBBox (X1, Y1, X2, Y2, LtX, LtY, RbX, RbY)
    int X1, Y1, X2, Y2, * LtX, * LtY, * RbX, * RbY;
 {
@@ -163,7 +184,7 @@ void UnSelNonVertexObjs (highlight)
       else
          sel_ptr->next->prev = sel_ptr->prev;
 
-      cfree (sel_ptr);
+      free(sel_ptr);
    }
 }
 
@@ -174,10 +195,10 @@ void JustRemoveAllVSel ()
    while (topVSel != NULL)
    {
       next_vsel = topVSel->next;
-      cfree (topVSel->v_index);
-      cfree (topVSel->x);
-      cfree (topVSel->y);
-      cfree (topVSel);
+      free(topVSel->v_index);
+      free(topVSel->x);
+      free(topVSel->y);
+      free(topVSel);
       topVSel = next_vsel;
    }
    botVSel = NULL;
@@ -191,7 +212,7 @@ void RemoveAllSel ()
    while (topSel != NULL)
    {
       next_sel = topSel->next;
-      cfree (topSel);
+      free(topSel);
       topSel = next_sel;
    }
    botSel = NULL;
@@ -199,10 +220,10 @@ void RemoveAllSel ()
    while (topVSel != NULL)
    {
       next_vsel = topVSel->next;
-      cfree (topVSel->v_index);
-      cfree (topVSel->x);
-      cfree (topVSel->y);
-      cfree (topVSel);
+      free(topVSel->v_index);
+      free(topVSel->x);
+      free(topVSel->y);
+      free(topVSel);
       topVSel = next_vsel;
    }
    botVSel = NULL;
@@ -223,12 +244,12 @@ struct AttrRec *FindObjAttrWithName(ObjPtr, AttrName)
 
    for (attr_ptr=ObjPtr->fattr; attr_ptr!=NULL; attr_ptr=attr_ptr->next) {
       if (compare_name) {
-         if (strcmp(attr_ptr->name, AttrName) == 0) {
+         if (strcmp(attr_ptr->attr_name.s, AttrName) == 0) {
             found_attr = attr_ptr;
             break;
          }
       } else {
-         if (strcmp(attr_ptr->s, AttrName) == 0) {
+         if (strcmp(attr_ptr->attr_value.s, AttrName) == 0) {
             found_attr = attr_ptr;
             break;
          }
@@ -236,9 +257,10 @@ struct AttrRec *FindObjAttrWithName(ObjPtr, AttrName)
    }
    if (attr_ptr == NULL) return NULL;
 
+   if (found_attr->obj->color == colorIndex) return found_attr;
    for (attr_ptr=found_attr->next; attr_ptr!=NULL; attr_ptr=attr_ptr->next) {
       if (compare_name) {
-         if (strcmp(attr_ptr->name, AttrName) == 0) {
+         if (strcmp(attr_ptr->attr_name.s, AttrName) == 0) {
             if (attr_ptr->obj->color == colorIndex) {
                break;
             } else {
@@ -246,7 +268,7 @@ struct AttrRec *FindObjAttrWithName(ObjPtr, AttrName)
             }
          }
       } else {
-         if (strcmp(attr_ptr->s, AttrName) == 0) {
+         if (strcmp(attr_ptr->attr_value.s, AttrName) == 0) {
             if (attr_ptr->obj->color == colorIndex) {
                break;
             } else {
@@ -290,20 +312,43 @@ struct ObjRec * FindAVertex (XOff, YOff, VIndex, AbsX, AbsY)
       {
          case OBJ_POLY:
             poly_ptr = obj_ptr->detail.p;
-            if (PtInPolyMark (XOff,YOff,poly_ptr->n,poly_ptr->vlist,VIndex))
+            if (PtInPolyMark (obj_ptr, XOff, YOff, poly_ptr->n,
+                  poly_ptr->vlist, VIndex))
             {
-               *AbsX = poly_ptr->vlist[*VIndex].x;
-               *AbsY = poly_ptr->vlist[*VIndex].y;
+               if (obj_ptr->ctm == NULL) {
+                  *AbsX = poly_ptr->vlist[*VIndex].x;
+                  *AbsY = poly_ptr->vlist[*VIndex].y;
+               } else {
+                  int x, y;
+
+                  TransformPointThroughCTM(
+                        poly_ptr->vlist[*VIndex].x-obj_ptr->x,
+                        poly_ptr->vlist[*VIndex].y-obj_ptr->y,
+                        obj_ptr->ctm, &x, &y);
+                  *AbsX = x+obj_ptr->x;
+                  *AbsY = y+obj_ptr->y;
+               }
                return (obj_ptr);
             }
             break;
          case OBJ_POLYGON:
             polygon_ptr = obj_ptr->detail.g;
-            if (PtInPolyMark (XOff, YOff, polygon_ptr->n, polygon_ptr->vlist,
-                  VIndex))
+            if (PtInPolyMark (obj_ptr, XOff, YOff, polygon_ptr->n,
+                  polygon_ptr->vlist, VIndex))
             {
-               *AbsX = polygon_ptr->vlist[*VIndex].x;
-               *AbsY = polygon_ptr->vlist[*VIndex].y;
+               if (obj_ptr->ctm == NULL) {
+                  *AbsX = polygon_ptr->vlist[*VIndex].x;
+                  *AbsY = polygon_ptr->vlist[*VIndex].y;
+               } else {
+                  int x, y;
+
+                  TransformPointThroughCTM(
+                        polygon_ptr->vlist[*VIndex].x-obj_ptr->x,
+                        polygon_ptr->vlist[*VIndex].y-obj_ptr->y,
+                        obj_ptr->ctm, &x, &y);
+                  *AbsX = x+obj_ptr->x;
+                  *AbsY = y+obj_ptr->y;
+               }
                return (obj_ptr);
             }
             break;
@@ -331,6 +376,10 @@ struct ObjRec * FindAnObj (XOff, YOff, OwnerObj, ConnectObj, ReturnedObjName)
    for (obj_ptr=topObj; returned_obj==NULL && obj_ptr!=NULL;
          obj_ptr=obj_ptr->next) {
       obj_ptr->tmp_child = NULL;
+      obj_ptr->tmp_parent = NULL;
+      if (colorLayers && !ObjInVisibleLayer(obj_ptr)) {
+         continue;
+      }
       actual_obj = obj_ptr;
       for (attr_ptr=obj_ptr->fattr; returned_obj==NULL && attr_ptr!=NULL;
             attr_ptr=attr_ptr->next) {
@@ -352,41 +401,49 @@ struct ObjRec * FindAnObj (XOff, YOff, OwnerObj, ConnectObj, ReturnedObjName)
             YOff >= OFFSET_Y(obj_ptr->bbox.lty)-3 &&
             XOff <= OFFSET_X(obj_ptr->bbox.rbx)+3 &&
             YOff <= OFFSET_Y(obj_ptr->bbox.rby)+3) {
-         struct ObjRec	* next_level_child=NULL;
+         struct ObjRec  * next_level_child=NULL;
 
          switch (obj_ptr->type) {
          case OBJ_TEXT:
-            returned_obj = obj_ptr;
+            if (FindGoodText (XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
             break;
          case OBJ_XBM:
-            if (FindGoodXBm(XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
+            if (FindGoodXBm (XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
             break;
          case OBJ_XPM:
-            if (FindGoodXPm(XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
+            if (FindGoodXPm (XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
             break;
          case OBJ_BOX:
-            if (FindGoodBox(XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
+            if (FindGoodBox (XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
             break;
          case OBJ_OVAL:
-            if (FindGoodOval(XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
+            if (FindGoodOval (XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
             break;
          case OBJ_POLY:
-            if (FindGoodPoly(XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
+            if (FindGoodPoly (XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
             break;
          case OBJ_POLYGON:
-            if (FindGoodPolygon(XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
+            if (FindGoodPolygon (XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
             break;
          case OBJ_ARC:
-            if (FindGoodArc(XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
+            if (FindGoodArc (XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
             break;
          case OBJ_RCBOX:
-            if (FindGoodRCBox(XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
+            if (FindGoodRCBox (XOff,YOff,obj_ptr)) returned_obj = obj_ptr;
             break;
 
          case OBJ_GROUP:
          case OBJ_SYM:
          case OBJ_ICON:
-            if (FindGoodObj(XOff, YOff, obj_ptr->detail.r->first, &sub_obj,
+            if (colorLayers) {
+               struct ObjRec *tmp_obj;
+
+               for (tmp_obj=obj_ptr->detail.r->first; tmp_obj != NULL;
+                     tmp_obj=tmp_obj->next) {
+                  tmp_obj->tmp_parent = obj_ptr;
+               }
+            }
+            if (FindGoodObj (XOff, YOff, obj_ptr->detail.r->first, &sub_obj,
                   &next_level_child)) {
                obj_ptr->tmp_child = next_level_child;
                if (sub_obj == NULL) {
@@ -412,7 +469,7 @@ struct ObjRec * FindAnObj (XOff, YOff, OwnerObj, ConnectObj, ReturnedObjName)
          for (obj_ptr=actual_obj; obj_ptr != NULL; obj_ptr=obj_ptr->tmp_child) {
             if ((attr_ptr=FindObjAttrWithName(obj_ptr, "name=")) != NULL) {
                *c_ptr++ = '!';
-               strcpy(c_ptr, attr_ptr->s);
+               strcpy(c_ptr, attr_ptr->attr_value.s);
                c_ptr = &c_ptr[strlen(c_ptr)];
             } else {
                if (prev_obj != NULL) prev_obj->tmp_child = NULL;
@@ -488,13 +545,15 @@ void AddNewSelObj (ObjPtr)
    register struct ObjRec	* obj_ptr = topObj;
    register struct SelRec	* sel_ptr = topSel, * new_sel_ptr;
 
-   new_sel_ptr = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+   new_sel_ptr = (struct SelRec *)malloc(sizeof(struct SelRec));
+   if (new_sel_ptr == NULL) FailAllocMessage();
    new_sel_ptr->obj = ObjPtr;
 
-   for ( ; sel_ptr != NULL && obj_ptr != ObjPtr; obj_ptr = obj_ptr->next)
-      if (obj_ptr == sel_ptr->obj)
+   for ( ; sel_ptr != NULL && obj_ptr != ObjPtr; obj_ptr = obj_ptr->next) {
+      if (obj_ptr == sel_ptr->obj) {
          sel_ptr = sel_ptr->next;
-
+      }
+   }
    if (sel_ptr == NULL)
    {  /* the object is below the last selected object */
       if (botSel == NULL)
@@ -570,12 +629,15 @@ struct VSelRec * SelectOneVertex (XOff, YOff)
    if ((obj_ptr = FindAVertex (XOff, YOff, &v_index, &x, &y)) == NULL)
       return (NULL);
 
-   topVSel = (struct VSelRec *) calloc (1, sizeof(struct VSelRec));
+   topVSel = (struct VSelRec *)malloc(sizeof(struct VSelRec));
+   if (topVSel == NULL) FailAllocMessage();
    topVSel->obj = obj_ptr;
    topVSel->max_v = 10;
-   topVSel->v_index = (int *) calloc (10, sizeof(int));
-   topVSel->x = (int *) calloc (10, sizeof(int));
-   topVSel->y = (int *) calloc (10, sizeof(int));
+   topVSel->v_index = (int*)malloc(10*sizeof(int));
+   if (topVSel->v_index == NULL) FailAllocMessage();
+   topVSel->x = (int*)malloc(10*sizeof(int));
+   topVSel->y = (int*)malloc(10*sizeof(int));
+   if (topVSel->x == NULL || topVSel->y == NULL) FailAllocMessage();
    topVSel->v_index[0] = v_index;
    topVSel->x[0] = x;
    topVSel->y[0] = y;
@@ -610,7 +672,8 @@ struct SelRec * SelectOneObj (XOff, YOff)
    }
    if (owner_obj != NULL) obj_ptr = owner_obj;
 
-   topSel = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+   topSel = (struct SelRec *)malloc(sizeof(struct SelRec));
+   if (topSel == NULL) FailAllocMessage();
    topSel->next = NULL;
    topSel->obj = obj_ptr;
    topSel->prev = NULL;
@@ -631,7 +694,7 @@ int FindVertices (X1, Y1, X2, Y2, TopVSel, BotVSel)
    register int			i;
    struct SelRec		* sel_ptr;
    struct BBRec			bbox;
-   XPoint			* v;
+   IntPoint			* v;
    int				n, count, max_count, j;
 
    *TopVSel = *BotVSel = NULL;
@@ -649,13 +712,29 @@ int FindVertices (X1, Y1, X2, Y2, TopVSel, BotVSel)
             obj_ptr->detail.g->vlist;
       n = (obj_ptr->type==OBJ_POLY) ? obj_ptr->detail.p->n :
             obj_ptr->detail.g->n;
-      for (i = 0, count = 0; i < n; i++)
-         if (v[i].x >= X1 && v[i].x <= X2 && v[i].y >= Y1 && v[i].y <= Y2)
-            count++;
+      if (obj_ptr->ctm == NULL) {
+         for (i = 0, count = 0; i < n; i++) {
+            if (v[i].x >= X1 && v[i].x <= X2 && v[i].y >= Y1 && v[i].y <= Y2) {
+               count++;
+            }
+         }
+      } else {
+         for (i = 0, count = 0; i < n; i++) {
+            int x, y;
+
+            TransformPointThroughCTM(v[i].x-obj_ptr->x, v[i].y-obj_ptr->y,
+                  obj_ptr->ctm, &x, &y);
+            if (x+obj_ptr->x >= X1 && x+obj_ptr->x <= X2 &&
+                  y+obj_ptr->y >= Y1 && y+obj_ptr->y <= Y2) {
+               count++;
+            }
+         }
+      }
 
       if (count != 0)
       {
-         vsel_ptr = (struct VSelRec *) calloc (1, sizeof(struct VSelRec));
+         vsel_ptr = (struct VSelRec *)malloc(sizeof(struct VSelRec));
+         if (vsel_ptr == NULL) FailAllocMessage();
          vsel_ptr->obj = obj_ptr;
          vsel_ptr->next = *TopVSel;
          vsel_ptr->prev = NULL;
@@ -668,18 +747,37 @@ int FindVertices (X1, Y1, X2, Y2, TopVSel, BotVSel)
          max_count = ((count%10) == 0) ? 10*((int)(count/10)) :
                10*((int)(count/10)+1);
          vsel_ptr->max_v = max_count;
-         vsel_ptr->v_index = (int *) calloc (max_count, sizeof(int));
-         vsel_ptr->x = (int *) calloc (max_count, sizeof(int));
-         vsel_ptr->y = (int *) calloc (max_count, sizeof(int));
+         vsel_ptr->v_index = (int*)malloc(max_count*sizeof(int));
+         if (vsel_ptr->v_index == NULL) FailAllocMessage();
+         vsel_ptr->x = (int*)malloc(max_count*sizeof(int));
+         vsel_ptr->y = (int*)malloc(max_count*sizeof(int));
+         if (vsel_ptr->x == NULL || vsel_ptr->y == NULL) FailAllocMessage();
 
-         for (i = 0, j = 0; i < n; i++)
-            if (v[i].x >= X1 && v[i].x <= X2 && v[i].y >= Y1 && v[i].y <= Y2)
-            {
-               vsel_ptr->v_index[j] = i;
-               vsel_ptr->x[j] = v[i].x;
-               vsel_ptr->y[j] = v[i].y;
-               j++;
+         if (obj_ptr->ctm == NULL) {
+            for (i = 0, j = 0; i < n; i++) {
+               if (v[i].x >= X1 && v[i].x <= X2 &&
+                     v[i].y >= Y1 && v[i].y <= Y2) {
+                  vsel_ptr->v_index[j] = i;
+                  vsel_ptr->x[j] = v[i].x;
+                  vsel_ptr->y[j] = v[i].y;
+                  j++;
+               }
             }
+         } else {
+            for (i = 0, j = 0; i < n; i++) {
+               int x, y;
+
+               TransformPointThroughCTM(v[i].x-obj_ptr->x, v[i].y-obj_ptr->y,
+                     obj_ptr->ctm, &x, &y);
+               if (x+obj_ptr->x >= X1 && x+obj_ptr->x <= X2 &&
+                     y+obj_ptr->y >= Y1 && y+obj_ptr->y <= Y2) {
+                  vsel_ptr->v_index[j] = i;
+                  vsel_ptr->x[j] = x+obj_ptr->x;
+                  vsel_ptr->y[j] = y+obj_ptr->y;
+                  j++;
+               }
+            }
+         }
       }
    }
    return (*TopVSel != NULL);
@@ -696,20 +794,26 @@ struct SelRec * FindObjects (X1, Y1, X2, Y2, TopSel, BotSel)
 
    *TopSel = *BotSel = NULL;
 
-   for (obj_ptr = botObj; obj_ptr != NULL; obj_ptr = obj_ptr->prev)
+   for (obj_ptr = botObj; obj_ptr != NULL; obj_ptr = obj_ptr->prev) {
+      obj_ptr->tmp_parent = NULL;
+      if (colorLayers && !ObjInVisibleLayer(obj_ptr)) {
+         continue;
+      }
       if (X1 <= obj_ptr->bbox.ltx && X2 >= obj_ptr->bbox.rbx &&
-            Y1 <= obj_ptr->bbox.lty && Y2 >= obj_ptr->bbox.rby)
-      {
-         sel_ptr = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+            Y1 <= obj_ptr->bbox.lty && Y2 >= obj_ptr->bbox.rby) {
+         sel_ptr = (struct SelRec *)malloc(sizeof(struct SelRec));
+         if (sel_ptr == NULL) FailAllocMessage();
          sel_ptr->next = *TopSel;
          sel_ptr->obj = obj_ptr;
          sel_ptr->prev = NULL;
-         if (*TopSel == NULL)
+         if (*TopSel == NULL) {
             *BotSel = sel_ptr;
-         else
+         } else {
             (*TopSel)->prev = sel_ptr;
+         }
          *TopSel = sel_ptr;
       }
+   }
 
    return (*TopSel);
 }
@@ -760,7 +864,8 @@ struct ObjRec * PtInObjList (XOff, YOff, FirstObjPtr)
       {
          switch (obj_ptr->type)
          {
-            case OBJ_TEXT: return (obj_ptr);
+            case OBJ_TEXT:
+               if (FindGoodText (XOff,YOff,obj_ptr)) return (obj_ptr); break;
             case OBJ_XBM:
                if (FindGoodXBm (XOff,YOff,obj_ptr)) return (obj_ptr); break;
             case OBJ_XPM:
@@ -818,7 +923,8 @@ struct ObjRec * PtInSelected (XOff, YOff)
       {
          switch (obj_ptr->type)
          {
-            case OBJ_TEXT: return (obj_ptr);
+            case OBJ_TEXT:
+               if (FindGoodText (XOff,YOff,obj_ptr)) return (obj_ptr); break;
             case OBJ_XBM:
                if (FindGoodXBm (XOff,YOff,obj_ptr)) return (obj_ptr); break;
             case OBJ_XPM:
@@ -897,10 +1003,10 @@ void ToggleVertexSelection (ObjPtr, VIndex, AbsX, AbsY)
          else
             vsel_ptr->next->prev = vsel_ptr->prev;
 
-         cfree (vsel_ptr->v_index);
-         cfree (vsel_ptr->x);
-         cfree (vsel_ptr->y);
-         cfree (vsel_ptr);
+         free(vsel_ptr->v_index);
+         free(vsel_ptr->x);
+         free(vsel_ptr->y);
+         free(vsel_ptr);
       }
       else
       {
@@ -923,20 +1029,24 @@ void ToggleVertexSelection (ObjPtr, VIndex, AbsX, AbsY)
    {
       if (vsel_ptr == NULL)
       {
-         vsel_ptr = (struct VSelRec *) calloc (1, sizeof(struct VSelRec));
+         vsel_ptr = (struct VSelRec *)malloc(sizeof(struct VSelRec));
+         if (vsel_ptr == NULL) FailAllocMessage();
          vsel_ptr->obj = ObjPtr;
          n = vsel_ptr->n = 1;
          vsel_ptr->max_v = 10;
-         vsel_ptr->v_index = (int *) calloc (10, sizeof(int));
-         vsel_ptr->x = (int *) calloc (10, sizeof(int));
-         vsel_ptr->y = (int *) calloc (10, sizeof(int));
+         vsel_ptr->v_index = (int*)malloc(10*sizeof(int));
+         if (vsel_ptr->v_index == NULL) FailAllocMessage();
+         vsel_ptr->x = (int*)malloc(10*sizeof(int));
+         vsel_ptr->y = (int*)malloc(10*sizeof(int));
+         if (vsel_ptr->x == NULL || vsel_ptr->y == NULL) FailAllocMessage();
 
          vsel_ptr->prev = NULL;
          vsel_ptr->next = topVSel;
-         if (topVSel == NULL)
+         if (topVSel == NULL) {
             botVSel = vsel_ptr;
-         else
+         } else {
             topVSel->prev = vsel_ptr;
+         }
          topVSel = vsel_ptr;
       }
       else
@@ -980,7 +1090,7 @@ void ToggleSelectedObjIfSelectedAlready (ObjPtr)
       else
          sel_ptr->next->prev = sel_ptr->prev;
 
-      cfree (sel_ptr);
+      free(sel_ptr);
       numObjSelected--;
    }
    else
@@ -1009,14 +1119,14 @@ void ContinueSel (XOff, YOff, ShiftKeyDown)
    end_x = XOff;
    end_y = YOff; 
 
-   SelBox (drawWindow, revDefaultGC, XOff, YOff, end_x, end_y);
+   SelBox(drawWindow, revDefaultGC, XOff, YOff, end_x, end_y);
    PixelToMeasurementUnit(w_buf, 0);
    PixelToMeasurementUnit(h_buf, 0);
    PixelToMeasurementUnit(x_buf, ABS_X(end_x));
    PixelToMeasurementUnit(y_buf, ABS_Y(end_y));
-   sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
-   StartShowMeasureCursor (end_x, end_y, buf, TRUE);
-   XGrabPointer (mainDisplay, drawWindow, False,
+   sprintf(buf, "x=%s\ny=%s\nw=%s\nh=%s", w_buf, h_buf, x_buf, y_buf);
+   StartShowMeasureCursor(end_x, end_y, buf, TRUE);
+   XGrabPointer(mainDisplay, drawWindow, False,
          PointerMotionMask | ButtonReleaseMask,
          GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
    
@@ -1033,7 +1143,7 @@ void ContinueSel (XOff, YOff, ShiftKeyDown)
          PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(end_y-YOff)));
          PixelToMeasurementUnit(x_buf, ABS_X(end_x));
          PixelToMeasurementUnit(y_buf, ABS_Y(end_y));
-         sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+         sprintf(buf, "x=%s\ny=%s\nw=%s\nh=%s", w_buf, h_buf, x_buf, y_buf);
          EndShowMeasureCursor (end_x, end_y, buf, TRUE);
          SelBox (drawWindow, revDefaultGC, XOff, YOff, end_x, end_y);
          done = TRUE;
@@ -1048,7 +1158,7 @@ void ContinueSel (XOff, YOff, ShiftKeyDown)
          PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(end_y-YOff)));
          PixelToMeasurementUnit(x_buf, ABS_X(end_x));
          PixelToMeasurementUnit(y_buf, ABS_Y(end_y));
-         sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+         sprintf(buf, "x=%s\ny=%s\nw=%s\nh=%s", w_buf, h_buf, x_buf, y_buf);
          ShowMeasureCursor (end_x, end_y, buf, TRUE);
          SelBox (drawWindow, revDefaultGC, XOff, YOff, end_x, end_y);
          end_x = new_end_x; end_y = new_end_y;
@@ -1057,7 +1167,7 @@ void ContinueSel (XOff, YOff, ShiftKeyDown)
          PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(end_y-YOff)));
          PixelToMeasurementUnit(x_buf, ABS_X(end_x));
          PixelToMeasurementUnit(y_buf, ABS_Y(end_y));
-         sprintf (buf, "%sx%s%s%s", w_buf, h_buf, x_buf, y_buf);
+         sprintf(buf, "x=%s\ny=%s\nw=%s\nh=%s", w_buf, h_buf, x_buf, y_buf);
          ShowMeasureCursor (end_x, end_y, buf, TRUE);
 
          MarkRulers (end_x, end_y);
@@ -1129,10 +1239,10 @@ void ContinueSel (XOff, YOff, ShiftKeyDown)
                      ToggleVertexSelection (obj_ptr, vsel_ptr->v_index[i],
                            vsel_ptr->x[i], vsel_ptr->y[i]);
                   next_vsel = vsel_ptr->next;
-                  cfree(vsel_ptr->v_index);
-                  cfree(vsel_ptr->x);
-                  cfree(vsel_ptr->y);
-                  cfree(vsel_ptr);
+                  free(vsel_ptr->v_index);
+                  free(vsel_ptr->x);
+                  free(vsel_ptr->y);
+                  free(vsel_ptr);
                }
                UpdSelBBox ();
             }
@@ -1190,7 +1300,7 @@ void ContinueSel (XOff, YOff, ShiftKeyDown)
                   next_sel = sel_ptr->next;
                   obj_ptr = sel_ptr->obj;
                   ToggleSelectedObjIfSelectedAlready (obj_ptr);
-                  cfree(sel_ptr);
+                  free(sel_ptr);
                }
                UpdSelBBox ();
             }
@@ -1225,7 +1335,6 @@ void Select (input)
    struct SelRec	* sel_ptr;
    struct VSelRec	* vsel_ptr;
    struct ObjRec	* obj_ptr;
-   int			mouse_x, mouse_y, grid_x, grid_y, corner;
    char			s[80];
    KeySym		key_sym;
    Time			click_time;
@@ -1319,6 +1428,8 @@ void Select (input)
    button_ev = &(input->xbutton);
    if (button_ev->button == Button1)
    {
+      int	mouse_x, mouse_y, grid_x, grid_y, corner;
+
       mouse_x = button_ev->x;
       mouse_y = button_ev->y;
       GridXY (mouse_x, mouse_y, &grid_x, &grid_y);
@@ -1395,6 +1506,14 @@ void Select (input)
             return;
          }
       }
+      else if (curChoice == ROTATEMODE && topSel != NULL)
+      {
+         if ((sel_ptr = PtInSelMark (mouse_x, mouse_y, &corner)) != NULL)
+         {
+            RotateShearSel (grid_x, grid_y, sel_ptr->obj, corner);
+            return;
+         }
+      }
       ContinueSel (mouse_x, mouse_y, FALSE);
       justDupped = FALSE;
    }
@@ -1410,12 +1529,12 @@ struct AttrRec *FindFileAttrWithName(AttrName)
 
    for (attr_ptr=tgifObj->fattr; attr_ptr != NULL; attr_ptr=attr_ptr->next) {
       if (compare_name) {
-         if (strcmp(attr_ptr->name, AttrName) == 0) {
+         if (strcmp(attr_ptr->attr_name.s, AttrName) == 0) {
             found_attr = attr_ptr;
             break;
          }
       } else {
-         if (strcmp(attr_ptr->s, AttrName) == 0) {
+         if (strcmp(attr_ptr->attr_value.s, AttrName) == 0) {
             found_attr = attr_ptr;
             break;
          }
@@ -1423,9 +1542,10 @@ struct AttrRec *FindFileAttrWithName(AttrName)
    }
    if (attr_ptr == NULL) return NULL;
 
+   if (found_attr->obj->color == colorIndex) return found_attr;
    for (attr_ptr=found_attr->next; attr_ptr!=NULL; attr_ptr=attr_ptr->next) {
       if (compare_name) {
-         if (strcmp(attr_ptr->name, AttrName) == 0) {
+         if (strcmp (attr_ptr->attr_name.s, AttrName) == 0) {
             if (attr_ptr->obj->color == colorIndex) {
                break;
             } else {
@@ -1433,7 +1553,7 @@ struct AttrRec *FindFileAttrWithName(AttrName)
             }
          }
       } else {
-         if (strcmp(attr_ptr->s, AttrName) == 0) {
+         if (strcmp (attr_ptr->attr_value.s, AttrName) == 0) {
             if (attr_ptr->obj->color == colorIndex) {
                break;
             } else {
@@ -1524,28 +1644,30 @@ int DoTeleport(teleport_attr)
    int do_not_save=FALSE, rc=TRUE, just_goto_page=FALSE;
 
    while (!DirIsRemote(curDir) && fileModified) {
-      switch (MsgBox("File modified.\n\nsave file before open? [ync](y)",
+      switch (MsgBox("File modified.\n\nSave file before open? [ync](y)",
             TOOL_NAME, YNC_MB)) {
       case MB_ID_YES: SaveFile(); break;
       case MB_ID_NO: do_not_save = TRUE; SetFileModified(FALSE); break;
       case MB_ID_CANCEL: return FALSE;
       }
    }
-   if (!FormNewFileName(curDir, teleport_attr->s,
-         (strcmp(teleport_attr->name,TELEPORT_ATTR)==0 ? OBJ_FILE_EXT : NULL),
-         file_name, &page_spec)) {
-      sprintf(msg, "Invalid teleport destination '%s'.", teleport_attr->s);
+   if (!FormNewFileName(curDir, teleport_attr->attr_value.s,
+         (strcmp(teleport_attr->attr_name.s,TELEPORT_ATTR)==0 ? OBJ_FILE_EXT :
+         NULL), file_name, &page_spec)) {
+      sprintf(msg, "Invalid teleport destination '%s'.",
+            teleport_attr->attr_value.s);
       MsgBox(msg, TOOL_NAME, INFO_MB);
       if (do_not_save) SetFileModified(TRUE);
       rc = FALSE;
    }
-   if (*teleport_attr->s == '#') just_goto_page = TRUE;
+   if (*teleport_attr->attr_value.s == '#') just_goto_page = TRUE;
 
    if (rc && page_spec != NULL && just_goto_page) {
       int new_page_num=(-1);
 
       if (!GetPageNumFromPageSpec(page_spec, &new_page_num)) {
-         sprintf(msg, "Invalid teleport destination '%s'.", teleport_attr->s);
+         sprintf(msg, "Invalid teleport destination '%s'.",
+               teleport_attr->attr_value.s);
          MsgBox(msg, TOOL_NAME, INFO_MB);
          if (do_not_save) SetFileModified(TRUE);
          rc = FALSE;
@@ -1562,11 +1684,11 @@ int DoTeleport(teleport_attr)
          CommitNavigate();
          justDupped = FALSE;
       }
-      if (page_spec != NULL) cfree(page_spec);
+      if (page_spec != NULL) free(page_spec);
       return rc;
    }
    if (!rc) {
-      if (page_spec != NULL) cfree(page_spec);
+      if (page_spec != NULL) free(page_spec);
       return rc;
    }
    if (FileIsRemote(file_name)) {
@@ -1599,7 +1721,8 @@ int DoTeleport(teleport_attr)
       int new_page_num=(-1);
 
       if (!GetPageNumFromPageSpec(page_spec, &new_page_num)) {
-         sprintf(msg, "Invalid teleport destination '%s'.", teleport_attr->s);
+         sprintf(msg, "Invalid teleport destination '%s'.",
+               teleport_attr->attr_value.s);
          MsgBox(msg, TOOL_NAME, INFO_MB);
          if (do_not_save) SetFileModified(TRUE);
          rc = FALSE;
@@ -1612,10 +1735,10 @@ int DoTeleport(teleport_attr)
          RedrawScrollBars();
          justDupped = FALSE;
       }
-      if (page_spec != NULL) cfree(page_spec);
+      if (page_spec != NULL) free(page_spec);
       return rc;
    }
-   if (page_spec != NULL) cfree(page_spec);
+   if (page_spec != NULL) free(page_spec);
    return rc;
 }
 
@@ -1635,16 +1758,17 @@ int DoPageTeleport(teleport_attr, do_by_page_name)
       for (i=1, page_ptr=firstPage; page_ptr!=NULL;
             page_ptr=page_ptr->next, i++) {
          if (page_ptr->name != NULL && strcmp(page_ptr->name,
-               teleport_attr->s) == 0) {
+               teleport_attr->attr_value.s) == 0) {
             if (curPageNum != i) SetCurPage(i);
             return TRUE;
          }
       }
-      sprintf(msg, "Can not find page \"%s\" to warp to.", teleport_attr->s);
+      sprintf(msg, "Can not find page \"%s\" to warp to.",
+            teleport_attr->attr_value.s);
       MsgBox(msg, TOOL_NAME, INFO_MB);
       rc = FALSE;
    } else {
-      i = atoi(teleport_attr->s);
+      i = atoi(teleport_attr->attr_value.s);
       if (i >= 1 && i <= lastPageNum) {
          if (curPageNum != i) SetCurPage(i);
       } else {
@@ -1686,7 +1810,7 @@ void DoExecLoop(obj_ptr, exec_attr)
       ResetDeckIndices();
       SaveStatusStringsIntoBuf(status_buf, &one_line_status);
       if (cmdToExecAfterHyperJump != NULL) {
-         cfree(cmdToExecAfterHyperJump);
+         free(cmdToExecAfterHyperJump);
          cmdToExecAfterHyperJump = NULL;
       }
       warpToAttr = NULL;
@@ -1799,16 +1923,21 @@ void SelAllObj (high_light)
    SetCurChoice (NOTHING);
    if (topSel!=NULL) { HighLightReverse (); RemoveAllSel (); }
 
-   for (obj_ptr = botObj; obj_ptr != NULL; obj_ptr = obj_ptr->prev)
-   {
-      sel_ptr = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+   for (obj_ptr = botObj; obj_ptr != NULL; obj_ptr = obj_ptr->prev) {
+      obj_ptr->tmp_parent = NULL;
+      if (colorLayers && !ObjInVisibleLayer(obj_ptr)) {
+         continue;
+      }
+      sel_ptr = (struct SelRec *)malloc(sizeof(struct SelRec));
+      if (sel_ptr == NULL) FailAllocMessage();
       sel_ptr->next = topSel;
       sel_ptr->obj = obj_ptr;
       sel_ptr->prev = NULL;
-      if (topSel == NULL)
+      if (topSel == NULL) {
          botSel = sel_ptr;
-      else
+      } else {
          topSel->prev = sel_ptr;
+      }
       topSel = sel_ptr;
    }
    UpdSelBBox ();
@@ -1895,7 +2024,7 @@ void DelAllSelObj ()
    register struct SelRec	* sel_ptr;
    register int			j, i;
    struct VSelRec		* vsel_ptr;
-   XPoint			* v=NULL;
+   IntPoint			* v=NULL;
    struct PolyRec		* poly_ptr=NULL;
    struct PolygonRec		* polygon_ptr=NULL;
    int				n=0;
@@ -1910,6 +2039,7 @@ void DelAllSelObj ()
       for (vsel_ptr=botVSel; vsel_ptr!=NULL; vsel_ptr=vsel_ptr->prev)
       {
          int	delete_it=FALSE, extra_vertex=FALSE;
+         char	*smooth=NULL;
 
          obj_ptr = vsel_ptr->obj;
 
@@ -1920,6 +2050,7 @@ void DelAllSelObj ()
                poly_ptr = obj_ptr->detail.p;
                v = poly_ptr->vlist;
                n = poly_ptr->n;
+               smooth = poly_ptr->smooth;
 
                if (vsel_ptr->n >= n-1) delete_it = TRUE;
                break;
@@ -1927,6 +2058,7 @@ void DelAllSelObj ()
                polygon_ptr = obj_ptr->detail.g;
                v = polygon_ptr->vlist;
                n = polygon_ptr->n;
+               smooth = polygon_ptr->smooth;
 
                for (j=0; j < vsel_ptr->n; j++)
                   if (vsel_ptr->v_index[j] == 0)
@@ -1966,12 +2098,13 @@ void DelAllSelObj ()
             topSel = saved_top_sel;
             botSel = saved_bot_sel;
 
-            cfree (sel_ptr);
+            free(sel_ptr);
             ChangeReplaceOneCmdToDeleteCmd ();
          }
          else
          {
-            mark = (short *) calloc (n, sizeof(short));
+            mark = (short *)malloc(n*sizeof(short));
+            if (mark == NULL) FailAllocMessage();
             for (j=0; j < n; j++) mark[j] = FALSE;
             for (j=0; j < vsel_ptr->n; j++) mark[vsel_ptr->v_index[j]] = TRUE;
 
@@ -1982,8 +2115,17 @@ void DelAllSelObj ()
                   {
                      if (mark[i])
                      {
-                        for (j=i+1; j < n; j++)
+                        for (j=i+1; j < n; j++) {
                            v[j-1] = v[j];
+                           if (smooth != NULL) smooth[j-1] = smooth[j];
+                        }
+                        if (smooth != NULL) {
+                           if (i == 0) {
+                              smooth[0] = FALSE;
+                           } else if (i == n-1) {
+                              smooth[n-2] = FALSE;
+                           }
+                        }
                         n--;
                      }
                   }
@@ -1995,25 +2137,31 @@ void DelAllSelObj ()
                      UpdPolyBBox (obj_ptr, poly_ptr->intn, poly_ptr->intvlist);
                   break;
                case OBJ_POLYGON:
-                  for (i=n-1; i >= 0; i--)
+                  for (i=n-2; i >= 0; i--)
                   {
                      if (mark[i])
                      {
-                        for (j=i+1; j < n; j++) v[j-1] = v[j];
+                        for (j=i+1; j < n; j++) {
+                           v[j-1] = v[j];
+                           if (smooth != NULL) smooth[j-1] = smooth[j];
+                        }
                         n--;
+                        if (smooth != NULL && i == 0) {
+                           smooth[n-1] = smooth[0];
+                        }
                      }
                   }
                   polygon_ptr->n -= vsel_ptr->n;
                   if (extra_vertex) v[polygon_ptr->n++] = v[0];
                   AdjObjSplineVs (obj_ptr);
-                  if (poly_ptr->curved != LT_INTSPLINE)
+                  if (polygon_ptr->curved != LT_INTSPLINE)
                      UpdPolyBBox (obj_ptr, polygon_ptr->n, polygon_ptr->vlist);
                   else
                      UpdPolyBBox (obj_ptr, polygon_ptr->intn,
                            polygon_ptr->intvlist);
                   break;
             }
-            cfree (mark);
+            free(mark);
             AdjObjBBox (obj_ptr);
             RecordReplaceAnObj (obj_ptr);
          }
@@ -2050,12 +2198,18 @@ void GroupSingleObj ()
          !(tmpTopObj->type == OBJ_GROUP || tmpTopObj->type == OBJ_SYM ||
          tmpTopObj->type == OBJ_ICON))
    {
+      struct AttrRec *attr_ptr;
+
       topObj->fattr = tmpTopObj->fattr;
       topObj->lattr = tmpTopObj->lattr;
       tmpTopObj->fattr = tmpTopObj->lattr = NULL;
+      for (attr_ptr=topObj->fattr; attr_ptr != NULL; attr_ptr=attr_ptr->next) {
+         attr_ptr->owner = topObj;
+      }
    }
 
-   topSel = botSel = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+   topSel = botSel = (struct SelRec *)malloc(sizeof(struct SelRec));
+   if (topSel == NULL) FailAllocMessage();
    topSel->obj = topObj;
    topSel->next = topSel->prev = NULL;
    numObjSelected = 1;
@@ -2085,7 +2239,8 @@ void GroupSelObj ()
    CreateGroupObj (tmpTopObj, tmpBotObj);
 
    RemoveAllSel ();
-   topSel = botSel = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+   topSel = botSel = (struct SelRec *)malloc(sizeof(struct SelRec));
+   if (topSel == NULL) FailAllocMessage();
    topSel->obj = topObj;
    topSel->next = topSel->prev = NULL;
    UpdSelBBox ();
@@ -2102,7 +2257,8 @@ void SelectTopObj ()
 {
    if (topObj == NULL) return;
 
-   topSel = botSel = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+   topSel = botSel = (struct SelRec *)malloc(sizeof(struct SelRec));
+   if (topSel == NULL) FailAllocMessage();
    topSel->obj = topObj;
    topSel->next = topSel->prev = NULL;
    UpdSelBBox ();
