@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,7 +29,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/setup.c,v 2.149 1995/07/08 22:24:27 william Exp $";
+      "@(#)$Header: /n/opus/u/guest/william/src/tgif/v3/RCS/setup.c,v 3.0 1996/05/06 16:07:31 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -50,6 +50,9 @@ static char RCSid[] =
 #include "file.e"
 #include "font.e"
 #include "grid.e"
+#include "help.e"
+#include "imgproc.e"
+#include "import.e"
 #include "mainloop.e"
 #include "mainmenu.e"
 #include "mark.e"
@@ -68,9 +71,11 @@ static char RCSid[] =
 #ifndef _NO_EXTERN
 #include "setup.e"
 #endif
+#include "shape.e"
 #include "shortcut.e"
 #include "spline.e"
 #include "stk.e"
+#include "stretch.e"
 #include "text.e"
 #include "xbitmap.e"
 #include "xpixmap.e"
@@ -94,18 +99,22 @@ extern int	atoi ARGS_DECL((char *));
 #define RULER_W 20
 #define BRDR_W 1
 
+#define DEF_VSBAR_H 64
+#define DEF_HSBAR_W 128
+
 #define DRAW_WINDOW_W (5*PIX_PER_INCH)
 #define DRAW_WINDOW_H (5*PIX_PER_INCH)
-#define CHOICE_WINDOW_W (12*CHOICE_IMAGE_W)
-#define CHOICE_WINDOW_H (2*CHOICE_IMAGE_H)
-#define VSBAR_H (DRAW_WINDOW_H+RULER_W+2*BRDR_W)
-#define HSBAR_W (DRAW_WINDOW_W+RULER_W+2*BRDR_W)
-#define TITLE_WINDOW_W (HSBAR_W+SCROLLBAR_W+2*BRDR_W)
-#define TITLE_WINDOW_H (MENU_IMAGE_H*2)
-#define TOOL_WINDOW_W (TITLE_WINDOW_W+2*BRDR_W)
-#define TOOL_WINDOW_H \
-      (TITLE_WINDOW_H+CHOICE_WINDOW_H+VSBAR_H+SCROLLBAR_W+8*BRDR_W)
-#define MSG_WINDOW_W (TITLE_WINDOW_W-CHOICE_WINDOW_W-2*BRDR_W)
+#define CHOICE_WINDOW_W (MAXCHOICEWINDOWCOLS*CHOICE_IMAGE_W)
+#define CHOICE_WINDOW_H (CHOICE_IMAGE_H<<1)
+#define VSBAR_H (DRAW_WINDOW_H+RULER_W+(BRDR_W<<1))
+#define HSBAR_W 128
+#define PAGE_DUMMY_WINDOW_W 3
+#define PAGE_WINDOW_W (DRAW_WINDOW_W+RULER_W-HSBAR_W-PAGE_DUMMY_WINDOW_W)
+#define COLOR_DUMMY_WINDOW_H 3
+#define COLOR_WINDOW_H (DRAW_WINDOW_H+RULER_W-VSBAR_H-COLOR_DUMMY_WINDOW_H)
+#define TITLE_WINDOW_W (DRAW_WINDOW_W+RULER_W+BRDR_W+SCROLLBAR_W+(BRDR_W<<1))
+#define TITLE_WINDOW_H (MENU_IMAGE_H<<1)
+#define MSG_WINDOW_W (TITLE_WINDOW_W-CHOICE_WINDOW_W-(BRDR_W<<1))
 #define MSG_WINDOW_H (CHOICE_WINDOW_H)
 #define ICON_WINDOW_W 64
 #define ICON_WINDOW_H 64
@@ -121,6 +130,10 @@ int	hSBarW = HSBAR_W;
 int	scrollBarW = SCROLLBAR_W;
 int	rulerW = RULER_W;
 int	brdrW = BRDR_W;
+int	pageWindowW = PAGE_WINDOW_W;
+int	pageDummyWindowW = PAGE_DUMMY_WINDOW_W;
+int	colorWindowH = COLOR_WINDOW_H;
+int	colorDummyWindowH = COLOR_DUMMY_WINDOW_H;
 int	msgWindowW = MSG_WINDOW_W;
 int	msgWindowH = MSG_WINDOW_H;
 int	choiceImageW = CHOICE_IMAGE_W;
@@ -165,6 +178,10 @@ Window	iconWindow=None;
 Window	iconBaseWindow=None;
 Window	menubarWindow=None;
 Window	statusWindow=None;
+Window	pageWindow=None;
+Window	pageDummyWindow=None;
+Window	colorWindow=None;
+Window	colorDummyWindow=None;
 
 #ifndef A4PAPER
 int	onePageWidth = (85*PIX_PER_INCH)/10;
@@ -267,18 +284,20 @@ void InitWinSizes ()
 {
    initDrawWinW = drawWinW;
    initDrawWinH = drawWinH;
-   titleWindowH = (showVersion) ? MENU_IMAGE_H*2 : MENU_IMAGE_H;
-   hSBarW = drawWinW+rulerW+2*brdrW;
-   vSBarH = drawWinH+rulerW+2*brdrW;
-   titleWindowW = hSBarW+scrollBarW+2*brdrW;
+   titleWindowH = (showVersion) ? (MENU_IMAGE_H<<1) : MENU_IMAGE_H;
+   pageWindowW = drawWinW+rulerW-hSBarW-pageDummyWindowW;
+   colorWindowH = drawWinH+rulerW-DEF_VSBAR_H-colorDummyWindowH;
+   vSBarH = drawWinH+rulerW+(brdrW<<1);
+   titleWindowW = drawWinW+rulerW+brdrW+scrollBarW+(brdrW<<1);
    menubarWindowW = statusWindowW = titleWindowW;
    CalcMenubarWindowHeight ();
-   msgWindowW = titleWindowW-choiceWindowW-2*brdrW;
-   mainWinW = titleWindowW+2*brdrW;
+   msgWindowW = titleWindowW-choiceWindowW-(brdrW<<1);
+   mainWinW = titleWindowW+(brdrW<<1);
    mainWinH =
          (noMenubar ? 0 : menubarWindowH+(brdrW<<1)) +
          (noStatusWindow ? 0 : statusWindowH+(brdrW<<1)) +
          titleWindowH+choiceWindowH+vSBarH+scrollBarW+8*brdrW;
+   if (colorLayers) vSBarH = DEF_VSBAR_H;
    CalcStatusSubWinGeom ();
 }
 
@@ -286,16 +305,18 @@ static
 void InverseInitWinSizes ()
    /* derive other win sizes from mainWinW and mainWinH */
 {
-   titleWindowW = mainWinW-2*brdrW;
+   titleWindowW = mainWinW-(brdrW<<1);
    menubarWindowW = statusWindowW = titleWindowW;
    CalcMenubarWindowHeight ();
-   msgWindowW = titleWindowW-choiceWindowW-2*brdrW;
+   msgWindowW = titleWindowW-choiceWindowW-(brdrW<<1);
    vSBarH = mainWinH - titleWindowH - choiceWindowH - scrollBarW - 8*brdrW -
          (noMenubar ? 0 : menubarWindowH+(brdrW<<1)) -
          (noStatusWindow ? 0 : statusWindowH+(brdrW<<1));
-   hSBarW = titleWindowW-scrollBarW-2*brdrW;
-   drawWinH = initDrawWinH = vSBarH-rulerW-2*brdrW;
-   drawWinW = initDrawWinW = hSBarW-rulerW-2*brdrW;
+   drawWinH = initDrawWinH = vSBarH-rulerW-(brdrW<<1);
+   drawWinW = initDrawWinW = titleWindowW-scrollBarW-rulerW-(brdrW<<2);
+   pageWindowW = titleWindowW-scrollBarW-hSBarW-pageDummyWindowW-(brdrW<<2);
+   colorWindowH = drawWinH+rulerW-DEF_VSBAR_H-colorDummyWindowH;
+   if (colorLayers) vSBarH = DEF_VSBAR_H;
    CalcMenubarWindowHeight ();
    CalcStatusSubWinGeom ();
 }
@@ -371,8 +392,8 @@ void Reconfigure (Forced)
    CalcMenubarWindowHeight ();
    if (canvasWindowOnly)
    {
-      initDrawWinW = mainWinW-2*brdrW;
-      initDrawWinH = mainWinH-2*brdrW;
+      initDrawWinW = mainWinW-(brdrW<<1);
+      initDrawWinH = mainWinH-(brdrW<<1);
    }
    else
    {
@@ -384,12 +405,13 @@ void Reconfigure (Forced)
    }
    drawWinW = initDrawWinW;
    drawWinH = initDrawWinH;
-   hSBarW = initDrawWinW + rulerW + 2*brdrW;
-   vSBarH = initDrawWinH + rulerW + 2*brdrW;
-   titleWindowW = hSBarW + scrollBarW + 2*brdrW;
+   pageWindowW = initDrawWinW+rulerW-hSBarW-pageDummyWindowW;
+   colorWindowH = initDrawWinH+rulerW-DEF_VSBAR_H-colorDummyWindowH-(brdrW<<1);
+   vSBarH = (colorLayers ? DEF_VSBAR_H : initDrawWinH + rulerW + (brdrW<<1));
+   titleWindowW = initDrawWinW + rulerW + scrollBarW + (brdrW<<2);
    menubarWindowW = statusWindowW = titleWindowW;
    CalcMenubarWindowHeight ();
-   msgWindowW = titleWindowW - choiceWindowW - 2*brdrW;
+   msgWindowW = titleWindowW - choiceWindowW - (brdrW<<1);
    CalcStatusSubWinGeom ();
    if (canvasWindowOnly)
       XResizeWindow (mainDisplay, drawWindow, drawWinW, drawWinH);
@@ -407,23 +429,37 @@ void Reconfigure (Forced)
       }
       XMoveResizeWindow (mainDisplay, msgWindow, 0, cur_y,
             msgWindowW, msgWindowH);
-      XMoveWindow (mainDisplay, choiceWindow, msgWindowW+2*brdrW, cur_y);
+      XMoveWindow (mainDisplay, choiceWindow, msgWindowW+(brdrW<<1), cur_y);
       cur_y += choiceWindowH + (brdrW<<1);
       XMoveResizeWindow (mainDisplay, dummyWindow1, 0, cur_y,
             drawWinW, rulerW);
       XMoveResizeWindow (mainDisplay, hRuleWindow, rulerW+(brdrW<<1), cur_y,
             drawWinW, rulerW);
-      XMoveResizeWindow (mainDisplay, vSBarWindow, drawWinW+rulerW+4*brdrW,
-            cur_y, scrollBarW, vSBarH);
+      if (colorLayers) {
+         XMoveResizeWindow (mainDisplay, colorWindow,
+               drawWinW+rulerW+(brdrW<<2), cur_y, scrollBarW, colorWindowH);
+         XMoveWindow (mainDisplay, colorDummyWindow,
+               drawWinW+rulerW+(brdrW<<2), cur_y+colorWindowH+(brdrW<<1));
+         XMoveResizeWindow (mainDisplay, vSBarWindow,
+               drawWinW+rulerW+(brdrW<<2),
+               cur_y+colorWindowH+colorDummyWindowH+(brdrW<<2), scrollBarW,
+               DEF_VSBAR_H);
+      } else {
+         XMoveResizeWindow (mainDisplay, vSBarWindow,
+               drawWinW+rulerW+(brdrW<<2), cur_y, scrollBarW, vSBarH);
+      }
       cur_y += rulerW + (brdrW<<1);
       XMoveResizeWindow (mainDisplay, vRuleWindow, 0, cur_y,
             rulerW, drawWinH);
       XMoveResizeWindow (mainDisplay, drawWindow, rulerW+(brdrW<<1), cur_y,
             drawWinW, drawWinH);
       cur_y += drawWinH + (brdrW<<1);
-      XMoveResizeWindow (mainDisplay, hSBarWindow, 0, cur_y, hSBarW,
-            scrollBarW);
-      XMoveWindow (mainDisplay, dummyWindow2, drawWinW+rulerW+4*brdrW,
+      XMoveResizeWindow (mainDisplay, pageWindow, 0, cur_y, pageWindowW,
+            scrollBarW+(brdrW<<1));
+      XMoveWindow (mainDisplay, pageDummyWindow, pageWindowW, cur_y);
+      XMoveWindow (mainDisplay, hSBarWindow,
+            pageWindowW+pageDummyWindowW+(brdrW<<1), cur_y);
+      XMoveWindow (mainDisplay, dummyWindow2, drawWinW+rulerW+(brdrW<<2),
             cur_y);
       cur_y += scrollBarW + (brdrW<<1);
       if (!noStatusWindow)
@@ -485,36 +521,44 @@ int mainWinEventHandler (input)
    }
 
    if (configure) Reconfigure (FALSE);
+   if (showCrossHair) {
+      SetNullCursor(drawWindow);
+   }
    return (INVALID);
 }
 
 void InitPaperSize ()
 {
-   psYOffStr = (char * *) calloc (MAXPAGESTYLES, sizeof(char *));
-   psYOffStr[0] = (char *) calloc (20, sizeof(char));
-   psYOffStr[1] = (char *) calloc (2, sizeof(char));
+   psYOffStr = (char**)malloc(MAXPAGESTYLES*sizeof(char *));
+   if (psYOffStr == NULL) FailAllocMessage();
+   psYOffStr[0] = (char*)malloc(20*sizeof(char));
+   psYOffStr[1] = (char*)malloc(2*sizeof(char));
+   if (psYOffStr[0] == NULL || psYOffStr[1] == NULL) FailAllocMessage();
    sprintf (psYOffStr[0], "%.2f",
          (float)(((float)onePageHeight)/((float)PIX_PER_INCH)));
    sprintf (psYOffStr[1], "0");
-   psYOff = (float *) calloc (MAXPAGESTYLES, sizeof(float));
+   psYOff = (float*)malloc(MAXPAGESTYLES*sizeof(float));
+   if (psYOff == NULL) FailAllocMessage();
    psYOff[0] = ((float)onePageHeight)/((float)PIX_PER_INCH);
    psYOff[1] = (float)0.0;
-   psPageWidthInInch = (float *) calloc (MAXPAGESTYLES, sizeof(float));
+   psPageWidthInInch = (float*)malloc(MAXPAGESTYLES*sizeof(float));
+   if (psPageWidthInInch == NULL) FailAllocMessage();
    psPageWidthInInch[0] = ((float)onePageWidth)/((float)PIX_PER_INCH);
    psPageWidthInInch[1] = ((float)onePageHeight)/((float)PIX_PER_INCH);
-   psPageHeightInInch = (float *) calloc (MAXPAGESTYLES, sizeof(float));
+   psPageHeightInInch = (float*)malloc(MAXPAGESTYLES*sizeof(float));
+   if (psPageHeightInInch == NULL) FailAllocMessage();
    psPageHeightInInch[0] = ((float)onePageHeight)/((float)PIX_PER_INCH);
    psPageHeightInInch[1] = ((float)onePageWidth)/((float)PIX_PER_INCH);
 }
 
 void CleanUpPaperSize ()
 {
-   cfree (psPageHeightInInch);
-   cfree (psPageWidthInInch);
-   cfree (psYOff);
-   cfree (psYOffStr[0]);
-   cfree (psYOffStr[1]);
-   cfree (psYOffStr);
+   free(psPageHeightInInch);
+   free(psPageWidthInInch);
+   free(psYOff);
+   free(psYOffStr[0]);
+   free(psYOffStr[1]);
+   free(psYOffStr);
    psYOffStr = NULL;
    psYOff = psPageWidthInInch = psPageHeightInInch = NULL;
 }
@@ -535,9 +579,11 @@ void Setup ()
       if ((strcmp (c_ptr, "on") == 0) || (strcmp (c_ptr, "On") == 0))
          XSynchronize (mainDisplay, True);
 
-   if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME, "DontShowVersion"))!=NULL)
-      if ((strcmp (c_ptr, "true") == 0) || (strcmp (c_ptr, "True") == 0))
-         showVersion = FALSE;
+   showVersion = FALSE;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "DontShowVersion")) != NULL &&
+         UtilStrICmp(c_ptr, "false") == 0) {
+      showVersion = TRUE;
+   }
 
    if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME, "PrintCommand")) != NULL)
       strcpy (printCommand, c_ptr);
@@ -577,6 +623,8 @@ void Setup ()
          whereToPrint = EPSI_FILE;
       else if (strcmp (c_ptr, "GIF") == 0)
          whereToPrint = GIF_FILE;
+      else if (strcmp (c_ptr, "HTML") == 0)
+         whereToPrint = HTML_FILE;
       else
          fprintf (stderr, "Unrecognized WhereToPrint:  %s\n", c_ptr);
    }
@@ -746,11 +794,9 @@ void Setup ()
    }
 
    groupedTextEditable = FALSE;
-   if ((c_ptr = XGetDefault (mainDisplay, TOOL_NAME,
-         "GroupedTextEditable")) != NULL)
-   {
-      if (strcmp ("true", c_ptr) == 0 || strcmp ("True", c_ptr) == 0)
-         groupedTextEditable = TRUE;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "GroupedTextEditable")) !=
+         NULL && UtilStrICmp(c_ptr, "true") == 0) {
+      groupedTextEditable = TRUE;
    }
 
    intrCheckInterval = 10;
@@ -924,6 +970,21 @@ void Setup ()
       }
    }
 
+   rotationIncrement = (45<<6); /* initially 45 degrees */
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"RotationIncrement")) != NULL) {
+      float fval;
+
+      if (sscanf(c_ptr, "%f", &fval) != 1 || fval < (float)0.0) {
+         fprintf (stderr, "Invalid %s*%s: '%s', 45 is used.\n",
+               TOOL_NAME, "RotationIncrement", c_ptr);
+      } else {
+         while (fval < (float)360.0) fval += (float)360.0;
+         while (fval >= (float)360.0) fval -= (float)360.0;
+         fval *= 64.0;
+         rotationIncrement = round(fval);
+      }
+   }
+
    initialMenubarWindowH = MENUBAR_WINDOW_H;
 
    strcpy (scanFileName, "No File");
@@ -1038,7 +1099,11 @@ void Setup ()
 
    InitFonts ();
    initialMenubarWindowH = defaultFontHeight;
-   statusWindowH = max(defaultFontHeight+(brdrW<<1)+4, btn1_height+2);
+   if (msgFontPtr == NULL) {
+      statusWindowH = max(defaultFontHeight+(brdrW<<1)+4, btn1_height+2);
+   } else {
+      statusWindowH = max(msgFontHeight+(brdrW<<1)+4, btn1_height+2);
+   }
    InitWinSizes ();
 
    if (canvasWindowOnly)
@@ -1069,13 +1134,13 @@ void Setup ()
       else
          sizehints.height = sizehints.min_height = mainWinH;
 
-      drawWinW = initDrawWinW = mainWinW-2*brdrW;
-      drawWinH = initDrawWinH = mainWinH-2*brdrW;
+      drawWinW = initDrawWinW = mainWinW-(brdrW<<1);
+      drawWinH = initDrawWinH = mainWinH-(brdrW<<1);
       if (msgWindowW <= 0) msgWindowW = 2;
    }
    else
    {
-      sizehints.min_width = choiceWindowW+4*brdrW+defaultFontWidth;
+      sizehints.min_width = choiceWindowW+(brdrW<<2)+defaultFontWidth;
       sizehints.min_height =
             (noMenubar ? 0 : menubarWindowH+(brdrW<<1))+
             (noStatusWindow ? 0 : statusWindowH+(brdrW<<1))+
@@ -1141,7 +1206,7 @@ void Setup ()
       Error ("setup", "Could not create the message window");
 
    if ((choiceWindow = XCreateSimpleWindow (mainDisplay, mainWindow,
-         msgWindowW+2*brdrW, cur_y, choiceWindowW, choiceWindowH,
+         msgWindowW+(brdrW<<1), cur_y, choiceWindowW, choiceWindowH,
          brdrW, myBorderPixel, myBgPixel)) == 0)
       Error ("setup", "Could not create the choice window");
    cur_y += msgWindowH + (brdrW<<1);
@@ -1153,13 +1218,24 @@ void Setup ()
       Error ("setup", "Could not create the left top dummy window");
 
    if ((hRuleWindow = XCreateSimpleWindow (mainDisplay, mainWindow,
-         rulerW+2*brdrW, cur_y, drawWinW, rulerW,
+         rulerW+(brdrW<<1), cur_y, drawWinW, rulerW,
          brdrW, myBorderPixel, myBgPixel)) == 0)
       Error ("setup", "Could not create the horizontal ruler window");
 
+   if ((colorWindow = XCreateSimpleWindow (mainDisplay, mainWindow,
+         (brdrW<<2)+rulerW+drawWinW, cur_y, scrollBarW, colorWindowH,
+         brdrW, myBorderPixel, myBgPixel)) == 0)
+      Error ("setup", "Could not create the color window");
+
+   if ((colorDummyWindow = XCreateSimpleWindow (mainDisplay, mainWindow,
+         (brdrW<<2)+rulerW+drawWinW, cur_y+colorWindowH+(brdrW<<1), scrollBarW,
+         colorDummyWindowH, brdrW, myBorderPixel, myBgPixel)) == 0)
+      Error ("setup", "Could not create the color dummy window");
+
    if ((vSBarWindow = XCreateSimpleWindow (mainDisplay, mainWindow,
-         4*brdrW+rulerW+drawWinW, cur_y, scrollBarW, vSBarH,
-         brdrW, myBorderPixel, myBgPixel)) == 0) 
+         (brdrW<<2)+rulerW+drawWinW,
+         colorLayers ? cur_y+colorWindowH+colorDummyWindowH+(brdrW<<2) : cur_y,
+         scrollBarW, vSBarH, brdrW, myBorderPixel, myBgPixel)) == 0) 
       Error ("setup", "Could not create the vertical scrollbar window");
    cur_y += rulerW + (brdrW<<1);
 
@@ -1168,17 +1244,27 @@ void Setup ()
       Error ("setup", "Could not create the vertical ruler window");
 
    if ((drawWindow = XCreateSimpleWindow (mainDisplay, mainWindow,
-         rulerW+2*brdrW, cur_y, drawWinW, drawWinH,
-         brdrW, myBorderPixel, myBgPixel)) == 0) 
+         rulerW+(brdrW<<1), cur_y, drawWinW, drawWinH,
+         0, myBorderPixel, myBgPixel)) == 0) 
       Error ("setup", "Could not create the drawing window");
    cur_y += drawWinH + (brdrW<<1);
 
-   if ((hSBarWindow = XCreateSimpleWindow (mainDisplay, mainWindow, 0, cur_y,
+   if ((pageWindow = XCreateSimpleWindow (mainDisplay, mainWindow, 0, cur_y,
+         pageWindowW, scrollBarW+(brdrW<<1), 0, myBorderPixel, myBgPixel)) == 0)
+      Error ("setup", "Could not create the page window");
+
+   if ((pageDummyWindow = XCreateSimpleWindow (mainDisplay, mainWindow,
+         pageWindowW, cur_y, pageDummyWindowW, scrollBarW, brdrW,
+         myBorderPixel, myBgPixel)) == 0) 
+      Error ("setup", "Could not create the page dummy window");
+
+   if ((hSBarWindow = XCreateSimpleWindow (mainDisplay, mainWindow,
+         pageWindowW+pageDummyWindowW+(brdrW<<1), cur_y,
          hSBarW, scrollBarW, brdrW, myBorderPixel, myBgPixel)) == 0) 
       Error ("setup", "Could not create the horizontal scrollbar window");
 
    if ((dummyWindow2 = XCreateSimpleWindow (mainDisplay, mainWindow,
-         rulerW+drawWinW+4*brdrW, cur_y,
+         rulerW+drawWinW+(brdrW<<2), cur_y,
          scrollBarW, scrollBarW, brdrW, myBorderPixel, myBgPixel)) == 0) 
       Error ("setup", "Could not create the right bottom dummy window");
    cur_y += scrollBarW + (brdrW<<1);
@@ -1275,18 +1361,40 @@ void Setup ()
             PointerMotionMask | EnterWindowMask);
 
       XMapWindow (mainDisplay, hRuleWindow);
-      XSelectInput (mainDisplay, hRuleWindow, ExposureMask | EnterWindowMask);
+      XSelectInput (mainDisplay, hRuleWindow,
+            ButtonPressMask | ExposureMask | EnterWindowMask);
       XMapWindow (mainDisplay, vRuleWindow);
-      XSelectInput (mainDisplay, vRuleWindow, ExposureMask | EnterWindowMask);
+      XSelectInput (mainDisplay, vRuleWindow,
+            ButtonPressMask | ExposureMask | EnterWindowMask);
 
       XMapWindow (mainDisplay, drawWindow); 
       XSelectInput (mainDisplay, drawWindow, ButtonReleaseMask |
             ButtonPressMask | PointerMotionMask | KeyPressMask |
             KeyReleaseMask | ExposureMask | EnterWindowMask);
 
+      if (colorLayers) {
+         XMapWindow(mainDisplay, colorWindow);
+         XSelectInput(mainDisplay, colorWindow, ButtonPressMask |
+               PointerMotionMask | ExposureMask | EnterWindowMask);
+         XMapWindow(mainDisplay, colorDummyWindow);
+         XSelectInput(mainDisplay, colorDummyWindow, ButtonPressMask |
+               ExposureMask | EnterWindowMask);
+      } else {
+         XSelectInput(mainDisplay, colorWindow, ButtonPressMask |
+               PointerMotionMask | ExposureMask | EnterWindowMask);
+         XSelectInput(mainDisplay, colorDummyWindow, ButtonPressMask |
+               ExposureMask | EnterWindowMask);
+      }
       XMapWindow (mainDisplay, vSBarWindow);
       XSelectInput (mainDisplay, vSBarWindow, ButtonPressMask | ExposureMask |
             EnterWindowMask);
+
+      XMapWindow (mainDisplay, pageWindow);
+      XSelectInput (mainDisplay, pageWindow, ButtonPressMask |
+            PointerMotionMask | ExposureMask | EnterWindowMask);
+      XMapWindow (mainDisplay, pageDummyWindow);
+      XSelectInput (mainDisplay, pageDummyWindow, ButtonPressMask |
+            ExposureMask | EnterWindowMask);
       XMapWindow (mainDisplay, hSBarWindow);
       XSelectInput (mainDisplay, hSBarWindow, ButtonPressMask | ExposureMask |
             EnterWindowMask);
@@ -1308,7 +1416,7 @@ void Setup ()
          XSelectInput (mainDisplay, statusWindow, ExposureMask |
                EnterWindowMask);
    }
-#else
+#else /* ~MAPBEFORESELECT */
    if (canvasWindowOnly)
    {
       XSelectInput (mainDisplay, drawWindow, ButtonReleaseMask |
@@ -1338,9 +1446,11 @@ void Setup ()
             PointerMotionMask | EnterWindowMask);
       XMapWindow (mainDisplay, choiceWindow);
 
-      XSelectInput (mainDisplay, hRuleWindow, ExposureMask | EnterWindowMask);
+      XSelectInput (mainDisplay, hRuleWindow,
+            ButtonPressMask | ExposureMask | EnterWindowMask);
       XMapWindow (mainDisplay, hRuleWindow);
-      XSelectInput (mainDisplay, vRuleWindow, ExposureMask | EnterWindowMask);
+      XSelectInput (mainDisplay, vRuleWindow,
+            ButtonPressMask | ExposureMask | EnterWindowMask);
       XMapWindow (mainDisplay, vRuleWindow);
 
       XSelectInput (mainDisplay, drawWindow, ButtonReleaseMask |
@@ -1348,9 +1458,29 @@ void Setup ()
             KeyReleaseMask | ExposureMask | EnterWindowMask);
       XMapWindow (mainDisplay, drawWindow); 
 
+      if (colorLayers) {
+         XSelectInput(mainDisplay, colorWindow, ButtonPressMask |
+               PointerMotionMask | ExposureMask | EnterWindowMask);
+         XMapWindow(mainDisplay, colorWindow);
+         XSelectInput(mainDisplay, colorDummyWindow, ButtonPressMask |
+               ExposureMask | EnterWindowMask);
+         XMapWindow(mainDisplay, colorDummyWindow);
+      } else {
+         XSelectInput(mainDisplay, colorWindow, ButtonPressMask |
+               PointerMotionMask | ExposureMask | EnterWindowMask);
+         XSelectInput(mainDisplay, colorDummyWindow, ButtonPressMask |
+               ExposureMask | EnterWindowMask);
+      }
       XSelectInput (mainDisplay, vSBarWindow, ButtonPressMask | ExposureMask |
             EnterWindowMask);
       XMapWindow (mainDisplay, vSBarWindow);
+
+      XSelectInput (mainDisplay, pageWindow, ButtonPressMask |
+            PointerMotionMask | ExposureMask | EnterWindowMask);
+      XMapWindow (mainDisplay, pageWindow);
+      XSelectInput (mainDisplay, pageDummyWindow, ButtonPressMask |
+            ExposureMask | EnterWindowMask);
+      XMapWindow (mainDisplay, pageDummyWindow);
       XSelectInput (mainDisplay, hSBarWindow, ButtonPressMask | ExposureMask |
             EnterWindowMask);
       XMapWindow (mainDisplay, hSBarWindow);
@@ -1376,6 +1506,7 @@ void Setup ()
 
    wmhints.flags = InputHint;
    wmhints.input = True;
+   wmhints.icon_pixmap = None;
 
    if (iconWindowCreated)
    {
@@ -1383,9 +1514,9 @@ void Setup ()
 
       sizehints.flags = PSize | PMinSize | PMaxSize;
       sizehints.width = sizehints.min_width = sizehints.max_width =
-            iconWindowW+2*brdrW;
+            iconWindowW+(brdrW<<1);
       sizehints.height = sizehints.min_height = sizehints.max_height =
-            iconWindowH+2*brdrW;
+            iconWindowH+(brdrW<<1);
 
       wmhints.flags |= IconWindowHint;
       wmhints.icon_window = iconBaseWindow;
@@ -1397,9 +1528,9 @@ void Setup ()
          if ((bitmask & XValue) && (bitmask & YValue))
          {
             if (bitmask & XNegative) sizehints.x += DisplayWidth (mainDisplay,
-                  mainScreen) - iconWindowW - 2*brdrW - 1;
+                  mainScreen) - iconWindowW - (brdrW<<1) - 1;
             if (bitmask & YNegative) sizehints.y += DisplayHeight (mainDisplay,
-                  mainScreen) - iconWindowH - 2*brdrW - 1;
+                  mainScreen) - iconWindowH - (brdrW<<1) - 1;
 
             sizehints.flags |= USPosition;
 
@@ -1429,6 +1560,9 @@ void Setup ()
       }
    }
    XSetWMHints (mainDisplay, mainWindow, &wmhints);
+   if ((wmhints.flags & IconPixmapHint) && wmhints.icon_pixmap != None) {
+      XFreePixmap(mainDisplay, wmhints.icon_pixmap);
+   }
 
    wmhints.flags = InputHint;
    wmhints.input = True;
@@ -1436,6 +1570,10 @@ void Setup ()
 
    InitText ();
    InitRemote ();
+   InitImageProc ();
+   InitImport ();
+   InitHelp ();
+   InitShape ();
 }
 
 void CleanUpResiduals ()
