@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,7 +29,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/special.c,v 2.100 1995/05/15 01:23:12 william Exp $";
+      "@(#)$Header: /n/opus/u/guest/william/src/tgif/v3/RCS/special.c,v 3.0 1996/05/06 16:07:43 william Exp $";
 #endif
 
 #include <stdio.h>
@@ -44,7 +44,7 @@ static char RCSid[] =
 #include "choice.e"
 #include "cmd.e"
 #include "color.e"
-#include "copypaste.e"
+#include "cutpaste.e"
 #include "cursor.e"
 #include "dialog.e"
 #include "drawing.e"
@@ -72,8 +72,14 @@ static char RCSid[] =
 #include "special.e"
 #endif
 #include "stk.e"
+#include "text.e"
+#ifdef _TGIF_WB
+#include "wb1.e"
+#endif /* _TGIF_WB */
 #include "util.e"
 
+int placingTopObj=FALSE;
+
 char * specialMenuStr[] =
    {  "MakeSymbolic   ^#m",
       "UnMakeSymbolic ^#n",
@@ -99,7 +105,11 @@ char * specialMenuStr[] =
       "ImportAttrs       ",
       "ExportAttrs       ",
       "MergeWithTable    ",
-      "ExportToTable     "
+      "ExportToTable     ",
+#ifdef _TGIF_WB
+      "WhiteBoard        ",
+#endif /* _TGIF_WB */
+      NULL
    };
 static char * specialMenuDescription[] =
    {  "Turn a selected grouped/icon object into a symbol object",
@@ -127,6 +137,9 @@ static char * specialMenuDescription[] =
       "Export attributes of a selected object to a text file",
       "Merge selected object with a table file",
       "Export attributes of selected objects to a table file",
+#ifdef _TGIF_WB
+      "Start/Enable the White Board communication",
+#endif /* _TGIF_WB */
       NULL
    };
 
@@ -142,25 +155,27 @@ struct ObjRec * ReadSymbol (FP)
 
    importingFile = TRUE; /* ignore 'state' info but set fileVersion */
    readingPageNum = 0;
-   while ((read_status = ReadObj (FP, &obj_ptr)) == TRUE)
-      if (obj_ptr != NULL)
-      {
-         if (obj_ptr->type == OBJ_SYM)
-         {
-            UnlockAnObj (obj_ptr);
+   while ((read_status=ReadObj(FP, &obj_ptr)) == TRUE) {
+      if (obj_ptr != NULL) {
+         if (obj_ptr->type == OBJ_SYM) {
+            UnlockAnObj(obj_ptr);
             obj_ptr->type = OBJ_ICON;
             attr_ptr = obj_ptr->lattr;
             for ( ; attr_ptr != NULL; attr_ptr = attr_ptr->prev)
                attr_ptr->inherited = TRUE;
             importingFile = FALSE;
-            SetDefaultCursor (mainWindow);
-            SetDefaultCursor (drawWindow);
-            return (obj_ptr);
+            SetDefaultCursor(mainWindow);
+            SetDefaultCursor(drawWindow);
+            return obj_ptr;
+         } else {
+            FreeObj(obj_ptr);
          }
-         else
-            FreeObj (obj_ptr);
       }
+   }
    importingFile = FALSE;
+   if (!PRTGIF && colorLayers && needToRedrawColorWindow) {
+      RedrawColorWindow();
+   }
    if (read_status == INVALID)
    {
       sprintf (gszMsgBox,
@@ -236,6 +251,7 @@ unsigned int PlaceTopObj (ObjPtr)
    XFlush (mainDisplay);
    XSync (mainDisplay, False);
 
+   placingTopObj = TRUE;
    if (XCheckMaskEvent (mainDisplay, ExposureMask, &ev) ||
          XCheckMaskEvent (mainDisplay, VisibilityChangeMask, &ev))
       ExposeEventHandler (&ev, TRUE);
@@ -245,7 +261,7 @@ unsigned int PlaceTopObj (ObjPtr)
    GridXY (obj_ltx, obj_lty, &grid_obj_ltx, &grid_obj_lty);
 
    dx = dy = 0;
-   XSetInputFocus (mainDisplay, drawWindow, RevertToNone, CurrentTime);
+/* XSetInputFocus (mainDisplay, drawWindow, RevertToNone, CurrentTime); */
    XGrabPointer (mainDisplay, drawWindow, False,
          PointerMotionMask | ButtonPressMask,
          GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
@@ -304,6 +320,8 @@ unsigned int PlaceTopObj (ObjPtr)
          MoveObj (ObjPtr, grid_dx, grid_dy);
          numRedrawBBox = 0;
          ShowInterrupt (1);
+         placingTopObj = FALSE;
+         ObjPtr->tmp_parent = NULL;
          DrawObj (drawWindow, ObjPtr);
          HideInterrupt();
       }
@@ -323,6 +341,7 @@ unsigned int PlaceTopObj (ObjPtr)
       }
    }
    XSync (mainDisplay, True);
+   placingTopObj = FALSE;
    return button_pressed;
 }
 
@@ -333,8 +352,8 @@ int UnMakeIconicOnInstantiate (obj_ptr)
    register struct AttrRec	* attr_ptr;
 
    for (attr_ptr=obj_ptr->lattr; attr_ptr!=NULL; attr_ptr=attr_ptr->prev)
-      if (*attr_ptr->name=='\0' &&
-            strcmp (attr_ptr->s, "unmakeiconic_on_instantiate")==0)
+      if (*attr_ptr->attr_name.s=='\0' &&
+            strcmp (attr_ptr->attr_value.s, "unmakeiconic_on_instantiate")==0)
          return (TRUE);
    return (FALSE);
 }
@@ -347,7 +366,7 @@ void DeleteUnMakeIconicOnInstantiateText ()
 
    for (sel_ptr=botSel; sel_ptr != NULL; sel_ptr=sel_ptr->prev)
       if (sel_ptr->obj->type == OBJ_TEXT &&
-            strcmp (sel_ptr->obj->detail.t->first->s,
+            strcmp (sel_ptr->obj->detail.t->first->dyn_str.s,
             "unmakeiconic_on_instantiate") == 0)
       {
          text_obj_ptr = sel_ptr->obj;
@@ -367,7 +386,8 @@ void DeleteUnMakeIconicOnInstantiateText ()
       return;
    }
    MakeQuiescent ();
-   topSel = (struct SelRec *) calloc (1, sizeof(struct SelRec));
+   topSel = (struct SelRec *)malloc(sizeof(struct SelRec));
+   if (topSel == NULL) FailAllocMessage();
    topSel->next = NULL;
    topSel->obj = text_obj_ptr;
    topSel->prev = NULL;
@@ -650,7 +670,7 @@ void MakeIconic ()
       if (writeFileFailed)
       {
          writeFileFailed = FALSE;
-         sprintf (s, "%s '%s'.  %s.  '%s' deleted.",
+         sprintf (s, "%s '%s'.\n\n%s.\n\n'%s' deleted.",
                "Fail to write to", file_name, "File system may be full",
                file_name);
          MsgBox (s, TOOL_NAME, INFO_MB);
@@ -736,7 +756,8 @@ void AddLine(buf)
 {
    struct LineRec *line_ptr;
 
-   line_ptr=(struct LineRec *)calloc(1, sizeof(struct LineRec));
+   line_ptr = (struct LineRec *)malloc(sizeof(struct LineRec));
+   if (line_ptr == NULL) FailAllocMessage();
    line_ptr->s = buf;
    line_ptr->next = NULL;
    line_ptr->prev = lastLine;
@@ -800,7 +821,7 @@ void ImportAnAttr(obj_ptr, attr_line_num, fname)
       } else {
          struct StrRec *next_s_ptr;
 
-         UtilStrCpy(attr_ptr->s, sizeof(attr_ptr->s), c_ptr);
+         DynStrSet(&attr_ptr->attr_value, c_ptr);
          text_ptr = attr_ptr->obj->detail.t;
          text_ptr->cached_zoom = 0;
          if (text_ptr->cached_bitmap != None) {
@@ -810,18 +831,18 @@ void ImportAnAttr(obj_ptr, attr_line_num, fname)
          for (s_ptr=text_ptr->first->next; s_ptr != NULL;
                s_ptr=next_s_ptr) {
             next_s_ptr = s_ptr->next;
-            cfree(s_ptr);
+            free(s_ptr);
          }
          text_ptr->last = text_ptr->first;
          text_ptr->first->next = text_ptr->first->prev = NULL;
          text_ptr->lines = 1;
          UpdAttr(attr_ptr);
       }
-      cfree(attr_name);
+      free(attr_name);
       for (line_ptr=firstLine->next; line_ptr!=NULL; line_ptr=line_ptr->next) {
-         s_ptr = (struct StrRec *)calloc(1, sizeof(struct StrRec));
+         s_ptr = (struct StrRec *)malloc(sizeof(struct StrRec));
          if (s_ptr == NULL) FailAllocMessage();
-         UtilStrCpy(s_ptr->s, sizeof(s_ptr->s), line_ptr->s);
+         DynStrSet(&s_ptr->dyn_str, line_ptr->s);
          s_ptr->prev = text_ptr->last;
          s_ptr->next = NULL;
          text_ptr->last->next = s_ptr;
@@ -835,8 +856,8 @@ void ImportAnAttr(obj_ptr, attr_line_num, fname)
    }
    for ( ; firstLine != NULL; firstLine=next_line) {
       next_line = firstLine->next;
-      if (firstLine->s != NULL) cfree(firstLine->s);
-      cfree(firstLine);
+      if (firstLine->s != NULL) free(firstLine->s);
+      free(firstLine);
    }
    lastLine = NULL;
    numLines = 0;
@@ -988,13 +1009,14 @@ void ExportAttrs()
       struct StrRec *s_ptr;
 
       if (attr_ptr != topSel->obj->lattr) fprintf(fp, "\n");
-      if (fprintf(fp, "%s%s\n", attr_ptr->name, attr_ptr->s) < 0) {
+      if (fprintf(fp, "%s%s\n", attr_ptr->attr_name.s,
+            attr_ptr->attr_value.s) < 0) {
          writeFileFailed = TRUE;
       }
       if (!writeFileFailed && (s_ptr=attr_ptr->obj->detail.t->first) != NULL) {
          for (s_ptr=s_ptr->next; !writeFileFailed && s_ptr != NULL;
                s_ptr=s_ptr->next) {
-            if (fprintf(fp, "%s\n", s_ptr->s) < 0) {
+            if (fprintf(fp, "%s\n", s_ptr->dyn_str.s) < 0) {
                writeFileFailed = TRUE;
             }
          }
@@ -1029,13 +1051,13 @@ int GetMergeSpec(obj_ptr, attr_name, pn_val, distance)
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return FALSE;
    }
-   if (distance && ((c_ptr=strstr(attr_ptr->s, "in")) != NULL ||
-         (c_ptr=strstr(attr_ptr->s, "In")) != NULL ||
-         (c_ptr=strstr(attr_ptr->s, "IN")) != NULL)) {
+   if (distance && ((c_ptr=strstr(attr_ptr->attr_value.s, "in")) != NULL ||
+         (c_ptr=strstr(attr_ptr->attr_value.s, "In")) != NULL ||
+         (c_ptr=strstr(attr_ptr->attr_value.s, "IN")) != NULL)) {
       char saved_ch=(*c_ptr);
 
       *c_ptr = '\0';
-      if (sscanf(attr_ptr->s, "%lf", &val) != 1) {
+      if (sscanf(attr_ptr->attr_value.s, "%lf", &val) != 1) {
          *c_ptr = saved_ch;
          sprintf(gszMsgBox, "Malformed '%s' attribute for MergeWithTable.",
                attr_name);
@@ -1045,13 +1067,14 @@ int GetMergeSpec(obj_ptr, attr_name, pn_val, distance)
       *c_ptr = saved_ch;
       val = val * ((double)PIX_PER_INCH);
       *pn_val = round(val);
-   } else if (distance && ((c_ptr=strstr(attr_ptr->s, "cm")) != NULL ||
-         (c_ptr=strstr(attr_ptr->s, "Cm")) != NULL ||
-         (c_ptr=strstr(attr_ptr->s, "CM")) != NULL)) {
+   } else if (distance &&
+         ((c_ptr=strstr(attr_ptr->attr_value.s, "cm")) != NULL ||
+         (c_ptr=strstr(attr_ptr->attr_value.s, "Cm")) != NULL ||
+         (c_ptr=strstr(attr_ptr->attr_value.s, "CM")) != NULL)) {
       char saved_ch=(*c_ptr);
 
       *c_ptr = '\0';
-      if (sscanf(attr_ptr->s, "%lf", &val) != 1) {
+      if (sscanf(attr_ptr->attr_value.s, "%lf", &val) != 1) {
          *c_ptr = saved_ch;
          sprintf(gszMsgBox, "Malformed '%s' attribute for MergeWithTable.",
                attr_name);
@@ -1062,7 +1085,7 @@ int GetMergeSpec(obj_ptr, attr_name, pn_val, distance)
       val = val * ((double)ONE_CM);
       *pn_val = round(val);
    } else {
-      if (sscanf(attr_ptr->s, "%d", pn_val) != 1) {
+      if (sscanf(attr_ptr->attr_value.s, "%d", pn_val) != 1) {
          sprintf(gszMsgBox, "Malformed '%s' attribute for MergeWithTable.",
                attr_name);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
@@ -1080,6 +1103,7 @@ void MergeWithTable()
    int i, short_name, columns_in_file, ok=TRUE, line_num, user_placement;
    int left_margin=0, top_margin=0, v_pitch=0, h_pitch=0, strip_double_quotes;
    int num_cols=0, num_rows=0, paper_w=0, paper_h=0, x, y, r, c;
+   int tab_separated=TRUE;
    FILE *fp;
    struct ObjRec *template_obj;
 
@@ -1150,10 +1174,16 @@ void MergeWithTable()
    columns_in_file = 1;
    val_ptr = spec;
    c_ptr = strchr(val_ptr, '\t');
+   if (c_ptr == NULL && (c_ptr=strchr(val_ptr, ';')) != NULL) {
+      tab_separated = FALSE;
+      Msg("';'s are used as separators.");
+   } else {
+      Msg("<TAB>s are used as separators.");
+   }
    while (val_ptr != NULL) {
       if (c_ptr != NULL) *c_ptr = '\0';
       if (strlen(val_ptr) == 0) {
-         cfree(spec);
+         free(spec);
          fclose(fp);
          sprintf(gszMsgBox,
                "Malformed column names in '%s'.\n\nMerging aborted.",
@@ -1162,16 +1192,15 @@ void MergeWithTable()
          return;
       }
       if (c_ptr == NULL) break;
-      *c_ptr++ = '\t';
+      *c_ptr++ = (tab_separated ? '\t' : ';');
       val_ptr = c_ptr;
-      c_ptr = strchr(val_ptr, '\t');
+      c_ptr = strchr(val_ptr, tab_separated ? '\t' : ';');
 
       columns_in_file++;
    }
-   col_names = (char**)calloc(columns_in_file+1, sizeof(char*));
+   col_names = (char**)malloc((columns_in_file+1)*sizeof(char*));
    if (col_names == NULL) {
-      if (col_names != NULL) cfree(col_names);
-      cfree(spec);
+      free(spec);
       fclose(fp);
       FailAllocMessage();
       return;
@@ -1179,35 +1208,47 @@ void MergeWithTable()
    col_names[columns_in_file] = NULL;
    i = 0;
    val_ptr = spec;
-   for (c_ptr=strchr(val_ptr, '\t'); val_ptr != NULL; i++) {
+   for (c_ptr=strchr(val_ptr, tab_separated ? '\t' : ';'); val_ptr != NULL;
+         i++) {
       int len;
 
       if (c_ptr != NULL) *c_ptr = '\0';
+      if (!tab_separated) {
+         UtilTrimBlanks(val_ptr);
+         if (strip_double_quotes && *val_ptr == '"') {
+            len = strlen(val_ptr);
+
+            if (val_ptr[len-1] == '"') {
+               val_ptr[len-1] = '\0';
+               val_ptr++;
+            }
+         }
+      }
       len = strlen(val_ptr);
-      if ((col_names[i]=(char*)calloc(len+2, sizeof(char))) == NULL) {
+      if ((col_names[i]=(char*)malloc((len+2)*sizeof(char))) == NULL) {
          FailAllocMessage();
          columns_in_file = i;
          for (i=0; i < columns_in_file; i++) {
             if (col_names[i] != NULL) {
-               cfree(col_names[i]);
+               free(col_names[i]);
             }
          }
-         cfree(col_names);
-         cfree(spec);
+         free(col_names);
+         free(spec);
          fclose(fp);
          return;
       }
       sprintf(col_names[i], "%s=", val_ptr);
       if (FindAttrWithName(template_obj, col_names[i], NULL) == NULL) {
-         cfree(col_names[i]);
+         free(col_names[i]);
          col_names[i] = NULL;
       }
       if (c_ptr == NULL) break;
-      *c_ptr++ = '\t';
+      *c_ptr++ = (tab_separated ? '\t' : ';');
       val_ptr = c_ptr;
-      c_ptr = strchr(val_ptr, '\t');
+      c_ptr = strchr(val_ptr, tab_separated ? '\t' : ';');
    }
-   cfree(spec);
+   free(spec);
 
    MakeQuiescent();
    UnlinkObj(template_obj);
@@ -1246,7 +1287,7 @@ void MergeWithTable()
 
       if (obj_ptr == NULL) {
          ok = FALSE;
-         cfree(buf);
+         free(buf);
          break;
       }
       sprintf(gszMsgBox, "Processing object %1d...", line_num);
@@ -1261,13 +1302,15 @@ void MergeWithTable()
       i = 0;
       val_ptr = buf;
       replaceAttrFirstValueRedraw = FALSE;
-      for (c_ptr=strchr(val_ptr, '\t'); val_ptr != NULL; i++) {
+      for (c_ptr=strchr(val_ptr, tab_separated ? '\t' : ';'); val_ptr != NULL;
+            i++) {
          struct AttrRec *attr_ptr;
 
          if (c_ptr != NULL) *c_ptr = '\0';
 
          if (col_names[i] != NULL && (attr_ptr=FindAttrWithName(obj_ptr,
                col_names[i], NULL)) != NULL) {
+            if (!tab_separated) UtilTrimBlanks(val_ptr);
             if (strip_double_quotes && *val_ptr == '"') {
                int len=strlen(val_ptr);
 
@@ -1283,9 +1326,9 @@ void MergeWithTable()
             }
          }
          if (c_ptr == NULL) break;
-         *c_ptr++ = '\t';
+         *c_ptr++ = (tab_separated ? '\t' : ';');
          val_ptr = c_ptr;
-         c_ptr = strchr(val_ptr, '\t');
+         c_ptr = strchr(val_ptr, tab_separated ? '\t' : ';');
       }
       replaceAttrFirstValueRedraw = TRUE;
       UnlinkObj(obj_ptr);
@@ -1305,7 +1348,8 @@ void MergeWithTable()
          if (user_placement) {
             if ((attr_ptr=FindAttrWithName(obj_ptr, "name=",
                   NULL)) != NULL) {
-               sprintf(gszMsgBox, "Placing object '%s'...", attr_ptr->s);
+               sprintf(gszMsgBox, "Placing object '%s'...",
+                     attr_ptr->attr_value.s);
                SetStringStatus(gszMsgBox);
                XSync(mainDisplay, False);
             }
@@ -1317,14 +1361,16 @@ void MergeWithTable()
          }
          AdjObjBBox(obj_ptr);
          AddObj(NULL, topObj, obj_ptr);
+         numRedrawBBox = 0;
+         obj_ptr->tmp_parent = NULL;
          DrawObj(drawWindow, obj_ptr);
          if ((attr_ptr=FindAttrWithName(obj_ptr, "EXEC_AFTER_MERGE=",
                NULL)) != NULL) {
             char name[MAXSTRING+1];
             struct AttrRec *next_attr_ptr=NULL;
 
-            if (*attr_ptr->s != '\0') {
-               sprintf(name, "%s=", attr_ptr->s);
+            if (*attr_ptr->attr_value.s != '\0') {
+               sprintf(name, "%s=", attr_ptr->attr_value.s);
                if ((next_attr_ptr=FindAttrWithName(obj_ptr, name, NULL)) !=
                      NULL) {
                   int saved_intr_check_interval=intrCheckInterval;
@@ -1338,7 +1384,7 @@ void MergeWithTable()
                   intrCheckInterval = saved_intr_check_interval;
                } else {
                   sprintf(gszMsgBox, "%s '%s'.\n\nMerging aborted.",
-                        "Can not find attribute named", attr_ptr->s);
+                        "Can not find attribute named", attr_ptr->attr_value.s);
                   MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
                   ok = FALSE;
                }
@@ -1376,7 +1422,7 @@ void MergeWithTable()
             }
          }
       }
-      cfree(buf);
+      free(buf);
    }
    fclose(fp);
 
@@ -1387,10 +1433,10 @@ void MergeWithTable()
    FreeObj(template_obj);
    for (i=0; i < columns_in_file; i++) {
       if (col_names[i] != NULL) {
-         cfree(col_names[i]);
+         free(col_names[i]);
       }
    }
-   cfree(col_names);
+   free(col_names);
    if (firstCmd != NULL) CleanUpCmds();
    SetFileModified(TRUE);
    justDupped = FALSE;
@@ -1410,7 +1456,8 @@ void ExportToTable()
       MsgBox("No object selected.\n\nNothing to export.", TOOL_NAME, INFO_MB);
       return;
    }
-   spec_attr = FindAttrWithName(NULL, "!.TABLE_ATTRS=", NULL);
+   strcpy(msg, "!.TABLE_ATTRS=");
+   spec_attr = FindAttrWithName(NULL, msg, NULL);
    if (spec_attr == NULL) {
       sprintf(gszMsgBox, "%s \"!.TABLE_ATTRS\" %s.\n\n%s.",
             "Can not find the", "file attribute",
@@ -1467,9 +1514,9 @@ void ExportToTable()
    total = 0;
    for (s_ptr=spec_attr->obj->detail.t->first; s_ptr != NULL;
          s_ptr=s_ptr->next) {
-      total += strlen(s_ptr->s)+1;
+      total += strlen(s_ptr->dyn_str.s)+1;
    }
-   if ((spec=(char*)calloc(1, total+1)) == NULL) {
+   if ((spec=(char*)malloc((total+1)*sizeof(char))) == NULL) {
       FailAllocMessage();
       fclose(fp);
       return;
@@ -1477,16 +1524,17 @@ void ExportToTable()
    c_ptr = spec;
    for (s_ptr=spec_attr->obj->detail.t->first; s_ptr != NULL;
          s_ptr=s_ptr->next) {
-      int len=(c_ptr==spec ? strlen(spec_attr->s) : strlen(s_ptr->s));
+      int len=(c_ptr==spec ? strlen(spec_attr->attr_value.s) :
+            strlen(s_ptr->dyn_str.s));
 
-      strcpy(c_ptr, (c_ptr==spec ? spec_attr->s : s_ptr->s));
+      strcpy(c_ptr, (c_ptr==spec ? spec_attr->attr_value.s : s_ptr->dyn_str.s));
       c_ptr += len;
       *c_ptr++ = ',';
    }
    *c_ptr = '\0';
    if ((spec_copy=UtilStrDup(spec)) == NULL) {
       FailAllocMessage();
-      cfree(spec);
+      free(spec);
       fclose(fp);
       return;
    }
@@ -1495,17 +1543,18 @@ void ExportToTable()
          c_ptr=strtok(NULL, " ,;\t\n\r")) {
       total++;
    }
-   cfree(spec_copy);
-   spec_copy = UtilStrDup(spec);
-   col_names = (char**)calloc(total+1, sizeof(char*));
+   free(spec_copy);
+
    buf_sz = 0x400;
-   buf = (char*)calloc(buf_sz+2, sizeof(char));
+   spec_copy = UtilStrDup(spec);
+   col_names = (char**)malloc((total+1)*sizeof(char*));
+   buf = (char*)malloc((buf_sz+2)*sizeof(char));
    if (spec_copy == NULL || col_names == NULL || buf == NULL) {
       FailAllocMessage();
-      if (col_names != NULL) cfree(col_names);
-      if (spec_copy != NULL) cfree(spec_copy);
-      if (buf != NULL) cfree(buf);
-      cfree(spec);
+      if (col_names != NULL) free(col_names);
+      if (spec_copy != NULL) free(spec_copy);
+      if (buf != NULL) free(buf);
+      free(spec);
       fclose(fp);
       return;
    }
@@ -1515,13 +1564,13 @@ void ExportToTable()
          c_ptr=strtok(NULL, " ,;\t\n\r"), i++) {
       int len=strlen(c_ptr);
 
-      if ((col_names[i]=(char*)calloc(len+2, sizeof(char))) == NULL) {
+      if ((col_names[i]=(char*)malloc((len+2)*sizeof(char))) == NULL) {
          FailAllocMessage();
-         for (total=0; total < i; total++) cfree(col_names[i]);
-         cfree(col_names);
-         cfree(spec_copy);
-         cfree(buf);
-         cfree(spec);
+         for (total=0; total < i; total++) free(col_names[i]);
+         free(col_names);
+         free(spec_copy);
+         free(buf);
+         free(spec);
          fclose(fp);
          return;
       }
@@ -1550,7 +1599,7 @@ void ExportToTable()
          if (attr_ptr != NULL) something_exported = TRUE;
 
          sprintf(gszMsgBox, "%s%s", (col==0 ? "" : "\t"),
-               (attr_ptr==NULL ? "": attr_ptr->s));
+               (attr_ptr==NULL ? "": attr_ptr->attr_value.s));
          len = strlen(gszMsgBox);
          while (len+cur_len >= buf_sz) {
             buf_sz += 0x400;
@@ -1579,11 +1628,11 @@ void ExportToTable()
    sprintf(gszMsgBox, "%1d objects exported.", num_exported);
    Msg(gszMsgBox);
 
-   for (i=0; i < total; i++) cfree(col_names[i]);
-   cfree(col_names);
-   cfree(spec_copy);
-   if (buf != NULL) cfree(buf);
-   cfree(spec);
+   for (i=0; i < total; i++) free(col_names[i]);
+   free(col_names);
+   free(spec_copy);
+   if (buf != NULL) free(buf);
+   free(spec);
    fclose(fp);
 
    if (writeFileFailed) {
@@ -1628,6 +1677,9 @@ void SpecialSubMenu (index)
       case SPECIAL_EXPORT_ATTRS: ExportAttrs (); break;
       case SPECIAL_MERGE_WITH_TABLE: MergeWithTable (); break;
       case SPECIAL_EXPORT_TO_TABLE: ExportToTable (); break;
+#ifdef _TGIF_WB
+      case SPECIAL_WHITE_BOARD: WhiteBoard (); break;
+#endif /* _TGIF_WB */
    }
 }
 
