@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,7 +29,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/spline.c,v 2.46 1995/05/15 01:23:15 william Exp $";
+      "@(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/spline.c,v 3.1 1996/10/30 22:59:52 william Exp $";
 #endif
 
 #include <math.h>
@@ -41,6 +41,7 @@ static char RCSid[] =
 #include "types.h"
 
 #include "dialog.e"
+#include "msg.e"
 #include "poly.e"
 #include "polygon.e"
 #include "raster.e"
@@ -149,7 +150,7 @@ void SetSplineVs (N, MaxN, X1, Y1, X2, Y2, X3, Y3, X4, Y4)
 
 XPoint * MakeSplinePolyVertex (N, XOff, YOff, NumVs, Vs)
    int		* N, XOff, YOff, NumVs;
-   XPoint	* Vs;
+   IntPoint	* Vs;
 {
    double	mx1, my1, mx2, my2, mx3, my3, mx4, my4, x1, y1, x2, y2;
    int		i, x_off, y_off, max_n;
@@ -165,10 +166,9 @@ XPoint * MakeSplinePolyVertex (N, XOff, YOff, NumVs, Vs)
       case 1:
          break;
       case 2:
-         splineVs = (XPoint *) calloc (NumVs+1, sizeof (XPoint));
-         if (splineVs == NULL)
-         {
-            fprintf (stderr, "Can not calloc() for open splines.\n");
+         splineVs = (XPoint*)malloc((NumVs+1)*sizeof(XPoint));
+         if (splineVs == NULL) {
+            FailAllocMessage();
             *N = 0;
             return (splineVs);
          }
@@ -185,10 +185,9 @@ XPoint * MakeSplinePolyVertex (N, XOff, YOff, NumVs, Vs)
          mx4 = ZOOMED_SIZE(Vs->x-x_off); my4 = ZOOMED_SIZE(Vs->y-y_off);
          mx3 = (x1+mx4)/2.0;             my3 = (y1+my4)/2.0;
          max_n = 100;
-         splineVs = (XPoint *) calloc (max_n+1, sizeof (XPoint));
-         if (splineVs == NULL)
-         {
-            fprintf (stderr, "Can not calloc() for open splines.\n");
+         splineVs = (XPoint*)malloc((max_n+1)*sizeof(XPoint));
+         if (splineVs == NULL) {
+            FailAllocMessage();
             *N = 0;
             return (splineVs);
          }
@@ -205,10 +204,9 @@ XPoint * MakeSplinePolyVertex (N, XOff, YOff, NumVs, Vs)
          mx3 = (3.0*x1+x2)/4.0;          my3 = (3.0*y1+y2)/4.0;
          mx4 = (x1+x2)/2.0;              my4 = (y1+y2)/2.0;
          max_n = 100;
-         splineVs = (XPoint *) calloc (max_n+1, sizeof (XPoint));
-         if (splineVs == NULL)
-         {
-            fprintf (stderr, "Can not calloc() for open splines.\n");
+         splineVs = (XPoint *)malloc((max_n+1)*sizeof(XPoint));
+         if (splineVs == NULL) {
+            FailAllocMessage();
             *N = 0;
             return (splineVs);
          }
@@ -246,7 +244,7 @@ typedef struct MultiSplineRec {
 XPoint * MakeMultiSplinePolyVertex (N, Smooth, XOff, YOff, NumVs, Vs)
    int		* N, XOff, YOff, NumVs;
    char		* Smooth;
-   XPoint	* Vs;
+   IntPoint	* Vs;
 {
    register int	i, j;
    int		segments=1, has_smooth_point=FALSE, start_index, seg_index;
@@ -262,8 +260,12 @@ XPoint * MakeMultiSplinePolyVertex (N, Smooth, XOff, YOff, NumVs, Vs)
       fprintf (stderr, "Please try to reproduce this error and\n");
       fprintf (stderr, "\tsend bug report to william@cs.ucla.edu.\n");
       fflush (stderr);
-      Dialog ("Possible fatal error.  Please read the terminal error output.",
-            NULL, NULL);
+      sprintf (gszMsgBox, "%s.\n\n%s.\n\n%s %s.",
+            "Bad poly in MakeMultiSplinePolyVertex()",
+            "Safest thing to do is to save file and exit",
+            "Please try to reproduce this error and",
+            "send bug report to william@cs.ucla.edu");
+      MsgBox (gszMsgBox, TOOL_NAME, STOP_MB);
       return (NULL);
    }
    for (i=1; i < NumVs-1; i++)
@@ -278,10 +280,12 @@ XPoint * MakeMultiSplinePolyVertex (N, Smooth, XOff, YOff, NumVs, Vs)
       return MakePolyVertex (XOff, YOff, NumVs, Vs);
    }
    if (segments == 1) return MakeSplinePolyVertex (N, XOff, YOff, NumVs, Vs);
-   msptr = (struct MultiSplineRec *) calloc (segments,
+   msptr = (struct MultiSplineRec *)malloc(segments *
          sizeof(struct MultiSplineRec));
-   if (msptr == NULL) return NULL;
-
+   if (msptr == NULL) {
+      FailAllocMessage();
+      return NULL;
+   }
    for (i=0; i < segments; i++) msptr[i].vlist = NULL;
    start_index = 0;
    seg_index = 0;
@@ -297,30 +301,28 @@ XPoint * MakeMultiSplinePolyVertex (N, Smooth, XOff, YOff, NumVs, Vs)
       }
    }
    if (total > 0) total++;
-   splineVs = (XPoint *) calloc (total+2, sizeof (XPoint));
+   splineVs = (XPoint *)malloc((total+2)*sizeof(XPoint));
+   if (splineVs == NULL) FailAllocMessage();
    xpptr = splineVs;
-   for (i=0; i < segments; i++)
-   {
-      if (msptr[i].vlist != NULL)
-      {
-         for (j=0; j < msptr[i].n; j++)
-         {
+   for (i=0; i < segments; i++) {
+      if (msptr[i].vlist != NULL) {
+         for (j=0; j < msptr[i].n; j++) {
             xpptr->x = msptr[i].vlist[j].x;
             xpptr->y = msptr[i].vlist[j].y;
             xpptr++;
          }
          xpptr--;
-         cfree (msptr[i].vlist);
+         free(msptr[i].vlist);
       }
    }
-   cfree (msptr);
+   free(msptr);
    *N = total;
    return (splineVs);
 }
 
 XPoint * MakeSplinePolygonVertex (N, XOff, YOff, NumVs, Vs)
    int		* N, XOff, YOff, NumVs;
-   XPoint	* Vs;
+   IntPoint	* Vs;
 {
    double	mx1, my1, mx2, my2, mx3, my3, mx4, my4, x1, y1, x2, y2;
    int		i, max_n, x_off, y_off;
@@ -332,10 +334,9 @@ XPoint * MakeSplinePolygonVertex (N, XOff, YOff, NumVs, Vs)
 
    if (NumVs <= 3)
    {
-      splineVs = (XPoint *) calloc (5, sizeof (XPoint));
-      if (splineVs == NULL)
-      {
-         fprintf (stderr, "Can not calloc() for open splines.\n");
+      splineVs = (XPoint *)malloc(5*sizeof(XPoint));
+      if (splineVs == NULL) {
+         FailAllocMessage();
          *N = 0;
          return (splineVs);
       }
@@ -353,10 +354,9 @@ XPoint * MakeSplinePolygonVertex (N, XOff, YOff, NumVs, Vs)
    mx4 = (x1 + x2) / 2.0;         my4 = (y1 + y2) / 2.0;
 
    max_n = 100;
-   splineVs = (XPoint *) calloc (max_n+1, sizeof (XPoint));
-   if (splineVs == NULL)
-   {
-      fprintf (stderr, "Can not calloc() for closed splines.\n");
+   splineVs = (XPoint*)malloc((max_n+1)*sizeof(XPoint));
+   if (splineVs == NULL) {
+      FailAllocMessage();
       *N = 0;
       return (splineVs);
    }
@@ -380,13 +380,14 @@ XPoint * MakeSplinePolygonVertex (N, XOff, YOff, NumVs, Vs)
 XPoint * MakeMultiSplinePolygonVertex (N, Smooth, XOff, YOff, NumVs, Vs)
    int		* N, XOff, YOff, NumVs;
    char		* Smooth;
-   XPoint	* Vs;
+   IntPoint	* Vs;
 {
    register int	i, j;
    int		num_smooth_points=0, num_hinge_points=0;
    int		start_index, seg_index, tmp_index;
    int		total=0, once_around=FALSE;
-   XPoint	* xpptr, * tmp_vs;
+   XPoint	*xpptr;
+   IntPoint	*tmp_vs;
    struct MultiSplineRec	* msptr;
 
    if (Smooth == NULL)
@@ -406,16 +407,16 @@ XPoint * MakeMultiSplinePolygonVertex (N, Smooth, XOff, YOff, NumVs, Vs)
    {
       return MakeSplinePolygonVertex (N, XOff, YOff, NumVs, Vs);
    }
-   msptr = (struct MultiSplineRec *) calloc (num_hinge_points,
+   msptr = (struct MultiSplineRec *)malloc(num_hinge_points *
          sizeof(struct MultiSplineRec));
-   if (msptr == NULL) fprintf (stderr, "Can not alloc().\n");
+   if (msptr == NULL) FailAllocMessage();
    for (i=0; i < num_hinge_points; i++) msptr[i].vlist = NULL;
 
    for (i=0; i < NumVs; i++)
       if (!Smooth[i])
          break;
-   tmp_vs = (XPoint *) calloc (NumVs+1, sizeof(XPoint));
-   if (tmp_vs == NULL) fprintf(stderr, "Can not calloc().\n");
+   tmp_vs = (IntPoint*)malloc((NumVs+1)*sizeof(IntPoint));
+   if (tmp_vs == NULL) FailAllocMessage();
    start_index = i;
    seg_index = 0;
 
@@ -443,25 +444,23 @@ XPoint * MakeMultiSplinePolygonVertex (N, Smooth, XOff, YOff, NumVs, Vs)
          once_around = TRUE;
       }
    }
-   if (tmp_vs != NULL) cfree (tmp_vs);
+   if (tmp_vs != NULL) free(tmp_vs);
    if (total > 0) total++;
-   splineVs = (XPoint *) calloc (total+2, sizeof (XPoint));
+   splineVs = (XPoint*)malloc((total+2)*sizeof(XPoint));
+   if (splineVs == NULL) FailAllocMessage();
    xpptr = splineVs;
-   for (i=0; i < num_hinge_points; i++)
-   {
-      if (msptr[i].vlist != NULL)
-      {
-         for (j=0; j < msptr[i].n; j++)
-         {
+   for (i=0; i < num_hinge_points; i++) {
+      if (msptr[i].vlist != NULL) {
+         for (j=0; j < msptr[i].n; j++) {
             xpptr->x = msptr[i].vlist[j].x;
             xpptr->y = msptr[i].vlist[j].y;
             xpptr++;
          }
          xpptr--;
-         cfree (msptr[i].vlist);
+         free(msptr[i].vlist);
       }
    }
-   cfree (msptr);
+   free(msptr);
    *N = total;
    return (splineVs);
 }
@@ -474,22 +473,28 @@ struct MtxRec {
 static
 void OpenSetupMatrix (NumPts, Vs)
    int			NumPts;
-   XPoint		* Vs;
+   IntPoint		* Vs;
 {
    register int		i;
 
-   mtxInfo.x = (double*)calloc(NumPts,sizeof(double));
-   mtxInfo.y = (double*)calloc(NumPts,sizeof(double));
-   mtxInfo.dx = (double*)calloc(NumPts,sizeof(double));
-   mtxInfo.dy = (double*)calloc(NumPts,sizeof(double));
-   for (i=0; i < NumPts; i++)
-   {
+   mtxInfo.x = (double*)malloc(NumPts*sizeof(double));
+   mtxInfo.y = (double*)malloc(NumPts*sizeof(double));
+   mtxInfo.dx = (double*)malloc(NumPts*sizeof(double));
+   mtxInfo.dy = (double*)malloc(NumPts*sizeof(double));
+   if (mtxInfo.x == NULL || mtxInfo.y == NULL || mtxInfo.dx == NULL ||
+         mtxInfo.dy == NULL) {
+      FailAllocMessage();
+   }
+   for (i=0; i < NumPts; i++) {
       mtxInfo.x[i] = mtxInfo.dx[i] = ((double)(Vs[i].x))*((double)theSum);
       mtxInfo.y[i] = mtxInfo.dy[i] = ((double)(Vs[i].y))*((double)theSum);
    }
-   mtxInfo.mtx = (double**)calloc(NumPts,sizeof(double*));
-   for (i=0; i < NumPts; i++)
-      mtxInfo.mtx[i] = (double*)calloc(3,sizeof(double));
+   mtxInfo.mtx = (double**)malloc(NumPts*sizeof(double*));
+   if (mtxInfo.mtx == NULL) FailAllocMessage();
+   for (i=0; i < NumPts; i++) {
+      mtxInfo.mtx[i] = (double*)malloc(3*sizeof(double));
+      if (mtxInfo.mtx[i] == NULL) FailAllocMessage();
+   }
    mtxInfo.mtx[0][0] = mtxInfo.mtx[NumPts-1][2] = (double)0.0;
    mtxInfo.mtx[0][1] = mtxInfo.mtx[NumPts-1][1] = (double)theSum;
    mtxInfo.mtx[0][2] = mtxInfo.mtx[NumPts-1][0] = (double)0.0;
@@ -534,25 +539,26 @@ void FreeMtxInfo (NumPts)
 {
    register int	i;
 
-   cfree (mtxInfo.x); mtxInfo.x = NULL;
-   cfree (mtxInfo.y); mtxInfo.y = NULL;
-   cfree (mtxInfo.dx); mtxInfo.dx = NULL;
-   cfree (mtxInfo.dy); mtxInfo.dy = NULL;
-   for (i=0; i < NumPts; i++) cfree (mtxInfo.mtx[i]);
-   cfree (mtxInfo.mtx); mtxInfo.mtx = NULL;
+   free(mtxInfo.x); mtxInfo.x = NULL;
+   free(mtxInfo.y); mtxInfo.y = NULL;
+   free(mtxInfo.dx); mtxInfo.dx = NULL;
+   free(mtxInfo.dy); mtxInfo.dy = NULL;
+   for (i=0; i < NumPts; i++) free(mtxInfo.mtx[i]);
+   free(mtxInfo.mtx); mtxInfo.mtx = NULL;
 }
 
 static
-XPoint * OpenControlPts (NumPts, N)
+IntPoint * OpenControlPts (NumPts, N)
    int	NumPts, * N;
 {
    register int	i;
    int		index=0;
    double	half=theSum/((double)2.0);
    double	weight=half-((double)1.0);
-   XPoint	* v;
+   IntPoint	*v;
 
-   v = (XPoint*) calloc (((NumPts-2)<<1)+2, sizeof(XPoint));
+   v = (IntPoint*)malloc((((NumPts-2)<<1)+2)*sizeof(IntPoint));
+   if (v == NULL) FailAllocMessage();
    v[index].x = (int)(mtxInfo.x[0]);
    v[index].y = (int)(mtxInfo.y[0]);
    index++;
@@ -582,7 +588,7 @@ XPoint * OpenControlPts (NumPts, N)
 
 XPoint * MakeIntSplinePolyVertex (N, CntrlN, CntrlVs, XOff, YOff, NumVs, Vs)
    int		* N, * CntrlN, XOff, YOff, NumVs;
-   XPoint	* * CntrlVs, * Vs;
+   IntPoint	* * CntrlVs, * Vs;
 {
    int x_off, y_off;
 
@@ -597,11 +603,11 @@ XPoint * MakeIntSplinePolyVertex (N, CntrlN, CntrlVs, XOff, YOff, NumVs, Vs)
       case 1:
          break;
       case 2:
-         *CntrlVs = (XPoint *) calloc (NumVs+1, sizeof (XPoint));
-         splineVs = (XPoint *) calloc (NumVs+1, sizeof (XPoint));
-         if (splineVs == NULL)
-         {
-            fprintf (stderr, "Can not calloc() for open splines.\n");
+         *CntrlVs = (IntPoint*)malloc((NumVs+1)*sizeof(IntPoint));
+         if (*CntrlVs == NULL) FailAllocMessage();
+         splineVs = (XPoint*)malloc((NumVs+1)*sizeof(XPoint));
+         if (splineVs == NULL) {
+            FailAllocMessage();
             *N = 0;
             return (splineVs);
          }
@@ -628,24 +634,30 @@ XPoint * MakeIntSplinePolyVertex (N, CntrlN, CntrlVs, XOff, YOff, NumVs, Vs)
 static
 void ClosedSetupMatrix (NumPts, Vs)
    int			NumPts;
-   XPoint		* Vs;
+   IntPoint		* Vs;
 {
    register int		i;
    register double	val;
 
-   mtxInfo.x = (double*)calloc(NumPts,sizeof(double));
-   mtxInfo.y = (double*)calloc(NumPts,sizeof(double));
-   mtxInfo.dx = (double*)calloc(NumPts,sizeof(double));
-   mtxInfo.dy = (double*)calloc(NumPts,sizeof(double));
-   for (i=0; i < NumPts; i++)
-   {
+   mtxInfo.x = (double*)malloc(NumPts*sizeof(double));
+   mtxInfo.y = (double*)malloc(NumPts*sizeof(double));
+   mtxInfo.dx = (double*)malloc(NumPts*sizeof(double));
+   mtxInfo.dy = (double*)malloc(NumPts*sizeof(double));
+   if (mtxInfo.x == NULL || mtxInfo.y == NULL || mtxInfo.dx == NULL ||
+         mtxInfo.dy == NULL) {
+      FailAllocMessage();
+   }
+   for (i=0; i < NumPts; i++) {
       mtxInfo.x[i] = mtxInfo.dx[i] = ((double)(Vs[i].x))*((double)theSum);
       mtxInfo.y[i] = mtxInfo.dy[i] = ((double)(Vs[i].y))*((double)theSum);
    }
    /* the first NumPts-2 rows have an extra column */
-   mtxInfo.mtx = (double**)calloc(NumPts,sizeof(double*));
-   for (i=0; i < NumPts; i++)
-      mtxInfo.mtx[i] = (double*)calloc(4,sizeof(double));
+   mtxInfo.mtx = (double**)malloc(NumPts*sizeof(double*));
+   if (mtxInfo.mtx == NULL) FailAllocMessage();
+   for (i=0; i < NumPts; i++) {
+      mtxInfo.mtx[i] = (double*)malloc(4*sizeof(double));
+      if (mtxInfo.mtx[i] == NULL) FailAllocMessage();
+   }
    mtxInfo.mtx[0][0] = mtxInfo.mtx[NumPts-1][2] = (double)1.0;
    mtxInfo.mtx[0][1] = mtxInfo.mtx[NumPts-1][1] = (double)SUM_MINUS_2;
    mtxInfo.mtx[0][2] = mtxInfo.mtx[NumPts-1][0] = (double)1.0;
@@ -786,16 +798,17 @@ void Gaussian (NumPts)
 }
 
 static
-XPoint * ClosedControlPts (NumPts, N)
+IntPoint * ClosedControlPts (NumPts, N)
    int	NumPts, * N;
 {
    register int	i;
    int		index=0;
    double	half=theSum/((double)2.0);
    double	weight=half-((double)1.0);
-   XPoint	* v;
+   IntPoint	*v;
 
-   v = (XPoint*) calloc ((NumPts<<1)+2, sizeof(XPoint));
+   v = (IntPoint*)malloc(((NumPts<<1)+2)*sizeof(IntPoint));
+   if (v == NULL) FailAllocMessage();
    for (i=0; i<NumPts; i++) {
       v[index].x = (int)((weight*mtxInfo.x[i]+mtxInfo.x[(i+1) % NumPts])/half);
       v[index].y = (int)((weight*mtxInfo.y[i]+mtxInfo.y[(i+1) % NumPts])/half);
@@ -816,7 +829,7 @@ XPoint * ClosedControlPts (NumPts, N)
 
 XPoint * MakeIntSplinePolygonVertex (N, CntrlN, CntrlVs, XOff, YOff, NumVs, Vs)
    int		* N, * CntrlN, XOff, YOff, NumVs;
-   XPoint	* * CntrlVs, * Vs;
+   IntPoint	* * CntrlVs, * Vs;
 {
    int	x_off, y_off;
 
@@ -827,10 +840,9 @@ XPoint * MakeIntSplinePolygonVertex (N, CntrlN, CntrlVs, XOff, YOff, NumVs, Vs)
 
    if (NumVs <= 3)
    {
-      splineVs = (XPoint *) calloc (5, sizeof (XPoint));
-      if (splineVs == NULL)
-      {
-         fprintf (stderr, "Can not calloc() for open splines.\n");
+      splineVs = (XPoint *)malloc(5*sizeof(XPoint));
+      if (splineVs == NULL) {
+         FailAllocMessage();
          *N = 0;
          return (splineVs);
       }
@@ -852,7 +864,7 @@ XPoint * MakeIntSplinePolygonVertex (N, CntrlN, CntrlVs, XOff, YOff, NumVs, Vs)
 void DumpCurvedPolyPoints (FP, NumPts, V, Indent)
    FILE			* FP;
    int			NumPts, Indent;
-   register XPoint	* V;
+   register IntPoint	* V;
 {
    register int	j, i;
    double	x1, y1, x2, y2;
@@ -886,7 +898,6 @@ void DumpCurvedPolyPoints (FP, NumPts, V, Indent)
       
          for (i = 2; i < NumPts-2; i++, V++)
          {
-/*          mx1 = mx4; my1 = my4; */
             mx2 = (x1 + 5.0*x2) / 6.0; my2 = (y1 + 5.0*y2) / 6.0;
             x1 = x2; y1 = y2;
 #ifdef stellar
@@ -904,7 +915,6 @@ void DumpCurvedPolyPoints (FP, NumPts, V, Indent)
             x2 = V->x; y2 = V->y;
 #endif
          }
-/*       mx1 = mx4; my1 = my4; */
          mx2 = (x1 + 5.0*x2) / 6.0; my2 = (y1 + 5.0*y2) / 6.0;
          x1 = x2; y1 = y2;
          mx3 = (2.0*x1 + V->x) / 3.0; my3 = (2.0*y1 + V->y) / 3.0;
@@ -917,7 +927,7 @@ void DumpCurvedPolyPoints (FP, NumPts, V, Indent)
 void DumpCurvedPolygonPoints (FP, NumPts, V, Indent)
    FILE			* FP;
    int			NumPts, Indent;
-   register XPoint	* V;
+   register IntPoint	* V;
 {
    register int	j;
    double	mx2, my2, mx3, my3, mx4, my4, x1, y1, x2, y2;
@@ -932,7 +942,6 @@ void DumpCurvedPolygonPoints (FP, NumPts, V, Indent)
 
    for (i = 1; i < NumPts; i++, V++)
    {
-/*    mx1 = mx4;             my1 = my4; */
       mx2 = (x1+5.0*x2)/6.0; my2 = (y1+5.0*y2)/6.0;
       x1 = x2;               y1 = y2;
 #ifdef stellar
@@ -956,7 +965,7 @@ void DumpMultiCurvedPolyPoints (FP, Smooth, Style, Curved, NumPts, V, Indent)
    FILE			* FP;
    char			* Smooth;
    int			Style, Curved, NumPts, Indent;
-   register XPoint	* V;
+   register IntPoint	* V;
 {
    register int	i, j;
    int		segments=1, has_smooth_point=FALSE, start_index;
@@ -973,8 +982,12 @@ void DumpMultiCurvedPolyPoints (FP, Smooth, Style, Curved, NumPts, V, Indent)
       fprintf (stderr, "Please try to reproduce this error and\n");
       fprintf (stderr, "\tsend bug report to william@cs.ucla.edu.\n");
       fflush (stderr);
-      Dialog ("Possible fatal error.  Please read the terminal error output.",
-            NULL, NULL);
+      sprintf (gszMsgBox, "%s.\n\n%s.\n\n%s %s.",
+            "Bad poly in DumpMultiCurvedPolyPoints()",
+            "Safest thing to do is to save file and exit",
+            "Please try to reproduce this error and",
+            "send bug report to william@cs.ucla.edu");
+      MsgBox (gszMsgBox, TOOL_NAME, STOP_MB);
       return;
    }
    for (i=1; i < NumPts-1; i++)
@@ -1037,13 +1050,13 @@ void DumpMultiCurvedPolygonPoints (FP, Smooth, Curved, NumPts, V, Indent)
    FILE			* FP;
    char			* Smooth;
    int			Curved, NumPts, Indent;
-   register XPoint	* V;
+   register IntPoint	* V;
 {
    register int	i, j;
    int		num_smooth_points=0, num_hinge_points=0, tmp_index;
    int		start_index;
    int		once_around=FALSE;
-   XPoint	* tmp_vs;
+   IntPoint	*tmp_vs;
 
    if (Curved == LT_INTSPLINE || Smooth == NULL)
    {
@@ -1068,13 +1081,14 @@ void DumpMultiCurvedPolygonPoints (FP, Smooth, Curved, NumPts, V, Indent)
       DumpCurvedPolygonPoints (FP, NumPts, V, Indent);
       return;
    }
-   tmp_vs = (XPoint *) calloc (NumPts+1, sizeof(XPoint));
-   if (tmp_vs == NULL) fprintf(stderr, "Can not calloc().\n");
+   tmp_vs = (IntPoint*)malloc((NumPts+1)*sizeof(IntPoint));
+   if (tmp_vs == NULL) FailAllocMessage();
 
-   for (i=0; i < NumPts; i++)
-      if (!Smooth[i])
+   for (i=0; i < NumPts; i++) {
+      if (!Smooth[i]) {
          break;
-
+      }
+   }
    for (j=0; j < Indent; j++) fprintf (FP, " ");
    fprintf (FP, "%1d %1d moveto\n", V[i].x, V[i].y);
    start_index = i;
@@ -1106,6 +1120,6 @@ void DumpMultiCurvedPolygonPoints (FP, Smooth, Curved, NumPts, V, Indent)
          once_around = TRUE;
       }
    }
-   if (tmp_vs != NULL) cfree (tmp_vs);
+   if (tmp_vs != NULL) free(tmp_vs);
 }
 
