@@ -5,7 +5,7 @@
  *
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1996, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -33,7 +33,7 @@
  */
 #ifndef lint
 static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/tcp.c,v 2.13 1995/05/15 01:23:22 william Exp $";
+      "@(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/tcp.c,v 3.0 1996/05/06 22:06:41 william Exp $";
 #endif
 
 #include <sys/types.h>
@@ -41,11 +41,16 @@ static char RCSid[] =
 #include <sys/time.h>
 #include <netinet/in.h>
 #include <netdb.h>
+#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <fcntl.h>
+#ifdef isc /* SunSoft/Interactive UNIX */
+#include <net/errno.h>
+#include <sys/bsdtypes.h>
+#endif
 
 #ifndef O_NONBLOCK
 #include <sys/ioctl.h>
@@ -63,7 +68,6 @@ static char RCSid[] =
 #endif /* !_NO_EXTERN */
 #include "util.e"
 
-static
 void SetSocketBlockingState(pn_socket, n_blocking)
    int *pn_socket, n_blocking;
 {
@@ -87,18 +91,35 @@ void SetSocketBlockingState(pn_socket, n_blocking)
 void TcpFreeBuf(buf)
    char *buf;
 {
-   cfree(buf);
+   free(buf);
+}
+
+static gnPipeBroken=FALSE;
+
+static
+void BrokenPipe(nSig)
+   int nSig;
+{
+   if (nSig == SIGPIPE) {
+      gnPipeBroken = TRUE;
+      signal(SIGPIPE, SIG_DFL);
+   }
 }
 
 int TcpDoConnect(psz_host, us_port, pn_socket)
    char *psz_host;
    int us_port, *pn_socket;
 {
+   static int not_initialized=TRUE;
    struct sockaddr_in soc_address;
    struct sockaddr_in *sin=(&soc_address);
    struct hostent *p_hostent=NULL;
    int status=TG_REMOTE_STATUS_OK;
 
+   if (not_initialized) {
+      not_initialized = FALSE;
+      signal(SIGPIPE, BrokenPipe);
+   }
    if (*psz_host >= '0' && *psz_host <= '9') {
       sin->sin_addr.s_addr = inet_addr(psz_host);
    } else {
@@ -141,8 +162,13 @@ int TcpDoConnect(psz_host, us_port, pn_socket)
             status = rc;
             break;
          } else if (rc > 0) {
+            gnPipeBroken = FALSE;
             status = connect(*pn_socket, (struct sockaddr*)&soc_address,
                   sizeof(soc_address));
+            if (gnPipeBroken) {
+               fprintf(stderr, "Broken pipe while contacting '%s'.\n",
+                     psz_host);
+            }
             if ((status < 0)&&(errno == EISCONN)) status = TG_REMOTE_STATUS_OK;
             if (errno == EALREADY)
                rc = 0;
@@ -201,7 +227,7 @@ int TcpDoRead(n_socket, ppsz_buf, pn_buf_sz)
    char **ppsz_buf;
 {
    int buf_sz=0x400, len=0, end_of_file=FALSE;
-   char *buf=(char*)calloc(buf_sz, sizeof(char));
+   char *buf=(char*)malloc(buf_sz*sizeof(char));
 
    if (pn_buf_sz != NULL) *pn_buf_sz = 0;
    *ppsz_buf = NULL;
@@ -223,11 +249,11 @@ int TcpDoRead(n_socket, ppsz_buf, pn_buf_sz)
       if (bytes_read <= 0) {
          if (bytes_read < 0 && (errno == ENOTCONN || errno == ECONNRESET ||
                errno == EPIPE)) {
-            cfree(buf);
+            free(buf);
             fprintf(stderr, "Network read error.\n");
             return TG_REMOTE_STATUS_READ;
          } else if (bytes_read < 0) {
-            cfree(buf);
+            free(buf);
             fprintf(stderr, "Network error.\n");
             return TG_REMOTE_STATUS_NET;
          }
