@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (william@cs.ucla.edu)
  *
- * Copyright (C) 1990-1995, William Cheng.
+ * Copyright (C) 1990-1997, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -26,14 +26,43 @@
  * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ * @(#)$Header: /u/halfmoon/home/william/src/tgif/v3/RCS/version.c,v 3.6 1997/10/24 02:54:40 william Exp $
  */
-#ifndef lint
-static char RCSid[] =
-      "@(#)$Header: /u/multimedia/william/X11/TGIF2/RCS/version.c,v 2.39 1995/05/15 01:23:34 william Exp $";
-#endif
+
+#include "const.h"
+#include "patchlvl.h"
 
 #ifndef _NO_EXTERN
 #include "version.e"
 #endif
 
-char	* version_string = "2.16";
+char *versionString="3.0";
+char *copyrightString="Copyright (C) 1990-1997, William Chia-Wei Cheng";
+char *homePageURL="http://bourbon.cs.umd.edu:8001/tgif/";
+char *currentReleaseURL="http://bourbon.cs.umd.edu:8001/tgif/current.html";
+char *hyperGraphicsURL="http://bourbon.cs.umd.edu:8001/tgif/index.obj";
+
+char fullVersionString[MAXSTRING];
+char fullToolName[MAXSTRING];
+char *specialBuild="";
+
+void SetFullVersionString()
+{
+   if (TGIF_PATCHLEVEL == 0) {
+      if (*specialBuild == '\0') {
+         sprintf(fullVersionString, "%s", versionString);
+      } else {
+         sprintf(fullVersionString, "%s-%s", versionString, specialBuild);
+      }
+   } else {
+      if (*specialBuild == '\0') {
+         sprintf(fullVersionString, "%s-p%1d", versionString, TGIF_PATCHLEVEL);
+      } else {
+         sprintf(fullVersionString, "%s-p%1d-%s", versionString,
+               TGIF_PATCHLEVEL, specialBuild);
+      }
+   }
+   sprintf(fullToolName, "%s-%s", TOOL_NAME, fullVersionString);
+}
+
